/*===========================================================================*/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*===========================================================================*/
#ifdef _WIN32
  #ifdef _CVI_
    #include <userint.h>
  #else
    #include <windows.h>
  #endif
#endif
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "rsfsh.h"
#include "rsfsh_utility.h"

#define IEEE_ERROR_BITS         0x3C    /* AND mask for ESR checking */
#define BUFFER_SIZE             2048UL    /* File I/O buffer size */

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rsfsh_invalidViBooleanRange (ViBoolean val)
{
    return(ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsfsh_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsfsh_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rsfsh_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}
/*=========================================================================*/
/* Function: Check Status                                                  */
/* Purpose:  This function reads Event Status Register to check for        */
/*           possible errors.                                              */
/*=========================================================================*/
ViStatus rsfsh_checkStatus (ViSession instrSession)
{
    ViStatus            rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange    instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Error Checking Enabled? */
    if (instrPtr != NULL)
    {
        if (instrPtr->errorChecking != VI_FALSE)
        {
            ViInt32 esrByte = 0;

            /*--------------- Read the Standard Event Register ------------------*/
            CHECKERR (viQueryf (instrSession, "*ESR?\n", "%ld", &esrByte));

            /*---------------- Check if any error or message bit was asserted ---*/
            rsfsh_status = ((esrByte & IEEE_ERROR_BITS) != 0) ? RSFSH_ERROR_INSTRUMENT_ERROR : VI_SUCCESS;
        }
    }

    
    return rsfsh_status;
}


/*===========================================================================*/
/* Function: Wait for OPC                                                    */
/* Purpose:  This function sends a command to the instrument and waits until */
/*           the instrument response is available (queries MAV in STB).      */
/*===========================================================================*/
ViStatus rsfsh_waitOPC (ViSession instrSession, ViPString command,
                ViPString instrResp, ViInt32 timeout)
{
    ViStatus    rsfsh_status    = VI_SUCCESS;
    ViUInt16    response        = 0;
    ViInt32     counter         = 0;
    ViInt32     oldTimeout      = 0;
    ViUInt16    intf       = VI_INTF_GPIB;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf));

    /* Interface specific operations */
    switch (intf)
        {
        case VI_INTF_ASRL: /* Serial intf */
            CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, (ViAttrState) timeout));
            if ((rsfsh_status = viQueryf (instrSession, "%s\n", "%[^\r\n]", command, instrResp)) < 0)
            {
                (void) viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, (ViAttrState) oldTimeout);
                return rsfsh_status;
            }
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, (ViAttrState) oldTimeout));
        break;
        default: /* Default case */
            CHECKERR (viPrintf (instrSession, "%s\n", command));

            while ((response & 0x10) == 0) /* (queries MAV in STB) */
                {

                CHECKERR (viReadSTB (instrSession, &response));

                (void) rsfsh_delay (instrSession, 0.005);
                counter+= 5;
                if (counter >= timeout)
                    return VI_ERROR_TMO;
                }

            CHECKERR (viScanf (instrSession, "%[^\r\n]", instrResp));
        break;
        }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Send Command                                                    */
/* Purpose:  This function sends command to the instrument with error check. */
/*===========================================================================*/
ViStatus rsfsh_sendCmd (ViSession instrSession, ViString cmd)
{
    ViStatus    rsfsh_status    = VI_SUCCESS;
    size_t      length          = 0;
    ViChar      *pbuffer;

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc(length + 1);
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;

    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    length++;
    GOCHECKERR (viWrite (instrSession, (ViPBuf) cmd, (ViUInt32) length, VI_NULL));
    GOCHECKERR (rsfsh_checkStatus (instrSession));

ERROR_LABEL:

    free(pbuffer);

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set ViInt32                                                     */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsfsh_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      *wrt_buf;
    ViStatus    rsfsh_status = VI_SUCCESS;

    if ((max - min) > 0)
        if (rsfsh_invalidViInt32Range (value, min, max) == VI_TRUE)
            return parameterErrorCode;

    
    wrt_buf = (ViChar*) malloc (strlen (cmd) + 20);
    
    sprintf (wrt_buf, "%s %ld", cmd, value);

    rsfsh_status = rsfsh_sendCmd (instrSession, wrt_buf);
    
    free (wrt_buf);
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get ViInt32                                                     */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsfsh_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR( viPrintf(instrSession, "%s\n", cmd));
    CHECKERR( viScanf(instrSession, "%ld", value));
    CHECKERR( rsfsh_checkStatus (instrSession));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set ViBoolean                                                   */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsfsh_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViStatus parameterErrorCode)
{
    if (rsfsh_invalidViBooleanRange (value) == VI_TRUE)
        return parameterErrorCode;

    return rsfsh_setViString (instrSession,
                              cmd,
                              (value == VI_TRUE) ? "ON" : "OFF",
                              0);
}

/*===========================================================================*/
/* Function: Get ViBoolean                                                   */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsfsh_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rsfsh_status;
    ViInt32     bool_val = 0;

    if (value == NULL)
        return parameterErrorCode;

    CHECKERR (rsfsh_getViInt32(instrSession,
                               cmd,
                               &bool_val,
                               parameterErrorCode));
    *value = (ViBoolean) bool_val;

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set ViBoolean With Path                                         */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsfsh_setViBooleanPath (ViSession instrSession,
                           ViString cmd,
                           ViInt32 path,
                           ViBoolean value,
                           ViStatus parameterErrorCode)
{
    ViChar  wrt_buf[BUFFER_SIZE];
    ViChar* p2buf;

    CHECKPATH;
    if (rsfsh_invalidViBooleanRange (value) != 0)
        return parameterErrorCode;

    p2buf = wrt_buf + sprintf(wrt_buf,cmd, path);
    sprintf (p2buf, " %s", (value != 0) ? "ON" : "OFF");

    return rsfsh_sendCmd (instrSession, wrt_buf);
}

/*===========================================================================*/
/* Function: Get ViBoolean With Path                                         */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsfsh_getViBooleanPath (ViSession instrSession,
                           ViString cmd,
                           ViInt32 path,
                           ViBoolean *value,
                           ViStatus parameterErrorCode)
{
    ViChar  wrt_buf[BUFFER_SIZE];

    CHECKPATH;

    sprintf(wrt_buf,cmd, path);

    return rsfsh_getViBoolean(instrSession,
                                   wrt_buf,
                                   value,
                                   parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set ViReal64                                                    */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rsfsh_setViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode)
{
    ViChar      buffer[BUFFER_SIZE];

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rsfsh_invalidViReal64Range (value, min, max) == VI_TRUE)
            return parameterErrorCode;

    sprintf(buffer,"%s %E", cmd, value);

    return rsfsh_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get ViReal64                                                    */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rsfsh_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if ( value == VI_NULL )
        return parameterErrorCode;

    CHECKERR(viPrintf(instrSession, "%s\n",cmd));
    CHECKERR(viScanf (instrSession, "%le",value));
    CHECKERR(rsfsh_checkStatus (instrSession));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 With Channel                                       */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rsfsh_setViReal64Path (ViSession instrSession,
                            ViString cmd,
                            ViInt32  path,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    CHECKPATH;

    sprintf(wrt_buf,cmd, path);

    return rsfsh_setViReal64(instrSession,
                             wrt_buf,
                             value,
                             min,
                             max,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get ViReal64 With Channel                                       */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rsfsh_getViReal64Path (ViSession instrSession,
                            ViString cmd,
                            ViInt32 path,
                            ViReal64 *value,
                            ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    CHECKPATH;

    sprintf(wrt_buf, cmd, path);

    return rsfsh_getViReal64(instrSession,
                             wrt_buf,
                             value,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set ViReal64 with Units                                         */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rsfsh_setViReal64Units (ViSession instrSession,
                                 ViString cmd,
                                 ViReal64 value,
                                 ViString units,
                                 ViReal64 min,
                                 ViReal64 max,
                                 ViStatus parameterErrorCode)
{
    ViChar      buffer[BUFFER_SIZE];

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rsfsh_invalidViReal64Range (value, min, max) == VI_TRUE)
            return parameterErrorCode;

    sprintf(buffer,"%s %.12f %s",cmd, value, units);

    return rsfsh_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Set String                                                      */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus rsfsh_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*) malloc (strlen(cmd) + strlen(value) + 4 );
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf, "%s %s\n", cmd, value);

    GOCHECKERR(viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32) (p2buf - wrt_buf), VI_NULL));
    GOCHECKERR(rsfsh_checkStatus(instrSession));

ERROR_LABEL:

    free(wrt_buf);

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get String                                                      */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus rsfsh_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViUInt32 buffer_size,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsfsh_status    = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    ViUInt32    count           = 0;
    size_t      length          = 0;

    if (value == NULL)
        return parameterErrorCode;

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    
    memcpy (pbuffer, cmd, length);
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;
    
    pbuffer[length] = '\n';

    GOCHECKERR (viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) (length + 1), VI_NULL));

    free (pbuffer);
    
    pbuffer = NULL;

    GOCHECKERR (rsfsh_readDataUnknownLength (instrSession, &pbuffer, &count));
    
    if (pbuffer == NULL)
        return rsfsh_status;
    
    GOCHECKERR (rsfsh_checkStatus (instrSession));

    if (count > buffer_size)
    {
        //rsfsh_status = RSFSH_WARNING_SMALL_BUFFER;
        count = buffer_size;
        memcpy (value, pbuffer, (size_t) count);
        value[count] = '\0';
    }
    else
        if ((p2buffer = strpbrk (pbuffer, "\r\n")) != VI_NULL)
        {
            rsfsh_status = VI_SUCCESS;
            memcpy (value, pbuffer, (size_t) (p2buffer - pbuffer));
            value[p2buffer - pbuffer] = '\0';
        }

ERROR_LABEL:

    if (pbuffer != NULL)
        free (pbuffer);

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Quoted String                                               */
/* Purpose:  This function sets command with quoted string parameter.        */
/*===========================================================================*/
ViStatus rsfsh_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*)malloc(strlen(cmd) + strlen(value) + 2*strlen(quotes) + 4);
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf,"%s %s%s%s\n", cmd, quotes, value, quotes);

    GOCHECKERR (viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32) (p2buf - wrt_buf), VI_NULL));
    GOCHECKERR (rsfsh_checkStatus(instrSession));

ERROR_LABEL:

    free (wrt_buf);

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set String With Channel                                         */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus rsfsh_setViStringPath (ViSession instrSession,
                            ViString cmd,
                            ViInt32 path,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*) malloc (strlen(cmd) + strlen(value) + 11);
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf, cmd, path);
    p2buf += sprintf(p2buf," %s\n", value);

    rsfsh_status = viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32)(p2buf - wrt_buf), VI_NULL);
    free(wrt_buf);

    CHECKERR(rsfsh_status);
    CHECKERR(rsfsh_checkStatus(instrSession));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get String with Path                                            */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus rsfsh_getViStringPath (ViSession instrSession,
                                ViString cmd,
                                ViInt32 path,
                                ViString value,
                                ViInt32 buffer_size,
                                ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    sprintf(wrt_buf,cmd, path);

    return rsfsh_getViString(instrSession,
                             wrt_buf,
                             value,
                             (ViUInt32)buffer_size,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsfsh_setEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus rsfsh_status = VI_SUCCESS;

    CHECKERR (rsfsh_checkIndexInList (value, list, parameterErrorCode));

    return rsfsh_setViString (instrSession, cmd, list[value], parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsfsh_getEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE] = "";

    CHECKERR( rsfsh_getViString(instrSession, cmd, buffer, BUFFER_SIZE, parameterErrorCode));
    CHECKERR( rsfsh_getIndexFromString (buffer, list, value));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Enum With Path                                              */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsfsh_setEnumPath (ViSession instrSession,
                        ViString cmd,
                        ViInt32 path,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus rsfsh_status = VI_SUCCESS;

    CHECKERR (rsfsh_checkIndexInList (value, list, parameterErrorCode));

    return rsfsh_setViStringPath (instrSession, cmd, path, list[value], parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get Enum Path                                                   */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsfsh_getEnumPath (ViSession instrSession,
                        ViString cmd,
                        ViInt32 path,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE] = "";

    CHECKERR(rsfsh_getViStringPath(instrSession, cmd, path, buffer, (ViInt32)BUFFER_SIZE, parameterErrorCode));
    CHECKERR( rsfsh_getIndexFromString (buffer, list, value));

    return rsfsh_status;

}

/*===========================================================================*/
/* Function: Set ViUInt32                                                    */
/* Purpose:  This function sets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rsfsh_setViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 value,
                           ViUInt32 min,
                           ViUInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    if ((max - min) > 0)
        if (rsfsh_invalidViUInt32Range (value, min, max) != 0)
            return parameterErrorCode;

    sprintf (wrt_buf, "%s %u", cmd, value);

    return rsfsh_sendCmd (instrSession, wrt_buf);
}

/*===========================================================================*/
/* Function: Get ViUInt32                                                    */
/* Purpose:  This function gets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rsfsh_getViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR( viPrintf(instrSession, "%s\n", cmd));
    CHECKERR( viScanf(instrSession, "%u", value));
    CHECKERR( rsfsh_checkStatus (instrSession));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Check Index List                                                */
/* Purpose:  This function checks if the passed index is valid for given list*/
/*===========================================================================*/
ViStatus rsfsh_checkIndexInList (ViInt32 index,
                                 ViString list[],
                                 ViStatus statusIfError)

{
    ViInt32     i=0;

    if (index < 0)
        return statusIfError;

    while (list[i++] != VI_NULL);
    if (index >= i)
        return statusIfError;

    return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Get Index From String                                           */
/* Purpose:  This function returns index for string.                         */
/*===========================================================================*/
ViStatus rsfsh_getIndexFromString (ViString string,
                                   ViString *list,
                                   ViInt32 *index)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViUInt32    i;
    size_t      len;

    for (i = 0; list[i]; i++)
        if ((len = strlen (string)) == strlen (list[i])) {
            if (strncmp(string, list[i], strlen (string)) == 0) break;
        }

    *index = (ViInt32) i;

    return (list[i]) ? rsfsh_status : VI_ERROR_INSTR_INTERPRETING_RESPONSE;
}

/*===========================================================================*/
/* Function: Get Catalog                                                     */
/*===========================================================================*/
ViStatus rsfsh_getCatalog (ViSession instrSession,
                           ViString cmd,
                           ViInt32 arraySize,
                           ViChar* array,
                           ViInt32 parameterErrorCode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    size_t      count  = 0,
                length = 0;
    ViUInt32    unknown_count = 0;

    if (array == NULL)
        return parameterErrorCode;

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;
    
    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    GOCHECKERR (viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) (length + 1), VI_NULL));

    free(pbuffer);
    
    pbuffer = NULL;

    GOCHECKERR (rsfsh_readDataUnknownLength (instrSession, &pbuffer, &unknown_count));
    GOCHECKERR (rsfsh_checkStatus (instrSession));

    if (count > (size_t) arraySize)
        {
        //rsfsh_status = RSFSH_WARNING_SMALL_BUFFER;
        count = (size_t)arraySize;
        memcpy (array, pbuffer, count);
        array[count-1] = '\0';
        }
    else
        {
        if ((p2buffer = strchr (pbuffer, ';')) != VI_NULL)
            {
            rsfsh_status = VI_SUCCESS;
            memcpy (array, pbuffer, (size_t)(p2buffer - pbuffer));
            array[p2buffer - pbuffer] = '\0';
            }
        else
            {
            memcpy (array, pbuffer, count);
            array[count-1] = '\0';
            }
        }
ERROR_LABEL:

    if (pbuffer != NULL)
        free (pbuffer);

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rsfsh_getViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViUInt32    local_count = 0;
    ViInt32     i = 0;

    if (value == VI_NULL)
        return parameterErrorCode;

    GOCHECKERR (viPrintf(instrSession, "%s\n",cmd));
    GOCHECKERR (rsfsh_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while ((p2buf != NULL) && (i < count))
    {
        value[i++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
    };

    GOCHECKERR (rsfsh_checkStatus (instrSession));

ERROR_LABEL:

    free(pbuffer);

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read Data With Unknown length                                   */
/* Purpose:  This function allocates buffer and reads data from the          */
/*           instrument                                                      */
/* WARNING:  You must deallocate buffer, if any pointer has been returned.   */
/*           This function does not work correctly, when you read bin data   */
/*           via serial intf                                                 */
/*===========================================================================*/
ViStatus rsfsh_readDataUnknownLength(ViSession instrSession,
                                     /*@null@*/ViChar **outBuffer,
                                     ViUInt32 *count)
{
    ViStatus rsfsh_status = VI_SUCCESS;
    ViUInt32 actSize, retCnt = 0;
    ViChar   *buffer = NULL, *p2buffer = NULL;

    *outBuffer = VI_NULL;
    if (count) *count = 0;

    /* Allocate Buffer */
    actSize = BUFFER_SIZE;
    buffer = (ViChar *)calloc((size_t)actSize, (size_t) sizeof(ViChar));
    if (buffer == VI_NULL) return VI_ERROR_ALLOC;

    p2buffer = buffer;
    do
    {
        if ((rsfsh_status = viRead (instrSession, (ViPBuf)p2buffer, BUFFER_SIZE, &retCnt)) < 0)
        {
            free(buffer);
            *outBuffer = VI_NULL;
            return rsfsh_status;
        }
        if (retCnt == BUFFER_SIZE) /* Buffer was small, reallocate it */
        {
            actSize += BUFFER_SIZE;
            buffer = (ViChar *)realloc(buffer, (size_t) actSize);
            if (buffer == VI_NULL) return VI_ERROR_ALLOC;
            p2buffer = buffer + (actSize - BUFFER_SIZE); /* Set pointer to end of data in rallocated buffer */
        }
        else
        {
            *(p2buffer+retCnt) = '\0'; /* For sure set end of string at the end of data */
        }
    } while (rsfsh_status == VI_SUCCESS_MAX_CNT); /* if buffer was small, continue with next read */

    *outBuffer = buffer;
    if (count != NULL) *count = (actSize - BUFFER_SIZE) + retCnt;

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read Data To File                                               */
/* Purpose:  This function read data from the instrument and                 */
/*           write it to a user specified file.                              */
/*===========================================================================*/
ViStatus rsfsh_readDataToFile(
    ViSession   instrSession,
    ViString    cmd,
    ViString    destination
)
{
    ViStatus    rsfsh_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViUInt32    count           = 0,
                lengthOfLength  = 0;
    ViChar      buffer[BUFFER_SIZE],
                format[32],
                *pbuffer = NULL, *p2buffer = NULL;
    FILE        *targetFile;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    sprintf (buffer, "%s\n", cmd);

    CHECKERR (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen(buffer), VI_NULL));
    CHECKERR (rsfsh_readDataUnknownLength (instrSession, &pbuffer, &count));

    p2buffer = pbuffer;

    if (sscanf (p2buffer, "#%1lu", &lengthOfLength) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    sprintf (format, "%%%lud", lengthOfLength);
    p2buffer += 2;

    if (sscanf (p2buffer, format, &count) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    p2buffer += lengthOfLength;

    if (intfType == VI_INTF_GPIB)
        if ((rsfsh_status = viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL)) < VI_SUCCESS)
            {
            free (pbuffer);
            return rsfsh_status;
            }

    if ((rsfsh_status = rsfsh_checkStatus (instrSession)) < VI_SUCCESS)
        {
        free (pbuffer);
        return rsfsh_status;
        }

    /* Store data from instrument to file on host computer */
    if ((targetFile = fopen (destination, "wb")) == NULL)
        {
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    if (count != (ViUInt32) fwrite (p2buffer, sizeof (ViByte), (size_t)(count), targetFile))
        {
        (void) fclose (targetFile);
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_WRITE;
        }

    (void) fclose (targetFile);

    if (pbuffer) /* If not NULL pointer, then deallocate memory. */
        free (pbuffer);

    if ((rsfsh_status = rsfsh_checkStatus (instrSession)) < 0)
        return rsfsh_status;

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read Trace Data                                                 */
/* Purpose:  This function reads out trace data from the instrument.         */
/*           It checks if data are ASCII or binary                           */
/*===========================================================================*/
ViStatus rsfsh_dataReadTrace (ViSession instrSession,
                                 ViString trace, 
                                 ViInt32 arrayLength, 
                                 /*@out@*/ViReal64 _VI_FAR traceData[],
                                 ViPInt32 noofPoints)
{
    ViChar      *extBuf=VI_NULL;
    ViReal32    *temp;
    ViInt32    i = 0;
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViUInt16    intf = VI_INTF_GPIB;
    ViInt32    count=0;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *p2buf=VI_NULL;
    ViUInt32    ret_cnt;
    ViInt32    len;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf));
/*Don't put parameter range checking here! 
All parameters has to be checked in high level function */      
    
        viPrintf (instrSession, ":FORM REAL,32;*CLS;:TRAC? %s\n", trace);
    
    //viCheckErr (rsspecan_readBinaryDataBlock(instrSession, &extBuf, &count));
    //Check first character            
    CHECKERR (viRead (instrSession, (ViBuf) buffer, 1, &ret_cnt)); 
    if (buffer[0]=='#')
        /*Binary data*/
    {
        /* Disable termination character */
        CHECKERR (viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
         /* Disable termination character on serial */
        if (intf == VI_INTF_ASRL)
        {
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
        }
        /* Parse header */
        CHECKERR (viRead (instrSession, (ViBuf) buffer, 1, &ret_cnt));
        buffer[1] = '\0';
        sscanf (buffer, "%ld", &len);
        CHECKERR (viRead (instrSession, (ViBuf) buffer, len, &ret_cnt));
        buffer[len] = '\0';
        sscanf (buffer, "%ld", &len);
        
        extBuf = (ViChar*) malloc (len);
        /* Read useful data */
        CHECKERR (viRead (instrSession, (ViBuf) extBuf, len, &ret_cnt));
    
        /*if (len != (ViInt32) ret_cnt)
            viCheckErr (RSFSH_ERROR_UNKNOWN_ERROR);*/

        /* Flush garbage */
        viScanf (instrSession,"%*[^\r\n]");
        /* Convert binary data into array of ViReal64 types. */
        count = (ViInt32)(ret_cnt/4);
        /* Binary data byte order (MOTOROLA or INTEL). */
        rsfsh_SWAP_32((ViPBuf)extBuf, count);
        /* Pointer to data */
        temp = (ViReal32 *)extBuf;
        if (noofPoints) *noofPoints=count;    
    
        count=(count<arrayLength)?count:arrayLength;
    
        for (i = 0; i < count; i++)
            if (i<arrayLength)
            *traceData++ = (ViReal64)*temp++;
    }
    else
        /*ASCII data*/
    {
        //read rest of first value
        p2buf=buffer;
        while((p2buf[0]!=',')&&(p2buf[0]!='\r')&&(p2buf[0]!='\n')){
            p2buf++;
            CHECKERR (viRead (instrSession, (ViBuf) p2buf, 1, &ret_cnt)); 
        }
        if(buffer[0]=='"'){ buffer[0]=' '; } // remove begining quotes
        traceData[0] = atof (buffer); 
        //read all values
        rsfsh_readDataUnknownLength(instrSession, &extBuf, &ret_cnt); 

        p2buf = strtok(extBuf, ",");
        i=1;
        while (p2buf) 
        {
            if (i < arrayLength)
            {
                traceData[i] = atof (p2buf);
            }   
            p2buf = strtok(NULL, ",");  
            i++;
        }
    
        if (noofPoints) *noofPoints=i; 
    
    }

    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
    switch (intf)
    {
        case VI_INTF_ASRL:
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
        break;    
    }    
    if (extBuf) free(extBuf);
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read Trace Data Only                                            */
/* Purpose:  This function reads out trace data from the instrument.         */
/*           It checks if data are ASCII or binary                           */
/*===========================================================================*/
ViStatus rsfsh_dataReadTraceOnly (ViSession instrSession,
                                 ViInt32 arrayLength, 
                                 /*@out@*/ViReal64 _VI_FAR traceData[],
                                 ViPInt32 noofPoints)
{
    ViChar      *extBuf=VI_NULL;
    ViReal32    *temp;
    ViInt32    i = 0;
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViUInt16    intf = VI_INTF_GPIB;
    ViInt32    count=0;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *p2buf=VI_NULL;
    ViUInt32    ret_cnt;
    ViInt32    len;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf));
    //Check first character            
    CHECKERR (viRead (instrSession, (ViBuf) buffer, 1, &ret_cnt)); 
    if (buffer[0]=='#')
        /*Binary data*/
    {
        /* Disable termination character */
        CHECKERR (viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
         /* Disable termination character on serial */
        if (intf == VI_INTF_ASRL)
        {
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
        }
        /* Parse header */
        CHECKERR (viRead (instrSession, (ViBuf) buffer, 1, &ret_cnt));
        buffer[1] = '\0';
        sscanf (buffer, "%ld", &len);
        CHECKERR (viRead (instrSession, (ViBuf) buffer, len, &ret_cnt));
        buffer[len] = '\0';
        sscanf (buffer, "%ld", &len);
        
        extBuf = (ViChar*) malloc (len);
        /* Read useful data */
        CHECKERR (viRead (instrSession, (ViBuf) extBuf, len, &ret_cnt));
    
        /*if (len != (ViInt32) ret_cnt)
            viCheckErr (RSFSH_ERROR_UNKNOWN_ERROR);*/

        /* Flush garbage */
        viScanf (instrSession,"%*[^\r\n]");
        /* Convert binary data into array of ViReal64 types. */
        count = (ViInt32)(ret_cnt/4);
        /* Binary data byte order (MOTOROLA or INTEL). */
        rsfsh_SWAP_32((ViPBuf)extBuf, count);
        /* Pointer to data */
        temp = (ViReal32 *)extBuf;
        if (noofPoints) *noofPoints=count;    
    
        count=(count<arrayLength)?count:arrayLength;
    
        for (i = 0; i < count; i++)
            if (i<arrayLength)
            *traceData++ = (ViReal64)*temp++;
    }
    else
        /*ASCII data*/
    {
        //read rest of first value
        p2buf=buffer;
        while((p2buf[0]!=',')&&(p2buf[0]!='\r')&&(p2buf[0]!='\n')){
            p2buf++;
            CHECKERR (viRead (instrSession, (ViBuf) p2buf, 1, &ret_cnt)); 
        }
        if(buffer[0]=='"'){ buffer[0]=' '; } // remove begining quotes
        traceData[0] = atof (buffer); 
        //read all values
        rsfsh_readDataUnknownLength(instrSession, &extBuf, &ret_cnt); 

        p2buf = strtok(extBuf, ",");
        i=1;
        while (p2buf) 
        {
            if (i < arrayLength)
            {
                traceData[i] = atof (p2buf);
            }   
            p2buf = strtok(NULL, ",");  
            i++;
        }
    
        if (noofPoints) *noofPoints=i; 
    
    }

    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
    switch (intf)
    {
        case VI_INTF_ASRL:
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
        break;    
    }    
    if (extBuf) free(extBuf);
    return rsfsh_status;
}

