/*= Rohde&Schwarz FSH series Spectrum Analyzer ==============================*/
/* LabWindows/CVI 6.0 Instrument Driver
/* Original Release: June 2003
/* By: Martin Koutny
/*
/* Should you have any technical questions please contact the hotline of
/* Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
/*
/* e-mail: CustomerSupport@rsd.rohde-schwarz.com
/*
/* Modification History:
/*
/*  see ChangeLog
/*
/*===========================================================================*/

#ifdef _WIN32
  #ifdef _CVI_
    #include <userint.h>
  #else
    #include <windows.h>
  #endif
#endif
#include <visa.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "rsfsh.h"
#include "rsfsh_utility.h"

/*= DEFINES =================================================================*/
#define RSFSH_REVISION     "Rev 2.7, 01/2014, CVI 2012" /* Instrument driver revision */
#define BUFFER_SIZE         512L                         /* File I/O buffer size */
#define TRACE_BUFFER_SIZE   4096 
#define LARGE_BUFFER_SIZE   10240
#define NUMERIC_PRECISION   1.0e-12

#define RSFSH_DEFAULT_TIMEOUT  10000   /* Default Timeout value in ms (10 sec) */

/*= Useful Macros ===========================================================*/
/* Error checking macro with clean up (rsfsh_init only) */
#define CHECKERR_CU(fCal) if (rsfsh_status = (fCal), rsfsh_status < VI_SUCCESS) \
    return rsfsh_initCleanUp (rmSession, instrSession, rsfsh_status); else

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString measurementModeArr[] = {"","SAN","","","","","","DTF","REC","","","","NAN", VI_NULL};
static ViString frequencyModeArr[] = {"CW","FIX","SWE", VI_NULL};
static ViString verticalScaleArr[] = {"LIN","LOG","PCT","LDB", VI_NULL};
static ViString amplitudeUnitsArr[] = {"DBM","DBUV","DBMV","V","W", VI_NULL};
static ViString traceModeArr[] = {"WRIT","AVER","MAXH","MINH","VIEW","RMS", VI_NULL};
static ViString detectorTypeArr[] = {"APE","NEG","POS","SAMP","RMS","AVER","QPE", VI_NULL};
static ViString polarityArr[] = {"POS","NEG", VI_NULL};
static ViString triggerSourceArr[] = {"IMM","VID","","","EXT", VI_NULL};
static ViString resolutionBandwidthArr[] = {"AUTO","100 Hz","300 Hz","1 kHz","3 kHz","10 kHz","30 kHz","100 kHz","300 kHz","1 MHz","200 kHz","3 MHz","1 Hz","3 Hz","10 Hz","30 Hz",VI_NULL};
static ViString videoBandwidthArr[] = {"AUTO","10 Hz","30 Hz","100 Hz","300 Hz","1 kHz","3 kHz","10 kHz","30 kHz","100 kHz","300 kHz","1 MHz","3 MHz", VI_NULL};
static ViString measModeArr[] = {"SAN","NAN","DTF","PM","BWCD","BC2K","BDO","REC"
                                 "GSM","BTDS","BWCD","LTEF","LTET",VI_NULL};
static ViString stepLinkArr[] = {"DIVT","OFF", VI_NULL};
static ViString freqInputModeArr[] = {"CHAN","FREQ", VI_NULL};
static ViString channelModeArr[] = {"FREQ","CHAN", VI_NULL};
static ViString padArr[] = {"RAM","RAZ","HZTE", VI_NULL};
static ViString attenuationModeArr[] = {"LDIS","LNO", VI_NULL};
static ViString markerDisplayModeArr[] = {"FREQ","CHAN", VI_NULL};
static ViString markerOutputFormatArr[] = {"NORM","RPDB","RPL","RSC","IMP","ADM","NIMP","NADM", VI_NULL};
static ViString channelPowerModeArr[] = {"CLR","MAX", VI_NULL};
static ViString channelPowerUnitArr[] = {"DBM","DBMV","DBUV","VOLT","WATT","DUVM","DUAM","V","W","V_M","W_M2", VI_NULL};
static ViString demodulationTypeArr[] = {"AM","FM", VI_NULL};
static ViString spacingArr[] = {"LIN","LOG", VI_NULL};
static ViString reflectionUnitArr[] = {"RHO","MRHO", VI_NULL};
static ViString powerSelectTypeArr[] = {"CPOW","OBW","TDMA","ACP", VI_NULL};
static ViString networkAnalyzerModeArr[] = {"SCAL","VECT","VVM","DTF","REFL","SPEC",VI_NULL};
static ViString measurementFormatArr[] = {"MAGN","PHAS","MPH","VSWR","REF","SMIT","LOSS","GDEL","VVM", VI_NULL};
static ViString transmissionMeasurementFormatArr[] = {"MAGN","PHAS","MPH","GDEL", VI_NULL};
static ViString dtfMeasurementFormatArr[] = {"MAGN","","","VSWR","REFL", VI_NULL};
static ViString colorSchemeArr[] = {"COL","BW", VI_NULL};
static ViString dateFormatArr[] = {"DDMM","MMDD", VI_NULL};
static ViString bncModeArr[] = {"REF","TRIG","BIAS","IF3","IFD", VI_NULL};
static ViString unitLengthArr[] = {"MET","FEET", VI_NULL};
static ViString powerUnitArr[] = {"DBM","DBUV","DBMV","V","W","DUVM","DUAM","V_M","W_M2", VI_NULL};
static ViString selStatusRegArr[] = {"",":POW",":LIM1",":LMAR1",":SYNC",":ACPL",
                                     ":FREQ",":LIM2",":LMAR2",":PNO",":DIQ"};
static ViString statusRegArr[] = {"OPER:EVEN","OPER:COND","QUES:EVEN","QUES:COND",
                                  "QUES:POW:EVEN","QUES:POW:COND","QUES:LIM:EVEN",
                                  "QUES:LIM:COND","QUES:LMAR:EVEN","QUES:LMAR:COND",
                                  "QUES:SYNC:EVEN","QUES:SYNC:COND","QUES:ACPL:EVEN",
                                  "QUES:ACPL:COND","QUES:FREQ:EVEN","QUES:FREQ:COND",
                                  "QUES:TRAN:EVEN","QUES:TRAN:COND","QUE",
                                  "QUES:LIM2:EVEN","QUES:LIM2:COND","QUES:LMAR2:EVEN",
                                  "QUES:LMAR2:COND","QUES:PNO:EVEN","QUES:PNO:COND",
                                  "OPER:PCAL:EVEN","OPER:PCAL:COND","QUES:DIQ:EVEN",
                                  "QUES:DIQ:COND"};
static ViString zoomFactorArr[] = {"2","4","8", VI_NULL};
static ViString powerParameterArr[] = {"CPOW","OBAN", VI_NULL};
static ViString powerSensorUnitArr[] = {"DBM","W","DB","VSWR", VI_NULL};
static ViString powerSensorMeasTimeArr[] = {"SHOR","NORM","LONG", VI_NULL};
static ViString powerSensorDisplayArr[] = {"AVER","PENV", VI_NULL};
static ViString gpsSignalQualityArr[] = {"INS","LOW","MED","HIG","EXC", VI_NULL};
static ViString acpUnitsArr[] = {"DBM","DBMV","DBUV","V","W", VI_NULL};
static ViString acpModeArr[] = {"ABS","REL", VI_NULL};
static ViString acpAutoRefModeArr[] = {"MIN","MAX","LHIG", VI_NULL};
static ViString sourceArr[] = {"INT","EXT", VI_NULL};
static ViString result3GPPBSArr[] = {"ACH","CPEV","CPP","FERR","PCEV","PCP","PSP","PTOT","SSP", VI_NULL};
static ViString powerModeArr[] = {"ABS","CPICH", VI_NULL};
static ViString SEMArr[] = {"AUTO","ESP", "LIST", VI_NULL};
static ViString resultEVDOArr[] = {"EVMP","FERR","PNF","PAV","PDAT","PMAC","PPIL","PTOT","RHOP","TACT","TAU", VI_NULL};
static ViString powerEVDOModeArr[] = {"PICH","TOT", VI_NULL};
static ViString antennaDiversityModeArr[] = {"OFF","1","2", VI_NULL};
static ViString resultBC2KArr[] = {"ACT","FERP","FERR","MACC","PNF","PPAV","PTOT","RHO","TAU",VI_NULL};
static ViString powerC2KModeArr[] = {"PICH","TOT", VI_NULL};
static ViString scanModeArr[] = {"FIX","SCAN", VI_NULL};
static ViString CISPRBandwidthArr[] = {"200.000000","9000.000000","120000.000000","1000000.000000", VI_NULL};
static ViString antennaDirectionArr[] = {"AUTO","X","Y","Z",VI_NULL};
static ViString linkDirectionArr[] = {"DOWN","UP",VI_NULL};
static ViString traceStyleArr[] = {"LIN","POLY",VI_NULL};
static ViString absRelArr[] = {"ABS", "REL", VI_NULL}; 
static ViString xUnitArr[] = {"HZ", "S", "M", VI_NULL}; 
static ViString yUnitArr[] = {"DB", "DBM", "DBUV", "DBMV", "DBUVM", "DBUAM", 
							  "VSWR", "RHO", "V", "W", VI_NULL};
static ViString storageArr[] = {"SDC", "USB", VI_NULL};
static ViString soevSourceArr[] = {"ALLS", "DIST", "LIM", "TIM", VI_NULL};
static ViString soevlimitsModeArr[] = {"FAIL", "STAR", "STOP", VI_NULL};
static ViString methodArr[] = {"F2PC","F2PN","F2PH","S11C","S11","S11N","S12C",
							   "S12N","S21C","S21N","S22C","S22","S22N", VI_NULL};

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
void rsfsh_SWAP_32(unsigned char *data, long points);
//ViBoolean rsfsh_invalidViBooleanRange (ViBoolean val);
ViBoolean rsfsh_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rsfsh_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViBoolean rsfsh_invalidViString (ViString val, ViUInt32 maxLength);
ViStatus rsfsh_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
                ViStatus currentStatus);
ViStatus rsfsh_defaultInstrSetup (ViSession openInstrSession);
ViStatus rsfsh_cmdCommand (ViSession instrSession, ViString cmdCommand);
ViStatus rsfsh_setCommand (ViSession instrSession, ViString setCommand);
ViStatus rsfsh_getCommand (ViSession instrSession, ViString getCommand, ViChar value[]);
ViStatus rsfsh_delay (ViSession instrSession, ViReal64 seconds);                

/*****************************************************************************/
/*------ INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS HERE ------*/
/*****************************************************************************/

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_init (ViRsrc resourceName, ViBoolean IDQuery,
                               ViBoolean resetDevice, ViPSession instrSession)
{
    ViUInt32 attr_dbg;
    ViUInt16 attr_dbg_sh;
    ViStatus            rsfsh_status = VI_SUCCESS;
    ViUInt16            intf = VI_INTF_GPIB;
    ViSession           rmSession = 0;
    ViChar              buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr = NULL;

    /*- Check input parameter ranges ----------------------------------------*/
    if (rsfsh_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    CHECKERR( viOpenDefaultRM (&rmSession));

    if ((rsfsh_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG,
                                 VI_NULL, instrSession)) < 0)
        {
        viClose (rmSession);
        return rsfsh_status;
        }
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR_CU( viGetAttribute (*instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr == NULL)
        instrPtr = malloc (sizeof (struct rsfsh_statusDataRanges));

    CHECKERR_CU(viSetAttribute (*instrSession, VI_ATTR_USER_DATA, (ViAttrState)instrPtr));

    /* Serial interface support */
    CHECKERR_CU( viGetAttribute (*instrSession, VI_ATTR_INTF_TYPE, &intf));

    if (intf == VI_INTF_ASRL)  /* Serial Port */
        {
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\r'));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR_EN,
                                     VI_TRUE));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN,
                                     VI_ASRL_END_TERMCHAR));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT,
                                     VI_ASRL_END_TERMCHAR));
        CHECKERR_CU( viFlush (*instrSession, VI_ASRL_IN_BUF_DISCARD ||
                              VI_ASRL_OUT_BUF_DISCARD));
        }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE,
                                 RSFSH_DEFAULT_TIMEOUT));
    CHECKERR_CU( viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000));
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));
    /* -------------------------------------------------------------------- */
    /* Drawback of Agilent VISA is patched by adding extra termination      */
    /* character "\n" to each I/O operation                                 */
    /*                                                                      */
    /* VI_ATTR_SEND_END_EN will stay with this attribute set to VI_TRUE     */
    /* -------------------------------------------------------------------- */

    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_SEND_END_EN, VI_TRUE));

    
    if (intf == VI_INTF_ASRL)  /* Serial Port */
    {
        viGetAttribute (*instrSession, VI_ATTR_ASRL_BAUD, &attr_dbg);
        viGetAttribute (*instrSession, VI_ATTR_ASRL_DATA_BITS, &attr_dbg_sh);
        viGetAttribute (*instrSession, VI_ATTR_ASRL_PARITY, &attr_dbg_sh);
        viGetAttribute (*instrSession, VI_ATTR_ASRL_STOP_BITS, &attr_dbg_sh);
        viGetAttribute (*instrSession, VI_ATTR_ASRL_FLOW_CNTRL, &attr_dbg_sh);
        viGetAttribute (*instrSession, VI_ATTR_ASRL_END_IN, &attr_dbg_sh);
        viGetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT, &attr_dbg_sh);
        instrPtr -> fsh3 = VI_TRUE;
    }
    else
    {
        instrPtr -> fsh3 = VI_FALSE;
    }
    
    /*- Go to the REMOTE state -*/
    /*
        CHECKERR_CU (rsfsh_cmdCommand (*instrSession, "REMOTE"));
    */

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery)
        {
        if (intf == VI_INTF_ASRL) 
            CHECKERR_CU( rsfsh_getCommand (*instrSession, "IDN?", buffer));
        else
            CHECKERR( viQueryf (*instrSession, "*IDN?\n","%s",
                                buffer));
        
        /* Search if instrument is supported or not */
        if ((strncmp ("Rohde&Schwarz", buffer, 13)) != 0)
            return rsfsh_initCleanUp (rmSession, instrSession,
                                      VI_ERROR_FAIL_ID_QUERY);
        }

    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice)
        CHECKERR_CU( rsfsh_reset (*instrSession));
    else  /*- Send Default Instrument Setup ---------------------------------*/
        CHECKERR_CU( rsfsh_defaultInstrSetup (*instrSession));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read Spectrum                                                   */
/* Purpose:  This function is a simple example how to use instrument driver  */
/*           functions. This function first sets FSH to the analyzer         */
/*           measurement mode, sets trace detector to RMS, sets frequency and*/
/*           level parameters, sends trigger and then reads out measured     */
/*           trace data.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_readSpectrum (ViSession instrSession,
                                      ViReal64 centerFrequency,
                                      ViReal64 frequencySpan,
                                      ViReal64 frequencyOffset,
                                      ViReal64 referenceLevel, ViInt32 unit,
                                      ViReal64 referenceLevelOffset,
                                      ViReal64 _VI_FAR traceData[],
                                      ViPInt32 samplesReturned,
                                      ViInt32 trace)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViInt32     timeout = 0;

    if (rsfsh_invalidViReal64Range (centerFrequency, 0.0, 6.0e9))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViReal64Range (frequencySpan, 0.0, 6.0e9))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViReal64Range (frequencyOffset, 0.0, 100.0e9))
        return VI_ERROR_PARAMETER4;
    /* Parameter 5 is not range checked */
    if (rsfsh_invalidViInt32Range (unit, 0, 7))
        return VI_ERROR_PARAMETER6;
    if (rsfsh_invalidViReal64Range (referenceLevelOffset, -100.0, 100.0))
        return VI_ERROR_PARAMETER7;
    if (!traceData)
        return VI_ERROR_PARAMETER8;
    if (!samplesReturned)
        return VI_ERROR_INSTR_PARAMETER9;

    CHECKERR (rsfsh_setMeasurementMode (instrSession, 1));
    CHECKERR (rsfsh_setTraceDetector (instrSession, 4));
    CHECKERR (rsfsh_configureFrequency (instrSession, centerFrequency, frequencySpan, frequencyOffset));
    CHECKERR (rsfsh_configureAmplitude (instrSession, referenceLevel, unit, referenceLevelOffset));
    /* Get session timeout */
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &timeout));
    CHECKERR (rsfsh_sendTriggerWopc (instrSession, timeout));
    CHECKERR (rsfsh_readTraceData (instrSession, trace, traceData, samplesReturned));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Measurement Mode
/* Purpose:  This function sets the measurement modes by means of text 
/*           parameters.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMeasMode(
    ViSession   instrSession,
    ViInt32 measurementMode
)
{
    return rsfsh_setEnum (instrSession,
                          "INST:SEL",
                          measurementMode,
                          measModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Configure Display Line
/* Purpose:  This function configures the display line.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureDisplayLine(
    ViSession   instrSession,
    ViBoolean   state,
    ViReal64    position
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    CHECKERR (rsfsh_setDisplayLineState (instrSession, state));
    CHECKERR (rsfsh_setDisplayLinePosition (instrSession, position));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Measurement Mode
/* Purpose:  This function queries the measurement modes by means of text 
/*           parameters.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMeasMode(
    ViSession   instrSession,
    ViInt32*    measurementMode
)
{
    return rsfsh_getEnum (instrSession,
                          "INST:SEL?",
                          measurementMode,
                          measModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Serial Baud Rate                                            */
/* Purpose:  This function sets the serial interface baud rate.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setSerialBaudRate (ViSession instrSession,
                                           ViInt32 baudRate)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     VISAbaudRate = baudRate;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
        switch (baudRate)
            {
            case 19200:
                baudRate = 0;
            break;
            case 38400:
                baudRate = 1;
            break;
            case 57600:
                baudRate = 2;
            break;
            case 115200:
                baudRate = 3;
            break;
            case 9600:
                baudRate = 4;
            break;
            default:
                return VI_ERROR_PARAMETER2;
        }

    sprintf (buffer, "BAUD,%ld", baudRate);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    CHECKERR (viSetAttribute (instrSession, VI_ATTR_ASRL_BAUD, VISAbaudRate));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Serial Baud Rate                                            */
/* Purpose:  This function gets the serial interface baud rate.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getSerialBaudRate (ViSession instrSession,
                                           ViPInt32 baudRate)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!baudRate)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "BAUD", buffer));
    sscanf (buffer, "%ld", baudRate);

    switch (*baudRate)
        {
        case 0:
            *baudRate = 19200;
        break;
        case 1:
            *baudRate = 38400;
        break;
        case 2:
            *baudRate = 57600;
        break;
        case 3:
            *baudRate = 115200;
        break;
        case 4:
            *baudRate = 9600;
        break;
        default:
            *baudRate = 0;
            return VI_ERROR_PARAMETER2;
        }
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Save Dataset                                                    */
/* Purpose:  This function saves the current setting and measurement in the  */
/*           specified dataset. If the dataset exists it will be overwritten.*/
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_saveDataset (ViSession instrSession,
                                     ViString datasetName)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (datasetName, 12))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SAVE,%s", datasetName);
    CHECKERR (rsfsh_cmdCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Recall Dataset                                                  */
/* Purpose:  This function recalls a stored dataset.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_recallDataset (ViSession instrSession,
                                       ViString datasetName)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (datasetName, 12))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "RECALL,%s", datasetName);
    CHECKERR (rsfsh_cmdCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Screenshot And Copy Trace                                       */
/* Purpose:  This function makes a screenshot of the current trace and stores
             it on the R&S FSH internal hard disk, following by the path and
             file name that the R&S FSH uses for storing screenshots.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_screenshotAndCopyTrace (ViSession instrSession,
                                                ViString path_andFileName)
{   
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    CHECKERR (rsfsh_setViString (instrSession, "MMEM:NAME %s", path_andFileName, VI_ERROR_PARAMETER2));
    
    CHECKERR (rsfsh_sendCmd (instrSession, "HCOP"));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get External Reference Status                                   */
/* Purpose:  This command returns the external reference status.             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getExternalReferenceStatus (ViSession instrSession,
                                                    ViInt32 *externalReferenceStatus)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!externalReferenceStatus)
        return VI_ERROR_PARAMETER2;
        
    CHECKERR (rsfsh_getCommand (instrSession, "EXTREF", buffer));
    sscanf (buffer, "%ld", externalReferenceStatus);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;      
}

/*===========================================================================*/
/* Function: Set Trigger Slope
/* Purpose:  This function sets the slope of the trigger signal. The 
/*           selected trigger slope applies to all trigger signal sources.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTriggerSlope(
    ViSession   instrSession,
    ViInt32 triggerSlope
)
{
    return rsfsh_setEnum (instrSession,
                          "TRIG:SLOP",
                          triggerSlope,
                          polarityArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Trigger Slope
/* Purpose:  This function queries the slope of the trigger signal. The 
/*           selected trigger slope applies to all trigger signal sources.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTriggerSlope(
    ViSession   instrSession,
    ViInt32*    triggerSlope
)
{
    return rsfsh_getEnum (instrSession,
                          "TRIG:SEQ:SLOP?",
                          triggerSlope,
                          polarityArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Send Trigger                                                    */
/* Purpose:  This function starts (re-starts) a new sweep.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_sendTrigger (ViSession instrSession)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    CHECKERR (rsfsh_cmdCommand (instrSession, "INIT"));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Send Trigger and Wait for OPC                                   */
/* Purpose:  This function starts (re-starts) a new sweep and waits for      */
/*           operation completed (OPC) before returning the status code.     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_sendTriggerWopc (ViSession instrSession,
                                         ViInt32 timeout)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViInt32     oldTimeout = 0;
    ViUInt16    intf = VI_INTF_ASRL;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    CHECKERR (rsfsh_cmdCommand (instrSession, "INIT"));
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf));

    /* Interface specific operations */
    switch (intf)
        {
        case VI_INTF_ASRL: /* Serial interface */
            CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, timeout));

            if ((rsfsh_status = rsfsh_cmdCommand (instrSession, "WAIT")) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout);
                return rsfsh_status;
                }
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));
        break;
        default:
            /*
                NOT SUPPORTED
            */
        break;
        }
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Remote (Local) Mode                                         */
/* Purpose:  This function turns the instrument into Remote or Local mode.   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setRemoteLocalMode (ViSession instrSession,
                                            ViBoolean mode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (mode))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "%s", (mode == VI_FALSE) ? "LOCAL" : "REMOTE");
    CHECKERR (rsfsh_cmdCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set External Input Connector                                    */
/* Purpose:  This function switches between external trigger input and       */
/*           external reference  input on the external input connector.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setExternalInputConnector (ViSession instrSession,
                                                   ViBoolean externalInputConnector)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (externalInputConnector))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "EXTINPUT,%ld", externalInputConnector);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get External Input Connector                                    */
/* Purpose:  This function returns either external trigger input or          */
/*           external reference input on the external input connector.       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getExternalInputConnector (ViSession instrSession,
                                                   ViBoolean *externalInputConnector)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!externalInputConnector)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "EXTINPUT", buffer));
    sscanf (buffer, "%hd", externalInputConnector);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Accessory                                                   */
/* Purpose:  This function sets the connected accessory.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAccessory (ViSession instrSession, ViInt32 accessory)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (accessory, 0, 5))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "ACCESSORY,%ld", accessory);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Accessory                                                   */
/* Purpose:  This function gets the connected accessory..                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAccessory (ViSession instrSession, ViPInt32 accessory)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!accessory)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "ACCESSORY", buffer));
    sscanf (buffer, "%ld", accessory);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Display                                                     */
/* Purpose:  This function turns the display on or off.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDisplay (ViSession instrSession, ViBoolean display)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (display))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "DISPLAY,%ld", display);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Display                                                     */
/* Purpose:  This function returns the display state.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDisplay (ViSession instrSession, ViBoolean *display)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!display)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "DISPLAY", buffer));
    sscanf (buffer, "%hd", display);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Display Line Position                                       */
/* Purpose:  This function defines the position of the display line. The     */
/*           display line enables the user to mark any levels in the diagram.*/
/*           The unit depends on the setting made prior to this command.     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDisplayLinePosition (ViSession instrSession, 
                                                ViReal64 position)
{
    return rsfsh_setViReal64 (instrSession,
                              "CALC:DLIN",
                              position,
                              0.0,0.0,
                              VI_ERROR_PARAMETER2);   
}

/*===========================================================================*/
/* Function: Get Display Line Position                                       */
/* Purpose:  This function queries the position of the display line. 
             This function is not compatible with FSH3 instrument.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDisplayLinePosition (ViSession instrSession, 
                                                ViReal64 *position)
{
    return rsfsh_getViReal64 (instrSession,
                              "CALC:DLIN?",
                              position,
                              VI_ERROR_PARAMETER2);   
}

/*===========================================================================*/
/* Function: Set Display Line State                                          */
/* Purpose:  This function switches the display line on or off.
             This function is not compatible with FSH3 instrument.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDisplayLineState (ViSession instrSession, 
                                             ViBoolean state)
{
    return rsfsh_setViBoolean (instrSession,
                               "CALC:DLIN:STAT",
                               state,
                               VI_ERROR_PARAMETER2);   
}

/*===========================================================================*/
/* Function: Get Display Line State                                          */
/* Purpose:  This function queries the display line state.
             This function is not compatible with FSH3 instrument.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDisplayLineState (ViSession instrSession, 
                                             ViBoolean *state)
{
    return rsfsh_getViBoolean (instrSession,
                               "CALC:DLIN:STAT?",
                               state,
                               VI_ERROR_PARAMETER2);   
}

/*===========================================================================*/
/* Function: Set Display Log Range                                           */
/* Purpose:  This function defines the display range of the yaxis (level    */
/*           axis) with logarithmic scaling                                  */
/*           (DISPlay[:WINDow]:TRACe<1|2>:Y:SPACing). For linear scaling, the*/
/*           display range is fixed and cannot be modified.                  */
/*           The numeric suffix <1|2> is irrelevant.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDisplayLogRange (ViSession instrSession, 
                                            ViReal64 range)
{
    return rsfsh_setViReal64 (instrSession,
                              "DISP:WIND:TRAC:Y:SCAL",
                              range,
                              10.0,200.0,
                              VI_ERROR_PARAMETER2);   
}

/*===========================================================================*/
/* Function: Get Display Log Range                                           */
/* Purpose:  This function queries the display range of the yaxis (level    */
/*           axis) with logarithmic scaling.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDisplayLogRange (ViSession instrSession, 
                                            ViReal64 *range)
{
    return rsfsh_getViReal64 (instrSession,
                              "DISP:WIND:TRAC:Y:SCAL?",
                              range,
                              VI_ERROR_PARAMETER2);   
}

/*===========================================================================*/
/* Function: Display Screenshot                                              */
/* Purpose:  This function makes a screenshot of the current trace.          */
/*                                                                           */
/*           This function is not compatible with FSH3 instrument.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_displayScreenshot (ViSession instrSession)
{
    return rsfsh_sendCmd (instrSession, "DISP:STOR");   
}

/*===========================================================================*/
/* Function: Configure Frequency                                             */
/* Purpose:  This function sets the center frequency, span and offset        */
/*           parameters. The start and stop frequency of the instrument is   */
/*           defined by the current center frequency and span                */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureFrequency (ViSession instrSession,
                                            ViReal64 centerFrequency,
                                            ViReal64 frequencySpan,
                                            ViReal64 frequencyOffset)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViReal64Range (centerFrequency, 0.0, 6.0e9))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViReal64Range (frequencySpan, 0.0, 6.0e9))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViReal64Range (frequencyOffset, 0.0, 100.0e9))
        return VI_ERROR_PARAMETER4;

    CHECKERR( rsfsh_setFrequencyOffset (instrSession, 0.0));
    CHECKERR( rsfsh_setCenterFrequency (instrSession, centerFrequency));
    CHECKERR( rsfsh_setFrequencyOffset (instrSession, frequencyOffset));
    CHECKERR( rsfsh_setFrequencySpan (instrSession, frequencySpan));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Frequency Start Stop                                  */
/* Purpose:  This function configures the frequency range of the spectrum    */
/*           analyzer using start frequency and stop frequency. If start     */
/*           frequency is equal to the stop frequency, then the spectrum     */
/*           analyzer is in time-domain mode.                                */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureFrequencyStartStop (ViSession instrSession,
                                                     ViReal64 startFrequency,
                                                     ViReal64 stopFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    CHECKERR( rsfsh_setStartFrequency (instrSession, startFrequency));
    CHECKERR( rsfsh_setStopFrequency (instrSession, stopFrequency));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Frequency Span Full                                   */
/* Purpose:  This function sets the frequency span in the analyzer mode to   */
/*           its maximum.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureFrequencySpanFull (ViSession instrSession)
{
    return rsfsh_sendCmd (instrSession, "SENS:FREQ:SPAN:FULL");
}

/*===========================================================================*/
/* Function: Set Center Frequency                                            */
/* Purpose:  This function sets the center frequency. The start and stop     */
/*           frequency of the instrument is defined by the current center    */
/*           frequency and span.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCenterFrequency (ViSession instrSession,
                                            ViReal64 centerFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    /* Not range checked */
    switch (instrPtr -> fsh3)
    {
    case FSH3:
    sprintf (buffer, "FREQ,%Lf", centerFrequency);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setViReal64 (instrSession,
                                      "SENS:FREQ:CENT",
                                      centerFrequency,
                                      0.0,0.0,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Center Frequency                                            */
/* Purpose:  This function gets the center frequency.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCenterFrequency (ViSession instrSession,
                                            ViPReal64 centerFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));


    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!centerFrequency)
        return VI_ERROR_PARAMETER2;
    
    CHECKERR (rsfsh_getCommand (instrSession, "FREQ", buffer));
    sscanf (buffer, "%Le", centerFrequency);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      "SENS:FREQ:CENT?",
                                      centerFrequency,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Frequency Span                                              */
/* Purpose:  This function sets the span. The start and stop frequency of the*/
/*           instrument is defined by the current center frequency and span. */
/*           The <numeric value> = 0 is used for Zero Span.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setFrequencySpan (ViSession instrSession,
                                          ViReal64 frequencySpan)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (frequencySpan, 0.0, 6.0e9))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "SPAN,%Lf", frequencySpan);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setViReal64 (instrSession,
                                      "SENS:FREQ:SPAN",
                                      frequencySpan,
                                      0.0,0.0,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Frequency Span                                              */
/* Purpose:  This function gets the frequency span.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getFrequencySpan (ViSession instrSession,
                                          ViPReal64 frequencySpan)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!frequencySpan)
        return VI_ERROR_PARAMETER2;
    
    CHECKERR (rsfsh_getCommand (instrSession, "SPAN", buffer));
    sscanf (buffer, "%Le", frequencySpan);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      "SENS:FREQ:SPAN?",
                                      frequencySpan,
                                      VI_ERROR_PARAMETER2);
    break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Frequency Offset                                            */
/* Purpose:  This function sets the frequency offset. The new center         */
/*           frequency is the current frequency plus the frequency offset.   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setFrequencyOffset (ViSession instrSession,
                                            ViReal64 frequencyOffset)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (frequencyOffset, -1.5e9, 100.0e9))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "FREQOFFS,%Lf", frequencyOffset);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setViReal64 (instrSession,
                                      "SENS:FREQ:OFFS",
                                      frequencyOffset,
                                      -100.0e+9, 100.0e+9,
                                      VI_ERROR_PARAMETER2);
    break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Frequency Offset                                            */
/* Purpose:  This function gets the frequency offset.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getFrequencyOffset (ViSession instrSession,
                                            ViPReal64 frequencyOffset)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!frequencyOffset)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "FREQOFFS", buffer));
    sscanf (buffer, "%Le", frequencyOffset);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      "SENS:FREQ:OFFS?",
                                      frequencyOffset,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Start Frequency                                             */
/* Purpose:  This function sets the start frequency.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setStartFrequency (ViSession instrSession,
                                           ViReal64 startFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViReal64    stopFrequency = 0.0,
                centerFrequency = 0.0,
                span = 0.0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Not range checked */

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    /* Read settings */
    CHECKERR (rsfsh_getCommand (instrSession, "FREQ", buffer));
    sscanf (buffer, "%Le", &centerFrequency);
    CHECKERR (rsfsh_getCommand (instrSession, "SPAN", buffer));
    sscanf (buffer, "%Le", &span);

    /* Calculate settings */
    stopFrequency = centerFrequency + span/2.0;
    span = stopFrequency - startFrequency;
    centerFrequency = startFrequency + span/2.0;

    /* Store settings */
    sprintf (buffer, "FREQ,%Lf", centerFrequency);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    sprintf (buffer, "SPAN,%Lf", span);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setViReal64 (instrSession,
                                      "SENS:FREQ:STAR",
                                      startFrequency,
                                      0.0,0.0,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Start Frequency                                             */
/* Purpose:  This function gets the start frequency.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getStartFrequency (ViSession instrSession,
                                           ViPReal64 startFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViReal64    centerFrequency = 0.0,
                span = 0.0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!startFrequency)
        return VI_ERROR_PARAMETER2;

    /* Read settings */
    CHECKERR (rsfsh_getCommand (instrSession, "FREQ", buffer));
    sscanf (buffer, "%Le", &centerFrequency);
    CHECKERR (rsfsh_getCommand (instrSession, "SPAN", buffer));
    sscanf (buffer, "%Le", &span);

    /* Calculate settings */
    *startFrequency = centerFrequency - span/2.0;
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      "SENS:FREQ:STAR?",
                                      startFrequency,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Stop Frequency                                              */
/* Purpose:  This function sets the stop frequency.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setStopFrequency (ViSession instrSession,
                                          ViReal64 stopFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViReal64    startFrequency = 0.0,
                centerFrequency = 0.0,
                span = 0.0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Not range checked */

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    /* Read settings */
    CHECKERR (rsfsh_getCommand (instrSession, "FREQ", buffer));
    sscanf (buffer, "%Le", &centerFrequency);
    CHECKERR (rsfsh_getCommand (instrSession, "SPAN", buffer));
    sscanf (buffer, "%Le", &span);

    /* Calculate settings */
    startFrequency = centerFrequency - span/2.0;
    span = stopFrequency - startFrequency;
    centerFrequency = stopFrequency - span/2.0;

    /* Store settings */
    sprintf (buffer, "FREQ,%Lf", centerFrequency);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    sprintf (buffer, "SPAN,%Lf", span);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setViReal64 (instrSession,
                                      "SENS:FREQ:STOP",
                                      stopFrequency,
                                      0.0,0.0,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Stop Frequency                                              */
/* Purpose:  This function gets the stop frequency.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getStopFrequency (ViSession instrSession,
                                          ViPReal64 stopFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViReal64    centerFrequency = 0.0,
                span = 0.0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!stopFrequency)
        return VI_ERROR_PARAMETER2;

    /* Read settings */
    CHECKERR (rsfsh_getCommand (instrSession, "FREQ", buffer));
    sscanf (buffer, "%Le", &centerFrequency);
    CHECKERR (rsfsh_getCommand (instrSession, "SPAN", buffer));
    sscanf (buffer, "%Le", &span);

    /* Calculate settings */
    *stopFrequency = centerFrequency + span/2.0;
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      "SENS:FREQ:STOP?",
                                      stopFrequency,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Auto Span Mode                                              */
/* Purpose:  This function switches span mode AUTO on/off (Measure Mode:     */
/*           Channel Power or Occupied BW).                                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAutoSpanMode (ViSession instrSession,
                                         ViBoolean spanMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (spanMode))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "AUTOSPAN,%ld", spanMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Auto Span Mode                                              */
/* Purpose:  This function returns span mode AUTO on/off (Measure Mode:      */
/*           Channel Power or Occupied BW).                                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAutoSpanMode (ViSession instrSession,
                                         ViBoolean *spanMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!spanMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "AUTOSPAN", buffer));
    sscanf (buffer, "%hd", spanMode);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Step Size Frequency                                         */
/* Purpose:  This function sets the step width of the center frequency.      */                                                          
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setStepSizeFrequency (ViSession instrSession,
                                              ViReal64 stepSize)

{
    return rsfsh_setViReal64 (instrSession,
                              "SENS:FREQ:CENT:STEP",
                              stepSize,
                              0.0,0.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Step Size Frequency                                         */
/* Purpose:  This function sets the step width of the center frequency.      */                                                          
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getStepSizeFrequency (ViSession instrSession,
                                              ViPReal64 stepSize)

{
    return rsfsh_getViReal64 (instrSession,
                              "SENS:FREQ:CENT:STEP?",
                              stepSize,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Step Link
/* Purpose:  This function couples the step size of the center frequency to 
/*           span (span >0) or to the resolution bandwidth (span = 0) or 
/*           cancels the couplings.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setStepLink(
    ViSession   instrSession,
    ViInt32 stepLink
)
{
    return rsfsh_setEnum (instrSession,
                          "SENS:FREQ:CENT:STEP:LINK",
                          stepLink,
                          stepLinkArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Step Link
/* Purpose:  This function querie the step size of the center frequency to 
/*           span (span >0) or to the resolution bandwidth (span = 0) or 
/*           cancels the couplings.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getStepLink(
    ViSession   instrSession,
    ViInt32*    stepLink
)
{
    return rsfsh_getEnum (instrSession,
                          "SENS:FREQ:CENT:STEP:LINK?",
                          stepLink,
                          stepLinkArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Frequency Input Mode
/* Purpose:  This function sets the frequency mode. Select the Channel 
/*           frequency mode only if you want to work with channel tables. The 
/*           input of the center frequency in this case is not done as a
/*           frequency value, but as a channel number.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setFrequencyInputMode(
    ViSession   instrSession,
    ViInt32 inputMode
)
{
    return rsfsh_setEnum (instrSession,
                          "SENS:FREQ:INP:MODE",
                          inputMode,
                          freqInputModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Frequency Input Mode
/* Purpose:  This function queries the frequency mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getFrequencyInputMode(
    ViSession   instrSession,
    ViInt32*    inputMode
)
{
    return rsfsh_getEnum (instrSession,
                          "SENS:FREQ:INP:MODE?",
                          inputMode,
                          freqInputModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Frequency Mode                                              */
/* Purpose:  This function switches between span > 0 (SWEep) and zero span   */
/*           (CW | FIXed) in the Spectrum Analyzer mode. For CW and FIXed,   */
/*           the frequency setting is via the SENSe:FREQuency:CENTer       */
/*           command. In the sweep mode, the setting is via the              */
/*           SENSe:FREQuency:STARt, SENSe:FREQuency:STOP,                */
/*           SENSe:FREQuency:CENTer, and SENSe:FREQuency:SPAN commands.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setFrequencyMode (ViSession instrSession,
                                          ViInt32 frequencyMode)

{
    return rsfsh_setEnum (instrSession,
                          "SENS:FREQ:MODE",
                          frequencyMode,
                          frequencyModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Frequency Mode                                              */
/* Purpose:  This function queries the frequency mode of the instrument.     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getFrequencyMode (ViSession instrSession,
                                          ViPInt32 frequencyMode)

{
    return rsfsh_getEnum (instrSession,
                          "SENS:FREQ:MODE?",
                          frequencyMode,
                          frequencyModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Channel Number
/* Purpose:  This function defines the channel number if the frequency mode is
/*           set ot "channel".
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setChannelNumber (ViSession instrSession,
                                          ViInt32 numberOfChannel)
{
    return rsfsh_setViInt32(instrSession,
                            "SENSe:CHANnel",
                            numberOfChannel,
                            37750,
                            38249,
                            VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Channel Number
/* Purpose:  This function returns the channel number if the frequency mode is
/*           set ot "channel".
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getChannelNumber (ViSession instrSession,
                                          ViInt32 *numberOfChannel)
{
    return rsfsh_getViInt32(instrSession,
                            "SENSe:CHANnel?",
                            numberOfChannel,
                            VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Configure Amplitude                                             */
/* Purpose:  This function sets the reference level, level units and         */
/*           reference level offset. The new reference level is the current  */
/*           reference level plus the reference level offset.                */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureAmplitude (ViSession instrSession,
                                            ViReal64 referenceLevel,
                                            ViInt32 unit,
                                            ViReal64 referenceLevelOffset)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    /* Not range checked */

    if (rsfsh_invalidViInt32Range (unit, 0, 8))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViReal64Range (referenceLevelOffset, -200.0, 200.0))
        return VI_ERROR_PARAMETER4;

    CHECKERR( rsfsh_setReferenceLevelOffset (instrSession, 0.0));
    CHECKERR( rsfsh_setLevelUnits (instrSession, unit));
    CHECKERR( rsfsh_setReferenceLevel (instrSession, referenceLevel));
    CHECKERR( rsfsh_setReferenceLevelOffset (instrSession, referenceLevelOffset));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Reference Level                                             */
/* Purpose:  This function sets the reference level using the current units. */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReferenceLevel (ViSession instrSession,
                                           ViReal64 referenceLevel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Not range checked */
    switch (instrPtr -> fsh3)
    {
    case FSH3:
    sprintf (buffer, "REFLVL,%Lf", referenceLevel);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setViReal64 (instrSession,
                                      "DISP:WIND:TRACe:Y:SCAL:RLEV",
                                      referenceLevel,
                                      0.0,0.0,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Reference Level                                             */
/* Purpose:  This function gets the reference level using the current units. */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReferenceLevel (ViSession instrSession,
                                           ViPReal64 referenceLevel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!referenceLevel)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "REFLVL", buffer));
    sscanf (buffer, "%Le", referenceLevel);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      "DISP:WIND:TRACe:Y:SCAL:RLEV?",
                                      referenceLevel,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Reference Level Offset                                      */
/* Purpose:  This function sets the reference level offset in dB. The new    */
/*           reference level is the current reference level plus the         */
/*           reference level offset.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReferenceLevelOffset (ViSession instrSession,
                                                 ViReal64 referenceLevelOffset)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (referenceLevelOffset, -100.0, 100.0))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "REFLVLOFFS,%Lf", referenceLevelOffset);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setViReal64 (instrSession,
                                      "DISP:WIND:TRAC:Y:SCAL:RLEV:OFFS",
                                      referenceLevelOffset,
                                      -200.0,200.0,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Reference Level Offset                                      */
/* Purpose:  This function gets the reference level offset in dB.            */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReferenceLevelOffset (ViSession instrSession,
                                                 ViPReal64 referenceLevelOffset)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!referenceLevelOffset)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "REFLVLOFFS", buffer));
    sscanf (buffer, "%Le", referenceLevelOffset);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      "DISP:WIND:TRAC:Y:SCAL:RLEV:OFFS?",
                                      referenceLevelOffset,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Vertical Scale                                              */
/* Purpose:  This function configures the scaling for the level display      */
/*           range.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setVerticalScale (ViSession instrSession,
                                          ViInt32 trace,
                                          ViInt32 verticalScale)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "DISP:WIND:TRAC%ld:Y:SPAC", trace);

    return rsfsh_setEnum (instrSession,
                          buffer,
                          verticalScale,
                          verticalScaleArr,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Vertical Scale                                              */
/* Purpose:  This function queries the scaling for the level display         */
/*           range.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVerticalScale (ViSession instrSession,
                                          ViInt32 trace,
                                          ViInt32 *verticalScale)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "DISP:WIND:TRAC%ld:Y:SPAC?", trace);

    return rsfsh_getEnum (instrSession,
                          buffer,
                          verticalScale,
                          verticalScaleArr,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Level Range                                                 */
/* Purpose:  This function defines the instrument's display range.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setLevelRange (ViSession instrSession, ViInt32 range)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (range, 0, 36))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "RANGE,%ld", range);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Level Range                                                 */
/* Purpose:  This function returns the instrument's display range.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLevelRange (ViSession instrSession,
                                       ViPInt32 range)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!range)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "RANGE", buffer));
    sscanf (buffer, "%ld", range);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Dynamic Range                                               */
/* Purpose:  This function defines the dynamic range.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDynamicRange (ViSession instrSession,
                                         ViBoolean range)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (range))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "DYNRANGE,%ld", range);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Dynamic Range                                               */
/* Purpose:  This function returns the dynamic range.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDynamicRange (ViSession instrSession,
                                         ViBoolean *range)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!range)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "DYNRANGE", buffer));
    sscanf (buffer, "%hd", range);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;     
}

/*===========================================================================*/
/* Function: Set Level Units                                                 */
/* Purpose:  This function defines the unit of the reference level.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setLevelUnits (ViSession instrSession, ViInt32 unit)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (unit, 0, 8))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "UNIT,%ld", unit);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Level Units                                                 */
/* Purpose:  This function returns the unit of the reference level.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLevelUnits (ViSession instrSession, ViPInt32 unit)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!unit)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "UNIT", buffer));
    sscanf (buffer, "%ld", unit);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set RF Input Impedance                                          */
/* Purpose:  This function defines the RF input impedance.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setRFInputImpedance (ViSession instrSession,
                                             ViInt32 RFInputImpedance)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (RFInputImpedance, 0, 1))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "RFINPUT,%ld", RFInputImpedance);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get RF Input Impedance                                          */
/* Purpose:  This function returns the RF input impedance.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getRFInputImpedance (ViSession instrSession,
                                             ViPInt32 RFInputImpedance)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!RFInputImpedance)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "RFINPUT", buffer));
    sscanf (buffer, "%ld", RFInputImpedance);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Preamplifier State                                          */
/* Purpose:  This function switches the preamplifier on/off.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPreamplifierState (ViSession instrSession,
                                              ViBoolean preamplifier)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (preamplifier))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "PREAMP,%ld", preamplifier);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setViBoolean (instrSession,
                                       "INP:GAIN:STAT",
                                       preamplifier,
                                       VI_ERROR_PARAMETER2);
    break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Preamplifier State                                          */
/* Purpose:  This function returns the preamplifier state (on/off).          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPreamplifierState (ViSession instrSession,
                                              ViPBoolean preamplifier)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!preamplifier)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "PREAMP", buffer));
    sscanf (buffer, "%hd", preamplifier);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViBoolean (instrSession,
                                       "INP:GAIN:STAT?",
                                       preamplifier,
                                       VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Amplitude Units                                             */
/* Purpose:  This function selects the default unit.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAmplitudeUnits (ViSession instrSession,
                                           ViInt32 amplitudeUnits)
{
    return rsfsh_setEnum (instrSession,
                          "UNIT:POW",
                          amplitudeUnits,
                          amplitudeUnitsArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Amplitude Units                                             */
/* Purpose:  This function queries the default unit.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAmplitudeUnits (ViSession instrSession,
                                           ViInt32 *amplitudeUnits)
{
    return rsfsh_getEnum (instrSession,
                          "UNIT:POW?",
                          amplitudeUnits,
                          amplitudeUnitsArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Select Transducer Factor
/* Purpose:  This function sets the transducer factor designated by <name>. 
/*           If <name> does not exist yet, a new transducer factor is 
/*           created.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_SelectTransducerFactor (ViSession instrSession,
                                             ViInt32 transducer,
                                             ViString transducerFactor)
{
    return rsfsh_setTrackingGeneratorCalibrationTransducerSelect (instrSession,
                                                                  transducer,
                                                                  transducerFactor);
    
}

/*===========================================================================*/
/* Function: Get Selected Transducer Factor
/* Purpose:  This function queries the transducer factor designated by 
/*           <name>. If <name> does not exist yet, a new transducer factor is 
/*           created.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getSelectedTransducerFactor (ViSession instrSession,
                                             ViInt32 transducer,
                                             ViChar transducerFactor[])
{
    return rsfsh_getTrackingGeneratorCalibrationTransducerSelect (instrSession,
                                                                  transducer,
                                                                  transducerFactor);
}

/*===========================================================================*/
/* Function: Set Transducer Factor State
/* Purpose:  This function sets the selected transducer factor on or off.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTransducerFactorState (ViSession instrSession,
                                            ViInt32 transducer, ViBoolean state)
{
    return rsfsh_setTrackingGeneratorCalibrationTransducerState (instrSession,
                                                                 transducer,
                                                                 state);
}

/*===========================================================================*/
/* Function: Get Transducer Factor State
/* Purpose:  This function queries the selected transducer factor on or off.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTransducerFactorState (ViSession instrSession,
                                            ViInt32 transducer, ViBoolean *state)
{
    return rsfsh_getTrackingGeneratorCalibrationTransducerState (instrSession,
                                                                 transducer,
                                                                 state);
}

/*===========================================================================*/
/* Function: Get Transducer Factor Unit
/* Purpose:  This function queries the unit of the transducer factor in use.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTransducerFactorUnit (ViSession instrSession,
                                           ViInt32 transducer, ViChar unit[])
{
    return rsfsh_getTrackingGeneratorCalibrationTransducerUnit (instrSession,
                                                                transducer,
                                                                unit);
}

/*===========================================================================*/
/* Function: Configure Bandwidth                                             */
/* Purpose:  This function sets the resolution and video bandwidth.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureBandwidth (ViSession instrSession,
                                            ViInt32 resolutionBandwidth,
                                            ViInt32 videoBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViInt32Range (videoBandwidth, 0, 12))
        return VI_ERROR_PARAMETER3;

    CHECKERR (rsfsh_setResolutionBandwidth (instrSession, resolutionBandwidth));
    CHECKERR (rsfsh_setVideoBandwidth (instrSession, videoBandwidth));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Resolution Bandwidth                                        */
/* Purpose:  This function sets the resolution bandwidth.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setResolutionBandwidth (ViSession instrSession,
                                                ViInt32 resolutionBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    if (rsfsh_invalidViInt32Range (resolutionBandwidth, 0, 15))
        return VI_ERROR_PARAMETER2;

    switch (instrPtr -> fsh3)
    {
    case FSH3:

    sprintf (buffer, "RBW,%ld", resolutionBandwidth);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    if (resolutionBandwidth == 0)
    {
        rsfsh_status = rsfsh_setAutoResolutionBandwidth (instrSession, VI_TRUE);
    }
    else
    {
        sprintf (buffer, "SENS:BAND:RES %s", resolutionBandwidthArr[resolutionBandwidth]);
        rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
    }
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Resolution Bandwidth                                        */
/* Purpose:  This function gets the resolution bandwidth.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getResolutionBandwidth (ViSession instrSession,
                                                ViPInt32 resolutionBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!resolutionBandwidth)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "RBW", buffer));
    sscanf (buffer, "%ld", resolutionBandwidth);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViInt32 (instrSession,
                                     "SENS:BAND:RES?",
                                     resolutionBandwidth,
                                     VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Auto Resolution Bandwidth                                   */
/* Purpose:  This function switches the auto coupling of the resolution      */
/*           bandwidth on/off.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAutoResolutionBandwidth (ViSession instrSession,
                                                    ViBoolean autoResolutionBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (autoResolutionBandwidth))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "AUTORBW,%ld", autoResolutionBandwidth);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setViBoolean (instrSession,
                                       "SENS:BAND:RES:AUTO",
                                       autoResolutionBandwidth,
                                       VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Auto Resolution Bandwidth                                   */
/* Purpose:  This function queries the setting of auto coupling of the       */
/*           resolution bandwidth.                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAutoResolutionBandwidth (ViSession instrSession,
                                                    ViBoolean *autoResolutionBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!autoResolutionBandwidth)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "AUTORBW", buffer));
    sscanf (buffer, "%hu", autoResolutionBandwidth);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViBoolean (instrSession,
                                       "SENS:BAND:RES:AUTO?",
                                       autoResolutionBandwidth,
                                       VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Video Bandwidth                                             */
/* Purpose:  This function sets the video bandwidth.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setVideoBandwidth (ViSession instrSession,
                                           ViInt32 videoBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (videoBandwidth, 0, 12))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "VBW,%ld", videoBandwidth);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    sprintf (buffer, "SENS:BAND:VID %s", videoBandwidthArr[videoBandwidth]);    
    
    rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Video Bandwidth                                             */
/* Purpose:  This function gets the video bandwidth.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVideoBandwidth (ViSession instrSession,
                                           ViPInt32 videoBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!videoBandwidth)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "VBW", buffer));
    sscanf (buffer, "%ld", videoBandwidth);
    case FSH4:
    rsfsh_status = rsfsh_getViInt32 (instrSession,
                                     "SENS:BAND:VID?",
                                     videoBandwidth,
                                     VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Auto Video Bandwidth                                        */
/* Purpose:  This function switches the auto coupling of the video           */
/*           bandwidth on/off.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAutoVideoBandwidth (ViSession instrSession,
                                               ViBoolean autoVideoBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (autoVideoBandwidth))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "AUTOVBW,%ld", autoVideoBandwidth);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    case FSH4:
    rsfsh_status = rsfsh_setViBoolean (instrSession,
                                       "SENS:BAND:VID:AUTO",
                                       autoVideoBandwidth,
                                       VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Auto Video Bandwidth                                        */
/* Purpose:  This function queries the setting of auto coupling of the       */
/*           video bandwidth.                                                */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAutoVideoBandwidth (ViSession instrSession,
                                               ViBoolean *autoVideoBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!autoVideoBandwidth)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "AUTOVBW", buffer));
    sscanf (buffer, "%hu", autoVideoBandwidth);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViBoolean (instrSession,
                                       "SENS:BAND:VID:AUTO?",
                                       autoVideoBandwidth,
                                       VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set CISPR Bandwidth                                             */
/* Purpose:  This function sets the CISPR bandwidth. This command is         */
/*           available in Receiver Mode only (R&S FSH-K3).                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCISPRBandwidth (ViSession instrSession,
                                           ViInt32 CISPRBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (CISPRBandwidth, 0, 3))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CISPRBW,%ld", CISPRBandwidth);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setEnum (instrSession,
                          "SENS:BAND:RES:CISP",
                          CISPRBandwidth,
                          CISPRBandwidthArr,
                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get CISPR Bandwidth                                             */
/* Purpose:  This function gets the CISPR bandwidth. This command is         */
/*           available in Receiver Mode only (R&S FSH-K3).                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCISPRBandwidth (ViSession instrSession,
                                           ViPInt32 CISPRBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!CISPRBandwidth)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CISPRBW", buffer));
    sscanf (buffer, "%ld", CISPRBandwidth);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getEnum (instrSession,
                          "SENS:BAND:RES:CISP?",
                          CISPRBandwidth,
                          CISPRBandwidthArr,
                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Auto CISPR Bandwidth                                        */
/* Purpose:  This function switches the auto setting of the CISPR bandwidth  */
/*           on/off. If AUTOCISPRBW is active the current setting of the     */
/*           CISPR bandwidth can be queried with the GET CISPRBW command.    */
/*           This command is available in Receiver Mode only (R&S FSH-K3).   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAutoCISPRBandwidth (ViSession instrSession,
                                               ViBoolean autoCISPRBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (autoCISPRBandwidth))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "AUTOCISPRBW,%ld", autoCISPRBandwidth);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setViBoolean (instrSession,
                          "SENS:BAND:RES:CISP:AUTO",
                          autoCISPRBandwidth,
                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Auto CISPR Bandwidth                                        */
/* Purpose:  This function queries the auto setting of the CISPR bandwidth.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAutoCISPRBandwidth (ViSession instrSession,
                                               ViBoolean *autoCISPRBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!autoCISPRBandwidth)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "AUTOCISPRBW", buffer));
    sscanf (buffer, "%hu", autoCISPRBandwidth);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getViBoolean (instrSession,
                          "SENS:BAND:RES:CISP:AUTO?",
                          autoCISPRBandwidth,
                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Sweep                                                 */
/* Purpose:  This function configures instrument's sweep parameters.         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureSweep (ViSession instrSession,
                                        ViInt32 sweepMode, ViReal64 sweepTime,
                                        ViInt32 triggerSource)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViInt32Range (sweepMode, 0, 1))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViReal64Range (sweepTime, 0.0, 1000.0))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViInt32Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER4;

    CHECKERR (rsfsh_setSweepMode (instrSession, sweepMode));
    CHECKERR (rsfsh_setSweepTime (instrSession, sweepTime));
    CHECKERR (rsfsh_setTriggerSource ( instrSession, triggerSource));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Trigger                                               */
/* Purpose:  This function specifies the triggering.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureTrigger (ViSession instrSession,
                                          ViInt32 videoLevel, 
                                          ViInt32 triggerPolarity,
                                          ViInt32 triggerSource)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViInt32Range (videoLevel, 0, 100))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (triggerPolarity, 0, 1))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViInt32Range (triggerSource, 0, 2))
        return VI_ERROR_PARAMETER4;

    CHECKERR (rsfsh_setTriggerVideoLevel (instrSession, videoLevel));
    CHECKERR (rsfsh_setTriggerPolarity (instrSession, triggerPolarity));
    CHECKERR (rsfsh_setTriggerSource ( instrSession, triggerSource));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Sweep Coupling Auto                                   */
/* Purpose:  This function activates auto coupling of selected coupling      */
/*           method.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureSweepCouplingAuto (ViSession instrSession,
                                                    ViInt32 sweepCoupling)
{
    ViChar      buffer[BUFFER_SIZE];
    
    switch (sweepCoupling)
    {
        case RSFSH_VAL_COUPLING_RBW:
            sprintf (buffer, "SENS:BAND:RES:AUTO ON");
            break;
            
        case RSFSH_VAL_COUPLING_VBW:
            sprintf (buffer, "SENS:BAND:VID:AUTO ON");
            break;
            
        case RSFSH_VAL_COUPLING_SWEEP_TIME:
            sprintf (buffer, "SENS:SWE:TIME:AUTO ON");
            break;
            
        default: 
            return VI_ERROR_PARAMETER2;
    }
    
    return rsfsh_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Configure Sweep Coupling                                        */
/* Purpose:  This function configures the coupling values of the spectrum    */
/*           analyzer.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureSweepCoupling (ViSession instrSession,
                                                ViInt32 sweepCoupling,
                                                ViReal64 couplingValue)
{
    ViChar      buffer[BUFFER_SIZE];
    
    switch (sweepCoupling)
    {
        case RSFSH_VAL_COUPLING_RBW:
            sprintf (buffer, "SENS:BAND:RES");
            break;
            
        case RSFSH_VAL_COUPLING_VBW:
            sprintf (buffer, "SENS:BAND:VID");
            break;
            
        case RSFSH_VAL_COUPLING_SWEEP_TIME:
            sprintf (buffer, "SENS:SWE:TIME");
            break;
            
        default: 
            return VI_ERROR_PARAMETER2;
    }
    
    return rsfsh_setViReal64 (instrSession, 
                              buffer,
                              couplingValue,
                              0.0,0.0,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Configure Acquisition                                           */
/* Purpose:  This function configures the sweep.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureAcquisition (ViSession instrSession,
                                              ViBoolean sweepModeContinuous,
                                              ViInt32 numberOfSweeps)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    CHECKERR (rsfsh_setSweepModeContinuous (instrSession, sweepModeContinuous));
    
    CHECKERR (rsfsh_setNumberOfSweeps (instrSession, numberOfSweeps));
    REMAP (2,3);
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Sweep Time                                                  */
/* Purpose:  This function sets the sweep time.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setSweepTime (ViSession instrSession,
                                      ViReal64 sweepTime)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      buffer2[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (sweepTime, 0.0, 1000.0))
        return VI_ERROR_PARAMETER2;

    if (sweepTime < NUMERIC_PRECISION)
    {
        sprintf (buffer, "AUTOSWPTIME,1");
        CHECKERR (rsfsh_setCommand (instrSession, buffer));
    }
    else
    {
        sprintf (buffer, "AUTOSWPTIME,0");
        CHECKERR (rsfsh_setCommand (instrSession, buffer));
        
        sprintf (buffer2, "SWPTIME,%Lf", sweepTime);
        CHECKERR (rsfsh_setCommand (instrSession, buffer2));
    }
    break;
    case FSH4:
    rsfsh_status = rsfsh_setViReal64 (instrSession,
                                      "SENS:SWE:TIME",
                                      sweepTime,
                                      0.0,0.0,
                                      VI_ERROR_PARAMETER2);
    break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Sweep Time                                                  */
/* Purpose:  This function gets the sweep time.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getSweepTime (ViSession instrSession,
                                      ViPReal64 sweepTime)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!sweepTime)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "SWPTIME", buffer));
    sscanf (buffer, "%Le", sweepTime);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      "SENS:SWE:TIME?",
                                      sweepTime,
                                      VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Auto Sweep Time                                             */
/* Purpose:  This function controls the automatic coupling of the sweep time */
/*           to the frequency span and bandwidth settings.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAutoSweepTime (ViSession instrSession,
                                          ViBoolean autoSweepTime)
{
    
    return rsfsh_setViBoolean (instrSession,
                               "SENS:SWE:TIME:AUTO",
                               autoSweepTime,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Auto Sweep Time                                             */
/* Purpose:  This function gets the auto sweep time.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAutoSweepTime (ViSession instrSession,
                                          ViBoolean *autoSweepTime)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!autoSweepTime)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "AUTOSWPTIME", buffer));
    sscanf (buffer, "%hd", autoSweepTime);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getViBoolean (instrSession,
                                       "SENS:SWE:TIME:AUTO?",
                                       autoSweepTime,
                                       VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;     
}

/*===========================================================================*/
/* Function: Set Sweep Mode                                                  */
/* Purpose:  This function sets the instrument to single sweep or continuous */
/*           sweep.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setSweepMode (ViSession instrSession,
                                      ViInt32 sweepMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (sweepMode, 0, 1))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SWPCONT,%ld", sweepMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Sweep Mode                                                  */
/* Purpose:  This function returns the instrument sweep mode.                */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getSweepMode (ViSession instrSession,
                                      ViPInt32 sweepMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!sweepMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "SWPCONT", buffer));
    sscanf (buffer, "%ld", sweepMode);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Number Of Sweeps                                            */
/* Purpose:  This function defines the number of sweeps started with single  */
/*           sweep, which are used for calculating the average or maximum    */
/*           value. If the values 0 or 1 are set, one sweep is performed.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setNumberOfSweeps (ViSession instrSession,
                                           ViInt32 numberOfSweeps)
{
    return rsfsh_setViInt32 (instrSession,
                             "SENS:SWE:COUN",
                             numberOfSweeps,
                             0, 32767,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Number Of Sweeps                                            */
/* Purpose:  This function queries the number of sweeps started with single  */
/*           sweep, which are used for calculating the average or maximum    */
/*           value.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getNumberOfSweeps (ViSession instrSession,
                                           ViPInt32 numberOfSweeps)
{
    return rsfsh_getViInt32 (instrSession,
                             "SENS:SWE:COUN?",
                             numberOfSweeps,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Sweep Mode Continuous                                       */
/* Purpose:  This function determines whether the trigger system is          */
/*           continuously initiated (continuous) or performs single          */
/*           measurements (single).                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setSweepModeContinuous (ViSession instrSession,
                                                ViBoolean sweepModeContinuous)
{
    return rsfsh_setViBoolean (instrSession,
                               "INIT:CONT",
                               sweepModeContinuous,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Sweep Mode Continuous                                       */
/* Purpose:  This function queries whether the trigger system is continuously*/
/*           initiated (continuous) or performs single measurements (single).*/
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getSweepModeContinuous (ViSession instrSession,
                                                ViBoolean *sweepModeContinuous)
{
    return rsfsh_getViBoolean (instrSession,
                               "INIT:CONT?",
                               sweepModeContinuous,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Trigger Source                                              */
/* Purpose:  This function selects the trigger source.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTriggerSource (ViSession instrSession,
                                          ViInt32 triggerSource)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRIGSRC,%ld", triggerSource);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setEnum (instrSession,
                                  "TRIG:SEQ:SOUR",
                                  triggerSource,
                                  triggerSourceArr,
                                  VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Trigger Source                                              */
/* Purpose:  This function returns the trigger source.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTriggerSource (ViSession instrSession,
                                          ViPInt32 triggerSource)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!triggerSource)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRIGSRC", buffer));
    sscanf (buffer, "%ld", triggerSource);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getEnum (instrSession,
                                  "TRIG:SEQ:SOUR?",
                                  triggerSource,
                                  triggerSourceArr,
                                  VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Trigger Level                                               */
/* Purpose:  This function defines the video trigger level.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTriggerLevel (ViSession instrSession,
                                         ViInt32 triggerLevel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRIGLVL,%ld", triggerLevel);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Trigger Level                                               */
/* Purpose:  This function returns the video trigger level.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTriggerLevel (ViSession instrSession,
                                         ViInt32 *triggerLevel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!triggerLevel)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRIGLVL", buffer));
    sscanf (buffer, "%ld", triggerLevel);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Trigger Delay                                               */
/* Purpose:  This function defines the trigger delay.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTriggerDelay (ViSession instrSession,
                                         ViReal64 triggerDelay)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (triggerDelay, 0.0, 100.0))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRIGDEL,%Lf", triggerDelay);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    CHECKERR (rsfsh_setViReal64 (instrSession,
                                 "TRIG:SEQ:HOLD:TIME",
                                 triggerDelay,
                                 0.0, 100.0,
                                 VI_ERROR_PARAMETER2));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Trigger Delay                                               */
/* Purpose:  This function returns the trigger delay.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTriggerDelay (ViSession instrSession,
                                         ViPReal64 triggerDelay)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!triggerDelay)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRIGDEL", buffer));
    sscanf (buffer, "%Le", triggerDelay);
    break;
    case FSH4:
    CHECKERR (rsfsh_getViReal64 (instrSession,
                                 "TRIG:SEQ:HOLD:TIME?",
                                 triggerDelay,
                                 VI_ERROR_PARAMETER2));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Trigger Polarity                                            */
/* Purpose:  This function selects the slope of the trigger signal. The      */
/*           selected trigger slope applies to all trigger signal sources.   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTriggerPolarity (ViSession instrSession,
                                            ViInt32 triggerPolarity)
{
    return rsfsh_setEnum (instrSession,
                          "TRIG:SEQ:SLOP",
                          triggerPolarity,
                          polarityArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Trigger Polarity                                            */
/* Purpose:  This function queries the slope of the trigger signal. The      */
/*           selected trigger slope applies to all trigger signal sources.   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTriggerPolarity (ViSession instrSession,
                                            ViInt32 *triggerPolarity)
{
    return rsfsh_getEnum (instrSession,
                          "TRIG:SEQ:SLOP?",
                          triggerPolarity,
                          polarityArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Trigger Video Level                                         */
/* Purpose:  This function sets the level of the video trigger source.       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTriggerVideoLevel (ViSession instrSession,
                                              ViInt32 videoLevel)
{
    return rsfsh_setViInt32 (instrSession,
                             "TRIG:SEQ:LEV:VID",
                             videoLevel,
                             0,100,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Trigger Video Level                                         */
/* Purpose:  This function queries the level of the video trigger source.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTriggerVideoLevel (ViSession instrSession,
                                              ViInt32 *videoLevel)
{
    return rsfsh_getViInt32 (instrSession,
                             "TRIG:SEQ:LEV:VID?",
                             videoLevel,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Configure Trace                                                 */
/* Purpose:  This function configures trace parameters such as trace mode and*/
/*           trace detector.                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureTrace (ViSession instrSession,
                                        ViInt32 traceMode, ViInt32 traceDetector)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViInt32Range (traceMode, 0, 4))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (traceDetector, 0, 5))
        return VI_ERROR_PARAMETER3;

    CHECKERR (rsfsh_setTraceMode (instrSession, traceMode));
    CHECKERR (rsfsh_setTraceDetector (instrSession, traceDetector));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Trace Detector                                        */
/* Purpose:  This function configures trace parameters such as trace mode and*/
/*           trace detector.                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureTraceDetector (ViSession instrSession,
                                                ViInt32 trace, 
                                                ViBoolean detectorTypeAuto,
                                                ViInt32 detectorType)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViInt32Range (trace, 1, 6))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViBooleanRange (detectorTypeAuto))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViInt32Range (detectorType, 0, 4))
        return VI_ERROR_PARAMETER4;

    CHECKERR (rsfsh_setTraceDetectorAuto (instrSession, trace, detectorTypeAuto));
    CHECKERR (rsfsh_setTraceDetectorType (instrSession, detectorType));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configre Trace Math
/* Purpose:  This function configures the mathematical relation of traces.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureTraceMath(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean   state,
    ViString    expression
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    if (state == VI_TRUE){
        sprintf (buffer, "CALC:MATH%ld:STAT", trace);
        
        CHECKERR (rsfsh_setViBoolean (instrSession,
                                      buffer,
                                      state,
                                      VI_ERROR_PARAMETER3));
    }
    
    sprintf (buffer, "CALC:MATH%ld:EXPR:DEF", trace);
    
    CHECKERR (rsfsh_setViString (instrSession,
                                 buffer,
                                 expression,
                                 VI_ERROR_PARAMETER4));
    
    return rsfsh_status;
        
}

/*===========================================================================*/
/* Function: Trace To Memory                                                 */
/* Purpose:  This function stores current trace into memory.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTraceToMemory (ViSession instrSession)
{
    return rsfsh_cmdCommand (instrSession, "TRACETOMEM");
}

/*===========================================================================*/
/* Function: Set Trace Mode                                                  */
/* Purpose:  This function defines the trace mode.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTraceMode (ViSession instrSession,
                                      ViInt32 traceMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (traceMode, 0, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "TRACEMODE,%ld", traceMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setEnum (instrSession,
                                  "DISP:WIND:TRAC:MODE",
                                  traceMode,
                                  traceModeArr,
                                  VI_ERROR_PARAMETER3);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Trace Mode                                                  */
/* Purpose:  This function returns the trace mode.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTraceMode (ViSession instrSession,
                                      ViPInt32 traceMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!traceMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRACEMODE", buffer));
    sscanf (buffer, "%ld", traceMode);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getEnum (instrSession,
                                  "DISP:WIND:TRAC:MODE?",
                                  traceMode,
                                  traceModeArr,
                                  VI_ERROR_PARAMETER3);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Trace Style                                                 */
/* Purpose:  This function sets the trace style.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_SetTraceStyle (ViSession instrSession,
                                       ViInt32 traceStyle)
{
	ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH4:
    	rsfsh_status = rsfsh_setEnum (instrSession,
                                  "DISP:WIND:TRAC:STYL",
                                  traceStyle,
                                  traceStyleArr,
                                  VI_ERROR_PARAMETER2);
    	break;
	default:
		break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Trace Style                                                 */
/* Purpose:  This function gets the trace style.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_GetTraceStyle (ViSession instrSession,
                                       ViInt32 *traceStyle)
{
	ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH4:
    	rsfsh_status = rsfsh_getEnum (instrSession,
                                  "DISP:WIND:TRAC:STYL?",
                                  traceStyle,
                                  traceStyleArr,
                                  VI_ERROR_PARAMETER2);
    	break;
	default:
		break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Trace State                                                 */
/* Purpose:  This function switches on or off the display of the             */
/*           corresponding trace.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTraceState (ViSession instrSession,
                                       ViInt32 trace,
                                       ViBoolean state)
{
    ViChar      buffer[BUFFER_SIZE];

    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "DISP:WIND:TRAC%ld:STAT", trace); 
    return rsfsh_setViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Trace State                                                 */
/* Purpose:  This function querie the state of the display of the            */
/*           corresponding trace.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTraceState (ViSession instrSession,
                                       ViInt32 trace,
                                       ViBoolean *state)
{
    ViChar      buffer[BUFFER_SIZE];

    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "DISP:WIND:TRAC%ld:STAT?", trace); 
    return rsfsh_getViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Memory Trace State
/* Purpose:  This function activates and deactivates the trace stored in the
/*           memory of the R&S FSH. The suffix at TRACe selects the memory
/*           slot the trace is stored in.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMemoryTraceState (ViSession instrSession,
                                             ViInt32 trace, ViBoolean state)
{
    ViChar      buffer[BUFFER_SIZE];

    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "DISPlay:WINDow:TRACe%ld:MEMory:STATe", trace); 
    return rsfsh_setViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Memory Trace State
/* Purpose:  This function returns state of the trace stored in the
/*           memory of the R&S FSH. The suffix at TRACe selects the memory
/*           slot the trace is stored in.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMemoryTraceState (ViSession instrSession,
                                             ViInt32 trace, ViBoolean *state)
{
    ViChar      buffer[BUFFER_SIZE];

    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "DISPlay:WINDow:TRACe%ld:MEMory:STATe?", trace); 
    return rsfsh_getViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Trace Detector                                              */
/* Purpose:  This function defines the trace detector used.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTraceDetector (ViSession instrSession,
                                          ViInt32 traceDetector)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (traceDetector, 0, 6))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRACEDET,%ld", traceDetector);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
    rsfsh_status = rsfsh_setEnum (instrSession,
                                  "SENS:DET:FUNC",
                                  traceDetector,
                                  detectorTypeArr,
                                  VI_ERROR_PARAMETER2);
    break;
    }
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Trace Detector                                              */
/* Purpose:  This function returns the trace detector used.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTraceDetector (ViSession instrSession,
                                          ViPInt32 traceDetector)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!traceDetector)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRACEDET", buffer));
    sscanf (buffer, "%ld", traceDetector);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getEnum (instrSession,
                                  "SENS:DET:FUNC?",
                                  traceDetector,
                                  detectorTypeArr,
                                  VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Trace Detector Auto                                         */
/* Purpose:  This function either couples the detector to the current trace  */
/*           setting or turns coupling off. The trace is selected by the     */
/*           numeric suffix at DETector.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTraceDetectorAuto (ViSession instrSession,
                                              ViInt32 trace,
                                              ViBoolean detectorTypeAuto)
{
    ViChar      buffer[BUFFER_SIZE];

    if (rsfsh_invalidViInt32Range (trace, 1, 6))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:DET%ld:FUNC:AUTO", trace);
    
    return rsfsh_setViBoolean (instrSession,
                               buffer,
                               detectorTypeAuto,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Trace Detector Auto                                         */
/* Purpose:  This function queries couples the detector to the current trace */
/*           setting or turns coupling off. The trace is selected by the     */
/*           numeric suffix at DETector.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTraceDetectorAuto (ViSession instrSession,
                                              ViInt32 trace,
                                              ViBoolean *detectorTypeAuto)
{
    ViChar      buffer[BUFFER_SIZE];

    if (rsfsh_invalidViInt32Range (trace, 1, 6))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:DET%ld:FUNC:AUTO?", trace);
    
    return rsfsh_getViBoolean (instrSession,
                               buffer,
                               detectorTypeAuto,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Trace Detector Type                                         */
/* Purpose:  This function selects the detector for recording measured values*/
/*           of the selected trace.                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTraceDetectorType (ViSession instrSession,
                                              ViInt32 detectorType)
{
    return rsfsh_setEnum (instrSession,
                          "SENS:DET:FUNC",
                          detectorType,
                          detectorTypeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Trace Detector Type                                         */
/* Purpose:  This function queries the detector for recording measured values*/
/*           of the selected trace.                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTraceDetectorType (ViSession instrSession,
                                              ViInt32 *detectorType)
{
    return rsfsh_getEnum (instrSession,
                          "SENS:DET:FUNC?",
                          detectorType,
                          detectorTypeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Trace Average                                               */
/* Purpose:  This function sets the number of traces used to calculate the   */
/*           trace average.                                                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTraceAverage (ViSession instrSession,
                                         ViInt32 numberOfTraces)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (numberOfTraces, 2, 999))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRACEAVG,%ld", numberOfTraces);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Trace Average                                               */
/* Purpose:  This function gets the number of traces used to calculate the   */
/*           trace average.                                                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTraceAverage (ViSession instrSession,
                                         ViInt32 *numberOfTraces)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!numberOfTraces)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRACEAVG", buffer));
    sscanf (buffer, "%ld", numberOfTraces);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Read Trace Data                                                 */
/* Purpose:  This function reads out the trace data. A trace consists of 301 */
/*           samples. If the Auto Peak detector is used, both Max and Min    */
/*           values are returned (602 samples). This function reads out the  */
/*           trace data in binary format.                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_readTraceData (ViSession instrSession,
                                       ViInt32 trace,
                                       ViReal64 _VI_FAR traceData[],
                                       ViPInt32 samplesReturned)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[TRACE_BUFFER_SIZE]; /* Data buffer */
    ViUInt16    intf = VI_INTF_GPIB;
    ViInt32     detector        = 0,
                units           = 0,
                count           = 0;
    ViReal64    dividerArr[]    = {1.0e3, 1.0e3, 1.0e3, 1.0e3, 1.0e3, 1.0e3, 1.0e6, 1.0e9,
                                   1.0e3, 1.0e6};
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!traceData)
        return VI_ERROR_PARAMETER2;
    if (!samplesReturned)
        return VI_ERROR_PARAMETER3;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf));
    /* Get number of trace samples + termination character */
    CHECKERR (rsfsh_getCommand (instrSession, "TRACEDET", buffer));
    sscanf (buffer, "%ld", &detector);
    count = (!detector) ? 602*4+2 : 301*4+1;
    /* Get current level units */
    CHECKERR (rsfsh_getCommand (instrSession, "UNIT", buffer));
    sscanf (buffer, "%ld", &units);
    /* Make sure that units are in allowed range */
    units = ((units < 0) || (units > 9)) ? 0 : units;
    /* Request for trace data */
    CHECKERR (rsfsh_writeInstrData (instrSession, "GET"));
    CHECKERR (rsfsh_writeInstrData (instrSession, "TRACEBIN"));

    /* Interface specific operations */
    switch (intf)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            {
            ViReal64    *p2output = traceData;
            ViInt32     *temp,
                        idx = 0;
            ViUInt32    retCnt;

            /* Disable termination character. */
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

            /* Read trace data */
            rsfsh_status = viRead (instrSession, (ViPBuf) buffer, count, &retCnt);
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);

            CHECKERR (rsfsh_status);

            /* Ignore any warnings */
            rsfsh_status = VI_SUCCESS;

            /* Convert binary data into array of ViReal64 types. */
            count /= 4;
            rsfsh_SWAP_32((unsigned char*) buffer, count);
            /* Set pointer to data */
            temp = (ViInt32*)(buffer);
            if (!detector)
                {
                /* samples 1 to 301 */
                for (idx = 0; idx < 301; idx++)
                    {
                    *p2output++ = *temp++;
                    traceData[idx] /= dividerArr[units];
                    }
                /* Shift pointer to data (skip one byte) */
                temp = (ViInt32*)(buffer + 1205);
                /* samples 302 to 602 */
                for (idx = 301; idx < 601; idx++)
                    {
                    *p2output++ = *temp++;
                    traceData[idx] /= dividerArr[units];
                    }
                }
            else
                {
                /* 301 samples */
                for (idx = 0; idx < 301; idx++)
                    {
                    *p2output++ = *temp++;
                    traceData[idx] /= dividerArr[units];
                    }
                }
            if (samplesReturned)
                *samplesReturned = count;
            }
        break;
        default:
            /*
                NOT SUPPORTED
            */
        break;
        }
    break;
    case FSH4:
        if (rsfsh_invalidViInt32Range (trace, 1, 2))
            return VI_ERROR_PARAMETER2;
        
        sprintf (buffer, "TRACE%ld", trace);
        CHECKERR (rsfsh_dataReadTrace (instrSession, buffer, 631, traceData, samplesReturned));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read Trace Data (ASCII)                                         */
/* Purpose:  This function reads out the trace data. A trace consists of 301 */
/*           samples. If the Auto Peak detector is used, both Max and Min    */
/*           values are returned (602 samples). This function reads out the  */
/*           trace data in alphanumeric format.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_readTraceDataASCII (ViSession instrSession,
                                               ViReal64 _VI_FAR traceData[],
                                               ViPInt32 samplesReturned)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     detector    = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!traceData)
        return VI_ERROR_PARAMETER2;
    if (!samplesReturned)
        return VI_ERROR_PARAMETER3;

    /* Get number of trace samples */
    CHECKERR (rsfsh_getCommand (instrSession, "TRACEDET", buffer));
    sscanf (buffer, "%ld", &detector);
    *samplesReturned = (!detector) ? 602 : 301;
    /* Request for trace data */
    CHECKERR (rsfsh_writeInstrData (instrSession, "GET"));
    CHECKERR (rsfsh_writeInstrData (instrSession, "TRACE"));
    CHECKERR (viScanf (instrSession, "%,#Lf", samplesReturned, traceData));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read SEM Peak List                                              */
/* Purpose:  This function returns the spurious emissions peak list.         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_ReadSEMPeakList (ViSession instrSession,
                                         ViInt32 noOfValues,
                                         ViInt32 rangeNumber[],
                                         ViReal64 startFrequency[],
                                         ViReal64 stopFrequency[],
                                         ViReal64 resolutionBandwidth[],
                                         ViReal64 frequency[],
                                         ViReal64 levelAbs[],
                                         ViReal64 levelRel[], ViReal64 delta[],
                                         ViInt32 limitCheck[],
                                         ViReal64 reserved1[],
                                         ViReal64 reserved2[],
                                         ViInt32 *returnedValues)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      *buffer = VI_NULL;
    ViChar      *pbuf=VI_NULL;
    ViInt32     count=0;
    ViUInt32    retCnt;
    ViInt32     i;
	    
    rsfsh_instrRange   instrPtr;
	
	if (rangeNumber == VI_NULL) return VI_ERROR_PARAMETER2;
    if (startFrequency == VI_NULL) return VI_ERROR_PARAMETER3;
    if (stopFrequency == VI_NULL) return VI_ERROR_PARAMETER4;
    if (resolutionBandwidth == VI_NULL) return VI_ERROR_PARAMETER5;
    if (frequency == VI_NULL) return VI_ERROR_PARAMETER6;
    if (levelAbs == VI_NULL) return VI_ERROR_PARAMETER7;
    if (levelRel == VI_NULL) return VI_ERROR_PARAMETER8;
    if (delta == VI_NULL) return VI_ERROR_INSTR_PARAMETER9;
    if (limitCheck == VI_NULL) return VI_ERROR_INSTR_PARAMETER10;
    if (reserved1 == VI_NULL) return VI_ERROR_INSTR_PARAMETER11;
    if (reserved2 == VI_NULL) return VI_ERROR_INSTR_PARAMETER12;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH4:
        CHECKERR (viPrintf (instrSession, "TRAC:DATA? LIST\n"));
	    CHECKERR (rsfsh_readDataUnknownLength (instrSession, &buffer, &retCnt));
		
	    pbuf=strtok(buffer, ",");
	    if (pbuf)
	        do{
	            if (count<noOfValues)
	            {
	                sscanf (pbuf, "%ld", &rangeNumber[count]);
	                pbuf=strtok(NULL, ","); 
	                sscanf (pbuf, "%le", &startFrequency[count]);
	                pbuf=strtok(NULL, ","); 
	                sscanf (pbuf, "%le", &stopFrequency[count]);
	                pbuf=strtok(NULL, ","); 
	                sscanf (pbuf, "%le", &resolutionBandwidth[count]);
	                pbuf=strtok(NULL, ","); 
	                sscanf (pbuf, "%le", &frequency[count]);
	                pbuf=strtok(NULL, ","); 
	                sscanf (pbuf, "%le", &levelAbs[count]);
	                pbuf=strtok(NULL, ","); 
	                sscanf (pbuf, "%le", &levelRel[count]);
	                pbuf=strtok(NULL, ","); 
	                sscanf (pbuf, "%le", &delta[count]);
	                pbuf=strtok(NULL, ","); 
	                sscanf (pbuf, "%ld", &limitCheck[count]);
	                pbuf=strtok(NULL, ",");
	                sscanf (pbuf, "%le", &reserved1[count]);
	                pbuf=strtok(NULL, ","); 
	                sscanf (pbuf, "%le", &reserved2[count]);
	                pbuf=strtok(NULL, ","); 
	            }               
	            else
	            {
	                for (i=0;i<11;i++)
	                    pbuf=strtok(NULL, ","); 
	            }       
	            count++;
	        }while (pbuf);  

	    if (returnedValues)
	        *returnedValues = count;
     
        break;
		
	default:
		break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read Trace Data                                                 */
/* Purpose:  This function reads out the complex trace data. A trace consists*/
/*           of 301 samples. If in DTF mode, 1024 values are returned.       */
/*           This function reads out the trace data in binary format.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_readComplexTraceData (ViSession instrSession,
                                              ViReal64 magnitude[],
                                              ViReal64 phase[],
                                              ViInt32 *samplesReturned)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViInt32     mode    = 255;
    ViInt32     count;
    ViPByte     pdata_arr;
    ViReal64    *p2output;
    ViInt32     *temp,
                idx = 0;
    ViUInt32    retCnt;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!magnitude)
        return VI_ERROR_PARAMETER2;
    if (!phase)
        return VI_ERROR_PARAMETER3;
    if (!samplesReturned)
        return VI_ERROR_PARAMETER4;

    /* Get number of trace samples */
    CHECKERR (rsfsh_getMeasurementMode (instrSession, &mode));
    *samplesReturned = (mode == 7) ? 1024 : 301;

    count = 2 + 2 * sizeof (ViReal32) * *samplesReturned;
    pdata_arr = (ViPByte) malloc (count);

    /* Request for trace data */
    rsfsh_status = rsfsh_writeInstrData (instrSession, "GET");
    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    rsfsh_status = rsfsh_writeInstrData (instrSession, "CTRACEBIN");
    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    /* Disable termination character. */
    CHECKERR (viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
    CHECKERR (viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

    rsfsh_status = viRead (instrSession, pdata_arr, count, &retCnt);

    viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);

    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    /* Ignore any warnings */
    rsfsh_status = VI_SUCCESS;

    /* Convert binary data into array of ViReal64 types. */
    count /= 4;
    rsfsh_SWAP_32(pdata_arr, count);
    /* Set pointer to data */
    temp = (ViInt32*)(pdata_arr);

    p2output = magnitude;

    for (idx = 0; idx < (*samplesReturned); idx++)
        *p2output++ = *temp++;

    p2output = phase;

    /* Shift pointer to data (skip one byte) */
    temp = (ViInt32*)(((ViByte*)temp) + 1);

    for (idx = 0; idx < (*samplesReturned); idx++)
        *p2output++ = *temp++;

    for (idx = 0; idx < *samplesReturned; idx++)
    {
        magnitude [idx] /= 1000.0;
        phase [idx] /= 1.0e6;
    }

    free (pdata_arr);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read Trace Data (ASCII)                                         */
/* Purpose:  This function reads out the complex trace data. A trace consists*/
/*           of 301 samples. If in DTF mode, 1024 values are returned.       */
/*           This function reads out the trace data in alphanumeric format.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_readComplexTraceDataASCII (ViSession instrSession,
                                                   ViReal64 magnitude[],
                                                   ViReal64 phase[],
                                                   ViInt32 *samplesReturned)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViInt32     mode    = 255;
    ViInt32     count;
    ViPReal64   pdata_arr;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!magnitude)
        return VI_ERROR_PARAMETER2;
    if (!phase)
        return VI_ERROR_PARAMETER3;
    if (!samplesReturned)
        return VI_ERROR_PARAMETER4;

    /* Get number of trace samples */
    CHECKERR (rsfsh_getMeasurementMode (instrSession, &mode));
    *samplesReturned = (mode == 7) ? 1024 : 301;
    pdata_arr = (ViPReal64) malloc (sizeof (ViReal64) * 2 * *samplesReturned);

    /* Request for trace data */
    rsfsh_status = rsfsh_writeInstrData (instrSession, "GET");
    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    rsfsh_status = rsfsh_writeInstrData (instrSession, "CTRACE");
    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    count = 2 * *samplesReturned;

    rsfsh_status = viScanf (instrSession, "%,#Lf", &count, pdata_arr);
    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    memcpy (magnitude, pdata_arr, sizeof (ViReal64) * *samplesReturned);
    memcpy (phase, pdata_arr + *samplesReturned, sizeof (ViReal64) * *samplesReturned);

    free (pdata_arr);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read Trace Data From Saved Data Set                             */
/* Purpose:  This function reads out the trace data from a previously saved  */
/*           data set. A trace consists of 301 samples. If the Auto Peak     */
/*           detector is used, both Max and Min values are returned (602     */
/*           samples).                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_readTraceDataFromSavedDataSet (ViSession instrSession,
                                                       ViString dataSet,
                                                       ViReal64 traceData[],
                                                       ViInt32 *samplesReturned)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[TRACE_BUFFER_SIZE]; /* Data buffer */
    ViChar      buffer2[BUFFER_SIZE];
    ViUInt16    intf = VI_INTF_GPIB;
    ViInt32     detector        = 0,
                units           = 0,
                count           = 0;
    ViReal64    dividerArr[]    = {1.0e3, 1.0e3, 1.0e3, 1.0e3, 1.0e3, 1.0e3, 1.0e6, 1.0e9,
                                   1.0e3, 1.0e6};
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!traceData)
        return VI_ERROR_PARAMETER2;
    if (!samplesReturned)
        return VI_ERROR_PARAMETER3;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf));
    /* Get number of trace samples + termination character */
    CHECKERR (rsfsh_getCommand (instrSession, "TRACEDET", buffer));
    sscanf (buffer, "%ld", &detector);
    count = (!detector) ? 602*4+2 : 301*4+1;
    /* Get current level units */
    CHECKERR (rsfsh_getCommand (instrSession, "UNIT", buffer));
    sscanf (buffer, "%ld", &units);
    /* Make sure that units are in allowed range */
    units = ((units < 0) || (units > 9)) ? 0 : units;
    /* Request for trace data */
    sprintf (buffer2, "MTRACEBIN,%s",dataSet);
    CHECKERR (rsfsh_writeInstrData (instrSession, "GET"));
    CHECKERR (rsfsh_writeInstrData (instrSession, buffer2));

    /* Interface specific operations */
    switch (intf)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            {
            ViReal64    *p2output = traceData;
            ViInt32     *temp,
                        idx = 0;
            ViUInt32    retCnt;

            /* Disable termination character. */
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

            /* Read trace data */
            rsfsh_status = viRead (instrSession, (ViPBuf) buffer, count, &retCnt);
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);

            CHECKERR (rsfsh_status);

            /* Ignore any warnings */
            rsfsh_status = VI_SUCCESS;

            /* Convert binary data into array of ViReal64 types. */
            count /= 4;
            rsfsh_SWAP_32((unsigned char*) buffer, count);
            /* Set pointer to data */
            temp = (ViInt32*)(buffer);
            if (!detector)
                {
                /* samples 1 to 301 */
                for (idx = 0; idx < 301; idx++)
                    {
                    *p2output++ = *temp++;
                    traceData[idx] /= dividerArr[units];
                    }
                /* Shift pointer to data (skip one byte) */
                temp = (ViInt32*)(buffer + 1205);
                /* samples 302 to 602 */
                for (idx = 301; idx < 601; idx++)
                    {
                    *p2output++ = *temp++;
                    traceData[idx] /= dividerArr[units];
                    }
                }
            else
                {
                /* 301 samples */
                for (idx = 0; idx < 301; idx++)
                    {
                    *p2output++ = *temp++;
                    traceData[idx] /= dividerArr[units];
                    }
                }
            if (samplesReturned)
                *samplesReturned = count;
            }
        break;
        default:
            /*
                NOT SUPPORTED
            */
        break;
        }
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Read Trace Data From Saved Data Set (ASCII)                     */
/* Purpose:  This function reads out the trace data from a previously saved  */
/*           data set. A trace consists of 301 samples. If the Auto Peak     */
/*           detector is used, both Max and Min values are returned (602     */
/*           samples).                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_readTraceDataFromSavedDataSetASCII (ViSession instrSession,
                                                            ViString dataSet,
                                                            ViReal64 traceData[],
                                                            ViInt32 *samplesReturned)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      buffer2[BUFFER_SIZE]; 
    ViInt32     detector    = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!traceData)
        return VI_ERROR_PARAMETER2;
    if (!samplesReturned)
        return VI_ERROR_PARAMETER3;

    /* Get number of trace samples */
    CHECKERR (rsfsh_getCommand (instrSession, "TRACEDET", buffer));
    sscanf (buffer, "%ld", &detector);
    *samplesReturned = (!detector) ? 602 : 301;
    /* Request for trace data */
    sprintf (buffer2, "MTRACE,%s",dataSet);
    CHECKERR (rsfsh_writeInstrData (instrSession, "GET"));
    CHECKERR (rsfsh_writeInstrData (instrSession, buffer2));
    CHECKERR (viScanf (instrSession, "%,#Lf", samplesReturned, traceData));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Read Complex Corrected Trace Data                               */
/* Purpose:  This function reads out the corrected magnitude and phase values*/ 
/*           (if applicable) in binary format. Dependent on the wrap mode the*/
/*           wrapped or unwrapped phase values are provided.                 */
/*           Phase information is only applicable when vector calibrated     */
/*           but not in group delay measurement mode.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_readComplexCorrectedTraceData (ViSession instrSession,
                                              ViReal64 magnitude[],
                                              ViReal64 phase[],
                                              ViInt32 *samplesReturned)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViInt32     mode    = 255;
    ViInt32     count;
    ViPByte     pdata_arr;
    ViReal64    *p2output;
    ViInt32     *temp,
                idx = 0;
    ViUInt32    retCnt;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!magnitude)
        return VI_ERROR_PARAMETER2;
    if (!phase)
        return VI_ERROR_PARAMETER3;
    if (!samplesReturned)
        return VI_ERROR_PARAMETER4;

    /* Get number of trace samples */
    CHECKERR (rsfsh_getMeasurementMode (instrSession, &mode));
    *samplesReturned = (mode == 7) ? 1024 : 301;

    count = 2 + 2 * sizeof (ViReal32) * *samplesReturned;
    pdata_arr = (ViPByte) malloc (count);

    /* Request for trace data */
    rsfsh_status = rsfsh_writeInstrData (instrSession, "GET");
    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    rsfsh_status = rsfsh_writeInstrData (instrSession, "CCORRTRACEBIN");
    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    /* Disable termination character. */
    CHECKERR (viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
    CHECKERR (viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

    rsfsh_status = viRead (instrSession, pdata_arr, count, &retCnt);

    viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);

    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    /* Ignore any warnings */
    rsfsh_status = VI_SUCCESS;

    /* Convert binary data into array of ViReal64 types. */
    count /= 4;
    rsfsh_SWAP_32(pdata_arr, count);
    /* Set pointer to data */
    temp = (ViInt32*)(pdata_arr);

    p2output = magnitude;

    for (idx = 0; idx < (*samplesReturned); idx++)
        *p2output++ = *temp++;

    p2output = phase;

    /* Shift pointer to data (skip one byte) */
    temp = (ViInt32*)(((ViByte*)temp) + 1);

    for (idx = 0; idx < (*samplesReturned); idx++)
        *p2output++ = *temp++;

    for (idx = 0; idx < *samplesReturned; idx++)
    {
        magnitude [idx] /= 1000.0;
        phase [idx] /= 1.0e6;
    }

    free (pdata_arr);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read Complex Corrected Trace Data (ASCII)                       */
/* Purpose: This function reads out the corrected magnitude and phase values */
/*          (if applicable) in alphanumeric format. The magnitude unit is    */
/*          dB, the phase unit is radiants. Dependent on the wrap mode the   */
/*          wrapped or unwrapped phase values are provided. Phase informaton */
/*          is only applicable when vector calibrated but not in group delay */
/*          measurement mode.                                                */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_readComplexCorrectedTraceDataASCII (ViSession instrSession,
                                                   ViReal64 magnitude[],
                                                   ViReal64 phase[],
                                                   ViInt32 *samplesReturned)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViInt32     mode    = 255;
    ViInt32     count;
    ViPReal64   pdata_arr;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!magnitude)
        return VI_ERROR_PARAMETER2;
    if (!phase)
        return VI_ERROR_PARAMETER3;
    if (!samplesReturned)
        return VI_ERROR_PARAMETER4;

    /* Get number of trace samples */
    CHECKERR (rsfsh_getMeasurementMode (instrSession, &mode));
    *samplesReturned = (mode == 7) ? 1024 : 301;
    pdata_arr = (ViPReal64) malloc (sizeof (ViReal64) * 2 * *samplesReturned);

    /* Request for trace data */
    rsfsh_status = rsfsh_writeInstrData (instrSession, "GET");
    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    rsfsh_status = rsfsh_writeInstrData (instrSession, "CCORRTRACE");
    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    count = 2 * *samplesReturned;

    rsfsh_status = viScanf (instrSession, "%,#Lf", &count, pdata_arr);
    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }

    memcpy (magnitude, pdata_arr, sizeof (ViReal64) * *samplesReturned);
    
    rsfsh_status = viScanf (instrSession, "%,#Lf", &count, pdata_arr);
    if (rsfsh_status < VI_SUCCESS)
    {
        free (pdata_arr);
        return rsfsh_status;
    }
    
    memcpy (phase, pdata_arr, sizeof (ViReal64) * *samplesReturned);

    free (pdata_arr);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}
    
    

/*===========================================================================*/
/* Function: Read Receiver Trace Data                                        */
/* Purpose:  This function reads out the trace data of the current           */
/*           measurement.                                                    */
/*                                                                           */
/*           This function is available in Receiver Mode only (R&S FSH-K3).  */
/*                                                                           */
/*           This function is only available for R&S FSH4/8 instrument.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_ReadReceiverTraceData (ViSession instrSession,
                                               ViUInt32 maximumTime,
                                               ViInt32 arrayLength,
                                               ViReal64 amplitude[],
                                               ViInt32 *actualPoints)
{
    ViStatus    rsfsh_status   = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     old_timeout = -1;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &old_timeout));
    CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, maximumTime));
            
    
    sprintf (buffer, "*CLS;FORM REAL,32;TRAC:DATA?\n");
    
    CHECKERR (viPrintf (instrSession, buffer));
    CHECKERR (rsfsh_dataReadTraceOnly (instrSession, arrayLength, amplitude, actualPoints));
    
    CHECKERR( rsfsh_checkStatus (instrSession));
    
    if (old_timeout >= 0)
        CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, (ViAttrState) old_timeout));
    
    return rsfsh_status;
    
}

/*===========================================================================*/
/* Function: Set Trace Math Mode                                             */
/* Purpose:  This function defines the trace math mode.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTraceMathMode (ViSession instrSession,
                                          ViInt32 mathMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (mathMode, 0, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf(buffer, "MATHMODE,%ld",mathMode);
    CHECKERR (rsfsh_setCommand(instrSession,buffer));
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Trace Math Mode                                             */
/* Purpose:  This function returns the trace math mode.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTraceMathMode (ViSession instrSession,
                                          ViInt32 *mathMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!mathMode)
        return VI_ERROR_PARAMETER2;
        
    CHECKERR (rsfsh_getCommand (instrSession, "MATHMODE", buffer));
    sscanf(buffer,"%ld",mathMode);
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Trace Math State
/* Purpose:  This function sets the mathematical relation of traces on or 
/*           off.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTraceMathState(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean   state
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    
    sprintf (buffer, "CALC:MATH%ld:STAT", trace);
    
    return rsfsh_setViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
                               
}

/*===========================================================================*/
/* Function: Get Trace Math State
/* Purpose:  This function queries the mathematical relation of traces on or 
/*           off.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTraceMathState(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean*  state
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MATH%ld:STAT?", trace);
    
    return rsfsh_getViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
                               
}

/*===========================================================================*/
/* Function: Set Trace Math Expression
/* Purpose:  This function sets the mathematical expression for relating 
/*           traces to trace1. The trace mathematics are activated with 
/*           CALCulate:MATH<1...2>:STATe.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTraceMathExpression(
    ViSession   instrSession,
    ViInt32 trace,
    ViString    expression
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MATH%ld", trace);
    
    return rsfsh_setViString (instrSession,
                              buffer,
                              expression,
                              VI_ERROR_PARAMETER3);
                               
}

/*===========================================================================*/
/* Function: Get Trace Math Expression
/* Purpose:  This function queries the mathematical expression for relating 
/*           traces to trace1. The trace mathematics are activated with 
/*           CALCulate:MATH<1...2>:STATe.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTraceMathExpression(
    ViSession   instrSession,
    ViInt32 trace,
    ViChar _VI_FAR  expression[]
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MATH%ld:EXPR:DEF?", trace);
    
    return rsfsh_getViString (instrSession,
                              buffer,
                              expression,
                              1024,
                              VI_ERROR_PARAMETER3);
                               
}

/*===========================================================================*/
/* Function: Copy Trace To Memory
/* Purpose:  This function stores the current trace into the memory trace of 
/*           the R&S FSH.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_copyTraceToMemory(
    ViSession   instrSession,
    ViInt32 trace
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MATH%ld:COPY:MEM", trace);
    
    return rsfsh_sendCmd (instrSession, buffer);
                               
}

/*===========================================================================*/
/* Function: Configure Input Impedance                                       */
/* Purpose:  This function sets the nominal input impedance of the           */
/*           instrument. The set impedance is taken into account in all level*/
/*           indications of results.                                         */
/*           The setting 75 should be selected, if the 50 input impedance is */
/*           transformed to a higher impedance using a 75 adapter of the RAZ */
/*           type (= 25 in series to the input impedance of the instrument). */
/*           The correction value in this case is 1.76 dB = 10 log ( 75 /    */
/*           50). 

             This function is not compatible with FSH3 instrument.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureInputImpedance (ViSession instrSession,
                                                 ViInt32 inputImpedance)
{
    return rsfsh_setViInt32 (instrSession,
                             "INP:IMP",
                             inputImpedance,
                             50,75,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Configure Attenuation                                           */
/* Purpose:  This function configures the input attenuation. 
             This function is not compatible with FSH3 instrument.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureAttenuation (ViSession instrSession,
                                              ViBoolean attenuationAuto,
                                              ViReal64 attenuation,
                                              ViInt32 attenuationMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    CHECKERR (rsfsh_setAttenuationAuto (instrSession, attenuationAuto));
    
    if (attenuationAuto != VI_TRUE){

        CHECKERR (rsfsh_setAttenuation (instrSession, attenuation));
        CHECKERR (rsfsh_setAttenuationMode (instrSession, attenuationMode));
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Input Impedance
/* Purpose:  This function sets the nominal input impedance of the 
/*           instrument. The set impedance is taken into account in all level 
/*           indications of results.
/*           The setting 75 should be selected, if the 50 input impedance is 
/*           transformed to a higher impedance using a 75 adapter of the RAZ 
/*           type (= 25 in series to the input impedance of the instrument). 
/*           The correction value in this case is 1.76 dB = 10 log ( 75 / 
/*           50).
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setInputImpedance(
    ViSession   instrSession,
    ViInt32 inputImpedance
)
{
    return rsfsh_setViInt32 (instrSession,
                             "INP:IMP",
                             inputImpedance,
                             50,75,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Input Impedance                                             */
/* Purpose:  This function queries the nominal input impedance of the        */
/*           instrument.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getInputImpedance (ViSession instrSession,
                                           ViInt32 *inputImpedance)
{
    return rsfsh_getViInt32 (instrSession,
                             "INP:IMP?",
                             inputImpedance,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Input Impedance Pad
/* Purpose:  This function sets the matching pad connected to the R&S FSH.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setInputImpedancePad(
    ViSession   instrSession,
    ViInt32 pad
)
{
    return rsfsh_setEnum (instrSession,
                          "INP:IMP:PAD",
                          pad,
                          padArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Input Impedance Pad
/* Purpose:  This function queries the matching pad connected to the R&S 
/*           FSH.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getInputImpedancePad(
    ViSession   instrSession,
    ViInt32*    pad
)
{
    return rsfsh_getEnum (instrSession,
                          "INP:IMP:PAD?",
                          pad,
                          padArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Attenuation Mode
/* Purpose:  This function sets the mode for controlling the RF attenuator.
/*           For further information on controlling the RF attenuator refer 
/*           to the Quick Start Guide, chapter 'Controlling the RF 
/*           Attenuator' or the Operating Manual, chapter "Setting the 
/*           Amplitude
/*           Parameters".
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAttenuationMode(
    ViSession   instrSession,
    ViInt32 attenuationMode
)
{
    return rsfsh_setEnum (instrSession,
                          "INP:ATT:MODE",
                          attenuationMode,
                          attenuationModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Attenuation Mode
/* Purpose:  This function queries the mode for controlling the RF 
/*           attenuator.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAttenuationMode(
    ViSession   instrSession,
    ViInt32*    attenuationMode
)
{
    return rsfsh_getEnum (instrSession,
                          "INP:ATT:MODE?",
                          attenuationMode,
                          attenuationModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Attenuation Auto                                            */
/* Purpose:  This function automatically couples the input attenuation to the*/
/*           reference level (state ON) or switches the input attenuation to */
/*           manual entry (state OFF).                                       */
/*           In receiver mode, this command automatically sets the           */
/*           attenuation so that a good S/N ratio is obtained without the    */
/*           receiver stages being overdriven (state ON). 

             This function is not compatible with FSH3 instrument.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAttenuationAuto (ViSession instrSession,
                                            ViBoolean attenuationAuto)
{
    return rsfsh_setViBoolean (instrSession,
                               "INP:ATT:AUTO",
                               attenuationAuto,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Attenuation Auto                                            */
/* Purpose:  This function queries the input attenuation auto.               */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAttenuationAuto (ViSession instrSession,
                                            ViBoolean *attenuationAuto)
{
    return rsfsh_getViBoolean (instrSession,
                               "INP:ATT:AUTO?",
                               attenuationAuto,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Attenuation                                                 */
/* Purpose:  This function programs the input attenuator. To protect the     */
/*           input mixer against damage from overloads, the setting 0 dB can */
/*           be obtained by entering numerals, not by using the DEC command. */
/*           The attenuation can be set in 5 dB steps. If the defined        */
/*           reference level cannot be set for the set RF attenuation, the   */
/*           reference level will be adjusted accordingly.                   */
/*           In the default state with Spectrum Analyzer mode, the           */
/*           attenuation set on the step attenuator is coupled to the        */
/*           reference level of the instrument. If the attenuation is        */
/*           programmed directly, the coupling to the reference level is     */
/*           switched off.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAttenuation (ViSession instrSession,
                                        ViReal64 attenuation)
{
    return rsfsh_setViReal64 (instrSession,
                              "INP:ATT",
                              attenuation,
                              0.0,0.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Attenuation                                                 */
/* Purpose:  This function queries the input attenuation.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAttenuation (ViSession instrSession,
                                        ViReal64 *attenuation)
{
    return rsfsh_getViReal64 (instrSession,
                              "INP:ATT?",
                              attenuation,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Configure Marker                                                */
/* Purpose:  This function configures marker (deltamarker) state and         */
/*           position.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureMarker (ViSession instrSession,
                                         ViInt32 marker,
                                         ViBoolean markerState,
                                         ViReal64 markerPosition,
                                         ViInt32 deltaMarker,
                                         ViBoolean deltamarkerState,
                                         ViReal64 deltamarkerPosition)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViBooleanRange (markerState))
        return VI_ERROR_PARAMETER3;
    /* Parameter 3 is not range checked */
    if (rsfsh_invalidViBooleanRange (deltamarkerState))
        return VI_ERROR_PARAMETER6;
    /* Parameter 5 is not range checked */

    CHECKERR (rsfsh_setMarkerState (instrSession, marker, markerState));
    if (markerState == VI_TRUE)
    {
    CHECKERR (rsfsh_setMarkerPosition (instrSession, marker, markerPosition));
    CHECKERR (rsfsh_setDeltamarkerState (instrSession, deltaMarker, deltamarkerState));
    if (deltamarkerState == VI_TRUE)
        {
        CHECKERR (rsfsh_setDeltamarkerPosition (instrSession, deltaMarker, deltamarkerPosition));
        }
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Marker Search Limit
/* Purpose:  This function configures marker search limits.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureMarkerSearchLimit(
    ViSession   instrSession,
    ViInt32     marker,
    ViBoolean   state,
    ViReal64    limitLeft,
    ViReal64    limitRight
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_setMarkerSearchLimitState (instrSession, marker, state));
    CHECKERR (rsfsh_setMarkerSearchLimitLeft (instrSession, limitLeft));
    CHECKERR (rsfsh_setMarkerSearchLimitRight (instrSession, limitRight));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Marker Zoom
/* Purpose:  This function configures the marker zoom.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureMarkerZoom(
    ViSession   instrSession,
    ViBoolean   zoom,
    ViInt32 zoomFactor,
    ViReal64    x,
    ViReal64    y
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    CHECKERR (rsfsh_setMarkerZoomState (instrSession, zoom));
    CHECKERR (rsfsh_setMarkerZoomFactor (instrSession, zoomFactor));
    CHECKERR (rsfsh_setMarkerZoomX (instrSession, x));
    CHECKERR (rsfsh_setMarkerZoomY (instrSession, y));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Marker Demodulation
/* Purpose:  This function configures the audio demodulator.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureMarkerDemodulation(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean   state,
    ViReal64    holdTime,
    ViInt32 type
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_setMarkerDemodulationState (instrSession, marker, state));
    CHECKERR (rsfsh_setMarkerDemodulationHoldTime (instrSession, holdTime));
    CHECKERR (rsfsh_setMarkerDemodulationType (instrSession, type));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Active Marker                                               */
/* Purpose:  This function selects one or all (multi) markers.               */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setActiveMarker (ViSession instrSession,
                                         ViInt32 marker)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    if (rsfsh_invalidViInt32Range (marker, 0, 6))
        return VI_ERROR_PARAMETER2;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> multiMarker = marker;
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Active Marker                                               */
/* Purpose:  This function returns the selected marker.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getActiveMarker (ViSession instrSession,
                                         ViPInt32 marker)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    *marker = instrPtr -> multiMarker;
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Marker  State                                               */
/* Purpose:  This function turns the one or all (multi) markers on / off.    */
/*           MARKON,1 is used for the marker in normal and in multi marker   */
/*           mode. MARKON,2 to MARKON,6 are available in multi marker mode   */
/*           only.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerState (ViSession instrSession,
                                        ViInt32 marker,
                                        ViBoolean markerState)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     multimarker  = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    marker = instrPtr -> multiMarker;
    
    if (rsfsh_invalidViBooleanRange (markerState))
        return VI_ERROR_PARAMETER3;
    
    CHECKERR (rsfsh_getCommand (instrSession, "MARKMODE", buffer));
    sscanf (buffer, "%ld", &multimarker);
    
    if (multimarker == 3)
    {
        if (marker == 0)
        {
            sprintf (buffer, "MARKALLON,%ld", markerState);
            CHECKERR (rsfsh_setCommand (instrSession, buffer));
        }
        else
        {
            sprintf (buffer, "MARKON,%ld,%ld", marker, markerState);
            CHECKERR (rsfsh_setCommand (instrSession, buffer));
        }
    }
    else
    {
        sprintf (buffer, "MARK1ON,%ld", markerState);
        CHECKERR (rsfsh_setCommand (instrSession, buffer));
    }
    break;
    case FSH4:
        
        if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
        
        sprintf (buffer, "CALC:MARK%ld:STAT", marker);
        
        rsfsh_status = rsfsh_setViBoolean (instrSession,
                                           buffer,
                                           markerState,
                                           VI_ERROR_PARAMETER3);
    break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Marker State                                                */
/* Purpose:  This function returns the one or all (multi) markers on / off.  */
/*           MARKON,1 is used for the marker in normal and in multi marker   */
/*           mode. MARKON,2 to MARKON,6 are available in multi marker mode   */
/*           only.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerState (ViSession instrSession,
                                        ViInt32    marker,
                                        ViBoolean *markerState)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      buffer2[BUFFER_SIZE];
    ViInt32     multimarker  = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    marker = instrPtr -> multiMarker; 

    if (!markerState)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "MARKMODE", buffer));
    sscanf (buffer, "%ld", &multimarker);
    
    if (multimarker == 3)
    {
        if (marker == 0)
        {
            CHECKERR (rsfsh_getCommand (instrSession, "MARKALLON", buffer));
            sscanf (buffer, "%hd", markerState);
        }
        else
        {
            sprintf (buffer2, "MARKON,%ld", marker);
            CHECKERR (rsfsh_getCommand (instrSession, buffer2, buffer));
            sscanf (buffer, "%hd", markerState);
        }
    }
    else
    {
        CHECKERR (rsfsh_getCommand (instrSession, "MARK1ON", buffer));
        sscanf (buffer, "%hd", markerState);
    }
    break;
    case FSH4:
        
        if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
        
        sprintf (buffer, "CALC:MARK%ld:STAT?", marker);
        
        rsfsh_status = rsfsh_getViBoolean (instrSession,
                                           buffer,
                                           markerState,
                                           VI_ERROR_PARAMETER3);
    break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Marker Position                                             */
/* Purpose:  This function sets the corresponding (multi) marker to the      */
/*           specified position. The marker unit depends on the unit of the  */
/*           x-axis which can be Hz, seconds or meter/feet depending of the  */
/*           measurement mode.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerPosition (ViSession instrSession,
                                           ViInt32   marker,
                                           ViReal64  markerPosition)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     multimarker  = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    marker = instrPtr -> multiMarker;

    CHECKERR (rsfsh_getCommand (instrSession, "MARKMODE", buffer));
    sscanf (buffer, "%ld", &multimarker);
    
    if (marker == 1)
    {
        sprintf (buffer, "MARK1,%Lf", markerPosition);
        CHECKERR (rsfsh_setCommand (instrSession, buffer));
    }
    else if (multimarker == 3)
    {
        sprintf (buffer, "MARK,%ld,%Lf", marker, markerPosition);
        CHECKERR (rsfsh_setCommand (instrSession, buffer));
    }
    break;
    case FSH4:
        sprintf (buffer, "CALC:MARK%ld:X", marker);
        rsfsh_status = rsfsh_setViReal64 (instrSession,
                                          buffer,
                                          markerPosition,
                                          0.0, 0.0,
                                          VI_ERROR_PARAMETER3);
        break;
    }
        
    return rsfsh_status;
    
}
    
/*===========================================================================*/
/* Function: Get Marker Position
/* Purpose:  This function queries the selected marker to the indicated 
/*           frequency (span > 0) or time (span = 0).
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerPosition(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64*   x
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:X?", marker);
    
    return rsfsh_getViReal64 (instrSession,
                              buffer,
                              x,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Marker Value                                                */
/* Purpose:  This function queries the current marker value. The marker unit */
/*           depends on the unit of the x-axis which can be Hz, seconds or   */
/*           meter/feet depending of the measurement mode. The unit of the   */
/*           level value depends on the current unit of the y-axis.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerValue (ViSession instrSession,
                                        ViPReal64 markerPosition,
                                        ViPReal64 markerLevel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      buffer2[BUFFER_SIZE];
    ViInt32     multimarker  = 0;
    ViInt32     marker;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    marker = instrPtr -> multiMarker;
    
    if (!markerPosition)
        return VI_ERROR_PARAMETER3;
    if (!markerLevel)
        return VI_ERROR_PARAMETER4;

    CHECKERR (rsfsh_getCommand (instrSession, "MARKMODE", buffer));
    sscanf (buffer, "%ld", &multimarker);
    
    if (marker == 1)
    {
        CHECKERR (rsfsh_getCommand (instrSession, "MARK1", buffer));
        sscanf (buffer, "%Le,%Le", markerPosition, markerLevel);
    }
    else if (multimarker == 3)
    {
        sprintf (buffer2, "MARK,%ld", marker);
        CHECKERR (rsfsh_getCommand (instrSession, buffer2, buffer));
        sscanf (buffer, "%Le,%Le", markerPosition, markerLevel);
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Disable All Markers
/* Purpose:  This function switches off all active markers and all delta 
/*           markers and active marker or delta marker measurement functions.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_disableAllMarkers(
    ViSession   instrSession
)
{
    return rsfsh_sendCmd (instrSession, "CALC:MARK:AOFF");
}

/*===========================================================================*/
/* Function: Set Active Delta Marker                                         */
/* Purpose:  This function selects one or all deltamarker.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setActiveDeltaMarker (ViSession instrSession,
                                              ViInt32 deltamarker)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    if (rsfsh_invalidViInt32Range (deltamarker, 0, 6))
        return VI_ERROR_PARAMETER2;
        
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> deltaMarker = deltamarker;

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Active Delta Marker                                         */
/* Purpose:  This function returns the state of one or all deltamarkers.     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getActiveDeltaMarker (ViSession instrSession,
                                              ViPInt32 deltamarker)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    *deltamarker = instrPtr -> deltaMarker;
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Delta Marker State                                          */
/* Purpose:  This function turns the deltamarker on/off.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDeltamarkerState (ViSession instrSession,
                                             ViInt32 deltamarker,
                                             ViBoolean deltamarkerState)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     multimarker  = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    
    deltamarker = instrPtr -> deltaMarker;
    if (rsfsh_invalidViBooleanRange (deltamarkerState))
        return VI_ERROR_PARAMETER2;
    
    CHECKERR (rsfsh_getCommand (instrSession, "MARKMODE", buffer));
    sscanf (buffer, "%ld", &multimarker);
    
    if (deltamarker == 1)
    {
        sprintf (buffer, "DELTA1ON,%ld", deltamarkerState);
        CHECKERR (rsfsh_setCommand (instrSession, buffer));
    }
    else if (multimarker == 3)
    {
        if (deltamarker == 0)
        {
            sprintf (buffer, "DELTAALLON,%ld", deltamarkerState);
            CHECKERR (rsfsh_setCommand (instrSession, buffer));
        }
        else
        {
            sprintf (buffer, "DELTAON,%ld,%ld", deltamarker, deltamarkerState);
            CHECKERR (rsfsh_setCommand (instrSession, buffer));
        }
    }
    break;
    case FSH4:
    if (rsfsh_invalidViInt32Range (deltamarker, 1, 6))
        return VI_ERROR_PARAMETER2;
        
    sprintf (buffer, "CALC:DELT%ld:STAT", deltamarker); 
    rsfsh_status = rsfsh_setViBoolean (instrSession,
                                       buffer,
                                       deltamarkerState,
                                       VI_ERROR_PARAMETER2);
    break;
    }
    
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Delta Marker State                                          */
/* Purpose:  This function returns the deltamarker state.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDeltamarkerState (ViSession instrSession,
                                             ViInt32 deltamarker,
                                             ViBoolean *deltamarkerState)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      buffer2[BUFFER_SIZE];
    ViInt32     multimarker  = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    deltamarker = instrPtr -> deltaMarker;
        
    if (!deltamarkerState)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "MARKMODE", buffer));
    sscanf (buffer, "%ld", &multimarker);
    
    if (deltamarker == 1)
    {
        CHECKERR (rsfsh_getCommand (instrSession, "DELTA1ON", buffer));
        sscanf (buffer, "%hd", deltamarkerState);
    }
    else if (multimarker == 3)
    {
        if (deltamarker == 0)
        {
            CHECKERR (rsfsh_getCommand (instrSession, "DELTAALLON", buffer));
            sscanf (buffer, "%hd", deltamarkerState);
        }
        else
        {
            sprintf (buffer2, "DELTAON,%ld", deltamarker);
            CHECKERR (rsfsh_getCommand (instrSession, buffer2, buffer));
            sscanf (buffer, "%hd", deltamarkerState);
        }
    }
    break;
    case FSH4:
    if (rsfsh_invalidViInt32Range (deltamarker, 1, 6))
        return VI_ERROR_PARAMETER2; 
    
    sprintf (buffer, "CALC:DELT%ld:STAT?", deltamarker);    
    rsfsh_status = rsfsh_getViBoolean (instrSession,
                                       buffer,
                                       deltamarkerState,
                                       VI_ERROR_PARAMETER2);
    break;
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Deltamarker Position                                        */
/* Purpose:  This function sets the deltamarker to the specified position in */
/*           relation to the marker. The deltamarker unit depends on the unit*/
/*           of the x-axis which can be Hz, seconds or meter/feet depending  */
/*           of the measurement mode.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDeltamarkerPosition (ViSession instrSession,
                                                ViInt32 deltamarker,
                                                ViReal64 deltamarkerPosition)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     multimarker  = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    deltamarker = instrPtr -> deltaMarker;
        
    CHECKERR (rsfsh_getCommand (instrSession, "MARKMODE", buffer));
    sscanf (buffer, "%ld", &multimarker);
    
    if (deltamarker == 1)
    {
        sprintf (buffer, "DELTA1,%Lf", deltamarkerPosition);
        CHECKERR (rsfsh_setCommand (instrSession, buffer));
    }
    else if (multimarker == 3)
    {
        sprintf (buffer, "DELTA,%ld,%Lf", deltamarker, deltamarkerPosition);
        CHECKERR (rsfsh_setCommand (instrSession, buffer));
    }
    break;
    case FSH4:
    if (rsfsh_invalidViInt32Range (deltamarker, 1, 6))
        return VI_ERROR_PARAMETER2; 
    
    sprintf (buffer, "CALC:DELT%ld:X", deltamarker);
    rsfsh_status = rsfsh_setViReal64 (instrSession,
                                      buffer,
                                      deltamarkerPosition,
                                      0.0, 0.0,
                                      VI_ERROR_PARAMETER2);
    break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Deltamarker Position                                        */
/* Purpose:  This function returns the absolute position of the deltamarker. */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDeltamarkerPosition (ViSession instrSession,
                                                ViInt32 deltamarker,
                                                ViReal64 *deltamarkerPosition)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    break;
    case FSH4:
    if (rsfsh_invalidViInt32Range (deltamarker, 1, 6))
        return VI_ERROR_PARAMETER2; 
    
    sprintf (buffer, "CALC:DELT%ld:X?", deltamarker);
    rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      buffer,
                                      deltamarkerPosition,
                                      VI_ERROR_PARAMETER3);
    break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Deltamarker Frequency or Time                               */
/* Purpose:  This function sets a delta marker to a position in relation to
             the reference marker..                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDeltamarkerFrequencyorTime (ViSession instrSession,
                                                       ViInt32 deltamarker,
                                                       ViReal64 frequencyorTime)
{
   ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (deltamarker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:DELT%ld:X:REL", deltamarker);
    
    return rsfsh_setViReal64 (instrSession,
                              buffer,
                              frequencyorTime,
                              0.0,0.0,  
                              VI_ERROR_PARAMETER3); 
}

/*===========================================================================*/
/* Function: Get Deltamarker Frequency or Time
/* Purpose:  This function queries the frequency (span > 0) or time (span = 
/*           0) of the selected delta marker relative to marker 1 or to the 
/*           reference position. The command activates the corresponding
/*           delta marker, if necessary.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDeltamarkerFrequencyorTime(
    ViSession   instrSession,
    ViInt32 deltamarker,
    ViReal64*   frequencyorTime
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (deltamarker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:DELT%ld:X:REL?", deltamarker);
    
    return rsfsh_getViReal64 (instrSession,
                              buffer,
                              frequencyorTime,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Deltamarker Value                                           */
/* Purpose:  This function queries the current deltamarker value. The        */
/*           deltamarker unit depends on the unit of the x-axis which can be */
/*           Hz, seconds or meter/feet depending of the measurement mode. The*/
/*           unit of the level value depends on the current unit of the      */
/*           y-axis.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDeltamarkerValue (ViSession instrSession,
                                             ViInt32   deltamarker,
                                             ViPReal64 deltamarkerPosition,
                                             ViPReal64 deltamarkerLevel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      buffer2[BUFFER_SIZE];
    ViInt32     multimarker  = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    deltamarker = instrPtr -> deltaMarker;
    
    if (!deltamarkerPosition)
        return VI_ERROR_PARAMETER2;
    if (!deltamarkerLevel)
        return VI_ERROR_PARAMETER3;

    CHECKERR (rsfsh_getCommand (instrSession, "MARKMODE", buffer));
    sscanf (buffer, "%ld", &multimarker);
    
    if (deltamarker == 1)
    {
        CHECKERR (rsfsh_getCommand (instrSession, "DELTA1", buffer));
        sscanf (buffer, "%Le,%Le", deltamarkerPosition, deltamarkerLevel);
    }
    else if (multimarker == 3)
    {
        sprintf (buffer2, "DELTA,%ld", deltamarker);
        CHECKERR (rsfsh_getCommand (instrSession, buffer2, buffer));
        sscanf (buffer, "%Le,%Le", deltamarkerPosition, deltamarkerLevel);
    }
    break;
    case FSH4:
    if (rsfsh_invalidViInt32Range (deltamarker, 1, 6))
        return VI_ERROR_PARAMETER2;
        
    sprintf (buffer, "CALC:DELT%ld:X:REL?", deltamarker);   
    rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      buffer,
                                      deltamarkerPosition,
                                      VI_ERROR_PARAMETER2);
    
    sprintf (buffer, "CALC:DELT%ld:Y?", deltamarker);   
    rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      buffer,
                                      deltamarkerLevel,
                                      VI_ERROR_PARAMETER3);
    break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Disable All Delta Markers                                       */
/* Purpose:  This function turns off all the delta markers.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_disableAllDeltaMarkers (ViSession instrSession)
{
    return rsfsh_sendCmd (instrSession, "CALC:DELT1:AOFF");
}

/*===========================================================================*/
/* Function: Delta Marker Search                                             */
/* Purpose:  This function specifies the type of delta marker search and     */ 
/*           performs the search.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_deltaMarkerSearch (ViSession instrSession,
                                           ViInt32 deltaMarker,
                                           ViInt32 markerSearch)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (deltaMarker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    switch (markerSearch)
    {
    case RSFSH_VAL_MARKER_SEARCH_HIGHEST:
        sprintf (buffer, "CALC:DELT%ld:MAX:PEAK", deltaMarker);
    break;
    case RSFSH_VAL_MARKER_SEARCH_NEXT_PEAK:
        sprintf (buffer, "CALC:DELT%ld:MAX:NEXT", deltaMarker);
    break;
    case RSFSH_VAL_MARKER_SEARCH_MINIMUM:
        sprintf (buffer, "CALC:DELT%ld:MIN:PEAK", deltaMarker);
    break;
    default:
        return VI_ERROR_PARAMETER3;
    }
    
    return rsfsh_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Marker Search
/* Purpose:  This function specifies the type of marker search and performs 
/*           the search.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_markerSearch(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32 markerSearch
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    switch (markerSearch)
    {
    case RSFSH_VAL_MARKER_SEARCH_HIGHEST:
        sprintf (buffer, "CALC:MARK%ld:MAX:PEAK", marker);
    break;
    case RSFSH_VAL_MARKER_SEARCH_NEXT_PEAK:
        sprintf (buffer, "CALC:MARK%ld:MAX:NEXT", marker);
    break;
    case RSFSH_VAL_MARKER_SEARCH_MINIMUM:
        sprintf (buffer, "CALC:MARK%ld:MIN:PEAK", marker);
    break;
    case RSFSH_VAL_MARKER_SEARCH_NEXT_MINIMUM:
        sprintf (buffer, "CALC:MARK%ld:MIN:NEXT", marker);
    break;
    default:
        return VI_ERROR_PARAMETER3;
    }
    
    return rsfsh_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Set Marker To                                                   */
/* Purpose:  This function sets marker to the peak (highest signal), next    */
/*           peak, as center frequency, as reference level or to the minumum */
/*           (lowest signal).                                                */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerTo (ViSession instrSession,
                                     ViInt32 markerTo)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViString    markerToArr[] = {"MARKPK","MARKNXTPK","MARKTOCENT","MARKTOLVL","MARKMIN"};
    ViInt32     multimarker  = 0;
    ViInt32     marker;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    marker = instrPtr -> multiMarker;
    
    if (rsfsh_invalidViInt32Range (markerTo, 0, 4))
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "MARKMODE", buffer));
    sscanf (buffer, "%ld", &multimarker);
    
    if (marker == 1)
    {
        sprintf (buffer, "%s", markerToArr[markerTo]);
        CHECKERR (rsfsh_cmdCommand (instrSession, buffer));
    }
    else if (multimarker == 3)
    {
        sprintf (buffer, "%s,%ld", markerToArr[markerTo], marker);
        CHECKERR (rsfsh_cmdCommand (instrSession, buffer));
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Marker Mode                                                 */
/* Purpose:  This function defines the marker mode.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerMode (ViSession instrSession,
                                       ViInt32 markerMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (markerMode, 0, 4))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "MARKMODE,%ld", markerMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Marker Mode                                                 */
/* Purpose:  This function returns the marker mode.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerMode (ViSession instrSession,
                                       ViPInt32 markerMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!markerMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "MARKMODE", buffer));
    sscanf (buffer, "%ld", markerMode);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Marker Demodulation Mode                                    */
/* Purpose:  This function defines the marker demodulation mode.             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerDemodulationMode (ViSession instrSession,
                                                   ViInt32 demodulation)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (demodulation, 0, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "MARKDEMOD,%ld", demodulation);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Marker Demodulation Mode                                    */
/* Purpose:  This function returns the marker demodulation mode.             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerDemodulationMode (ViSession instrSession,
                                                   ViInt32 *demodulation)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!demodulation)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "MARKDEMOD", buffer));
    sscanf (buffer, "%ld", demodulation);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Marker Demodulation Time                                    */
/* Purpose:  This function defines the demodulation time at the current      */
/*           marker position.                                                */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerDemodulationTime (ViSession instrSession,
                                                   ViReal64 demodulationTime)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (demodulationTime, 0.1, 500))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "MARKTIME,%Lf", demodulationTime);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Marker Demodulation AF Output Volume                        */
/* Purpose:  This function sets the volume of the demodulation AF output.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerDemodulationAFOutputVolume (ViSession instrSession,
                                                             ViInt32 demodulationVolume)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (demodulationVolume, 1, 100))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "MARKVOL,%ld", demodulationVolume);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Marker Impedance Reference                                  */
/* Purpose:  This function sets the impedance reference in Ohm.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerImpedanceReference (ViSession instrSession,
                                              ViInt32 markerImpedanceRef)

{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* parameter 2 not range checked */

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    sprintf (buffer, "MARKIMPREF,%ld", markerImpedanceRef);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Marker Impedance Reference                                  */
/* Purpose:  This function gets the impedance reference in Ohm.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerImpedanceReference (ViSession instrSession,
                                             ViInt32 *markerImpedanceRef)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!markerImpedanceRef)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "MARKIMPREF", buffer));
    sscanf (buffer, "%ld", markerImpedanceRef);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}   

/*===========================================================================*/
/* Function: Set Marker Measurement Mode                                     */
/* Purpose:  This function defines the marker marker format.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerMeasurementMode (ViSession instrSession,
                                                   ViInt32 markerMeasMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (markerMeasMode, 0, 6))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "MARKMEASY,%ld", markerMeasMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Marker Measurement Mode                                     */
/* Purpose:  This function returns the marker demodulation mode.             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerMeasurementMode (ViSession instrSession,
                                                   ViInt32 *markerMeasMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!markerMeasMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "MARKMEASY", buffer));
    sscanf (buffer, "%ld", markerMeasMode);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Marker Frequency Counter
/* Purpose:  This function sets on or off the frequency counter at the 
/*           marker position.
/*           The count result is queried with  
/*           CALCulate:MARKer<1...6>:COUNt:FREQuency?.
/*           Frequency counting is possible only for one marker at a time. 
/*           If it is activated for another marker, it is automatically 
/*           deactivated for the previous marker.
/*           It should be noted that a complete sweep must be performed 
/*           after switching on the frequency counter to ensure that the 
/*           frequency to be measured is actually reached. The 
/*           synchronization to
/*           the sweep end required for this is only possible in single 
/*           sweep mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerFrequencyCounter(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean   frequencyCounter
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:COUN:STAT", marker);
    
    return rsfsh_setViBoolean (instrSession,
                               buffer,
                               frequencyCounter,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Marker Frequency Counter
/* Purpose:  This function queries the state of the frequency counter at the 
/*           marker position.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerFrequencyCounter(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean*  frequencyCounter
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:COUN:STAT?", marker);
    
    return rsfsh_getViBoolean (instrSession,
                               buffer,
                               frequencyCounter,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Marker Frequency Counter Result
/* Purpose:  This function queries the result of the frequency counter for 
/*           the indicated marker. Before the command, the frequency counter 
/*           should be switched on and a complete measurement performed to 
/*           obtain a correct count result. Therefore, a single sweep with 
/*           synchronization must be performed between switching on the 
/*           frequency counter and querying the count result.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerFrequencyCounterResult(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64*   result
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:COUN:FREQ?", marker);
    
    return rsfsh_getViReal64 (instrSession,
                              buffer,
                              result,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Marker Display Mode
/* Purpose:  This function sets the marker frequency display mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerDisplayMode(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32 displayMode
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FREQ:MODE", marker);
    
    return rsfsh_setEnum (instrSession,
                          buffer,
                          displayMode,
                          markerDisplayModeArr,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Marker Display Mode
/* Purpose:  This function queries the marker frequency display mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerDisplayMode(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32*    displayMode
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FREQ:MODE?", marker);
    
    return rsfsh_getEnum (instrSession,
                          buffer,
                          displayMode,
                          markerDisplayModeArr,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Marker Center Frequency
/* Purpose:  This function matches the center frequency to the frequency of a
             marker
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerCenterFrequency (ViSession instrSession,
                                                  ViInt32 marker)
{
   ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:CENT", marker);
    
    return rsfsh_sendCmd (instrSession, buffer);  
}

/*===========================================================================*/
/* Function: Set Marker Reference Frequency
/* Purpose:  This function matches the reference level to the power level of
             a marker
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerReferenceFrequency (ViSession instrSession,
                                                     ViInt32 marker)
{
   ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:REF", marker);
    
    return rsfsh_sendCmd (instrSession, buffer);  
}

/*===========================================================================*/
/* Function: Set Marker Output Format
/* Purpose:  This function sets the output format for the (complex) value of 
/*           the marker.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerOutputFormat(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32 outputFormat
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:MODE", marker);
    
    return rsfsh_setEnum (instrSession,
                          buffer,
                          outputFormat,
                          markerOutputFormatArr,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Marker Output Format
/* Purpose:  This function queries the output format for the (complex) value 
/*           of the marker.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerOutputFormat(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32*    outputFormat
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:MODE?", marker);
    
    return rsfsh_getEnum (instrSession,
                          buffer,
                          outputFormat,
                          markerOutputFormatArr,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Marker TDMA Burst Length
/* Purpose:  This function sets the burst length of the TDMA signal.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerTDMABurstLength(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64    burstLength
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:TDMA:BURS", marker);
    
    return rsfsh_setViReal64 (instrSession,
                              buffer,
                              burstLength,
                              31.7e-3, 19.0,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Marker TDMA Burst Length
/* Purpose:  This function queries the burst length of the TDMA signal.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerTDMABurstLength(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64*   burstLength
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:TDMA:BURS?", marker);
    
    return rsfsh_getViReal64 (instrSession,
                              buffer,
                              burstLength,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Marker Search Limit State
/* Purpose:  This function activates and deactivates marker search limits.
/*           If the power measurement in zero span is active, this command 
/*           limits the evaluation range on the trace.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerSearchLimitState(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean   state
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:X:SLIM:STAT", marker);
    
    return rsfsh_setViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Marker Search Limit State
/* Purpose:  This function queries the state of the marker search limits.
/*           If the power measurement in zero span is active, this command 
/*           limits the evaluation range on the trace.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerSearchLimitState(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean*  state
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:X:SLIM:STAT?", marker);
    
    return rsfsh_getViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Marker Search Limit Left
/* Purpose:  This function sets the left limit of the search range for 
/*           markers and delta markers. Depending on the span setting of the 
/*           xaxis the indicated value defines a frequency (span > 0) or 
/*           time (span = 0).
/*           The numeric suffix <1...6> is irrelevant for this command.
/*           If the power measurement in zero span is active, this command 
/*           limits the evaluation range to the trace.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerSearchLimitLeft(
    ViSession   instrSession,
    ViReal64    limitLeft
)
{
    return rsfsh_setViReal64 (instrSession,
                              "CALC:MARK:X:SLIM:LEFT",
                              limitLeft,
                              0.0,0.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Marker Search Limit Left
/* Purpose:  This function queries the left limit of the search range for 
/*           markers and delta markers.
/*           The numeric suffix <1...6> is irrelevant for this command.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerSearchLimitLeft(
    ViSession   instrSession,
    ViReal64*   leftLimit
)
{
    return rsfsh_getViReal64 (instrSession,
                              "CALC:MARK:X:SLIM:LEFT?",
                              leftLimit,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Marker Search Limit Right
/* Purpose:  This function sets the right limit of the search range for 
/*           markers and delta markers. Depending on the span setting of the 
/*           xaxis the indicated value defines a frequency (span > 0) or 
/*           time (span = 0).
/*           The numeric suffix <1...6> is irrelevant for this command.
/*           If the power measurement in zero span is active, this command 
/*           limits the evaluation range to the trace.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerSearchLimitRight(
    ViSession   instrSession,
    ViReal64    rightLimit
)
{
    return rsfsh_setViReal64 (instrSession,
                              "CALC:MARK:X:SLIM:RIGH",
                              rightLimit,
                              0.0,0.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Marker Search Limit Right
/* Purpose:  This function queries the right limit of the search range for 
/*           markers and delta markers.
/*           The numeric suffix <1...6> is irrelevant for this command.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerSearchLimitRight(
    ViSession   instrSession,
    ViReal64*   rightLimit
)
{
    return rsfsh_getViReal64 (instrSession,
                              "CALC:MARK:X:SLIM:RIGH?",
                              rightLimit,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Marker Y
/* Purpose:  This function queries the measured value of the selected 
/*           marker. The corresponding marker is activated before or switched 
/*           to marker mode, if necessary.
/*           To obtain a correct query result, a complete sweep with 
/*           synchronization to the sweep end must be performed between the 
/*           activation of the marker and the query of the Y value. This is 
/*           only possible in single sweep mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerY(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64*   y
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:Y?", marker);
    
    return rsfsh_getViReal64 (instrSession,
                              buffer,
                              y,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Marker Reference Level to Marker
/* Purpose:  This function sets the reference level to that of the level of 
/*           the marker indicated by the suffix <1...6>.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerReferenceLeveltoMarker(
    ViSession   instrSession,
    ViInt32 marker
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:LEV:ONCE", marker);
    
    return rsfsh_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get Marker List                                                 */
/* Purpose:  This function queries the current list of multimarkers. Each    */
/*           multimarker returns three numbers: multimarker number, x-axis   */
/*           value, y-axis value. This GET command is available in multi     */
/*           marker mode only.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerList (ViSession instrSession,
                                       ViInt32 *markerCount,
                                       ViInt32 markerNumberArray[], 
                                       ViReal64 _VI_FAR markerPosition[],
                                       ViReal64 _VI_FAR markerValue[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      separators[]   = " ,\t\n";
    ViChar*     next;
    ViInt32     count = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!markerCount)
        return VI_ERROR_PARAMETER2;
    if (!markerNumberArray)
        return VI_ERROR_PARAMETER3;
    if (!markerPosition)
        return VI_ERROR_PARAMETER4;
    if (!markerValue)
        return VI_ERROR_PARAMETER5;        

    CHECKERR (rsfsh_getCommand (instrSession, "MARKALL?", buffer));
    
    next = strtok (buffer, separators);
    do
    {
        markerNumberArray[count] = atoi (next);
        markerPosition[count] = atof (strtok (NULL, separators));
        markerValue[count] = atof (strtok (NULL, separators));
        count++;
    } while ((next = strtok (NULL, separators)) != NULL);

    *markerCount = count;
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Deltamarker List                                            */
/* Purpose:  This function queries the current list of multi deltamarkers    */
/*           values. Each delta multimarker returns three numbers: delta     */
/*           multimarker number, x-axis value, y-axis value. This GET        */
/*           command is available in multi marker mode only.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDeltaMarkerList (ViSession instrSession,
                                            ViInt32 *deltamarkerCount,
                                            ViInt32 deltaMarkerNumberArray[],
                                            ViReal64 _VI_FAR deltamarkerPosition[],
                                            ViReal64 _VI_FAR deltamarkerValue[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      separators[]   = " ,\t\n";
    ViChar*     next;
    ViInt32     count = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!deltamarkerCount)
        return VI_ERROR_PARAMETER2;
    if (!deltamarkerPosition)
        return VI_ERROR_PARAMETER3;
    if (!deltamarkerValue)
        return VI_ERROR_PARAMETER4;        

    CHECKERR (rsfsh_getCommand (instrSession, "DELTAALL?", buffer));
    
    next = strtok (buffer, separators);
    do
    {
        deltaMarkerNumberArray[count] = atoi (next);
        deltamarkerPosition[count] = atof (strtok (NULL, separators));
        deltamarkerValue[count] = atof (strtok (NULL, separators));
        count++;
    } while ((next = strtok (NULL, separators)) != NULL);
        
    *deltamarkerCount = count;
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Marker Zoom State
/* Purpose:  This function sets the state of the marker zoom function in a 
/*           Smith chart.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerZoomState(
    ViSession   instrSession,
    ViBoolean   zoom
)
{
    return rsfsh_setViBoolean (instrSession,
                               "DISP:ZOOM:AREA:STAT",
                               zoom,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Marker Zoom State
/* Purpose:  This function queries the state of the marker zoom function in 
/*           a Smith chart.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerZoomState(
    ViSession   instrSession,
    ViBoolean*  zoom
)
{
    return rsfsh_getViBoolean (instrSession,
                               "DISP:ZOOM:AREA:STAT?",
                               zoom,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Marker Zoom Factor
/* Purpose:  This function sets the zoom factor of the marker zoom function 
/*           in a Smith chart.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerZoomFactor(
    ViSession   instrSession,
    ViInt32 zoomFactor
)
{
    return rsfsh_setEnum (instrSession,
                          "DISP:ZOOM:FACT",
                          zoomFactor,
                          zoomFactorArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Marker Zoom Factor
/* Purpose:  This function queries the zoom factor of the marker zoom 
/*           function in a Smith chart.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerZoomFactor(
    ViSession   instrSession,
    ViInt32*    zoomFactor
)
{
    return rsfsh_getEnum (instrSession,
                          "DISP:ZOOM:FACT?",
                          zoomFactor,
                          zoomFactorArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Marker Zoom X
/* Purpose:  This function sets the zoom window on the x-axis of the Smith 
/*           chart. '0%' marks the center of the x-axis in the Smith chart.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerZoomX(
    ViSession   instrSession,
    ViReal64    x
)
{
    return rsfsh_setViReal64 (instrSession,
                              "DISP:ZOOM:X",
                              x,
                              -50.0,50.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Marker Zoom X
/* Purpose:  This function queries the zoom window on the x-axis of the 
/*           Smith chart. '0%' marks the center of the x-axis in the Smith 
/*           chart.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerZoomX(
    ViSession   instrSession,
    ViReal64*   x
)
{
    return rsfsh_getViReal64 (instrSession,
                              "DISP:ZOOM:X?",
                              x,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Marker Zoom Y
/* Purpose:  This function sets the zoom window on the y-axis of the Smith 
/*           chart. '0%' marks the center of the y-axis in the Smith chart.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerZoomY(
    ViSession   instrSession,
    ViReal64    y
)
{
    return rsfsh_setViReal64 (instrSession,
                              "DISP:ZOOM:Y",
                              y,
                              -50.0,50.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Marker Zoom Y
/* Purpose:  This function queries the zoom window on the y-axis of the 
/*           Smith chart. '0%' marks the center of the y-axis in the Smith 
/*           chart.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerZoomY(
    ViSession   instrSession,
    ViReal64*   y
)
{
    return rsfsh_getViReal64 (instrSession,
                              "DISP:ZOOM:Y?",
                              y,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Channel Power Mode
/* Purpose:  This function sets the power display mode to Clear Write or Max 
/*           Hold for channel power measurements.
/*           The numeric suffix <1...6> is irrelevant.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setChannelPowerMode(
    ViSession   instrSession,
    ViInt32 mode
)
{
    return rsfsh_setEnum (instrSession,
                          "CALC:MARK:FUNC:CPOW:MODE",
                          mode,
                          channelPowerModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Channel Power Mode
/* Purpose:  This function queries the power display mode to Clear Write or 
/*           Max Hold for channel power measurements.
/*           The numeric suffix <1...6> is irrelevant.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getChannelPowerMode(
    ViSession   instrSession,
    ViInt32*    mode
)
{
    return rsfsh_getEnum (instrSession,
                          "CALC:MARK:FUNC:CPOW:MODE?",
                          mode,
                          channelPowerModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Marker Demodulation State
/* Purpose:  This function sets the audio demodulator when the indicated 
/*           marker is reached. With span > 0 the hold time can be defined at 
/*           the corresponding marker position with
/*           CALCulate:MARKer<1...6>:FUNCtion:DEModulation:HOLDoff. In zero 
/*           span the demodulation is permanently active.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerDemodulationState(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean   state
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:DEM:STAT", marker);
    
    return rsfsh_setViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Marker Demodulation State
/* Purpose:  This function queries the audio demodulator when the indicated 
/*           marker is reached.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerDemodulationState(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean*  state
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:DEM:STAT?", marker);
    
    return rsfsh_getViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Marker Demodulation Hold Time
/* Purpose:  This function sets the hold time at the marker position for the 
/*           demodulation with span > 0.
/*           The setting is independent of the selected marker, the suffixes 
/*           and <1...6> are irrelevant.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerDemodulationHoldTime(
    ViSession   instrSession,
    ViReal64    holdTime
)
{
    return rsfsh_setViReal64 (instrSession,
                              "CALC:MARK:FUNC:DEM:HOLD",
                              holdTime,
                              10.0e-3,500.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Marker Demodulation Hold Time
/* Purpose:  This function queries the hold time at the marker position for 
/*           the demodulation with span > 0.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerDemodulationHoldTime(
    ViSession   instrSession,
    ViReal64*   holdTime
)
{
    return rsfsh_getViReal64 (instrSession,
                              "CALC:MARK:FUNC:DEM:HOLD?",
                              holdTime,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Marker Demodulation Type
/* Purpose:  This function sets the demodulation type for the audio 
/*           demodulator.
/*           The command is independent of the selected marker, the suffixes 
/*           1 to 6 are irrelevant.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerDemodulationType(
    ViSession   instrSession,
    ViInt32 type
)
{
    return rsfsh_setEnum (instrSession,
                          "CALC:MARK:FUNC:DEM:SEL",
                          type,
                          demodulationTypeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Marker Demodulation Type
/* Purpose:  This function queries the demodulation type for the audio 
/*           demodulator.
/*           The command is independent of the selected marker, the suffixes 
/*           1 to 6 are irrelevant.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerDemodulationType(
    ViSession   instrSession,
    ViInt32*    type
)
{
    return rsfsh_getEnum (instrSession,
                          "CALC:MARK:FUNC:DEM:SEL?",
                          type,
                          demodulationTypeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Marker Noise State
/* Purpose:  This function sets the noise measurement on or off for all 
/*           markers. The noise power density is measured at the position of 
/*           the markers. The result can be queried with
/*           CALCulate:MARKer<1...6>:FUNCtion:NOISe:RESult?.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerNoiseState(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean   state
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:NOIS:STAT", marker);
    
    return rsfsh_setViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Marker Noise State
/* Purpose:  This function queries the noise measurement state for all 
/*           markers.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerNoiseState(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean*  state
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:NOIS:STAT?", marker);
    
    return rsfsh_getViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Marker Noise Result
/* Purpose:  This function queries the result of the noise measurement.
/*           A complete sweep with synchronization to the sweep end must be 
/*           performed between switching on the function and querying the 
/*           measured value in order to obtain a correct query result. This
/*           is only possible in single sweep mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerNoiseResult(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64*   result
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:NOIS:RES?", marker);
    
    return rsfsh_getViReal64 (instrSession,
                              buffer,
                              result,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Occupied Bandwidth Channel Bandwidth
/* Purpose:  This function sets the channel bandwidth for occupied bandwidth 
/*           measurements.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setOccupiedBandwidthChannelBandwidth(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64    bandwidth
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:OBAN:BAND", marker);
    
    return rsfsh_setViReal64 (instrSession,
                              buffer,
                              bandwidth,
                              0.0,0.0,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Occupied Bandwidth Channel Bandwidth
/* Purpose:  This function queries the channel bandwidth for occupied 
/*           bandwidth measurements.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getOccupiedBandwidthChannelBandwidth(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64*   bandwidth
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:OBAN:BAND?", marker);
    
    return rsfsh_getViReal64 (instrSession,
                              buffer,
                              bandwidth,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Occupied Bandwidth Channel Bandwidth Percentage
/* Purpose:  This function sets the percentage of the total power that 
/*           defines the occupied bandwidth.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setOccupiedBandwidthChannelBandwidthPercentage(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32 percentage
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:OBAN:BAND:PCT", marker);
    
    return rsfsh_setViInt32 (instrSession,
                             buffer,
                             percentage,
                             10,99,
                             VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Occupied Bandwidth Channel Bandwidth Percentage
/* Purpose:  This function queries the percentage of the total power that 
/*           defines the occupied bandwidth.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getOccupiedBandwidthChannelBandwidthPercentage(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32*    percentage
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:FUNC:OBAN:BAND:PCT?", marker);
    
    return rsfsh_getViInt32 (instrSession,
                             buffer,
                             percentage,
                             VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set n dB Down Marker State
/* Purpose:  This function activates and deactivates the n dB Down marker
/*           function. The n dB down function sets two temporary markers n dB
/*           below the level of the reference marker (marker 1), one on the
/*           right and the other marker to the left of the reference marker.
/*           
/*           The function is independent of the selected marker, the suffixes
/*           1 to 6 are irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setndBDownMarkerState (ViSession instrSession,
                                               ViInt32 marker, ViBoolean down)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:MARKer%ld:FUNCtion:NDBDown:STATe", marker);
    
    return rsfsh_setViBoolean (instrSession,
                             buffer,
                             down,
                             VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get n dB Down Marker State
/* Purpose:  This function returns state of the n dB Down marker
/*           function. The n dB down function sets two temporary markers n dB
/*           below the level of the reference marker (marker 1), one on the
/*           right and the other marker to the left of the reference marker.
/*           
/*           The function is independent of the selected marker, the suffixes
/*           1 to 6 are irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getndBDownMarkerState (ViSession instrSession,
                                               ViInt32 marker,ViBoolean *down)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:MARKer%ld:FUNCtion:NDBDown:STATe?", marker);
    
    return rsfsh_getViBoolean (instrSession,
                             buffer,
                             down,
                             VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set n dB Down Marker Distance
/* Purpose:  This function defines the distance in dB of the two temporary n
/*           dB down markers to the reference marker.
/*           
/*           The function is independent of the selected marker, the suffixes
/*           1 to 6 are irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setndBDownMarkerDistance (ViSession instrSession,
                                                  ViInt32 marker, ViReal64 distance)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:MARKer%ld:FUNCtion:NDBDown", marker);
    
    return rsfsh_setViReal64 (instrSession,
                             buffer,
                             distance,
                             -100,100,
                             VI_ERROR_PARAMETER3);    
}

/*===========================================================================*/
/* Function: Get n dB Down Marker Distance
/* Purpose:  This function returns the distance in dB of the two temporary n
/*           dB down markers to the reference marker.
/*           
/*           The function is independent of the selected marker, the suffixes
/*           1 to 6 are irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getndBDownMarkerDistance (ViSession instrSession,
                                                  ViInt32 marker, ViReal64 *distance)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:MARKer%ld:FUNCtion:NDBDown?", marker);
    
    return rsfsh_getViReal64 (instrSession,
                             buffer,
                             distance,
                             VI_ERROR_PARAMETER3);    
}

/*===========================================================================*/
/* Function: Get n dB Down Marker Frequency
/* Purpose:  This function returns the frequencies of the two temporary n dB
/*           down markers.
/*           
/*           The command is independent of the selected marker, the suffixes
/*           1 to 6 are irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getndBDownMarkerFrequency (ViSession instrSession,
                                                   ViInt32 marker,
                                                   ViInt32 *frequency1,
                                                   ViInt32 *frequency2)
{

    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      buffer2[BUFFER_SIZE];

    if (!frequency1)
        return VI_ERROR_PARAMETER3;
    if (!frequency2)
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALCulate:MARKer%ld:FUNCtion:NDBDown:FREQuency?", marker);

    //CHECKERR(rsfsh_writeInstrData (instrSession, buffer));
    CHECKERR(rsfsh_getViString(instrSession,buffer,buffer2,BUFFER_SIZE,VI_ERROR_PARAMETER2));
    //CHECKERR(viScanf (instrSession, "%[^\r\n]", buffer2));

    sscanf (buffer2, "%ld,%ld",frequency1,frequency2 );
    
    return rsfsh_status;
   
}

/*===========================================================================*/
/* Function: Get n dB Down Marker Bandwidth
/* Purpose:  This function returns the frequency spacing or bandwidth of the
/*           two temporary n dB down markers.
/*           
/*           The command is independent of the selected marker, the suffixes
/*           1 to 6 are irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getndBDownMarkerBandwidth (ViSession instrSession,
                                                   ViInt32 marker,
                                                   ViReal64 *bandwidth)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:MARKer%ld:FUNCtion:NDBDown:RESult?", marker);
    
    return rsfsh_getViReal64 (instrSession,
                             buffer,
                             bandwidth,
                             VI_ERROR_PARAMETER3); 
}

/*===========================================================================*/
/* Function: Initiate Measurement                                            */
/* Purpose:  This function initiates an acquisition. After calling this      */
/*           function, the spectrum analyzer leaves the idle state and waits */
/*           for a trigger.                                                  */
/*                                                                           */
/*           This function does not check the instrument status. Typically,  */
/*           the end-user calls this function only in a sequence of calls to */
/*           other low-level driver functions. The sequence performs one     */
/*           operation. The end-user uses the low-level functions to optimize*/
/*           one or more aspects of interaction with the instrument.         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_initiateMeasurement (ViSession instrSession,
                                             ViUInt32 timeout)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKERR (rsfsh_waitOPC (instrSession, "INIT:IMM;*OPC?", buffer, timeout));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Abort Measurement                                               */
/* Purpose:  This function aborts a current measurement and resets the       */
/*           trigger system.                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_abortMeasurement (ViSession instrSession)
{
    return rsfsh_sendCmd (instrSession, "ABOR");
}

/*===========================================================================*/
/* Function: Configure Transducer                                            */
/* Purpose:  This function configures transducer (transducer dB) table.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureTransducer (ViSession instrSession,
                                             ViString transducerName,
                                             ViString transducerdBName)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViString (transducerName, 64))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViString (transducerdBName, 64))
        return VI_ERROR_PARAMETER3;

    CHECKERR (rsfsh_setTransducer (instrSession, transducerName));
    CHECKERR (rsfsh_setTransducerdB (instrSession, transducerdBName));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Transducer                                            */
/* Purpose:  This function configures transducer (transducer dB) table.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureDirectionTransducer (ViSession instrSession,
                                                      ViString transducerXName,
                                                      ViString transducerYName,
                                                      ViString transducerZName)

{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViString (transducerXName, 64))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViString (transducerYName, 64))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViString (transducerZName, 64))
        return VI_ERROR_PARAMETER4;

    CHECKERR (rsfsh_setXTransducer (instrSession, transducerXName));
    CHECKERR (rsfsh_setYTransducer (instrSession, transducerYName));
    CHECKERR (rsfsh_setZTransducer (instrSession, transducerZName));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Limit Lines                                           */
/* Purpose:  This function configures lower and upper limit lines.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureLimitLines (ViSession instrSession,
                                             ViInt32  limit,
                                             ViString lowerLimitLine,
                                             ViString upperLimitLine,
                                             ViBoolean state,
                                             ViBoolean audioBeep)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViString (lowerLimitLine, 64))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViString (upperLimitLine, 64))
        return VI_ERROR_PARAMETER4;
    if (rsfsh_invalidViBooleanRange (state))
        return VI_ERROR_PARAMETER5;
    if (rsfsh_invalidViBooleanRange (audioBeep))
        return VI_ERROR_PARAMETER6;

    CHECKERR (rsfsh_setLowerLimitLine (instrSession, lowerLimitLine));
    CHECKERR (rsfsh_setUpperLimitLine (instrSession, upperLimitLine));
    CHECKERR (rsfsh_setLimitLineState (instrSession, limit, state));
    CHECKERR (rsfsh_setLimitLineAudioBeep (instrSession, audioBeep));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure n dB Down Value                                       */
/* Purpose:  This function defines the n dB down value. The range is -100.0*/
/*           dB to 100.0 dB with a resolution of 0.1 dB. This command will   */
/*           automatically set the marker mode to n db down.                */
/*                                                                           */
/*           This function is not compatible with FSH4 & FSH8 instrument.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configurendBDownValue (ViSession instrSession,
                                               ViReal64  ndBDown)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (ndBDown, -100.0, 100.0))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "NDBDOWNVAL,%Lf", ndBDown);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Query n dB Down Result                                          */
/* Purpose:  This function returns the measured n dB down bandwidth value. */
/*                                                                           */
/*           This function is not compatible with FSH4 & FSH8 instrument.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_queryndBDownResult (ViSession instrSession,
                                            ViReal64  *ndBDown)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    
    CHECKERR (rsfsh_getCommand (instrSession, "NDBDOWN", buffer));
    sscanf (buffer, "%Lf", ndBDown);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Meas Mode                                                   */
/* Purpose:  This function defines the measurement mode. Using this function */
/*           the instrument can turned OFF if the power adapter is connected.*/
/*           If the instrument is OFF it can be turned ON programmatically by*/
/*           selecting one of the measurement modes.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMeasurementMode (ViSession instrSession,
                                            ViInt32 measurementMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     mode = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (measurementMode, 0, 12))
        return VI_ERROR_PARAMETER2;

    /* Check what is the current measurement mode */
    CHECKERR (rsfsh_getCommand (instrSession, "MEAS", buffer));
    sscanf (buffer, "%ld", &mode);
    /* Set desired measurement mode */
    if (measurementMode != mode)
        {
        sprintf (buffer, "MEAS,%ld", measurementMode);
        CHECKERR (rsfsh_setCommand (instrSession, buffer));
        /*
            if (measurementMode > 0)
                CHECKERR (rsfsh_cmdCommand (instrSession, "REMOTE"));
        */
        }
    break;
    case FSH4:
    rsfsh_status = rsfsh_setEnum (instrSession,
                                  "INST:SEL",
                                  measurementMode,
                                  measurementModeArr,
                                  VI_ERROR_PARAMETER2);
    break;
    }
    
    CHECKERR (rsfsh_delay (instrSession, 10.0));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Meas Mode                                                   */
/* Purpose:  This function returns the measurement mode.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMeasurementMode (ViSession instrSession,
                                            ViPInt32 measurementMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!measurementMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "MEAS", buffer));
    sscanf (buffer, "%ld", measurementMode);
    break;
    case FSH4:
    rsfsh_status = rsfsh_getEnum (instrSession,
                                  "INST:SEL?",
                                  measurementMode,
                                  measurementModeArr,
                                  VI_ERROR_PARAMETER2);
    break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Transducer                                                  */
/* Purpose:  This function selects a transducer table.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTransducer (ViSession instrSession,
                                       ViString transducerName)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (transducerName, 64))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRD1,%s", transducerName);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Transducer                                                  */
/* Purpose:  This function returns currently used transducer table.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTransducer (ViSession instrSession,
                                       ViChar _VI_FAR transducerName[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!transducerName)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRD1", buffer));
    sscanf (buffer, "%64[^\r]", transducerName);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}


/*===========================================================================*/
/* Function: Set Transducer X                                                */
/* Purpose:  This function selects an X-direction transducer (dBuV/m only)   */
/*           for the Isotropic Antenna measurement.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setXTransducer (ViSession instrSession,
                                       ViString transducerName)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (transducerName, 64))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRD1X,%s", transducerName);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Transducer X                                                */
/* Purpose:  This function returns an X-direction transducer (dBuV/m only)   */
/*           for the Isotropic Antenna measurement.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getXTransducer (ViSession instrSession,
                                       ViChar _VI_FAR transducerName[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!transducerName)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRD1X", buffer));
    sscanf (buffer, "%64[^\r]", transducerName);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}
    
/*===========================================================================*/
/* Function: Set Transducer Y                                                */
/* Purpose:  This function selects an Y-direction transducer (dBuV/m only)   */
/*           for the Isotropic Antenna measurement.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setYTransducer (ViSession instrSession,
                                       ViString transducerName)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (transducerName, 64))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRD1Y,%s", transducerName);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Transducer Y                                                */
/* Purpose:  This function returns an Y-direction transducer (dBuV/m only)   */
/*           for the Isotropic Antenna measurement.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getYTransducer (ViSession instrSession,
                                       ViChar _VI_FAR transducerName[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!transducerName)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRD1Y", buffer));
    sscanf (buffer, "%64[^\r]", transducerName);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Transducer Z                                                */
/* Purpose:  This function selects an Z-direction transducer (dBuV/m only)   */
/*           for the Isotropic Antenna measurement.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setZTransducer (ViSession instrSession,
                                       ViString transducerName)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (transducerName, 64))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRD1Z,%s", transducerName);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Transducer Z                                                */
/* Purpose:  This function returns an Z-direction transducer (dBuV/m only)   */
/*           for the Isotropic Antenna measurement.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getZTransducer (ViSession instrSession,
                                       ViChar _VI_FAR transducerName[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!transducerName)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRD1Z", buffer));
    sscanf (buffer, "%64[^\r]", transducerName);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Transducer dB                                               */
/* Purpose:  This function selects a transducer (dB only) table.             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTransducerdB (ViSession instrSession,
                                         ViString transducerName)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (transducerName, 64))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRD2,%s", transducerName);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Transducer dB                                               */
/* Purpose:  This function returns currently used transducer (dB only) table.*/
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTransducerdB (ViSession instrSession,
                                         ViChar _VI_FAR transducerName[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!transducerName)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRD2", buffer));
    sscanf (buffer, "%64[^\r]", transducerName);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Lower Limit Line                                            */
/* Purpose:  This function selects the lower limit line. To turn a limit line*/
/*           off, use the string "NONE".                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setLowerLimitLine (ViSession instrSession,
                                           ViString lowerLimitLine)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (lowerLimitLine, 64))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "LIMLOW,%s", lowerLimitLine);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Lower Limit Line                                            */
/* Purpose:  This function returns the lower limit line.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLowerLimitLine (ViSession instrSession,
                                           ViChar _VI_FAR lowerLimitLine[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!lowerLimitLine)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "LIMLOW", buffer));
    sscanf (buffer, "%64[^\r]", lowerLimitLine);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Upper Limit Line                                            */
/* Purpose:  This function selects the upper limit line. To turn a limit line*/
/*           off, use the string "NONE".                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setUpperLimitLine (ViSession instrSession,
                                           ViString upperLimitLine)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (upperLimitLine, 64))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "LIMUPP,%s", upperLimitLine);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Upper Limit Line                                            */
/* Purpose:  This function returns the upper limit line.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getUpperLimitLine (ViSession instrSession,
                                           ViChar _VI_FAR upperLimitLine[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!upperLimitLine)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "LIMUPP", buffer));
    sscanf (buffer, "%64[^\r]", upperLimitLine);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Limit Check Status                                          */
/* Purpose:  This function returns the limit check status.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLimitCheckStatus (ViSession instrSession,
                                             ViPInt32 checkStatus)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!checkStatus)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "LIMPASS", buffer));
    sscanf (buffer, "%ld", checkStatus);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Lower Threshold Line                                        */
/* Purpose:  This function defines the lower threshold line (value). The     */
/*           unit of the threshold value is specified with the Unit          */
/*           parameter. This command is available in Receiver / Scan Mode    */
/*           only (R&S FSH-K3).                                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setLowerThresholdLine (ViSession instrSession,
                                               ViReal64 lowerThresholdLine)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* parameter 2 not range checked */

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    sprintf (buffer, "THRLOW,%Lf", lowerThresholdLine);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Lower Threshold Line                                        */
/* Purpose:  This function queries the setting of the lower threshold line   */
/*           (value). The unit of the threshold value is specified with the  */
/*           Unit parameter. This command is available in Receiver / Scan    */
/*           Mode only (R&S FSH-K3).                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLowerThresholdLine (ViSession instrSession,
                                               ViPReal64 lowerThresholdLine)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!lowerThresholdLine)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "THRLOW", buffer));
    sscanf (buffer, "%Le", lowerThresholdLine);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Upper Threshold Line                                        */
/* Purpose:  This function defines the upper threshold line (value). The     */
/*           unit of the threshold value is specified with the Unit          */
/*           parameter. This command is available in Receiver / Scan Mode    */
/*           only (R&S FSH-K3).                                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setUpperThresholdLine (ViSession instrSession,
                                               ViReal64 upperThresholdLine)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* parameter 2 not range checked */

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    sprintf (buffer, "THRUPP,%Lf", upperThresholdLine);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Upper Threshold Line                                        */
/* Purpose:  This function queries the setting of the upper threshold line   */
/*           (value). The unit of the threshold value is specified with the  */
/*           Unit parameter. This command is available in Receiver / Scan    */
/*           Mode only (R&S FSH-K3).                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getUpperThresholdLine (ViSession instrSession,
                                               ViPReal64 upperThresholdLine)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!upperThresholdLine)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "THRUPP", buffer));
    sscanf (buffer, "%Le", upperThresholdLine);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Threshold Line Check Status                                 */
/* Purpose:  This function gets the threshold line check status. This        */
/*           command is available in Receiver / Scan Mode only (R&S FSH-K3). */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getThresholdLineCheckStatus (ViSession instrSession,
                                                     ViPInt32 checkStatus)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

/*
    Current Condition:
                        When trying to get the return value the instrument
                        returns EXECUTION ERROR, no matter if the threshold line
                        status should return PASS or FAIL or UNKNOWN. This was
                        tested using "Write To Instrument" function - with
                        commands "GET , THRPASS".

    Future Work:
                        Unknown.
*/

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!checkStatus)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "THRPASS", buffer));
    sscanf (buffer, "%ld", checkStatus);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Threshold Off                                               */
/* Purpose:  This function sets the threshold off. This command is available */
/*           in receiver mode only (R&S FSH-K3).                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setThresholdOff (ViSession instrSession)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    CHECKERR (rsfsh_cmdCommand (instrSession, "THROFF"));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;

}

/*===========================================================================*/
/* Function: Get n dB Down Value                                             */
/* Purpose:  This function queries the n dB down value.                     */
/*                                                                           */
/*           This function is not compatible with FSH4 & FSH8 instrument.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getndBDownValue (ViSession instrSession,
                                         ViReal64  *ndBDown)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    
    CHECKERR (rsfsh_getCommand (instrSession, "NDBDOWNVAL", buffer));
    sscanf (buffer, "%Lf", ndBDown);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Number Of Measurement Points                                */
/* Purpose:  This function queries the number of measurement points during   */
/*           one sweep..                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getNumberOfMeasurementPoints (ViSession instrSession,
                                                      ViInt32 *numberOfMeasurementPoints)
{
   
    return rsfsh_getViInt32 (instrSession, 
                             "SWE:POIN?",
                             numberOfMeasurementPoints,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Define Limit Line                                               */
/* Purpose:  This function defines limit line.                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_defineLimitLine (ViSession instrSession,
                                         ViString name, ViString description,
                                         ViInt32 xUnit, ViInt32 xScale,
                                         ViInt32 yUnit, ViInt32 arraySize,
                                         ViReal64 xValues[], ViReal64 yValues[])
{
    ViStatus rsfsh_status = VI_SUCCESS;
    ViChar      buffer[3*BUFFER_SIZE];
    ViChar      *p2buf;
    ViInt32     i;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    switch (instrPtr -> fsh3)
    {
    case FSH3:
	    if (rsfsh_invalidViString (name, 64))
	        return VI_ERROR_PARAMETER2;
	    if (rsfsh_invalidViString (description, 64))
	        return VI_ERROR_PARAMETER3;
	    if (rsfsh_invalidViInt32Range (xUnit, 0, 2))
	        return VI_ERROR_PARAMETER4;
	    if (rsfsh_invalidViInt32Range (xScale, 0, 1))
	        return VI_ERROR_PARAMETER5;
	    if (rsfsh_invalidViInt32Range (yUnit, 0, 13))
	        return VI_ERROR_PARAMETER6;

	    p2buf = buffer + sprintf (buffer, "LIMDEF,%s,%s,%ld,%ld,%ld,", name,
	                                                                   description,
	                                                                   xUnit,
	                                                                   xScale,
	                                                                   yUnit);
    
	    for (i = 0; i < arraySize; i++)
	        p2buf += sprintf (p2buf, "%le,%le,", xValues[i], yValues[i]);
    
	    *--p2buf = '\0';
    
	    rsfsh_status = rsfsh_setCommand (instrSession, buffer);
	    break;
	
    case FSH4:
		if (rsfsh_invalidViString (name, 64))
	        return VI_ERROR_PARAMETER2;
	    if (rsfsh_invalidViString (description, 64))
	        return VI_ERROR_PARAMETER3;
	    if (rsfsh_invalidViInt32Range (xUnit, 0, 2))
	        return VI_ERROR_PARAMETER4;
	    if (rsfsh_invalidViInt32Range (xScale, 0, 1))
	        return VI_ERROR_PARAMETER5;
	    if (rsfsh_invalidViInt32Range (yUnit, 0, 9))
	        return VI_ERROR_PARAMETER6;

	    p2buf = buffer + sprintf (buffer, "CALC:LIM:DEF '%s','%s',%s,%s,%s,", name,
	                                                                   description,
	                                                                   xUnitArr[xUnit],
	                                                                   absRelArr[xScale],
	                                                                   yUnitArr[yUnit]);
    
	    for (i = 0; i < arraySize; i++)
	        p2buf += sprintf (p2buf, "%le,%le,", xValues[i], yValues[i]);
    
	    *--p2buf = '\0';
    
	    CHECKERR (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) strlen(buffer), VI_NULL));
	
        break;
    }
    return rsfsh_status;
    
}

/*===========================================================================*/
/* Function: Delete Limit Line                                               */
/* Purpose:  This function deletes limit line.                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_deleteLimitLine (ViSession instrSession,
                                         ViInt32 limit,
                                         ViString name)
{
    ViStatus rsfsh_status = VI_SUCCESS;
    ViChar  buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (name, 64))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "LIMDEL,%s", name);
    rsfsh_status = rsfsh_setCommand (instrSession, buffer);
    break;
    case FSH4:
        if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
        
        sprintf (buffer, "CALC:LIM%ld:DEL", limit);
    
        rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
        break;
    }
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Available Limit Lines List                                      */
/* Purpose:  This function queries list of available limit lines.            */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_limitLinesList (ViSession instrSession,
                                        ViInt32 bufferSize, ViChar list[])
{
    ViStatus rsfsh_status = VI_SUCCESS;
    ViUInt32 actSize, retCnt = 0;
    ViChar   *buffer = VI_NULL, *p2buffer;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    CHECKERR(rsfsh_writeInstrData (instrSession, "GET"));
    CHECKERR(rsfsh_writeInstrData (instrSession, "LIMLIST"));

    /* Allocate Buffer */
    actSize = BUFFER_SIZE;
    buffer = (ViChar *)calloc(actSize, sizeof(ViChar));
    if (buffer == VI_NULL) return VI_ERROR_ALLOC;

    p2buffer = buffer;
    do
        {
        if ((rsfsh_status = viRead (instrSession, (ViPBuf) p2buffer, BUFFER_SIZE, &retCnt)) < 0)
            {
            free(buffer);
            return rsfsh_status;
            }
        if (retCnt == BUFFER_SIZE) /* Buffer was small, reallocate it */
        {
            actSize += BUFFER_SIZE;
            buffer = (ViChar *)realloc(buffer, actSize);
            if (buffer == VI_NULL) return VI_ERROR_ALLOC;
            p2buffer = buffer + (actSize - BUFFER_SIZE); /* Set pointer to end of data in rallocated buffer */
        }
        else
        {
            *(p2buffer+retCnt) = '\0'; /* For sure set end of string on the end of data */
        }
       } while (rsfsh_status == 0x3FFF0006); /* if buffer was small, continue with next read */

    strncpy (list, buffer, bufferSize);
    
    free (buffer);
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Limit Line Audio Beep
/* Purpose:  This function activates or deactivates the audio beep. If 
/*           active, the R&S FSH outputs a beep each time a limit is 
/*           exceeded.
/*           The numeric suffix <1|2> is irrelevant.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setLimitLineAudioBeep(
    ViSession   instrSession,
    ViBoolean   audioBeep
)
{
    return rsfsh_setViBoolean (instrSession,
                               "CALC:LIM:BEEP:STAT",
                               audioBeep,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Limit Line Audio Beep
/* Purpose:  This function queries the audio beep state.
/*           The numeric suffix <1|2> is irrelevant.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLimitLineAudioBeep(
    ViSession   instrSession,
    ViBoolean*  audioBeep
)
{
    return rsfsh_getViBoolean (instrSession,
                               "CALC:LIM:BEEP:STAT?",
                               audioBeep,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Limit Line Description
/* Purpose:  This function queries the description of the limit line 
/*           indicated by the numeric suffix <1|2>. The maximum number of 
/*           characters for the comment is 32.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLimitLineDescription(
    ViSession   instrSession,
    ViInt32 limit,
    ViChar _VI_FAR  description[]
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:LIM%ld:COMM?", limit);
    
    return rsfsh_getViString (instrSession,
                              buffer,
                              description,
                              32,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Limit Line Check Result
/* Purpose:  This function queries the result of the limit check of the 
/*           limit line indicated by the numeric suffix <1|2>.
/*           
/*           This function is not compatible with FSH3 instrument.
/*           
/*           Note(s):
/*           Note that a complete sweep must have been performed to obtain a 
/*           correct result. A synchronization with *OPC, *OPC? or *WAI 
/*           should therefore be provided. The result of the limit check is 
/*           given with 0 for PASS, 1 for FAIL.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLimitLineCheckResult(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32*    limitCheck
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:LIM%ld:FAIL?", limit);
    
    return rsfsh_getViInt32 (instrSession, 
                             buffer,
                             limitCheck,
                             VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Select Limit Line Upper
/* Purpose:  This function selects the upper limit line.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_selectLimitLineUpper(
    ViSession   instrSession,
    ViString    limitLineName
)
{
    return rsfsh_setViStringQuoted (instrSession,
                                    "CALC:LIM:UPP:SEL",
                                    "'",
                                    limitLineName,
                                    VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Select Limit Line Lower
/* Purpose:  This function selects the lower limit line.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_selectLimitLineLower(
    ViSession   instrSession,
    ViString    limitLineName
)
{
    return rsfsh_setViStringQuoted (instrSession,
                                    "CALC:LIM:LOW:SEL",
                                    "'",
                                    limitLineName,
                                    VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Limit Line State
/* Purpose:  This function sets the limit line state.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setLimitLineState(
    ViSession   instrSession,
    ViInt32 limit,
    ViBoolean   state
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:LIM%ld:STAT", limit);
    
    return rsfsh_setViBoolean (instrSession, 
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Limit Line State
/* Purpose:  This function queries the limit line state.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLimitLineState(
    ViSession   instrSession,
    ViInt32 limit,
    ViBoolean*  state
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:LIM%ld:STAT?", limit);
    
    return rsfsh_getViBoolean (instrSession, 
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Limit Line Unit X
/* Purpose:  This function queries the unit of the x-axis.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLimitLineUnitX(
    ViSession   instrSession,
    ViInt32 limit,
    ViChar _VI_FAR  unitX[]
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:LIM%ld:UNIT:X?", limit);
    
    return rsfsh_getViString (instrSession,
                              buffer,
                              unitX,
                              1024,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Limit Line Unit Y
/* Purpose:  This function queries the unit of the level axis (y-unit).
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLimitLineUnitY(
    ViSession   instrSession,
    ViInt32 limit,
    ViChar _VI_FAR  unitY[]
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:LIM%ld:UNIT:Y?", limit);
    
    return rsfsh_getViString (instrSession,
                              buffer,
                              unitY,
                              1024,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Configure Tracking Generator Reference Level
/* Purpose:  This function the trtacking generator reference level.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureTrackingGeneratorReferenceLevel(
    ViSession   instrSession,
    ViInt32 type,
    ViReal64    referenceLevel,
    ViInt32 referencePosition
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    switch (type)
    {
        case RSFSH_VAL_TRACKING_GEN_TYPE_CLOSS:
            CHECKERR (rsfsh_setCableLossReferenceLevel (instrSession, referenceLevel));
            CHECKERR (rsfsh_setCableLossReferencePosition (instrSession, referencePosition));
            break;
        case RSFSH_VAL_TRACKING_GEN_TYPE_MAGN:
            CHECKERR (rsfsh_setMagnitudeReferenceLevel (instrSession, referenceLevel));
            CHECKERR (rsfsh_setMagnitudeReferencePosition (instrSession, referencePosition));
            break;
        case RSFSH_VAL_TRACKING_GEN_TYPE_PHAS:
            CHECKERR (rsfsh_setPhaseReferenceLevel (instrSession, referenceLevel));
            CHECKERR (rsfsh_setPhaseReferencePosition (instrSession, referencePosition));
            break;
        default:
            rsfsh_status = VI_ERROR_PARAMETER2;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Tracking Generator Y Scale
/* Purpose:  This function the trtacking generator reference level.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureTrackingGeneratorYScale(
    ViSession   instrSession,
    ViInt32 type,
    ViReal64    yScale
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    switch (type)
    {
        case RSFSH_VAL_TRACKING_GEN_TYPE_CLOSS:
            CHECKERR (rsfsh_setCableLossYScale (instrSession, yScale));
            break;
        case RSFSH_VAL_TRACKING_GEN_TYPE_MAGN:
            CHECKERR (rsfsh_setMagnitudeYScale (instrSession, yScale));
            break;
        case RSFSH_VAL_TRACKING_GEN_TYPE_PHAS:
            CHECKERR (rsfsh_setPhaseYScale (instrSession, (ViUInt32) yScale));
            break;
        case RSFSH_VAL_TRACKING_GEN_TYPE_REFL:
            CHECKERR (rsfsh_setReflectionCoefficientYScale (instrSession, (ViInt32) yScale));
            break;  
        default:
            rsfsh_status = VI_ERROR_PARAMETER2;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Tracking Generator Calibration
/* Purpose:  This function configures the selected transducer factor.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureTrackingGeneratorCalibration(
    ViSession   instrSession,
    ViInt32 transducer,
    ViBoolean   state,
    ViString    transducerFactor
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    CHECKERR (rsfsh_setTrackingGeneratorCalibrationTransducerSelect (instrSession, transducer, transducerFactor));
    CHECKERR (rsfsh_setTrackingGeneratorCalibrationTransducerState (instrSession, transducer, state));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Tracking Generator Mode                                     */
/* Purpose:  This function sets the tracking generator mode.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTrackingGeneratorMode (ViSession instrSession,
                                                  ViInt32 trackingGeneratorMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (trackingGeneratorMode, 0, 4))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TGMODE,%ld", trackingGeneratorMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Tracking Generator Mode                                     */
/* Purpose:  This function gets the tracking generator mode.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTrackingGeneratorMode (ViSession instrSession,
                                                  ViInt32 *trackingGeneratorMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!trackingGeneratorMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TGMODE", buffer));
    sscanf (buffer, "%ld", trackingGeneratorMode);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Tracking Generator Offset Length
/* Purpose:  This function sets the offset length. The numeric values must 
/*           be between 0 and 100 m with resolution of 0.01 mm.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTrackingGeneratorOffsetLength(
    ViSession   instrSession,
    ViReal64    length
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (length, 0.00, 100.00))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "OFFSETLEN,%Lf", length);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Tracking Generator Offset Length
/* Purpose:  This function queries the offset length.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTrackingGeneratorOffsetLength(
    ViSession   instrSession,
    ViReal64*   length
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!length)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "OFFSETLEN", buffer));
    sscanf (buffer, "%Lf", length);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Transm Calib State                                          */
/* Purpose:  This function queries the state of the transmission calibration.*/
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTransmCalibState (ViSession instrSession,
                ViPInt32 transmissionCalibrated)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!transmissionCalibrated)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRANSCAL", buffer));
    sscanf (buffer, "%ld", transmissionCalibrated);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Transm Vector Calib State                                   */
/* Purpose:  This function queries the state of the transmission vector      */
/*           calibration. Applies to option FSH-K2 only.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTransmVectorCalibState (ViSession instrSession,
                ViPInt32 transmissionVectorCalibrated)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!transmissionVectorCalibrated)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TRANSVECTCAL", buffer));
    sscanf (buffer, "%ld", transmissionVectorCalibrated);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Reflect Calib State                                         */
/* Purpose:  This function queries the state of the reflection calibration.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReflectCalibState (ViSession instrSession,
                ViPInt32 reflectionCalibrated)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!reflectionCalibrated)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "REFLCAL", buffer));
    sscanf (buffer, "%ld", reflectionCalibrated);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Reflect Vector Calib State                                  */
/* Purpose:  This function queries the state of the reflection vector        */
/*           calibration. Applies to option FSH-K2 only.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReflectVectorCalibState (ViSession instrSession,
                ViPInt32 reflectionVectorCalibrated)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!reflectionVectorCalibrated)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "REFLVECTCAL", buffer));
    sscanf (buffer, "%ld", reflectionVectorCalibrated);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Tracking Generator Level Attenuation                        */
/* Purpose:  This function defines the tracking generator output level       */
/*           attenuation.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTrackingGeneratorLevelAttenuation (ViSession instrSession,
                                                              ViInt32 levelAttenuation)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (levelAttenuation, 0, 20))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TGATT,%ld", levelAttenuation);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Tracking Generator Level Attenuation                        */
/* Purpose:  This function returns the tracking generator output level       */
/*           attenuation.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTrackingGeneratorLevelAttenuation (ViSession instrSession,
                                                              ViInt32 *levelAttenuation)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!levelAttenuation)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TGATT", buffer));
    sscanf (buffer, "%ld", levelAttenuation);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Tracking Generator Level                                    */
/* Purpose:  This function defines the tracking generator output level.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTrackingGeneratorLevel (ViSession instrSession,
                ViReal64 trackingGeneratorLevel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (trackingGeneratorLevel, -20.0, 0.0))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TGLVL,%Lf", trackingGeneratorLevel);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Tracking Generator Level                                    */
/* Purpose:  This function returns the tracking generator output level.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTrackingGeneratorLevel (ViSession instrSession,
                ViPReal64 trackingGeneratorLevel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!trackingGeneratorLevel)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TGLVL", buffer));
    sscanf (buffer, "%Le", trackingGeneratorLevel);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Tracking Generator Output Level
/* Purpose:  This function sets the output level of the tracking generator.
/*           You can set the output level by selecting an attenuation value. 
/*           The range is from 0 dB to 50 dB.
/*           Setting the attenuation to 0 dB results in an output level of 0 
/*           dBm. If you set the attenuation to 50 dB, the resulting output 
/*           level is -50 dBm.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTrackingGeneratorOutputLevel(
    ViSession   instrSession,
    ViReal64    outputLevel
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "SOUR:TG:ATT",
                              outputLevel,
                              0.0, 50.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Tracking Generator Output Level
/* Purpose:  This function queries the output level of the tracking 
/*           generator.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTrackingGeneratorOutputLevel(
    ViSession   instrSession,
    ViReal64*   outputLevel
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "SOUR:TG:ATT?",
                              outputLevel,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Measured Cable Loss                                         */
/* Purpose:  This function queries the measured cable loss.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMeasuredCableLoss (ViSession instrSession,
                                              ViReal64 *cableLoss)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!cableLoss)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CABLELOSS", buffer));
    sscanf (buffer, "%Le", cableLoss);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Electrical Cable Length                                     */
/* Purpose:  This function returns the electrical cable length.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getElectricalCableLength (ViSession instrSession,
                                              ViReal64 *electricalCableLength)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!electricalCableLength)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "ELCABLENVAL", buffer));
    sscanf (buffer, "%Le", electricalCableLength);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Electrical Cable State
/* Purpose:  This function sets the electrical cable length state.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setElectricalCableState(
    ViSession   instrSession,
    ViBoolean   state
)
{
    return rsfsh_setViBoolean (instrSession, 
                               "CALC:TRAC:CABL:LENG:STAT",
                               state,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Electrical Cable State
/* Purpose:  This function queries the electrical cable length state.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getElectricalCableState(
    ViSession   instrSession,
    ViBoolean*  state
)
{
    return rsfsh_getViBoolean (instrSession, 
                               "CALC:TRAC:CABL:LENG:STAT?",
                               state,
                               VI_ERROR_PARAMETER2);
}

/// HIFN  This function sets the electrical cable time state.
/// HIFN     
/// HIFN     This function is not compatible with FSH3 instrument.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate:TRACe:CABLe:TIME[:STATe]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control sets the electrical cable time state.
ViStatus _VI_FUNC rsfsh_setElectricalCableTimeState(
	ViSession	instrSession,
	ViBoolean	state
)
{
    return rsfsh_setViBoolean (instrSession, 
                               "CALC:TRAC:CABL:TIME:STAT",
                               state,
                               VI_ERROR_PARAMETER2);
}


/// HIFN  This function queries the electrical cable time state.
/// HIFN     
/// HIFN     This function is not compatible with FSH3 instrument.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate:TRACe:CABLe:TIME[:STATe]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control displays the electrical cable time state.
ViStatus _VI_FUNC rsfsh_getElectricalCableTimeState(
	ViSession	instrSession,
	ViBoolean*	state
)
{
    return rsfsh_getViBoolean (instrSession, 
                               "CALC:TRAC:CABL:TIME:STAT?",
                               state,
                               VI_ERROR_PARAMETER2);
}


/// HIFN  This function returns the electrical cable time.
/// HIFN     
/// HIFN     This function is not compatible with FSH3 instrument.
/// HIFN     
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate:TRACe:CABLe:TIME:RESult?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR electricalCableTime/Returns the electrical cable time.
ViStatus _VI_FUNC rsfsh_getElectricalCableTimeLength(
	ViSession	instrSession,
	ViReal64*	electricalCableTime
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "CALC:TRAC:CABL:TIME:RES?",
                              electricalCableTime,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set VSWR Y Scale
/* Purpose:  This function sets the display range of the y-axis for the VSWR 
/*           measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setVSWRYScale(
    ViSession   instrSession,
    ViReal64    YScale
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "DISP:VSWR:Y:SCAL",
                              YScale,
                              1.0, 71.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get VSWR Y Scale
/* Purpose:  This function queries the display range of the y-axis for the 
/*           VSWR measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVSWRYScale(
    ViSession   instrSession,
    ViReal64*   YScale
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "DISP:VSWR:Y:SCAL?",
                              YScale,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set VSWR Y Scale Min
/* Purpose:  This function sets the bottom value of the vertical axis for the
/*           VSWR measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setVSWRYScaleMin (ViSession instrSession,
                                          ViReal64 yScaleMinimum)
{
    return rsfsh_setViReal64 (instrSession, 
                              "DISP:VSWR:Y:SCAL:MIN",
                              yScaleMinimum,
                              1.0, 70.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get VSWR Y Scale Min
/* Purpose:  This function queries the bottom value of the vertical axis for the
/*           VSWR measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVSWRYScaleMin (ViSession instrSession,
                                          ViReal64 *yScaleMinimum)
{
    return rsfsh_getViReal64 (instrSession, 
                              "DISP:VSWR:Y:SCAL:MIN?",
                              yScaleMinimum,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set VSWR Y Scale Max
/* Purpose:  This function sets the top value of the vertical axis for the
/*           VSWR measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setVSWRYScaleMax (ViSession instrSession,
                                          ViReal64 yScaleMaximum)
{
    return rsfsh_setViReal64 (instrSession, 
                              "DISP:VSWR:Y:SCAL:MAX",
                              yScaleMaximum,
                              1.0, 70.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get VSWR Y Scale Max
/* Purpose:  This function queries the top value of the vertical axis for the
/*           VSWR measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVSWRYScaleMax (ViSession instrSession,
                                          ViReal64 *yScaleMaximum)
{
    return rsfsh_getViReal64 (instrSession, 
                              "DISP:VSWR:Y:SCAL:MAX?",
                              yScaleMaximum,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get VSWR Fail
/* Purpose:  This function queries the results of the limit check for the 
/*           VSWR measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVSWRFail(
    ViSession   instrSession,
    ViChar _VI_FAR  fail[]
)
{
    return rsfsh_getViString (instrSession,
                              "CALC:TRAC:LIM:VSWR:FAIL?",
                              fail,
                              1024,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Smith Chart Reference Impedance
/* Purpose:  This function sets the reference impedance for the Smith chart 
/*           measurement format. The impedance can be between 1mohm and 10 
/*           kohm.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setSmithChartReferenceImpedance(
    ViSession   instrSession,
    ViReal64    referenceImpedance
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "DISP:IMP:REF:POS",
                              referenceImpedance,
                              1.0e-3,10.0e+3,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Smith Chart Reference Impedance
/* Purpose:  This function queries the reference impedance for the Smith 
/*           chart measurement format. The impedance can be between 1mohm and 
/*           10 kohm.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getSmithChartReferenceImpedance(
    ViSession   instrSession,
    ViReal64*   referenceImpedance
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "DISP:IMP:REF:POS?",
                              referenceImpedance,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Cable Loss Reference Level
/* Purpose:  This function sets the reference level for the Cable Loss 
/*           measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCableLossReferenceLevel(
    ViSession   instrSession,
    ViReal64    referenceLevel
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "DISP:LOSS:REF",
                              referenceLevel,
                              0.0,0.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Cable Loss Reference Level
/* Purpose:  This function queries the reference level for the Cable Loss 
/*           measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCableLossReferenceLevel(
    ViSession   instrSession,
    ViReal64*   referenceLevel
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "DISP:LOSS:REF?",
                              referenceLevel,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Cable Loss Reference Position
/* Purpose:  This function sets the position of the reference value on the 
/*           display for the Cable Loss measurement format. Each step shifts 
/*           the reference position one grid line up or down.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCableLossReferencePosition(
    ViSession   instrSession,
    ViInt32 referencePosition
)
{
    return rsfsh_setViInt32 (instrSession, 
                             "DISP:LOSS:REF:POS",
                             referencePosition,
                             1,10,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Cable Loss Reference Position
/* Purpose:  This function queries the position of the reference value on 
/*           the display for the Cable Loss measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCableLossReferencePosition(
    ViSession   instrSession,
    ViInt32*    referencePosition
)
{
    return rsfsh_getViInt32 (instrSession, 
                             "DISP:LOSS:REF:POS?",
                             referencePosition,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Cable Loss Y Scale
/* Purpose:  This function sets the display range of the y-axis with 
/*           logarithmic scaling for the Cable Loss measurement format. The 
/*           switch to logarithmic scaling is achieved with the
/*           DISPlay[:WINDow]:TRACe:Y:SPACing command.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCableLossYScale(
    ViSession   instrSession,
    ViReal64    yScale
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "DISP:LOSS:Y:SCAL",
                              yScale,
                              0.0,0.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Cable Loss Y Scale
/* Purpose:  This function queries the display range of the y-axis with 
/*           logarithmic scaling for the Cable Loss measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCableLossYScale(
    ViSession   instrSession,
    ViReal64*   yScale
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "DISP:LOSS:Y:SCAL?",
                              yScale,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Magnitude Reference Level
/* Purpose:  This function sets the reference level for the Magnitude 
/*           measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMagnitudeReferenceLevel(
    ViSession   instrSession,
    ViReal64    referenceLevel
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "DISP:MAGN:REF",
                              referenceLevel,
                              0.0,0.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Magnitude Reference Level
/* Purpose:  This function queries the reference level for the Magnitude 
/*           measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMagnitudeReferenceLevel(
    ViSession   instrSession,
    ViReal64*   referenceLevel
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "DISP:MAGN:REF?",
                              referenceLevel,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Magnitude Reference Position
/* Purpose:  This function sets the position of the reference value on the 
/*           display for the Magnitude measurement format. Each step shifts 
/*           the reference position one grid line up or down.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMagnitudeReferencePosition(
    ViSession   instrSession,
    ViInt32 referencePosition
)
{
    return rsfsh_setViInt32 (instrSession, 
                              "DISP:MAGN:REF:POS",
                              referencePosition,
                              1,10,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Magnitude Reference Position
/* Purpose:  This function queries the position of the reference value on 
/*           the display for the Magnitude measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMagnitudeReferencePosition(
    ViSession   instrSession,
    ViInt32*    referencePosition
)
{
    return rsfsh_getViInt32 (instrSession, 
                             "DISP:MAGN:REF:POS?",
                             referencePosition,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Magnitude Y Scale
/* Purpose:  This function sets the display range of the y-axis with 
/*           logarithmic scaling for the Magnitude measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMagnitudeYScale(
    ViSession   instrSession,
    ViReal64    yScale
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "DISP:MAGN:Y:SCAL",
                              yScale,
                              0.0,0.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Magnitude Y Scale
/* Purpose:  This function queries the display range of the y-axis with 
/*           logarithmic scaling for the Magnitude measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMagnitudeYScale(
    ViSession   instrSession,
    ViReal64*   yScale
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "DISP:MAGN:Y:SCAL?",
                              yScale,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Magnitude Spacing
/* Purpose:  This function sets the scaling of the y-axis for the Magnitude 
/*           measurement format to either linear or logarithmic.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMagnitudeSpacing(
    ViSession   instrSession,
    ViInt32 spacing
)
{
    return rsfsh_setEnum (instrSession, 
                          "DISP:MAGN:Y:SPAC",
                          spacing,
                          spacingArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Magnitude Spacing
/* Purpose:  This function queries the scaling of the y-axis for the 
/*           Magnitude measurement format to either linear or logarithmic.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMagnitudeSpacing(
    ViSession   instrSession,
    ViInt32*    spacing
)
{
    return rsfsh_getEnum (instrSession, 
                          "DISP:MAGN:Y:SPAC?",
                          spacing,
                          spacingArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Phase Reference Level
/* Purpose:  This function sets the reference level for the Phase 
/*           measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPhaseReferenceLevel(
    ViSession   instrSession,
    ViReal64    referenceLevel
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "DISP:PHAS:REF",
                              referenceLevel,
                              0.0,0.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Phase Reference Level
/* Purpose:  This function queries the reference level for the Phase 
/*           measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPhaseReferenceLevel(
    ViSession   instrSession,
    ViReal64*   referenceLevel
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "DISP:PHAS:REF?",
                              referenceLevel,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Phase Reference Position
/* Purpose:  This function sets the position of the reference value on the 
/*           display for the Phase measurement format. Each step shifts the 
/*           reference position one grid line up or down.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPhaseReferencePosition(
    ViSession   instrSession,
    ViInt32 referencePosition
)
{
    return rsfsh_setViInt32 (instrSession, 
                             "DISP:PHAS:REF:POS",
                             referencePosition,
                             1,10,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Phase Reference Position
/* Purpose:  This function queries the position of the reference value on 
/*           the display for the Phase measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPhaseReferencePosition(
    ViSession   instrSession,
    ViInt32*    referencePosition
)
{
    return rsfsh_getViInt32 (instrSession, 
                             "DISP:PHAS:REF:POS?",
                             referencePosition,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Phase Y Scale
/* Purpose:  This function sets the display range of the y-axis for the 
/*           Phase measurement format. Phase can have values from 0 to 
/*           100000.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPhaseYScale(
    ViSession   instrSession,
    ViUInt32    yScale
)
{
    return rsfsh_setViUInt32 (instrSession, 
                              "DISP:PHAS:Y:SCAL",
                              yScale,
                              0,100000,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Phase Y Scale
/* Purpose:  This function queries the display range of the y-axis for the 
/*           Phase measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPhaseYScale(
    ViSession   instrSession,
    ViUInt32*   yScale
)
{
    return rsfsh_getViUInt32 (instrSession, 
                              "DISP:PHAS:Y:SCAL?",
                              yScale,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Phase Unwrap
/* Purpose:  This function removes the restriction limiting the value range 
/*           to +/- 180. Phase can have values from 0 to 100000.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPhaseUnwrap(
    ViSession   instrSession,
    ViBoolean   phaseUnwrap
)
{
    return rsfsh_setViBoolean (instrSession, 
                               "DISP:PHAS:Y:UNWR",
                               phaseUnwrap,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Phase Unwrap
/* Purpose:  This function queries the restriction limiting the value range 
/*           to +/- 180.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPhaseUnwrap(
    ViSession   instrSession,
    ViBoolean*  phaseUnwrap
)
{
    return rsfsh_getViBoolean (instrSession, 
                               "DISP:PHAS:Y:UNWR?",
                               phaseUnwrap,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Reflection Coefficient Y Scale
/* Purpose:  This function sets the display range of the y-axis for the 
/*           Reflection Coeffiecient measurement format.
/*           The switch to logarithmic scaling is achieved with the
/*           DISPlay[:WINDow]:TRACe:Y:SPACing command.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReflectionCoefficientYScale(
    ViSession   instrSession,
    ViInt32 yScale
)
{
    return rsfsh_setViInt32 (instrSession, 
                             "DISP:REFL:Y:SCAL",
                             yScale,
                             0, 1000,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Reflection Coefficient Y Scale
/* Purpose:  This function queries the display range of the y-axis for the 
/*           Reflection Coeffiecient measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReflectionCoefficientYScale(
    ViSession   instrSession,
    ViInt32*    yScale
)
{
    return rsfsh_getViInt32 (instrSession, 
                             "DISP:REFL:Y:SCAL?",
                             yScale,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Reflection Coefficient Unit
/* Purpose:  This function sets the unit for the reflection coefficient.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReflectionCoefficientUnit(
    ViSession   instrSession,
    ViInt32 unit
)
{
    return rsfsh_setEnum (instrSession, 
                          "DISP:REFL:UNIT",
                          unit,
                          reflectionUnitArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Reflection Coefficient Unit
/* Purpose:  This function queries the unit for the reflection coefficient.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReflectionCoefficientUnit(
    ViSession   instrSession,
    ViInt32*    unit
)
{
    return rsfsh_getEnum (instrSession, 
                          "DISP:REFL:UNIT?",
                          unit,
                          reflectionUnitArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Tracking Generator Reference Value
/* Purpose:  This function requires a tracking generator and active
/*           normalization in the Tracking Generator mode. It defines the
/*           position of the reference value.
/*
/*           The numeric suffixes <1|2> is irrelevant.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTrackingGeneratorReferenceValue(
    ViSession   instrSession,
    ViInt32 referenceValue
)
{
    return rsfsh_setViInt32 (instrSession, 
                             "DISP:TRAC:Y:RPOS",
                             referenceValue,
                             0, 100,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Tracking Generator Reference Value
/* Purpose:  This function queries the reference value.
/*
/*           The numeric suffixes <1|2> is irrelevant.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTrackingGeneratorReferenceValue(
    ViSession   instrSession,
    ViInt32*    referenceValue
)
{
    return rsfsh_getViInt32 (instrSession, 
                             "DISP:TRAC:Y:RPOS?",
                             referenceValue,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Adjust Y Scale
/* Purpose:  This function automatically sets the scaling of the yaxis to 
/*           best fit the measurement results of the indicated trace.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_adjustYScale(
    ViSession   instrSession,
    ViInt32     trace
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "DISP:TRAC%ld:Y:ADJ", trace);
    
    return rsfsh_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get Isotropic Antenna State
/* Purpose:  This function if the currently selected transducer is an
/*           isotropic antenna.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getIsotropicAntennaState (ViSession instrSession,
                                                  ViInt32 transducer,
                                                  ViBoolean *state)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (transducer, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "SENSe:CORRection:TRANsducer%ld:ISOTropic:STATe?", transducer);
    
    return rsfsh_getViBoolean (instrSession,
                             buffer,
                             state,
                             VI_ERROR_PARAMETER3); 
}

/*===========================================================================*/
/* Function: Set Isotropic Antenna
/* Purpose:  This function configures the Antenna Settings.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setIsotropicAntenna (ViSession instrSession,
                                             ViBoolean antenna)
{
   return rsfsh_setViBoolean (instrSession,
                              "INP:ANT:STAT",
                              antenna,
                              VI_ERROR_PARAMETER2);   
}

/*===========================================================================*/
/* Function: Get Isotropic Antenna
/* Purpose:  This function returns the isotropic antenna state.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getIsotropicAntenna (ViSession instrSession,
                                             ViBoolean *antenna)
{
   return rsfsh_getViBoolean (instrSession,
                              "INP:ANT:STAT?",
                               antenna,
                               VI_ERROR_PARAMETER2); 
}

/*===========================================================================*/
/* Function: Set Antenna Direction
/* Purpose:  This function configures the direction that the isotropic
/*           antenna measures.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAntennaDirection (ViSession instrSession,
                                             ViInt32 direction)
{
    return rsfsh_setEnum (instrSession,
		                  "INP:ANT:MEAS",
						  direction,
						  antennaDirectionArr,
						  VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Antenna Direction
/* Purpose:  This function queries the direction that the isotropic
/*           antenna measures.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAntennaDirection (ViSession instrSession,
                                             ViInt32 *direction)
{
    return rsfsh_getEnum (instrSession,
		                  "INP:ANT:MEAS?",
						  direction,
						  antennaDirectionArr,
						  VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Scalar Transmission Calibration                                 */
/* Purpose:  This function queries the power level measured by the sensor.   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_trackingGeneratorScalarTransmissionCalibration (ViSession instrSession)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[10] = "";
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    switch (instrPtr -> fsh3)
    {
    case FSH3:
    CHECKERR (rsfsh_cmdCommand (instrSession, "CAL_TGSCLTRN"));
    CHECKERR (rsfsh_readInstrData (instrSession, 10, buffer, VI_NULL));
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Initiate Tracking Generator Calibration                         */
/* Purpose:  This function initiates first phase of selected calibration     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_trackingGeneratorInitiateCalibration (ViSession instrSession,
                                                              ViInt32 trackingGeneratorCalibration)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer [10] = "";
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    switch (trackingGeneratorCalibration)
        {
        case 0://Scalar Reflection
            CHECKERR (rsfsh_cmdCommand (instrSession, "CAL_TGSCLRFL"));
        break;
        
        case 1://Vector Reflection
            CHECKERR (rsfsh_cmdCommand (instrSession, "CAL_TGVECRFL"));
        break;
        
        case 2://Vector Transmission
            CHECKERR (rsfsh_cmdCommand (instrSession, "CAL_TGVECTRN"));
        break;
        
        default:
            rsfsh_status = VI_ERROR_PARAMETER2;         
        }
/* wait for response */
    CHECKERR (rsfsh_readInstrData (instrSession, 10, buffer, VI_NULL));
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Next Phase Tracking Generator Calibration                       */
/* Purpose:  This function initiates next phase of selected calibration      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_trackingGeneratorNextPhaseCalibration (ViSession instrSession,
                                                               ViInt32 trackingGeneratorCalibration)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer [10] = "";
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    switch (trackingGeneratorCalibration)
        {
        case 0://Scalar Reflection
            CHECKERR (rsfsh_writeInstrData (instrSession, "CAL_TGSCLRFL"));
        break;
        
        case 1://Vector Reflection
            CHECKERR (rsfsh_writeInstrData (instrSession, "CAL_TGVECRFL"));
        break;
        
        case 2://Vector Transmission
            CHECKERR (rsfsh_writeInstrData (instrSession, "CAL_TGVECTRN"));
        break;
        
        default:
            rsfsh_status = VI_ERROR_PARAMETER2;         
        }
/* wait for response */
    CHECKERR (rsfsh_readInstrData (instrSession, 10, buffer, VI_NULL));
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Tracking Generator Calibration Transducer State
/* Purpose:  This function sets the selected transducer factor on or off.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTrackingGeneratorCalibrationTransducerState(
    ViSession   instrSession,
    ViInt32 transducer,
    ViBoolean   state
)
{
    ViChar buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (transducer, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "SENS:CORR:TRAN%ld:STAT", transducer);
    
    return rsfsh_setViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
    
}

/*===========================================================================*/
/* Function: Get Tracking Generator Calibration Transducer State
/* Purpose:  This function queries the selected transducer factor on or off.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTrackingGeneratorCalibrationTransducerState(
    ViSession   instrSession,
    ViInt32 transducer,
    ViBoolean*  state
)
{
    ViChar buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (transducer, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "SENS:CORR:TRAN%ld:STAT?", transducer);
    
    return rsfsh_getViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);
    
}

/*===========================================================================*/
/* Function: Set Tracking Generator Calibration Transducer Select
/* Purpose:  This function sets the transducer factor designated by <name>. 
/*           If <name> does not exist yet, a new transducer factor is 
/*           created.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTrackingGeneratorCalibrationTransducerSelect(
    ViSession   instrSession,
    ViInt32 transducer,
    ViString    transducerFactor
)
{
    ViChar buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (transducer, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "SENS:CORR:TRAN%ld:SEL", transducer);
    
    return rsfsh_setViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    transducerFactor,
                                    VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Tracking Generator Calibration Transducer Select
/* Purpose:  This function queries the transducer factor designated by 
/*           <name>. If <name> does not exist yet, a new transducer factor is 
/*           created.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTrackingGeneratorCalibrationTransducerSelect(
    ViSession   instrSession,
    ViInt32 transducer,
    ViChar _VI_FAR  transducerFactor[]
)
{
    ViChar buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (transducer, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "SENS:CORR:TRAN%ld:SEL?", transducer);
    
    return rsfsh_getViString (instrSession,
                              buffer,
                              transducerFactor,
                              1024,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Tracking Generator Calibration Transducer Unit
/* Purpose:  This function queries the unit of the transducer factor in use.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTrackingGeneratorCalibrationTransducerUnit(
    ViSession   instrSession,
    ViInt32 transducer,
    ViChar _VI_FAR  unit[]
)
{
    ViChar buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (transducer, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "SENS:CORR:TRAN%ld:UNIT?", transducer);
    
    return rsfsh_getViString (instrSession,
                              buffer,
                              unit,
                              1024,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Power Level                                                 */
/* Purpose:  This function queries the power level measured by the sensor.   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerLevel (ViSession instrSession,
                                       ViPReal64 powerLevel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!powerLevel)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "PWR", buffer));
    sscanf (buffer, "%Le", powerLevel);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Reflection                                                  */
/* Purpose:  This function queries the reflection measured by the power      */
/*           sensor R&S FSH-Z44.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReflection (ViSession instrSession,
                                       ViReal64 *reflection)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!reflection)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "REFL", buffer));
    sscanf (buffer, "%Le", reflection);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Power Sensor Zeroing                                            */
/* Purpose:  This function initiates the power sensor zeroing.               */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_powerSensorZeroing (ViSession instrSession)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if ((rsfsh_status = rsfsh_cmdCommand (instrSession, "ZERO")) == VI_ERROR_TMO)
        {
        /* Power sensor is not connected, generate execution error */
        CHECKERR(viScanf (instrSession, "%*[^\r\n]"));
        rsfsh_status = RSFSH_ERROR_EXECUTION_ERROR;
        }
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Power To Reference                                              */
/* Purpose:  This function defines the current power level as the reference  */
/*           value.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_powerToReference (ViSession instrSession)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    CHECKERR (rsfsh_cmdCommand (instrSession, "PWRTOREF"));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Measurement Time                                            */
/* Purpose:  This function defines the measurement time for the power sensor */
/*           (FSH-Z1 and FSH-Z18).                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMeasurementTime (ViSession instrSession,
                                            ViInt32 measurementTime)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (measurementTime, 0, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "MEASTIME,%ld", measurementTime);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Measurement Time                                            */
/* Purpose:  This function returns the measurement time for the power sensor */
/*           (FSH-Z1 and FSH-Z18).                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMeasurementTime (ViSession instrSession,
                                            ViPInt32 measurementTime)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!measurementTime)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "MEASTIME", buffer));
    sscanf (buffer, "%ld", measurementTime);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Reflection Unit                                             */
/* Purpose:  This control defines the reflection unit for the power sensor   */
/*           R&S FSH-Z44.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReflectionUnit (ViSession instrSession,
                                           ViBoolean reflectionUnit)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (reflectionUnit))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "REFLUNIT,%ld", reflectionUnit);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Reflection Unit                                             */
/* Purpose:  This control returns the reflection unit for the power sensor   */
/*           R&S FSH-Z44.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReflectionUnit (ViSession instrSession,
                                           ViBoolean *reflectionUnit)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!reflectionUnit)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "REFLUNIT", buffer));
    sscanf (buffer, "%hd", reflectionUnit);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Power Sensor Standard                                       */
/* Purpose:  This function defines the standard used for the power sensor    */ 
/*           R&S FSH-Z44 measurements.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerSensorStandard (ViSession instrSession,
                                                ViInt32 channelPowerStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     standard = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (channelPowerStandard, 0, 8))
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "PWRSSTD", buffer));
    sscanf (buffer, "%ld", &standard);

    if (channelPowerStandard != standard)
    {
        sprintf (buffer, "PWRSSTD,%ld", channelPowerStandard);
        CHECKERR (rsfsh_setCommand (instrSession, buffer));
    }
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Power Sensor Standard                                       */
/* Purpose:  This function returns the standard used for the power sensor    */ 
/*           R&S FSH-Z44 measurements.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerSensorStandard (ViSession instrSession,
                                                ViInt32 *channelPowerStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelPowerStandard)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "PWRSSTD", buffer));
    sscanf (buffer, "%ld", channelPowerStandard);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Configure Channel Power                                         */
/* Purpose:  This function configures channel power measurement parameters.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureChannelPower (ViSession instrSession,
                                               ViInt32 channelPowerStandard,
                                               ViReal64 channelPowerBandwidth,
                                               ViInt32 channelPowerUnit,
                                               ViInt32 mode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (rsfsh_invalidViInt32Range (channelPowerStandard, 0, 3))
        return VI_ERROR_PARAMETER2;
    /* Parameter 3 is not range checked */
    if (rsfsh_invalidViInt32Range (channelPowerUnit, 0, 10))
        return VI_ERROR_PARAMETER4;
    if (rsfsh_invalidViInt32Range (mode, 0, 1))
        return VI_ERROR_PARAMETER5;

    switch (instrPtr -> fsh3)
    {
    case FSH3:
        CHECKERR (rsfsh_setChannelPowerStandard (instrSession, channelPowerStandard));
        CHECKERR (rsfsh_setChannelPowerUnit (instrSession, channelPowerUnit));
        CHECKERR (rsfsh_setChannelPowerBandwidth (instrSession, channelPowerBandwidth));
        break;
        
    case FSH4:
        CHECKERR (rsfsh_setChannelPowerUnit (instrSession, channelPowerUnit));
        CHECKERR (rsfsh_setChannelPowerBandwidth (instrSession, channelPowerBandwidth));
        CHECKERR (rsfsh_setChannelPowerMode (instrSession, mode));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Channel Power                                               */
/* Purpose:  This function queries the measured channel power.               */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getChannelPower (ViSession instrSession,
                                         ViPReal64 channelPower)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelPower)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CHPWR", buffer));
    sscanf (buffer, "%Le", channelPower);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Channel Power Standard                                      */
/* Purpose:  This function defines the standard used for the channel power   */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setChannelPowerStandard (ViSession instrSession,
                                                 ViInt32 channelPowerStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (channelPowerStandard, 0, 3))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CHPWRSTD,%ld", channelPowerStandard);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Channel Power Standard                                      */
/* Purpose:  This function returns the standard used for the channel power   */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getChannelPowerStandard (ViSession instrSession,
                                                 ViPInt32 channelPowerStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelPowerStandard)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CHPWRSTD", buffer));
    sscanf (buffer, "%ld", channelPowerStandard);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Channel Power Customized Standard                           */
/* Purpose:  This function selects the customized standard for the channel   */
/*           power measurement previously loaded with R&S FSHView software.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setChannelPowerCustomizedStandard (ViSession instrSession,
                                                           ViString channelPowerStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (channelPowerStandard, 15))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CHPWRCSTD,%s", channelPowerStandard);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Channel Power Customized Standard                           */
/* Purpose:  This function returns the customized standard for the channel   */
/*           power measurement previously loaded with R&S FSHView software.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getChannelPowerCustomizedStandard (ViSession instrSession,
                                                           ViPString channelPowerStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelPowerStandard)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CHPWRCSTD", buffer));
    //sscanf (buffer, "%[^\r\n]", channelPowerStandard);
    strcpy (channelPowerStandard, buffer);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Channel Power Unit                                          */
/* Purpose:  This function defines the unit used for the channel power       */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setChannelPowerUnit (ViSession instrSession,
                                             ViInt32 channelPowerUnit)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (channelPowerUnit, 0, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CHPWRUNIT,%ld", channelPowerUnit);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        CHECKERR (rsfsh_setEnum (instrSession,
                                 "CALC:MARK:FUNC:CPOW:UNIT",
                                 channelPowerUnit,
                                 channelPowerUnitArr,
                                 VI_ERROR_PARAMETER2));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Channel Power Unit                                          */
/* Purpose:  This function returns the unit used for the channel power       */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getChannelPowerUnit (ViSession instrSession,
                                             ViPInt32 channelPowerUnit)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelPowerUnit)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CHPWRUNIT", buffer));
    sscanf (buffer, "%ld", channelPowerUnit);
    break;
    case FSH4:
        CHECKERR (rsfsh_getEnum (instrSession,
                                 "CALC:MARK:FUNC:CPOW:UNIT?",
                                 channelPowerUnit,
                                 channelPowerUnitArr,
                                 VI_ERROR_PARAMETER2));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Channel Power Bandwidth                                     */
/* Purpose:  This function defines the bandwidth used for channel power      */
/*           measurements.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setChannelPowerBandwidth (ViSession instrSession,
                                                  ViReal64 channelPowerBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Not range checked */

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    sprintf (buffer, "CHPWRBW,%Lf", channelPowerBandwidth);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        CHECKERR (rsfsh_setViReal64 (instrSession,
                                     "CALC:MARK:FUNC:CPOW:BAND",
                                     channelPowerBandwidth,
                                     0.0,0.0,
                                     VI_ERROR_PARAMETER2));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Channel Power Bandwidth                                     */
/* Purpose:  This function defines the bandwidth used for channel power      */
/*           measurements.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getChannelPowerBandwidth (ViSession instrSession,
                                                  ViPReal64 channelPowerBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelPowerBandwidth)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CHPWRBW", buffer));
    sscanf (buffer, "%Le", channelPowerBandwidth);
    break;
    case FSH4:
        CHECKERR (rsfsh_getViReal64 (instrSession,
                                     "CALC:MARK:FUNC:CPOW:BAND?",
                                     channelPowerBandwidth,
                                     VI_ERROR_PARAMETER2));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Channel Power Level Adjust                                      */
/* Purpose:  This function initiates a level adjustment for the channel power*/
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_channelPowerLevelAdjust (ViSession instrSession)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    CHECKERR (rsfsh_cmdCommand (instrSession, "LVLADJUST"));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Power Sensor Frequency 
/* Purpose:  This function sets the frequency of the power sensor.
/*           
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerSensorFrequency (ViSession instrSession,
                                                 ViReal64 frequency)
{
    return rsfsh_setViReal64 (instrSession,
                                     "SENSe:PMETer:FREQuency",
                                     frequency,
                                     10000000.0,18000000000.0,
                                     VI_ERROR_PARAMETER2);    
}

/*===========================================================================*/
/* Function: Get Power Sensor Frequency 
/* Purpose:  This function returns the frequency of the power sensor.
/*           
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerSensorFrequency (ViSession instrSession,
                                                 ViReal64 *frequency)
{
    return rsfsh_getViReal64 (instrSession,
                                     "SENSe:PMETer:FREQuency?",
                                     frequency,
                                     VI_ERROR_PARAMETER2);    
}

/*===========================================================================*/
/* Function: Set Power Sensor Relative Reference Level 
/* Purpose:  This function sets the reference level for relative measurements.
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerSensorRelativeReferenceLevel (ViSession instrSession,
                                                              ViReal64 level)
{
    return rsfsh_setViReal64 (instrSession,
                                     "CALCulate:PMETer:RELative:MAGNitude",
                                     level,
                                     -100.0,100.0,
                                     VI_ERROR_PARAMETER2);    
}

/*===========================================================================*/
/* Function: Get Power Sensor Relative Reference Level 
/* Purpose:  This function returns the reference level for relative measurements.
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerSensorRelativeReferenceLevel (ViSession instrSession,
                                                              ViReal64 *level)
{
    return rsfsh_getViReal64 (instrSession,
                                     "CALCulate:PMETer:RELative:MAGNitude?",
                                     level,
                                     VI_ERROR_PARAMETER2);    
}

/*===========================================================================*/
/* Function: Set Power Sensor Relative 
/* Purpose:  This function sets the current measurement result as the
/*           reference level for relative measurements.
/*           
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerSensorRelative (ViSession instrSession)
{
    return rsfsh_sendCmd(instrSession,
        "CALCulate:PMETer:RELative:MAGNitude:AUTO ONCE");
}

/*===========================================================================*/
/* Function: Set Power Sensor Reference Level Offset 
/* Purpose:  This function sets the offset of the reference level for
/*           relative measurements.
/*           
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerSensorReferenceLevelOffset (ViSession instrSession,
                                                            ViReal64 offset)
{
    return rsfsh_setViReal64 (instrSession,
                                     "CALCulate:PMETer:RELative:MAGNitude:OFFSet",
                                     offset,
                                     0.0,0.0,
                                     VI_ERROR_PARAMETER2);    
}

/*===========================================================================*/
/* Function: Get Power Sensor Reference Level Offset 
/* Purpose:  This function returns the offset of the reference level for
/*           relative measurements.
/*           
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerSensorReferenceLevelOffset (ViSession instrSession,
                                                            ViReal64 *offset)
{
    return rsfsh_getViReal64 (instrSession,
                                     "CALCulate:PMETer:RELative:MAGNitude:OFFSet?",
                                     offset,
                                     VI_ERROR_PARAMETER2);    
}

/*===========================================================================*/
/* Function: Set Power Sensor Units 
/* Purpose:  This function selects the unit of the power sensor.
/*           
/*           The suffix at UNIT has the following effects:
/*           Power Measurement with R&S FSH-Z1 and R&S FSH-Z18:
/*           Unit 1 Power unit
/*           Unit 2 not available.
/*           
/*           Power Measurement with R&S FSH-Z14 and R&S FSH-Z44:
/*           Unit 1 Forward Power
/*           Unit 2 Reflected Power
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerSensorUnits (ViSession instrSession,
                                             ViInt32 unit, ViInt32 units)
{
    ViChar buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (unit, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (units, 0, 3))
        return VI_ERROR_PARAMETER3;
    
    sprintf (buffer, "UNIT%ld:PMETer:POWer", unit);
    
    return rsfsh_setEnum (instrSession,
                              buffer,
                              units,
                              powerSensorUnitArr,
                              VI_ERROR_PARAMETER3);    
}

/*===========================================================================*/
/* Function: Get Power Sensor Units
/* Purpose:  This function returns the unit of the power sensor.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerSensorUnits (ViSession instrSession,
                                             ViInt32 unit, ViInt32 *units)
{
    ViChar buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (unit, 1, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "UNIT%ld:PMETer:POWer?", unit);
    
    return rsfsh_getEnum (instrSession,
                              buffer,
                              units,
                              powerSensorUnitArr,
                              VI_ERROR_PARAMETER3);    
}


/*===========================================================================*/
/* Function: Get Power Sensor Video Bandwidth 
/* Purpose:  This function returns the video bandwidth for powe sensor
/*           measurements.
/*           
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerSensorVideoBandwidth (ViSession instrSession,
                                                      ViReal64 *videoBandwidth)
{
    return rsfsh_getViReal64 (instrSession,
                                     "CALCulate:PMETer:PRESet:BANDwidth:VIDeo?",
                                     videoBandwidth,
                                     VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Power Sensor Measurement Time 
/* Purpose:  This function sets the duration of measurements with the power
/*           sensor.
/*           
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerSensorMeasurementTime (ViSession instrSession,
                                                       ViInt32 measurementTime)
{
    return rsfsh_setEnum (instrSession,
                              "SENSe:PMETer:MTIMe",
                              measurementTime,
                              powerSensorMeasTimeArr,
                              VI_ERROR_PARAMETER3); 
}

/*===========================================================================*/
/* Function: Get Power Sensor Measurement Time 
/* Purpose:  This function returns the duration of measurements with the power
/*           sensor.
/*           
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerSensorMeasurementTime (ViSession instrSession,
                                                       ViInt32 *measurementTime)
{
    return rsfsh_getEnum (instrSession,
                              "SENSe:PMETer:MTIMe?",
                              measurementTime,
                              powerSensorMeasTimeArr,
                              VI_ERROR_PARAMETER3); 
}

/*===========================================================================*/
/* Function: Set Power Sensor Zeroing 
/* Purpose:  This function starts to zero the power sensor.
/*           
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerSensorZeroing (ViSession instrSession)
{
    return rsfsh_setViString(instrSession,
                            "CALibration:PMETer:ZERO:AUTO",
                            "ONCE",
                            VI_ERROR_PARAMETER1);
}

/*===========================================================================*/
/* Function: Set Power Sensor Power Display 
/* Purpose:  This function selects the forward power display of the power
/*           sensor.
/*           
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerSensorPowerDisplay (ViSession instrSession,
                                                    ViInt32 powerDisplay)
{
    return rsfsh_setEnum (instrSession,
                              "SENSe:PMETer:DETector:FUNCtion",
                              powerDisplay,
                              powerSensorDisplayArr,
                              VI_ERROR_PARAMETER3);     
}

/*===========================================================================*/
/* Function: Get Power Sensor Power Display 
/* Purpose:  This function returns the forward power display of the power
/*           sensor.
/*           
/*           Note that the listed remote commands take effect only if the
/*           power sensors R&S FSH-Z1, R&S FSH-Z18, R&S FSH-Z14 or R&S
/*           FSH-Z44 are installed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerSensorPowerDisplay (ViSession instrSession,
                                                    ViInt32 *powerDisplay)
{
    return rsfsh_getEnum (instrSession,
                              "SENSe:PMETer:DETector:FUNCtion?",
                              powerDisplay,
                              powerSensorDisplayArr,
                              VI_ERROR_PARAMETER3);     
}

/*===========================================================================*/
/* Function: Get Power Sensor Measurements Results 
/* Purpose:  This function queries the results of measurements with the power
/*           sensor.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerSensorMeasurementsResults (ViSession instrSession,
                                                           ViInt32 unit,
                                                           ViReal64 *results)
{
    ViChar buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (unit, 1, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "FETch%ld:PMETer?", unit);
    
    return rsfsh_getViReal64 (instrSession,
                              buffer,
                              results,
                              VI_ERROR_PARAMETER3);      
}

/*===========================================================================*/
/* Function: Set Power Sensor Use Standard 
/* Purpose:  This function activates and deactivates the usage of a standard
/*           for power sensor measurements.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerSensorUseStandard (ViSession instrSession,
                                                   ViBoolean state)
{
    return rsfsh_setViBoolean(instrSession,
                     "CALCulate:PMETer:PRESet:STATe",
                     state,
                     VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Power Sensor Use Standard 
/* Purpose:  This function returns state of the usage of a standard
/*           for power sensor measurements.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerSensorUseStandard (ViSession instrSession,
                                                   ViBoolean *state)
{
    return rsfsh_getViBoolean(instrSession,
                     "CALCulate:PMETer:PRESet:STATe?",
                     state,
                     VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Select Channel Table
/* Purpose:  This function sets the specified channel table for upcoming 
/*           measurements.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_selectChannelTable(
    ViSession   instrSession,
    ViString    channelTable
)
{
    return rsfsh_setViStringQuoted (instrSession,
                                    "SENS:CHAN:TABL:SEL",
                                    "'",
                                    channelTable,
                                    VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Channel Table Link Direction
/* Purpose:  This function sets the link direction for measurements with channel tables.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setChannelTableLinkDirection (ViSession instrSession,
                                                            ViInt32 linkDirection)
{
    return rsfsh_setEnum (instrSession,
		                  "SENS:CHAN:TABL:SET",
						  linkDirection,
						  linkDirectionArr,
						  VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Channel Table Link Direction
/* Purpose:  This function queries the link direction for measurements with channel tables.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getChannelTableLinkDirection (ViSession instrSession,
                                                      ViInt32 *linkDirection)
{
    return rsfsh_getEnum (instrSession,
		                  "SENS:CHAN:TABL:SET?",
						  linkDirection,
						  linkDirectionArr,
						  VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Channel Table UL DL Signal
/* Purpose:  This function sets a channel table configured for downlink /
/*           uplink signals.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setChannelTableULDLSignal (ViSession instrSession,
                                                         ViInt32 signal,
                                                         ViString channelTable)
{
	ViStatus rsfsh_status = VI_SUCCESS;
	
	if (signal == RSFHS_VAL_DIRECTION_DOWNLINK)
		rsfsh_status = rsfsh_setViString (instrSession, "SENS:CHAN:TABL:SEL:DOWN", channelTable, VI_ERROR_PARAMETER3);
	else
		rsfsh_status = rsfsh_setViString (instrSession, "SENS:CHAN:TABL:SEL:UPL", channelTable, VI_ERROR_PARAMETER3);
		
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Channel Table UL DL Signal
/* Purpose:  This function queries a channel table configured for downlink /
/*           uplink signals.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getChannelTableULDLSignal (ViSession instrSession,
                                                   ViInt32 signal,
                                                   ViUInt32 arraySize,
                                                   ViChar channelTable[])
{
	ViStatus rsfsh_status = VI_SUCCESS;
	
	if (signal == RSFHS_VAL_DIRECTION_DOWNLINK)
		rsfsh_status = rsfsh_getViString (instrSession, "SENS:CHAN:TABL:SEL:DOWN?", channelTable, arraySize, VI_ERROR_PARAMETER4);
	else
		rsfsh_status = rsfsh_getViString (instrSession, "SENS:CHAN:TABL:SEL:UPL?", channelTable, arraySize, VI_ERROR_PARAMETER4);
		
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Occupied Bandwidth                                    */
/* Purpose:  This function configures occupied power measurement parameters. */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureOccupiedBandwidth (ViSession instrSession,
                                            ViInt32 marker,
                                            ViInt32 occupiedBandwidthStandard,
                                            ViReal64 occupiedChannelBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (occupiedBandwidthStandard, 0, 3))
        return VI_ERROR_PARAMETER3;
    

    CHECKERR (rsfsh_setOccupiedBandwidthStandard (instrSession,
              occupiedBandwidthStandard));
    CHECKERR (rsfsh_setOccupiedBandwidthChBW (instrSession,
              marker, occupiedChannelBandwidth));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Occupied Bandwidth                                          */
/* Purpose:  This function queries the measured occupied bandwidth.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getOccupiedBandwidth (ViSession instrSession,
                ViPReal64 occupiedBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!occupiedBandwidth)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "OBW", buffer));
    sscanf (buffer, "%Le", occupiedBandwidth);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Occupied Bandwidth Standard                                 */
/* Purpose:  This function defines the standard used for the occupied power  */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setOccupiedBandwidthStandard (ViSession instrSession,
                ViInt32 occupiedBandwidthStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (occupiedBandwidthStandard, 0, 3))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "OBWSTD,%ld", occupiedBandwidthStandard);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Occupied Bandwidth Standard                                 */
/* Purpose:  This function returns the standard used for the occupied power  */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getOccupiedBandwidthStandard (ViSession instrSession,
                ViPInt32 occupiedBandwidthStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!occupiedBandwidthStandard)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "OBWSTD", buffer));
    sscanf (buffer, "%ld", occupiedBandwidthStandard);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Occupied Bandwidth Customized Standard                      */
/* Purpose:  This function selects the customized standard for the occupied  */
/*           bandwidth measurement previously loaded with R&S FSHView        */
/*           software.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setOccupiedBandwidthCustomizedStandard (ViSession instrSession,
                                                                ViString occupiedBandwidthStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (occupiedBandwidthStandard, 15))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "OBWCSTD,%s", occupiedBandwidthStandard);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Channel Power Customized Standard                           */
/* Purpose:  This function returns the customized standard for the channel   */
/*           power measurement previously loaded with R&S FSHView software.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getOccupiedBandwidthCustomizedStandard (ViSession instrSession,
                                                                ViString *occupiedBandwidthStandard[])
{
	ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!occupiedBandwidthStandard)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "OBWCSTD", buffer));
    sscanf (buffer, "%[^\r\n]", occupiedBandwidthStandard);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;     
}

/*===========================================================================*/
/* Function: Set Occupied Bandwidth Ch BW                                    */
/* Purpose:  This function defines the bandwidth used for occupied bandwidth */
/*           measurements.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setOccupiedBandwidthChBW (ViSession instrSession,
                ViInt32 marker,
                ViReal64 occupiedChannelBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Not range checked */

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    sprintf (buffer, "OBWCHBW,%Lf", occupiedChannelBandwidth);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        if (rsfsh_invalidViInt32Range (marker, 1, 6))
            return VI_ERROR_PARAMETER2;
    
        sprintf (buffer, "CALC:MARK%ld:FUNC:OBAN:BAND", marker);
    
        CHECKERR (rsfsh_setViReal64 (instrSession,
                                     buffer,
                                     occupiedChannelBandwidth,
                                     0.0,0.0,
                                     VI_ERROR_PARAMETER3));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Occupied Bandwidth Ch BW                                    */
/* Purpose:  This function returns the bandwidth used for occupied bandwidth */
/*           measurements.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getOccupiedBandwidthChBW (ViSession instrSession,
                ViInt32 marker,
                ViPReal64 occupiedChannelBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!occupiedChannelBandwidth)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "OBWCHBW", buffer));
    sscanf (buffer, "%Le", occupiedChannelBandwidth);
    break;
    case FSH4:
        if (rsfsh_invalidViInt32Range (marker, 1, 6))
            return VI_ERROR_PARAMETER2;
    
        sprintf (buffer, "CALC:MARK%ld:FUNC:OBAN:BAND?", marker);
    
        CHECKERR (rsfsh_getViReal64 (instrSession,
                                     buffer,
                                     occupiedChannelBandwidth,
                                     VI_ERROR_PARAMETER3));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Occupied Bandwidth Level Adjust                                 */
/* Purpose:  This function initiates a level adjustment for the occupied     */
/*           bandwidth measurement.                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_occupiedBandwidthLevelAdjust (ViSession instrSession)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    CHECKERR (rsfsh_cmdCommand (instrSession, "LVLADJUST"));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure TDMA Power                                            */
/* Purpose:  This function configures TDMA power measurement parameters such */
/*           as TDMA power standard and measurement time.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureTDMAPower (ViSession instrSession,
                                            ViInt32 marker,
                                            ViInt32 TDMAPowerStandard,
                                            ViReal64 measurementTime)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (TDMAPowerStandard, 0, 1))
        return VI_ERROR_PARAMETER2;
    /* Parameter 3 is not range checked */

    CHECKERR (rsfsh_setTDMAPowerStandard (instrSession, TDMAPowerStandard));
    CHECKERR (rsfsh_setTDMAPowerMeasurementTime (instrSession, marker, measurementTime));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get TDMA Power                                                  */
/* Purpose:  This function queries the measured TDMA power.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTDMAPower (ViSession instrSession,
                                      ViPReal64 TDMAPower)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!TDMAPower)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TDMAPWR", buffer));
    sscanf (buffer, "%Le", TDMAPower);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set TDMA Power Standard                                         */
/* Purpose:  This function defines the standard used for the TDMA power      */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTDMAPowerStandard (ViSession instrSession,
                                              ViInt32 TDMAPowerStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (TDMAPowerStandard, 0, 1))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TDMASTD,%ld", TDMAPowerStandard);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get TDMA Power Standard                                         */
/* Purpose:  This function returns the standard used for the TDMA power      */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTDMAPowerStandard (ViSession instrSession,
                                              ViPInt32 TDMAPowerStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!TDMAPowerStandard)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TDMASTD", buffer));
    sscanf (buffer, "%ld", TDMAPowerStandard);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set TDMA Power Customized Standard                              */
/* Purpose:  This function selects the customized standard for the TDMA      */
/*           power measurement previously loaded with R&S FSHView software.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTDMAPowerCustomizedStandard (ViSession instrSession,
                                                        ViString TDMAStandard)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (TDMAStandard, 15))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TDMACSTD,%s", TDMAStandard);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get TDMA Power Customized Standard                              */
/* Purpose:  This function returns the customized standard for the TDMA      */
/*           power measurement previously loaded with R&S FSHView software.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTDMAPowerCustomizedStandard (ViSession instrSession,
                                                        ViString *TDMAStandard)
{
	ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!TDMAStandard)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TDMACSTD", buffer));
    sscanf (buffer, "%[^\r\n]", TDMAStandard);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set TDMA Power Measurement Time                                 */
/* Purpose:  This function defines the measurement time for the TDMA power   */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setTDMAPowerMeasurementTime (ViSession instrSession,
                                                     ViInt32  marker,
                                                     ViReal64 measurementTime)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Not range checked */

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    sprintf (buffer, "MEASTIME,%Lf", measurementTime);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        if (rsfsh_invalidViInt32Range (marker, 1, 6))
            return VI_ERROR_PARAMETER2;
        
        sprintf (buffer, "CALC:MARK%ld:FUNC:TDMA:BURS", marker);
        CHECKERR (rsfsh_setViReal64 (instrSession,
                                     buffer,
                                     measurementTime,
                                     31.7e-3, 19.0,
                                     VI_ERROR_PARAMETER3));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get TDMA Power Measurement Time                                 */
/* Purpose:  This function returns the measurement time for the TDMA power   */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTDMAPowerMeasurementTime (ViSession instrSession,
                                                     ViInt32   marker,
                                                     ViPReal64 measurementTime)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!measurementTime)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "MEASTIME", buffer));
    sscanf (buffer, "%Le", measurementTime);
    break;
    case FSH4:
        if (rsfsh_invalidViInt32Range (marker, 1, 6))
            return VI_ERROR_PARAMETER2;
        
        sprintf (buffer, "CALC:MARK%ld:FUNC:TDMA:BURS?", marker);
        CHECKERR (rsfsh_getViReal64 (instrSession,
                                     buffer,
                                     measurementTime,
                                     VI_ERROR_PARAMETER3));
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: TDMA Power Level Adjust                                         */
/* Purpose:  This function initiates a level adjustment for the TDMA power   */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_TDMAPowerLevelAdjust (ViSession instrSession)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    CHECKERR (rsfsh_cmdCommand (instrSession, "LVLADJUST"));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Receiver                                              */
/* Purpose:  This function configures receiver parameters such as Frequency, */
/*           Reference Level, Resolution Bandwidth, Trace Detector,          */
/*           Measurement Time, Channel Mode and Scan Mode.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureReceiver (ViSession instrSession,
                                           ViReal64 frequency,
                                           ViReal64 referenceLevel,
                                           ViInt32 resolutionBandwidth,
                                           ViInt32 traceDetector, /*valid values 2,4,5,6*/
                                           ViReal64 measurementTime,
                                           ViInt32 channelMode, 
                                           ViInt32 scanMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViReal64Range (frequency, 100.0e3, 6.0e9))
        return VI_ERROR_PARAMETER2;
    /* parameter 3 not range checked */
    if (rsfsh_invalidViInt32Range (resolutionBandwidth, 3, 9))
        return VI_ERROR_PARAMETER4;
    if (rsfsh_invalidViInt32Range (traceDetector, 2, 6) || (traceDetector == 3))
        return VI_ERROR_PARAMETER5;
    if (rsfsh_invalidViReal64Range (measurementTime, 1.0e-3, 100.0))
        return VI_ERROR_PARAMETER6;
    if (rsfsh_invalidViInt32Range (channelMode, 0, 1))
        return VI_ERROR_PARAMETER7;        
    if (rsfsh_invalidViInt32Range (scanMode, 0, 1))
        return VI_ERROR_PARAMETER8;
    
    CHECKERR (rsfsh_setReceiverFrequency (instrSession, frequency));
    CHECKERR (rsfsh_setReferenceLevel (instrSession, referenceLevel));
    CHECKERR (rsfsh_setResolutionBandwidth (instrSession, resolutionBandwidth));
    CHECKERR (rsfsh_setTraceDetector (instrSession, traceDetector));
    CHECKERR (rsfsh_setReceiverMeasurementTime (instrSession, measurementTime));
    CHECKERR (rsfsh_setReceiverChannelMode (instrSession, channelMode));
    CHECKERR (rsfsh_setReceiverScanMode (instrSession, scanMode));
                      
    return rsfsh_status;        
}

/*===========================================================================*/
/* Function: Set Receiver Frequency                                          */
/* Purpose:  This function sets receiver frequency.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverFrequency (ViStatus instrSession,
                                              ViReal64 frequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (frequency, 100.0e3, 6.0e9))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "FREQ,%Lf", frequency);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setViReal64 (instrSession,
                                      "SENS:FREQ:CENT",
                                      frequency,
                                      0.0,0.0,
                                      VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}
/*===========================================================================*/
/* Function: Get Receiver Frequency                                          */
/* Purpose:  This function returns receiver frequency.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverFrequency (ViSession instrSession,
                                              ViReal64 *frequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!frequency)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "FREQ", buffer));
    sscanf (buffer, "%Le", frequency);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getViReal64 (instrSession,
                                      "SENS:FREQ:CENT?",
                                      frequency,
                                      VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Receiver Channel Mode                                       */
/* Purpose:  This function switches between the channel mode and frequency   */
/*           mode. Use the command CHTABLE first to define a channel table   */
/*           for the channel mode.                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverChannelMode (ViSession instrSession,
                                                ViInt32 channelMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (channelMode, 0, 1))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CHMODE,%ld", channelMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setEnum (instrSession,
                          "SENS:FREQ:INP:MODE",
                          channelMode,
                          channelModeArr,
                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Receiver Channel Mode                                       */
/* Purpose:  This function queries the mode of receiver that can be either   */
/*           the channel mode or the frequency mode.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverChannelMode (ViSession instrSession,
                                                ViInt32 *channelMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CHMODE", buffer));
    sscanf (buffer, "%ld", channelMode);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getEnum (instrSession,
                          "SENS:FREQ:INP:MODE?",
                          channelMode,
                          channelModeArr,
                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Receiver Channel Number                                     */
/* Purpose:  This function defines the channel number. Use CHTABLE command   */
/*           to select a channel table first.                                */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverChannelNumber (ViSession instrSession,
                                                  ViInt32 channelNumber)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (channelNumber, 1, 9999))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CHANNEL,%ld", channelNumber);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setViInt32(instrSession,
                            "SENSe:CHANnel",
                            channelNumber,
                            1, 9999,
                            VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Receiver Channel Number                                     */
/* Purpose:  This function returns the channel number. Use CHTABLE command   */
/*           to select a channel table first.                                */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverChannelNumber (ViSession instrSession,
                                                  ViInt32 *channelNumber)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelNumber)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CHANNEL", buffer));
    sscanf (buffer, "%ld", channelNumber);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getViInt32(instrSession,
                            "SENSe:CHANnel?",
                            channelNumber,
                            VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Receiver Channel Table                                      */
/* Purpose:  This function selects a channel table for the channel mode.     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverChannelTable (ViSession instrSession,
                                                 ViString channelTable)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (channelTable, 15))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CHTABLE,%s", channelTable);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setViStringQuoted (instrSession,
                                    "SENS:CHAN:TABL:SEL",
                                    "'",
                                    channelTable,
                                    VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Receiver Channel Table                                      */
/* Purpose:  This function returns a channel table for the channel mode.     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverChannelTable (ViSession instrSession,
                                                 ViChar _VI_FAR channelTable[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelTable)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CHTABLE", buffer));
    sscanf (buffer, "%[^\r\n]", channelTable);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getViString (instrSession,
                                    "SENS:CHAN:TABL:SEL?",
                                    channelTable,
                                    BUFFER_SIZE,
                                    VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Receiver Coupled To Reference Channel State
/* Purpose:  This function couples the frequency of the noise channel in 
/*           Carrier / Noise measurements to the frequency of the reference 
/*           channel.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverCoupledToReferenceChannelState(
    ViSession   instrSession,
    ViBoolean   coupledToReferenceChannel
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (coupledToReferenceChannel))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "COUPLEDTOREF,%ld", coupledToReferenceChannel);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;     
}

/*===========================================================================*/
/* Function: Get Receiver Coupled To Reference Channel State
/* Purpose:  This function queries the couples the frequency of the noise 
/*           channel in Carrier / Noise measurements to the frequency of the 
/*           reference channel.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverCoupledToReferenceChannelState(
    ViSession   instrSession,
    ViBoolean*  coupledToReferenceChannel
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!coupledToReferenceChannel)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "COUPLEDTOREF", buffer));
    sscanf (buffer, "%hd", coupledToReferenceChannel);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Receiver Coupled To Reference Channel
/* Purpose:  This function couples the frequency of the noise channel in 
/*           Carrier / Noise measurements to the frequency of the reference 
/*           channel. The command sets the frequency offset of the noise 
/*           channel from the reference channel.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverCoupledToReferenceChannel(
    ViSession   instrSession,
    ViReal64    frequencyOffset
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    
    sprintf (buffer, "CTRFREQOFFS,%lf", frequencyOffset);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Receiver Coupled To Reference Channel
/* Purpose:  This function queries the frequency of the noise channel in 
/*           Carrier / Noise measurements to the frequency of the reference 
/*           channel.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverCoupledToReferenceChannel(
    ViSession   instrSession,
    ViReal64*   frequencyOffset
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!frequencyOffset)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CTRFREQOFFS", buffer));
    sscanf (buffer, "%lf", frequencyOffset);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Receiver Channel Signal Level                                   */
/* Purpose:  This function queries the signal level measured (fixed          */
/*           frequency/channel mode only).                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverSignalLevel (ViSession instrSession,
                                                ViReal64 *signalLevel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!signalLevel)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "LEVEL", buffer));
    sscanf (buffer, "%Le", signalLevel);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Receiver Measurement Time                                   */
/* Purpose:  This function defines the measurement time for the receiver     */
/*           mode.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverMeasurementTime (ViSession instrSession,
                                                    ViReal64 measurementTime)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (measurementTime, 1.0e-3, 100.0))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "MEASTIME,%Lf", measurementTime);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setViReal64 (instrSession, 
                              "SENS:SWE:TIME",
                              measurementTime,
                              1.0e-3, 100.0,
                              VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Receiver Measurement Time                                   */
/* Purpose:  This function returns the measurement time for the receiver     */
/*           mode.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverMeasurementTime (ViSession instrSession,
                                                    ViReal64 *measurementTime)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!measurementTime)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "MEASTIME", buffer));
    sscanf (buffer, "%Le", measurementTime);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getViReal64 (instrSession, 
                              "SENS:SWE:TIME?",
                              measurementTime,
                              VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Receiver Scan Mode                                          */
/* Purpose:  This function switches between frequency/channel scan mode and  */
/*           fixed frequency/channel mode. Use the command CHMODE to toggle  */
/*           between frequency and channel mode.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverScanMode (ViSession instrSession,
                                             ViInt32 scanMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (scanMode, 0, 1))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SCANMODE,%ld", scanMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setEnum (instrSession,
                          "SENS:FREQ:MODE",
                          scanMode,
                          scanModeArr,
                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Receiver Scan Mode                                          */
/* Purpose:  This function queries the frequency/channel scan mode and       */
/*           fixed frequency/channel mode. Use the command CHMODE to toggle  */
/*           between frequency and channel mode.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverScanMode (ViSession instrSession,
                                             ViInt32 *scanMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!scanMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "SCANMODE", buffer));
    sscanf (buffer, "%ld", scanMode);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getEnum (instrSession,
                          "SENS:FREQ:MODE?",
                          scanMode,
                          scanModeArr,
                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Receiver Freq Scan Start                                    */
/* Purpose:  This function defines the scan start frequency for the          */
/*           frequency scan mode.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverFreqScanStart (ViSession instrSession,
                                                  ViReal64 scanStartFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (scanStartFrequency, 100.0e3, 6.0e9))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SCANSTART,%Lf", scanStartFrequency);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setViReal64 (instrSession,
                                          "SENS:SCAN:STAR",
                                          scanStartFrequency,
                                          0.0, 0.0,
                                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Receiver Freq Scan Start                                    */
/* Purpose:  This function returns the scan start frequency for the          */
/*           frequency scan mode.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverFreqScanStart (ViSession instrSession,
                                                  ViReal64 *scanStartFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!scanStartFrequency)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "SCANSTART", buffer));
    sscanf (buffer, "%Le", scanStartFrequency);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getViReal64 (instrSession,
                                          "SENS:SCAN:STAR?",
                                          scanStartFrequency,
                                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Receiver Freq Scan Stop                                     */
/* Purpose:  This function defines the scan stop frequency for the frequency */
/*           scan mode.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverFreqScanStop (ViSession instrSession,
                                                 ViReal64 scanStopFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (scanStopFrequency, 100.0e3, 6.0e9))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SCANSTOP,%Lf", scanStopFrequency);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setViReal64 (instrSession,
                                          "SENS:SCAN:STOP",
                                          scanStopFrequency,
                                          0.0, 0.0,
                                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Receiver Freq Scan Stop                                     */
/* Purpose:  This function returns the scan stop frequency for the frequency */
/*           scan mode.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverFreqScanStop (ViSession instrSession,
                                                 ViReal64 *scanStopFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!scanStopFrequency)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "SCANSTOP", buffer));
    sscanf (buffer, "%Le", scanStopFrequency);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getViReal64 (instrSession,
                                          "SENS:SCAN:STOP?",
                                          scanStopFrequency,
                                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Receiver Freq Scan Step                                     */
/* Purpose:  This function defines the scan step frequency for the frequency */
/*           scan mode.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverFreqScanStep (ViSession instrSession,
                                                 ViReal64 scanStepFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (scanStepFrequency, 100.0e3, 6.0e9))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SCANSTEP,%Lf", scanStepFrequency);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setViReal64 (instrSession,
                                          "SENS:SCAN:STEP",
                                          scanStepFrequency,
                                          0.0, 0.0,
                                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Receiver Freq Scan Step                                     */
/* Purpose:  This function returns the scan step frequency for the frequency */
/*           scan mode.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverFreqScanStep (ViSession instrSession,
                                                 ViReal64 *scanStepFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!scanStepFrequency)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "SCANSTEP", buffer));
    sscanf (buffer, "%Le", scanStepFrequency);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getViReal64 (instrSession,
                                          "SENS:SCAN:STEP?",
                                          scanStepFrequency,
                                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Receiver Channel Scan Start                                 */
/* Purpose:  This function defines the start frequency for the channel scan  */
/*           mode.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverChannelScanStart (ViSession instrSession,
                                                     ViReal64 channelScanStartFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (channelScanStartFrequency, 100.0e3, 6.0e9))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "FREQSTART,%Lf", channelScanStartFrequency);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Receiver Channel Scan Start                                 */
/* Purpose:  This function returns the start frequency for the channel scan  */
/*           mode.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverChannelScanStart (ViSession instrSession,
                                                     ViReal64 *channelScanStartFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelScanStartFrequency)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "FREQSTART", buffer));
    sscanf (buffer, "%Le", channelScanStartFrequency);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Receiver Channel Scan Stop                                  */
/* Purpose:  This function defines the stop frequency for the channel scan   */
/*           mode.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReceiverChannelScanStop (ViSession instrSession,
                                                    ViReal64 channelScanStopFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (channelScanStopFrequency, 100.0e3, 6.0e9))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "FREQSTOP,%Lf", channelScanStopFrequency);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Receiver Channel Scan Stop                                  */
/* Purpose:  This function returns the stop frequency for the channel scan   */
/*           mode.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReceiverChannelScanStop (ViSession instrSession,
                                                    ViReal64 *channelScanStopFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelScanStopFrequency)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "FREQSTOP", buffer));
    sscanf (buffer, "%Le", channelScanStopFrequency);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Carrier Noise                                         */
/* Purpose:  This function configures carrier noise parameters such as       */
/*           channel bandwidth, units, norm, frequency, measurement mode.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureCarrierNoise (ViSession instrSession,
                                               ViReal64 channelBandwidth,
                                               ViInt32 measurementMode,
                                               ViReal64 centerFrequency,
                                               ViBoolean norm,
                                               ViBoolean cNNoiseMeasurementActive,
                                               ViInt32 unit)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViReal64Range (channelBandwidth, 8.33e3, 8.0e6))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (measurementMode, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViReal64Range (centerFrequency, 4.8e6, 3.0e9))
        return VI_ERROR_PARAMETER4;
    if (rsfsh_invalidViBooleanRange (norm))
        return VI_ERROR_PARAMETER5;
    if (rsfsh_invalidViBooleanRange (cNNoiseMeasurementActive))
        return VI_ERROR_PARAMETER6;        
    if (rsfsh_invalidViInt32Range (unit, 0, 2))
        return VI_ERROR_PARAMETER7;
    
    CHECKERR (rsfsh_setCarrierNoiseChannelBandwidth (instrSession, channelBandwidth));
    CHECKERR (rsfsh_setCarrierNoiseMeasurementMode (instrSession, measurementMode));
    CHECKERR (rsfsh_setCenterFrequency (instrSession, centerFrequency));
    CHECKERR (rsfsh_setCarrierNoiseNorm (instrSession, norm));
    CHECKERR (rsfsh_setCarrierNoiseMode (instrSession, cNNoiseMeasurementActive));
    
    if(!cNNoiseMeasurementActive)
        CHECKERR (rsfsh_setCarrierNoiseUnit (instrSession, unit));
    
    return rsfsh_status;     
}

/*===========================================================================*/
/* Function: Set Carrier Noise Channel Bandwidth                             */
/* Purpose:  This function defines the bandwidth used for carrier noise      */
/*           measurements.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseChannelBandwidth (ViSession instrSession,
                                                         ViReal64 channelBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (channelBandwidth, 8.33e3, 8.0e6))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNCHBW,%Lf", channelBandwidth);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Carrier Noise Channel Bandwidth                             */
/* Purpose:  This function returns the bandwidth used for carrier noise      */
/*           measurements.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseChannelBandwidth (ViSession instrSession,
                                                         ViReal64 *channelBandwidth)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!channelBandwidth)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNCHBW", buffer));
    sscanf (buffer, "%Le", channelBandwidth);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Noise Customized Standard
/* Purpose:  This function selects the customized standard for the carrier 
/*           noise measurement previously loaded with R&S FSHView software.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseCustomizedStandard(
    ViSession   instrSession,
    ViString    customizedStandard
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (customizedStandard, 15))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNCSTD,%s", customizedStandard);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Noise Manual Ref Power Active                       */
/* Purpose:  This function defines if the manual ref power is active.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseManualRefPowerActive (ViSession instrSession,
                                                             ViBoolean manualRefPowerActive)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (manualRefPowerActive))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNMANREFPWR,%ld", manualRefPowerActive);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;     
}

/*===========================================================================*/
/* Function: Get Carrier Noise Manual Ref Power Active                       */
/* Purpose:  This function returns if the manual ref power is active.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseManualRefPowerActive (ViSession instrSession,
                                                             ViBoolean *manualRefPowerActive)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!manualRefPowerActive)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNMANREFPWR", buffer));
    sscanf (buffer, "%hd", manualRefPowerActive);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Noise Measurement Mode                              */
/* Purpose:  This function defines the measurement mode used for the carrier */
/*           noise measurement.                                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseMeasurementMode (ViSession instrSession,
                                                        ViInt32 measurementMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (measurementMode, 0, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNMEASMODE,%ld", measurementMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Carrier Noise Measurement Mode                              */
/* Purpose:  This function returns the measurement mode used for the carrier */
/*           noise measurement.                                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseMeasurementMode (ViSession instrSession,
                                                        ViInt32 *measurementMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!measurementMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNMEASMODE", buffer));
    sscanf (buffer, "%ld", measurementMode);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}
 
/*===========================================================================*/
/* Function: Set Carrier Noise Mode                                          */
/* Purpose:  This function defines if the C/N noise measurement is active.   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseMode (ViSession instrSession,
                                             ViBoolean cNNoiseMeasurementActive)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (cNNoiseMeasurementActive))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNMODE,%ld", cNNoiseMeasurementActive);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Carrier Noise Mode                                          */
/* Purpose:  This function returns if the C/N noise measurement is active.   */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseMode (ViSession instrSession,
                                             ViBoolean *cNNoiseMeasurementActive)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!cNNoiseMeasurementActive)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNMODE", buffer));
    sscanf (buffer, "%hd", cNNoiseMeasurementActive);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Noise Noise Correction
/* Purpose:  This function defines if the C/N noise correction mode.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseNoiseCorrection(
    ViSession   instrSession,
    ViBoolean   cNNoiseCorrection
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (cNNoiseCorrection))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNNOISECOR,%ld", cNNoiseCorrection);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Carrier Noise Noise Correction
/* Purpose:  This function returns if the C/N noise correction mode is 
/*           active.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseNoiseCorrection(
    ViSession   instrSession,
    ViBoolean*  cNNoiseCorrection
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!cNNoiseCorrection)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNNOISECOR", buffer));
    sscanf (buffer, "%hd", cNNoiseCorrection);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Noise Norm                                          */
/* Purpose:  This function defines the norm used for the carrier noise       */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseNorm (ViSession instrSession,
                                             ViBoolean norm)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (norm))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNNORM,%ld", norm);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Carrier Noise Norm                                          */
/* Purpose:  This function returns the norm used for the carrier noise       */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseNorm (ViSession instrSession,
                                             ViBoolean *norm)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!norm)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNNORM", buffer));
    sscanf (buffer, "%hd", norm);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Noise Pilot Frequency                               */
/* Purpose:  This function defines the pilot frequency used for digital TV   */
/*           carrier noise measurements.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoisePilotFrequency (ViSession instrSession,
                                                       ViReal64 pilotFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (pilotFrequency, 2.2e6, 2.99e9)) 
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNPILOTFRQ,%Lf", pilotFrequency);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Carrier Noise Pilot Frequency                               */
/* Purpose:  This function returns the pilot frequency used for digital TV   */
/*           carrier noise measurements.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoisePilotFrequency (ViSession instrSession,
                                                       ViReal64 *pilotFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!pilotFrequency)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNPILOTFRQ", buffer));
    sscanf (buffer, "%Le", pilotFrequency);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;     
}

/*===========================================================================*/
/* Function: Set Carrier Noise Power Display                                 */
/* Purpose:  This function turns the C/N power display on or off.            */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoisePowerDisplay (ViSession instrSession,
                                                     ViBoolean powerDisplay)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (powerDisplay))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNPWRDISP,%ld", powerDisplay);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;     
}

/*===========================================================================*/
/* Function: Get Carrier Noise Power Display                                 */
/* Purpose:  This function returns the C/N power display state.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoisePowerDisplay (ViSession instrSession,
                                                     ViBoolean *powerDisplay)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!powerDisplay)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNPWRDISP", buffer));
    sscanf (buffer, "%hd", powerDisplay);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Noise Ratio Channel Bandwidth
/* Purpose:  This function sets the ratio channel bandwidth used for carrier 
/*           noise measurements.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseRatioChannelBandwidth(
    ViSession   instrSession,
    ViReal64    ratioChannelBandwidth
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    
    sprintf (buffer, "CNRATIOCHBW,%Lf", ratioChannelBandwidth);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Carrier Noise Ratio Channel Bandwidth
/* Purpose:  This function queries the ratio channel bandwidth used for 
/*           carrier noise measurements.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseRatioChannelBandwidth(
    ViSession   instrSession,
    ViReal64*   ratioChannelBandwidth
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!ratioChannelBandwidth)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNRATIOCHBW", buffer));
    sscanf (buffer, "%Le", ratioChannelBandwidth);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Noise Reference Measurement Mode
/* Purpose:  This function sets the reference measurement mode.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseReferenceMeasurementMode(
    ViSession   instrSession,
    ViInt32 referenceMeasurementMode
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (referenceMeasurementMode, 0, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNREFMEASMODE,%ld", referenceMeasurementMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Carrier Noise Reference Measurement Mode
/* Purpose:  This function queries the reference measurement mode.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseReferenceMeasurementMode(
    ViSession   instrSession,
    ViInt32*    referenceMeasurementMode
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!referenceMeasurementMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNREFMEASMODE", buffer));
    sscanf (buffer, "%ld", referenceMeasurementMode);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Noise Ref Power                                     */
/* Purpose:  This function defines the reference power used for carrier      */
/*           noise measurements.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseRefPower (ViSession instrSession,
                                                 ViReal64 refPower)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Parameter 2 not range checked */
    
    switch (instrPtr -> fsh3)
    {
    case FSH3:
    sprintf (buffer, "CNREFPWR,%Lf", refPower);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;     
}

/*===========================================================================*/
/* Function: Get Carrier Noise Ref Power                                     */
/* Purpose:  This function returns the reference power used for carrier      */
/*           noise measurements.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseRefPower (ViSession instrSession,
                                                 ViReal64 *refPower)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!refPower)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNREFPWR", buffer));
    sscanf (buffer, "%Le", refPower);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Noise Unit                                          */
/* Purpose:  This function defines the unit used for the carrier noise       */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseUnit (ViSession instrSession,
                                             ViInt32 unit)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (unit, 0, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNUNIT,%ld", unit);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Carrier Noise Unit                                          */
/* Purpose:  This function returns the unit used for the carrier noise       */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseUnit (ViSession instrSession,
                                             ViInt32 *unit)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!unit)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNUNIT", buffer));
    sscanf (buffer, "%ld", unit);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Noise Unit                                          */
/* Purpose:  This function defines the vision frequency used for analog TV   */
/*           carrier noise measurements.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCarrierNoiseVisionFrequency (ViSession instrSession,
                                                        ViReal64 visionFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (visionFrequency, 2.05e6, 2.99e9)) 
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CNVISIONFRQ,%Lf", visionFrequency);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;     
}

/*===========================================================================*/
/* Function: Get Carrier Noise Unit                                          */
/* Purpose:  This function returns the vision frequency used for analog TV   */
/*           carrier noise measurements.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseVisionFrequency (ViSession instrSession,
                                                        ViReal64 *visionFrequency)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!visionFrequency)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNVISIONFRQ", buffer));
    sscanf (buffer, "%Le", visionFrequency);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Carrier Noise Measurement Value                             */
/* Purpose:  This function queries the measured C/N value. The value depends */
/*           on the CNMODE selected (either C/N reference measurement or C/N */
/*           Noise measurement).                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCarrierNoiseMeasurementValue (ViSession instrSession,
                                                         ViReal64 *measurementValue)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!measurementValue)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CNVALUE", buffer));
    sscanf (buffer, "%Le", measurementValue);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Carrier Noise Unit                                          */
/* Purpose:  This function initiates a level adjustment for the channel      */
/*           power measurement.                                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_LevelAdjustment (ViSession instrSession)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    CHECKERR (rsfsh_cmdCommand (instrSession, "LVLADJUST"));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Configure Group Delay
/* Purpose:  This function configures the group delay.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureGroupDelay(
    ViSession   instrSession,
    ViInt32 apertureSteps,
    ViInt32 referenceLevel,
    ViInt32 referencePosition,
    ViReal64    yScale
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViInt32Range (apertureSteps, 1, 630))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (referenceLevel, 1, 10000))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViInt32Range (referencePosition, 1, 10))
        return VI_ERROR_PARAMETER4;
    
    CHECKERR (rsfsh_setGroupDelayApertureSteps (instrSession, apertureSteps));
    CHECKERR (rsfsh_setGroupDelayReferenceLevel (instrSession, referenceLevel));
    CHECKERR (rsfsh_setGroupDelayReferencePosition (instrSession, referencePosition));
    CHECKERR (rsfsh_setGroupDelayYScale (instrSession, yScale));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Group Delay Aperture Steps
/* Purpose:  This function sets the aperture steps for Group Delay 
/*           measurements.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setGroupDelayApertureSteps(
    ViSession   instrSession,
    ViInt32 apertureSteps
)
{
    return rsfsh_setViInt32 (instrSession, 
                             "DISP:GDEL:APER:STEP",
                             apertureSteps,
                             1, 630,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Group Delay Aperture Steps
/* Purpose:  This function queries the aperture steps for Group Delay 
/*           measurements.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGroupDelayApertureSteps(
    ViSession   instrSession,
    ViInt32*    apertureSteps
)
{
    return rsfsh_getViInt32 (instrSession, 
                             "DISP:GDEL:APER:STEP?",
                             apertureSteps,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Group Delay Reference Level
/* Purpose:  This function sets the reference level for the Cable Loss 
/*           measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setGroupDelayReferenceLevel(
    ViSession   instrSession,
    ViInt32 referenceLevel
)
{
    return rsfsh_setViInt32 (instrSession, 
                             "DISP:GDEL:REF",
                             referenceLevel,
                             1, 10000,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Group Delay Reference Level
/* Purpose:  This function queries the reference level for the Cable Loss 
/*           measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGroupDelayReferenceLevel(
    ViSession   instrSession,
    ViInt32*    referenceLevel
)
{
    return rsfsh_getViInt32 (instrSession, 
                             "DISP:GDEL:REF?",
                             referenceLevel,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Group Delay Reference Position
/* Purpose:  This function sets the position of the reference value on the 
/*           display for the Group Delay measurement format. Each step shifts 
/*           the reference position one grid line up or down.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setGroupDelayReferencePosition(
    ViSession   instrSession,
    ViInt32 referencePosition
)
{
    return rsfsh_setViInt32 (instrSession, 
                             "DISP:GDEL:REF:POS",
                             referencePosition,
                             1, 10,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Group Delay Reference Position
/* Purpose:  This function queries the position of the reference value on 
/*           the display for the Group Delay measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGroupDelayReferencePosition(
    ViSession   instrSession,
    ViInt32*    referencePosition
)
{
    return rsfsh_getViInt32 (instrSession, 
                             "DISP:GDEL:REF:POS?",
                             referencePosition,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Group Delay Y Scale
/* Purpose:  This function sets the display range of the y-axis for the 
/*           Group Delay measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setGroupDelayYScale(
    ViSession   instrSession,
    ViReal64    yScale
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "DISP:GDEL:Y:SCAL",
                              yScale,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Group Delay Y Scale
/* Purpose:  This function queries the display range of the y-axis for the 
/*           Group Delay measurement format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGroupDelayYScale(
    ViSession   instrSession,
    ViReal64*   yScale
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "DISP:GDEL:Y:SCAL?",
                              yScale,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Vector Voltmeter Reference State
/* Purpose:  This function sets the current measurement values as reference
/*           values. The reference values can be used for comparison
/*           measurements.
/*           To disable the vector voltage reference with FSH3 use
/*           VVREF.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setVectorVoltmeterReferenceState(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
        if (rsfsh_invalidViBooleanRange (state))
            return VI_ERROR_PARAMETER2;

        switch (state)
        {
            case VI_TRUE:
                CHECKERR (rsfsh_cmdCommand (instrSession, "VVTOREF"));
                break;
            case VI_FALSE:  
                CHECKERR (rsfsh_setCommand (instrSession, "VVREF,0"));
                break;
        }
        
    break;
    case FSH4:
        rsfsh_status = rsfsh_setViBoolean (instrSession, 
                                           "CALC:VVM:REF:STAT",
                                           state,
                                           VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get Vector Voltmeter Reference Values
/* Purpose:  This function queries the vector voltage reference values.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVectorVoltmeterReferenceValues(
    ViSession   instrSession,
    ViReal64*   magnitude,
    ViReal64*   phase
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *pbuff=VI_NULL;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!magnitude)
        return VI_ERROR_PARAMETER2;
    if (!phase)
        return VI_ERROR_PARAMETER3;

    CHECKERR (rsfsh_getCommand (instrSession, "VVREF", buffer));
    
    pbuff = strtok (buffer, ",");
    sscanf (pbuff, "%Le", magnitude);
    pbuff = strtok (NULL, ",");
    sscanf (pbuff, "%Le", phase);
    
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}


/*===========================================================================*/
/* Function: Get Vector Voltmeter Magnitude Reference Values
/* Purpose:  This function queries the reference values for the return loss 
/*           of the DUT in vector voltmeter mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVectorVoltmeterMagnitudeReferenceValues(
    ViSession   instrSession,
    ViReal64*   referenceValue
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "CALC:VVM:MAGN:REF?",
                              referenceValue,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Vector Voltmeter Magnitude Values
/* Purpose:  This function queries the numerical values for the return loss 
/*           of the DUT in vector voltmeter mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVectorVoltmeterMagnitudeValues(
    ViSession   instrSession,
    ViReal64*   value
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "CALC:VVM:MAGN:RES?",
                              value,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Vector Voltmeter Phase Reference Values
/* Purpose:  This function queries the reference values for the phase of the 
/*           DUT in vector voltmeter mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVectorVoltmeterPhaseReferenceValues(
    ViSession   instrSession,
    ViReal64*   referenceValue
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "CALC:VVM:PHAS:REF?",
                              referenceValue,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Vector Voltmeter Phase Values
/* Purpose:  This function queries the numerical values for the phase of the 
/*           DUT in vector voltmeter mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVectorVoltmeterPhaseValues(
    ViSession   instrSession,
    ViReal64*   value
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "CALC:VVM:PHAS:RES?",
                              value,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Vector Voltmeter Voltage
/* Purpose:  This function queries the measured vector voltage values.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getVectorVoltmeterVoltage(
    ViSession   instrSession,
    ViReal64*   magnitude,
    ViReal64*   phase
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *pbuff=VI_NULL;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!magnitude)
        return VI_ERROR_PARAMETER2;
    if (!phase)
        return VI_ERROR_PARAMETER3;

    CHECKERR (rsfsh_getCommand (instrSession, "VECTVOLT", buffer));
    
    pbuff = strtok (buffer, ",");
    sscanf (pbuff, "%Le", magnitude);
    pbuff = strtok (NULL, ",");
    sscanf (pbuff, "%Le", phase);
    
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Vector Voltage Reference
/* Purpose:  This function sets the state of the vector voltage reference.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setVectorVoltageReference(
    ViSession   instrSession,
    ViBoolean   vectorVoltageReference
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    
    sprintf (buffer, "VVREF,%ld", vectorVoltageReference);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Power State
/* Purpose:  This function activates and deactivates the selected power 
/*           measurement.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerState(
    ViSession   instrSession,
    ViBoolean   state
)
{
    return rsfsh_setViBoolean (instrSession, 
                               "CALC:MARK:FUNC:POW:STAT",
                               state,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Load Power Standard
/* Purpose:  This function loads and initializes a predefined Standard from 
/*           the specified file, and switches to the corresponding 
/*           measurement mode. A Standard comprises of parameters specifying
/*           default values for settings related to power measurements. 
/*           Parameters such as channel bandwidth, resolution bandwidth, 
/*           detector type, and sweep time are examples of parameters
/*           stored in a Standard.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_loadPowerStandard(
    ViSession   instrSession,
    ViString    standard
)
{
    return rsfsh_setViStringQuoted (instrSession, 
                                    "CALC:MARK:FUNC:POW:PRES",
                                    "'",
                                    standard,
                                    VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Power Standard Check
/* Purpose:  This function whether the current instrument state is still in 
/*           compliance with the Standard currently in use. Modifying 
/*           instrument settings that are not included in a Standard do
/*           not break Standard compliance; modifying settings that are 
/*           included in a Standard, will break compliance.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_powerStandardCheck(
    ViSession   instrSession,
    ViInt32*    check
)
{
    
    return rsfsh_getViInt32 (instrSession, 
                             "CALC:MARK:FUNC:POW:PRES:CHEC?",
                             check,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Power Standard Result
/* Purpose:  This function queries the result of the performed power 
/*           measurement. If necessary, the measurement is switched on prior 
/*           to the query.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_powerStandardResult(
    ViSession   instrSession,
    ViInt32     parameter,
    ViReal64*   result
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (parameter, 0, 1))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK:FUNC:POW:RES? %s", powerParameterArr[parameter]);
    
    return rsfsh_getViReal64 (instrSession, 
                              buffer,
                              result,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Power Measurement Type
/* Purpose:  This function sets the power measurement setting for a standard 
/*           and previously switches on the corresponding measurement, if 
/*           required.
/*           The configuration for a standard comprises of the parameters 
/*           weighting filter, channel bandwidth and spacing, resolution and 
/*           video bandwidth, as well as detector and sweep time.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPowerMeasurementType(
    ViSession   instrSession,
    ViInt32 type
)
{
    return rsfsh_setEnum (instrSession, 
                          "CALC:MARK:FUNC:POW:SEL",
                          type,
                          powerSelectTypeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Power Measurement Type
/* Purpose:  This function queries the power measurement setting for a 
/*           standard and previously switches on the corresponding 
/*           measurement, if required.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerMeasurementType(
    ViSession   instrSession,
    ViInt32*    type
)
{
    return rsfsh_getEnum (instrSession, 
                          "CALC:MARK:FUNC:POW:SEL?",
                          type,
                          powerSelectTypeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Spacing
/* Purpose:  This function sets the spacing between the carrier and its
/*           adjacent channel.
/*           A change of the adjacent channel spacing causes a change in the
/*           spacing of all alternate channels above the adjacent channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPAdjacentSpacing (ViSession instrSession,
                                       ViInt32 spacing)
{
    return rsfsh_setViInt32(instrSession,
                            "SENSe:POWer:ACHannel:SPACing:ACHannel",
                            spacing,
                            100,
                            2000000000,
                            VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Spacing
/* Purpose:  This function returns the spacing between the carrier and its
/*           adjacent channel.
/*           A change of the adjacent channel spacing causes a change in the
/*           spacing of all alternate channels above the adjacent channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPAdjacentSpacing (ViSession instrSession,
                                       ViInt32 *spacing)
{
    return rsfsh_getViInt32(instrSession,
                            "SENSe:POWer:ACHannel:SPACing:ACHannel?",
                            spacing,
                            VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Alternate Spacing
/* Purpose:  This function sets the spacing between transmission channel and
/*           the alternate channels. A change of the alternate channel
/*           spacing causes a change in the spacing of all alternate channels
/*           above the modifed alternate channel. Alternate channels below
/*           the modified remain the same.
/*           The numeric suffix <1...11> at ALTernate selects the alternate
/*           channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPAlternateSpacing (ViSession instrSession,
                                                ViInt32 spacing,
                                                ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 11))
        return VI_ERROR_PARAMETER3;
    
    sprintf (buffer, "SENSe:POWer:ACHannel:SPACing:ALTernate%ld", alternateChannel);
    
    return rsfsh_setViInt32 (instrSession,
                          buffer,
                          spacing,
                          100,
                          2000000000,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Alternate Spacing
/* Purpose:  This function returns the spacing between transmission channel
/*           and the alternate channels. A change of the alternate channel
/*           spacing causes a change in the spacing of all alternate channels
/*           above the modifed alternate channel. Alternate channels below
/*           the modified remain the same.
/*           
/*           The numeric suffix <1...11> at ALTernate selects the alternate
/*           channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPAlternateSpacing (ViSession instrSession,
                                                ViInt32 *spacing,
                                                ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 11))
        return VI_ERROR_PARAMETER3;
    
    sprintf (buffer, "SENSe:POWer:ACHannel:SPACing:ALTernate%ld?", alternateChannel);
    
    return rsfsh_getViInt32 (instrSession,
                          buffer,
                          spacing,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Alternate Channel Spacing
/* Purpose:  This function defines the channel spacing of the transmission
/*           channels.
/*           The numeric suffix <1...11> at ALTernate selects the alternate
/*           channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPAlternateChannelSpacing (ViSession instrSession,
                                                       ViInt32 spacing,
                                                       ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 11))
        return VI_ERROR_PARAMETER3;
    
    sprintf (buffer, "SENSe:POWer:ACHannel:SPACing:CHANnel%ld", alternateChannel);
    
    return rsfsh_setViInt32 (instrSession,
                          buffer,
                          spacing,
                          100,
                          2000000000,
                          VI_ERROR_PARAMETER2);
    
}

/*===========================================================================*/
/* Function: Get ACP Alternate Channel Spacing 
/* Purpose:  This function returns the channel spacing of the transmission
/*           channels.
/*           The numeric suffix <1...11> at ALTernate selects the alternate
/*           channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPAlternateChannelSpacing (ViSession instrSession,
                                                       ViInt32 *spacing,
                                                       ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 11))
        return VI_ERROR_PARAMETER3;
    
    sprintf (buffer, "SENSe:POWer:ACHannel:SPACing:CHANnel%ld?", alternateChannel);
    
    return rsfsh_getViInt32 (instrSession,
                          buffer,
                          spacing,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Configure ACP Channels
/* Purpose:  This function sets the number of transmission channels and
/*           adjacent channels pair to the left and to the right of the Tx
/*           channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureACPChannels (ViSession instrSession,
                                              ViInt32 transmissionChannels,
                                              ViInt32 adjacentChannels)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    CHECKERR (rsfsh_setACPTransmissionChannel(instrSession,
                                                transmissionChannels));
    CHECKERR (rsfsh_setACPChannelsPair( instrSession,
                                                adjacentChannels));
    return  rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure ACP Channel Bandwidth
/* Purpose:  This function sets the channel, adjacent and alternate channel
/*           bandwidth of the radio communication system.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureACPChannelBandwidth (ViSession instrSession,
                                                      ViReal64 channelBandwidth,
                                                      ViInt32 adjacentChannelBandwidth,
                                                      ViInt32 alternateChannelBandwidth,
                                                      ViInt32 alternateChannel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    CHECKERR ( rsfsh_setACPChannelBandwidth(instrSession,channelBandwidth));
    CHECKERR ( rsfsh_setACPAdjacentChannelBandwidth(instrSession,adjacentChannelBandwidth));
    CHECKERR ( rsfsh_setACPAlternateChannelBandwidth(instrSession,alternateChannelBandwidth, alternateChannel));
    return  rsfsh_status;
    
}

/*===========================================================================*/
/* Function: Configure ACP Spacing
/* Purpose:  This function sets the ACP spacing.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureACPSpacing (ViSession instrSession,
                                             ViInt32 TXSpacing,
                                             ViInt32 TXChannel,
                                             ViInt32 adjacentSpacing,
                                             ViInt32 alternateSpacing,
                                             ViInt32 alternateChannel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    CHECKERR ( rsfsh_setACPAlternateChannelSpacing(instrSession,TXSpacing,TXChannel));
    CHECKERR ( rsfsh_setACPAdjacentSpacing(instrSession,adjacentSpacing));
    CHECKERR ( rsfsh_setACPAlternateSpacing(instrSession,alternateSpacing, alternateChannel));
    
    return  rsfsh_status;
}

/*===========================================================================*/
/* Function: Set ACP Unit
/* Purpose:  This function defines the unit used for the adjacent channel
/*           power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPUnit (ViSession instrSession,
                                    ViInt32 adjacentChannelPowerUnit)
{
    return rsfsh_setEnum (instrSession,
                          "CALCulate:MARKer:FUNCtion:ACPower:UNIT",
                          adjacentChannelPowerUnit,
                          acpUnitsArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Unit
/* Purpose:  This function returns the unit used for the adjacent channel
/*           power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPUnit (ViSession instrSession,
                                    ViInt32 *adjacentChannelPowerUnit)
{
    return rsfsh_getEnum (instrSession,
                          "CALCulate:MARKer:FUNCtion:ACPower:UNIT?",
                          adjacentChannelPowerUnit,
                          acpUnitsArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Mode
/* Purpose:  This function selects either absolute or relative adjacent
/*           channel measurements. The command is available if span > 0 and
/*           if the number of adjacent channel is greater than 0.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPMode (ViSession instrSession, ViInt32 mode)
{
    return rsfsh_setEnum (instrSession,
                          "SENSe:POWer:ACHannel:MODE",
                          mode,
                          acpModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Mode
/* Purpose:  This function returns either absolute or relative adjacent
/*           channel measurements. The command is available if span > 0 and
/*           if the number of adjacent channel is greater than 0.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPMode (ViSession instrSession, ViInt32 *mode)
{
    return rsfsh_getEnum (instrSession,
                          "SENSe:POWer:ACHannel:MODE?",
                          mode,
                          acpModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Reference Level
/* Purpose:  This function adjust the reference level to that of the
/*           currently measured channel power. It also initiates an adjacent
/*           channel power measurement.
/*           This ensures that the signal path of the instrument is not
/*           overloaded. Since the measurement bandwidth is significantly
/*           smaller than the signal bandwidth in channel power measurements,
/*           the signal path can be overloaded although the trace is still
/*           significantly below the reference level. If the measured channel
/*           power equals the reference level, the signal path is not
/*           overloaded.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPReferenceLevel (ViSession instrSession)
{
    return rsfsh_sendCmd(instrSession,"SENSe:POWer:ACHannel:PRESet:RLEVel");
}

/*===========================================================================*/
/* Function: Set ACP Channels Pair
/* Purpose:  This function sets the number of adjacent channels pair to the
/*           left and to the right of the Tx channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPChannelsPair (ViSession instrSession,
                                            ViInt32 channels)
{
    return rsfsh_setViInt32(instrSession,
                            "SENSe:POWer:ACHannel:ACPairs",
                            channels,
                            1,
                            12,
                            VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Channels Pair
/* Purpose:  This function returns the number of adjacent channels pair to
/*           the left and to the right of the Tx channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPChannelsPair (ViSession instrSession,
                                            ViInt32 *channels)
{
    return rsfsh_getViInt32(instrSession,
                            "SENSe:POWer:ACHannel:ACPairs?",
                            channels,
                            VI_ERROR_PARAMETER2
                            );
}

/*===========================================================================*/
/* Function: Set ACP Channel Bandwidth
/* Purpose:  This function sets the channel bandwidth of the radio
/*           communication system. The bandwidths of adjacent channels are
/*           not influenced by this command.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPChannelBandwidth (ViSession instrSession,
                                                ViReal64 bandwidth)
{
    return rsfsh_setViReal64(instrSession,
                            "SENSe:POWer:ACHannel:BANDwidth:CHANnel",
                            bandwidth,
                            100,
                            1000e+6,
                            VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Channel Bandwidth
/* Purpose:  This function returns the channel bandwidth of the radio
/*           communication system. The bandwidths of adjacent channels are
/*           not influenced by this command.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPChannelBandwidth (ViSession instrSession,
                                                ViReal64 *bandwidth)
{
    return rsfsh_getViReal64(instrSession,
                            "SENSe:POWer:ACHannel:BANDwidth:CHANnel?",
                            bandwidth,
                            VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Adjacent Channel Bandwidth
/* Purpose:  This function sets the channel bandwidth of the adjacent channel
/*           in the radio communication system. If you change the bandwidth
/*           of the adjacent channel, the bandwidth of all other alternate
/*           channels is adjusted to the same value.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPAdjacentChannelBandwidth (ViSession instrSession,
                                                        ViInt32 bandwidth)
{
    return rsfsh_setViInt32(instrSession,
                            "SENSe:POWer:ACHannel:BANDwidth:ACHannel",
                            bandwidth,
                            100,
                            1000000000,
                            VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Adjacent Channel Bandwidth
/* Purpose:  This function returns the channel bandwidth of the adjacent channel
/*           in the radio communication system. If you change the bandwidth
/*           of the adjacent channel, the bandwidth of all other alternate
/*           channels is adjusted to the same value.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPAdjacentChannelBandwidth (ViSession instrSession,
                                                        ViInt32 *bandwidth)
{
    return rsfsh_getViInt32(instrSession,
                            "SENSe:POWer:ACHannel:BANDwidth:ACHannel?",
                            bandwidth,
                            VI_ERROR_PARAMETER2);
    
}

/*===========================================================================*/
/* Function: Set ACP Alternate Channel Bandwidth
/* Purpose:  This function sets the bandwidth of the alternate channels in
/*           the radio communication system.
/*           If you change the bandwidth of the first alternate channel, the
/*           bandwidth of all other alternate channels is adjusted to the
/*           same value.
/*           
/*           The suffix <1...11> selects the number of the alternate channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPAlternateChannelBandwidth (ViSession instrSession,
                                                         ViInt32 bandwidth,
                                                         ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 11))
        return VI_ERROR_PARAMETER3;
    
    sprintf (buffer, "SENSe:POWer:ACHannel:BANDwidth:ALTernate%ld", alternateChannel);
    
    return rsfsh_setViInt32 (instrSession,
                          buffer,
                          bandwidth,
                          1,
                          1000000000,
                          VI_ERROR_PARAMETER2);
    
}

/*===========================================================================*/
/* Function: Get ACP Alternate Channel Bandwidth
/* Purpose:  This function returns the bandwidth of the alternate channels in
/*           the radio communication system.
/*           If you change the bandwidth of the first alternate channel, the
/*           bandwidth of all other alternate channels is adjusted to the
/*           same value.
/*           
/*           The suffix <1...11> selects the number of the alternate channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPAlternateChannelBandwidth (ViSession instrSession,
                                                         ViInt32 *bandwidth,
                                                         ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 11))
        return VI_ERROR_PARAMETER3;
    
    sprintf (buffer, "SENSe:POWer:ACHannel:BANDwidth:ALTernate%ld?", alternateChannel);
    
    return rsfsh_getViInt32 (instrSession,
                          buffer,
                          bandwidth,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Automatic Reference Channel Selection
/* Purpose:  This function activates the automatic selection of a
/*           transmission channel to be used as a reference channel in
/*           relative ACP measurements.
/*           You can define the Tx channel with the highest power, the lowest
/*           power, or the Tx channel nearest to the adjacent channels as the
/*           reference channel.
/*           The command is available only for multi-carrier channel and
/*           adjacent-channel power measurements with span > 0
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPAutomaticReferenceChannelSelection (ViSession instrSession,
                                                                  ViInt32 mode)
{
    return rsfsh_setEnum (instrSession,
                          "SENSe:POWer:ACHannel:REFerence:TXCHannel:AUTO",
                          mode,
                          acpAutoRefModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Automatic Reference Channel Selection
/* Purpose:  This function returns the automatic selection of a transmission
/*           channel to be used as a reference channel in relative ACP
/*           measurements.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPAutomaticReferenceChannelSelection (ViSession instrSession,
                                                                  ViInt32 *mode)
{
    return rsfsh_getEnum (instrSession,
                          "SENSe:POWer:ACHannel:REFerence:TXCHannel:AUTO?",
                          mode,
                          acpAutoRefModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Transmission Channel
/* Purpose:  This function sets the number of transmission channels.
/*           
/*           The function is available for measurements with span > 0.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPTransmissionChannel (ViSession instrSession,
                                                   ViInt32 transmissionChannel)
{
    return rsfsh_setViInt32(instrSession,
                            "SENSe:POWer:ACHannel:TXCHannel:COUNt",
                            transmissionChannel,
                            1,
                            12,
                            VI_ERROR_PARAMETER2);
}


/*===========================================================================*/
/* Function: Get ACP Transmission Channel
/* Purpose:  This function returns the number of transmission channels.
/*           
/*           The function is available for measurements with span > 0.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPTransmissionChannel (ViSession instrSession,
                                                   ViInt32 *transmissionChannel)
{
    return rsfsh_getViInt32(instrSession,
                            "SENSe:POWer:ACHannel:TXCHannel:COUNt?",
                            transmissionChannel,
                            VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Reference Transmission Channel
/* Purpose:  This function sets a transmission channel as the reference
/*           channel for relative ACP measurements.
/*           
/*           The command is available only for multi-carrier channel and
/*           adjacent-channel power measurements with span > 0
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPReferenceTransmissionChannel (ViSession instrSession,
                                                   ViInt32 channel)
{
    return rsfsh_setViInt32(instrSession,
                            "SENSe:POWer:ACHannel:REFerence:TXCHannel:MANual",
                            channel,
                            1,
                            12,
                            VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Reference Transmission Channel
/* Purpose:  This function returns a transmission channel as the reference
/*           channel for relative ACP measurements.
/*           
/*           The command is available only for multi-carrier channel and
/*           adjacent-channel power measurements with span > 0
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPReferenceTransmissionChannel (ViSession instrSession,
                                                   ViInt32 *channel)
{
    return rsfsh_getViInt32(instrSession,
                            "SENSe:POWer:ACHannel:REFerence:TXCHannel:MANual?",
                            channel,
                            VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Limit Line State
/* Purpose:  This function activates and deactivates the limit check for ACP
/*           measurements. You have to use
/*           CALCulate:LIMit<1|2>:ACPower:ACHannel[:RELative]:STATe or
/*           CALCulate:LIMit<1...2>:ACPower:ALTernate<1...11>[:RELative]:STAT
/*           e in combination with this command to specify whether the limit
/*           check should be performed for the upper / lower adjacent channel
/*           or for the alternate adjacent channels.
/*           
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPLimitLineState (ViSession instrSession,
                                              ViInt32 limit, ViBoolean state)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:STATe", limit);
    
    return rsfsh_setViBoolean (instrSession,
                          buffer,
                          state,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get ACP Limit Line State
/* Purpose:  This function returns the limit check for ACP measurements.
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPLimitLineState (ViSession instrSession,
                                              ViInt32 limit, ViBoolean *state)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:STATe?", limit);
    
    return rsfsh_getViBoolean (instrSession,
                          buffer,
                          state,
                          VI_ERROR_PARAMETER3);

}

/*===========================================================================*/
/* Function: Set ACP Relative Limit
/* Purpose:  This function sets the relative limit of the upper / lower
/*           adjacent channel. The reference value for the relative limit
/*           value is the measured channel power.
/*           
/*           Note that the relative limit has no effect on the limit check if
/*           it is below the absolute limit value (see
/*           CALCulate:LIMit<1|2>:ACPower:ACHannel:ABSolute). This mechanism
/*           allows automatic checking of the absolute basic values of
/*           adjacent-channel power as defined in mobile radio standards.
/*           
/*           The numeric suffixe <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPRelativeLimit (ViSession instrSession,
                                            ViInt32 limit,
                                             ViReal64 adjacentRelativeLimit)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ACHannel:RELative", limit);
    

    return rsfsh_setViReal64 (instrSession,
                          buffer,
                          adjacentRelativeLimit,
                          0,100,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Relative Limit
/* Purpose:  This function returns the relative limit of the upper / lower
/*           adjacent channel. The reference value for the relative limit
/*           value is the measured channel power.
/*           Note that the relative limit has no effect on the limit check if
/*           it is below the absolute limit value (see
/*           CALCulate:LIMit<1|2>:ACPower:ACHannel:ABSolute). This mechanism
/*           allows automatic checking of the absolute basic values of
/*           adjacent-channel power as defined in mobile radio standards.
/*           
/*           The numeric suffixe <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPRelativeLimit (ViSession instrSession,
                                            ViInt32 limit,
                                             ViReal64 *adjacentRelativeLimit)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ACHannel:RELative?", limit);
    
    return rsfsh_getViReal64 (instrSession,
                          buffer,
                          adjacentRelativeLimit,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Relaitve Limit Line State
/* Purpose:  This function activates the relative limit check of the adjacent
/*           channel. The limit check must be activated with
/*           CALCulate:LIMit<1|2>:ACPower[:STATe] For the command to take
/*           effect. CALCulate:LIMit<1|2>:ACPower:ACHannel:RESult? queries
/*           the result.
/*           Note that a complete measurement must be performed between
/*           switching on the limit check and the result query, since
/*           otherwise no correct results are available.
/*           
/*           The numeric suffixe <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPRelaitveLimitLineState (ViSession instrSession,
                                                      ViInt32 limit,
                                                      ViBoolean state)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ACHannel:RELative:STATe", limit);
    
    return rsfsh_setViBoolean (instrSession,
                          buffer,
                          state,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get ACP Relaitve Limit Line State
/* Purpose:  This function returns limit check of the adjacent channel. The
/*           limit check must be activated with
/*           CALCulate:LIMit<1|2>:ACPower[:STATe] For the command to take
/*           effect. CALCulate:LIMit<1|2>:ACPower:ACHannel:RESult? queries
/*           the result.
/*           Note that a complete measurement must be performed between
/*           switching on the limit check and the result query, since
/*           otherwise no correct results are available.
/*           
/*           The numeric suffixe <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPRelaitveLimitLineState (ViSession instrSession,
                                                      ViInt32 limit,
                                                      ViBoolean *state)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ACHannel:RELative:STATe?", limit);
    
    return rsfsh_getViBoolean (instrSession,
                          buffer,
                          state,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set ACP Absolute Limit
/* Purpose:  This function defines the absolute limit for the lower / upper
/*           adjacent channel. It should be noted that the absolute limit has
/*           no effect on the limit check if it is below the relative limit
/*           (see CALCulate:LIMit<1|2>:ACPower:ACHannel[:RELative]). This
/*           mechanism allows automatic checking of the absolute basic values
/*           of adjacent-channel power as defined in mobile radio standards.
/*           
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPAbsoluteLimit (ViSession instrSession,
                                            ViInt32 limit,
                                             ViReal64 adjacentAbsoluteLimit)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ACHannel:ABSolute", limit);
    

    return rsfsh_setViReal64 (instrSession,
                          buffer,
                          adjacentAbsoluteLimit,
                          -200,200,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Absolute Limit
/* Purpose:  This function returns the absolute limit for the lower / upper
/*           adjacent channel. It should be noted that the absolute limit has
/*           no effect on the limit check if it is below the relative limit
/*           (see CALCulate:LIMit<1|2>:ACPower:ACHannel[:RELative]). This
/*           mechanism allows automatic checking of the absolute basic values
/*           of adjacent-channel power as defined in mobile radio standards.
/*           
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPAbsoluteLimit (ViSession instrSession,
                                            ViInt32 limit,
                                             ViReal64 *adjacentAbsoluteLimit)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ACHannel:ABSolute?", limit);
    

    return rsfsh_getViReal64 (instrSession,
                          buffer,
                          adjacentAbsoluteLimit,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Absolute Limit Line State
/* Purpose:  This function activates the absolute limit check of the adjacent
/*           channel. The limit check must be activated with
/*           CALCulate:LIMit<1|2>:ACPower[:STATe] For this command to take
/*           effect. CALCulate:LIMit<1|2>:ACPower:ACHannel:RESult? queries
/*           the result.
/*           Note that a complete measurement must be performed between
/*           switching on the limit check and the result query, since
/*           otherwise no correct results are available.
/*           The numeric suffixe <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPAbsoluteLimitLineState (ViSession instrSession,
                                                      ViInt32 limit,
                                                      ViBoolean state)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ACHannel:ABSolute:STATe", limit);
    

    return rsfsh_setViBoolean (instrSession,
                          buffer,
                          state,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Absolute Limit Line State
/* Purpose:  This function returns the absolute limit check of the adjacent
/*           channel. The limit check must be activated with
/*           CALCulate:LIMit<1|2>:ACPower[:STATe] For this command to take
/*           effect. CALCulate:LIMit<1|2>:ACPower:ACHannel:RESult? queries
/*           the result.
/*           Note that a complete measurement must be performed between
/*           switching on the limit check and the result query, since
/*           otherwise no correct results are available.
/*           The numeric suffixe <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPAbsoluteLimitLineState (ViSession instrSession,
                                                      ViInt32 limit,
                                                      ViBoolean *state)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ACHannel:ABSolute:STATe?", limit);
    

    return rsfsh_getViBoolean (instrSession,
                          buffer,
                          state,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Configure ACP Limit
/* Purpose:  This function sets the limit of the upper / lower adjacent
/*           channel.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureACPLimit (ViSession instrSession,
                                           ViInt32 mode, ViInt32 limit,
                                           ViReal64 adjacentLimit,
                                           ViBoolean state)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER3;

    if(mode==RSFSH_VAL_ACP_LIMIT_REL){
        CHECKERR (rsfsh_setACPRelativeLimit(instrSession,limit,adjacentLimit));
        CHECKERR (rsfsh_setACPRelaitveLimitLineState(instrSession,limit,state));
    } else if (mode==RSFSH_VAL_ACP_LIMIT_ABS){
        CHECKERR (rsfsh_setACPAbsoluteLimit(instrSession,limit,adjacentLimit));
        CHECKERR (rsfsh_setACPAbsoluteLimitLineState(instrSession,limit,state));
    } else {
        return VI_ERROR_PARAMETER2;
    }
    

    return  rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure ACP Limit Alternate
/* Purpose:  This function sets the limit of the alternate channels.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configuretACPLimitAlternate (ViSession instrSession,
                                                    ViInt32 mode,                                                 
                                                    ViInt32 limit,
                                                     ViReal64 alternateChannellimit,
                                                     ViInt32 alternateChannel,
                                                     ViBoolean state)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER3;

    if(mode==RSFSH_VAL_ACP_LIMIT_REL){
        CHECKERR (rsfsh_setACPRelativeLimitAlternate(instrSession,
                limit,alternateChannellimit,alternateChannel));
        CHECKERR (rsfsh_setACPRelativeLimitAlternateState(instrSession,limit,
                state, alternateChannel));
    } else if (mode==RSFSH_VAL_ACP_LIMIT_ABS){
        CHECKERR (rsfsh_setACPAbsoluteLimitAlternate(instrSession,
                limit,alternateChannellimit,alternateChannel));
        CHECKERR (rsfsh_setACPAbsoluteLimitAlternateState(instrSession,limit,
                state, alternateChannel));
    } else {
        return VI_ERROR_PARAMETER2;
    }
    

    return  rsfsh_status;
  
}

/*===========================================================================*/
/* Function: Get ACP Limit Line Check Result
/* Purpose:  This function queries the result of the limit check of the upper
/*           / lower adjacent channel. In case the ACP measurement is not
/*           active, the command returns a error.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPLimitLineCheckResult (ViSession instrSession,
                                                    ViInt32 limit,
                                                    ViChar limitCheck[],
                                                    ViInt32 bufferSize)
{
    ViChar      buffer[BUFFER_SIZE];

    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ACHannel:RESult? ", limit);

    return rsfsh_getViString (instrSession, 
                             buffer,
                             limitCheck,
                             bufferSize,
                             VI_ERROR_PARAMETER3);
    
}

/*===========================================================================*/
/* Function: Set ACP Relative Limit Alternate
/* Purpose:  This function sets the relative limit of the alternate channels.
/*           The reference value for the relative limit value is the measured
/*           channel power.
/*           Note that the relative limit has no effect on the limit check if
/*           it is below the absolute limit value (see
/*           CALCulate:LIMit<1|2>:ACPower:ALTernate<111>:ABSolute). This
/*           mechanism allows automatic checking of the absolute basic values
/*           of adjacent-channel power as defined in mobile radio standards.
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPRelativeLimitAlternate (ViSession instrSession,
                                                      ViInt32 limit,
                                                      ViReal64 relativeLimit,
                                                      ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 12))
        return VI_ERROR_PARAMETER4;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ALTernate%ld:RELative", limit, alternateChannel);
    

    return rsfsh_setViReal64 (instrSession,
                          buffer,
                          relativeLimit,
                          0.0,100.0,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Relative Limit Alternate
/* Purpose:  This function returns the relative limit of the alternate
/*           channels. The reference value for the relative limit value is
/*           the measured channel power.
/*           Note that the relative limit has no effect on the limit check if
/*           it is below the absolute limit value (see
/*           CALCulate:LIMit<1|2>:ACPower:ALTernate<111>:ABSolute). This
/*           mechanism allows automatic checking of the absolute basic values
/*           of adjacent-channel power as defined in mobile radio standards.
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPRelativeLimitAlternate (ViSession instrSession,
                                                      ViInt32 limit,
                                                      ViReal64 *relativeLimit,
                                                      ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 12))
        return VI_ERROR_PARAMETER4;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ALTernate%ld:RELative?", limit, alternateChannel);
    

    return rsfsh_getViReal64 (instrSession,
                          buffer,
                          relativeLimit,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Relative Limit Alternate State
/* Purpose:  This function activates the relative limit check of the
/*           alternate channels. The limit check must be activated with
/*           CALCulate:LIMit<1|2>:ACPower[:STATe] for the command to take
/*           effect.. CALCulate:LIMit<1|2>:ACPower:ACHannel:RESult? queries
/*           the result.
/*           
/*           Note that a complete measurement must be performed between
/*           switching on the limit check and the result query, since
/*           otherwise no correct results are available.
/*           
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPRelativeLimitAlternateState (ViSession instrSession,
                                                           ViInt32 limit,   
                                                           ViBoolean state,
                                                           ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 12))
        return VI_ERROR_PARAMETER4;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ALTernate%ld:RELative:STATe", limit, alternateChannel);
    

    return rsfsh_setViBoolean (instrSession,
                          buffer,
                          state,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Relative Limit Alternate State
/* Purpose:  This function returns the relative limit check of the alternate
/*           channels. The limit check must be activated with
/*           CALCulate:LIMit<1|2>:ACPower[:STATe] for the command to take
/*           effect.. CALCulate:LIMit<1|2>:ACPower:ACHannel:RESult? queries
/*           the result.
/*           
/*           Note that a complete measurement must be performed between
/*           switching on the limit check and the result query, since
/*           otherwise no correct results are available.
/*           
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPRelativeLimitAlternateState (ViSession instrSession,
                                                           ViInt32 limit,
                                                           ViBoolean *state,
                                                           ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 12))
        return VI_ERROR_PARAMETER4;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ALTernate%ld:RELative:STATe?", limit, alternateChannel);
    

    return rsfsh_getViBoolean (instrSession,
                          buffer,
                          state,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Absolute Limit Alternate
/* Purpose:  This function sets the absolute limit for the lower / upper
/*           adjacent channel. It should be noted that the absolute limit has
/*           no effect on the limit check if it is below the relative limit
/*           (see CALCulate:LIMit<1|2>:ACPower:ALTernate<111>[:RELative]).
/*           This mechanism allows automatic checking of the absolute basic
/*           values of adjacent-channel power as defined in mobile radio
/*           standards.
/*           
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPAbsoluteLimitAlternate (ViSession instrSession,
                                                      ViInt32 limit,
                                                      ViReal64 absoluteLimit,
                                                      ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 12))
        return VI_ERROR_PARAMETER4;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ALTernate%ld:ABSolute", limit, alternateChannel);
    
    return rsfsh_setViReal64 (instrSession,
                          buffer,
                          absoluteLimit,
                          -200.0,200.0,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Absolute Limit Alternate
/* Purpose:  This function returns the absolute limit for the lower / upper
/*           adjacent channel. It should be noted that the absolute limit has
/*           no effect on the limit check if it is below the relative limit
/*           (see CALCulate:LIMit<1|2>:ACPower:ALTernate<111>[:RELative]).
/*           This mechanism allows automatic checking of the absolute basic
/*           values of adjacent-channel power as defined in mobile radio
/*           standards.
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPAbsoluteLimitAlternate (ViSession instrSession,
                                                      ViInt32 limit,
                                                      ViReal64 *absoluteLimit,
                                                      ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 12))
        return VI_ERROR_PARAMETER4;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ALTernate%ld:ABSolute?", limit, alternateChannel);
    
    return rsfsh_getViReal64 (instrSession,
                          buffer,
                          absoluteLimit,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set ACP Absolute Limit Alternate State
/* Purpose:  This function activates the absolute limit check of the
/*           alternate channels. The limit check must be activated with
/*           CALCulate:LIMit<1|2>:ACPower[:STATe] for the command to take
/*           effect.. CALCulate:LIMit<1|2>:ACPower:ACHannel:RESult? queries
/*           the result.
/*           
/*           Note that a complete measurement must be performed between
/*           switching on the limit check and the result query, since
/*           otherwise no correct results are available.
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setACPAbsoluteLimitAlternateState (ViSession instrSession,
                                                           ViInt32 limit,
                                                           ViBoolean state,
                                                           ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 12))
        return VI_ERROR_PARAMETER4;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ALTernate%ld:ABSolute:STATe", limit, alternateChannel);
    

    return rsfsh_setViBoolean (instrSession,
                          buffer,
                          state,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Absolute Limit Alternate State
/* Purpose:  This function returns the absolute limit check of the
/*           alternate channels. The limit check must be activated with
/*           CALCulate:LIMit<1|2>:ACPower[:STATe] for the command to take
/*           effect.. CALCulate:LIMit<1|2>:ACPower:ACHannel:RESult? queries
/*           the result.
/*           
/*           Note that a complete measurement must be performed between
/*           switching on the limit check and the result query, since
/*           otherwise no correct results are available.
/*           The numeric suffix <1|2> at LIMit is irrelevant.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPAbsoluteLimitAlternateState (ViSession instrSession,
                                                           ViInt32 limit,
                                                           ViBoolean *state,
                                                           ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (alternateChannel, 1, 12))
        return VI_ERROR_PARAMETER4;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ALTernate%ld:ABSolute:STATe?", limit, alternateChannel);
    

    return rsfsh_getViBoolean (instrSession,
                          buffer,
                          state,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get ACP Alternate Limit Line Check Result
/* Purpose:  This function queries the result of the limit check of the upper
/*           / lower alternate channels. In case the ACP measurement is not
/*           active, the command returns an error.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getACPAlternateLimitLineCheckResult (ViSession instrSession,
                                                             ViInt32 limit,
                                                             ViChar limitCheck[],
                                                             ViInt32 bufferSize,
                                                             ViInt32 alternateChannel)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (limit, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALCulate:LIMit%ld:ACPower:ALTernate%ld:RESult?", limit,alternateChannel);
    
    return rsfsh_getViString (instrSession, 
                             buffer,
                             limitCheck,
                             bufferSize,
                             VI_ERROR_PARAMETER3);
    
}

/*===========================================================================*/
/* Function: Set Harmonic Distortion State
/* Purpose:  This function activates and deactivates the harmonic distortion
             measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setHarmonicDistortionState (ViSession instrSession,
                                                    ViBoolean state)
{
    return rsfsh_setViBoolean (instrSession, 
                               "CALC:MARK:FUNC:HARM",
                               state,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Harmonic Distortion State
/* Purpose:  This function queries the harmonic distortion measurement state.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getHarmonicDistortionState (ViSession instrSession,
                                                    ViBoolean *state)
{
    return rsfsh_getViBoolean (instrSession, 
                               "CALC:MARK:FUNC:HARM?",
                               state,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Adjust Harmonic Distortion Measurement
/* Purpose:  This function adjusts and optimizes the settings for the
             harmonic distortion measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_adjustHarmonicDistortionMeasurement (ViSession instrSession)
{
    return rsfsh_sendCmd (instrSession,"CALC:MARK:FUNC:HARM:PRES\n");
}

/*===========================================================================*/
/* Function: Get Total Harmonics Distortion
/* Purpose:  This function queries the total harmonic distortion of the
             signal, it returns two values, one for the total harmonic
             distortion in % and the second in dB.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getTotalHarmonicsDistortion (ViSession instrSession,
                                                     ViReal64 *totalHarmonicDistortion_in,
                                                     ViReal64 *totalHarmonicDistortion_in_dB)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      buffer2[BUFFER_SIZE];

    if (!totalHarmonicDistortion_in)
        return VI_ERROR_PARAMETER2;
    if (!totalHarmonicDistortion_in_dB)
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:MARK:FUNC:HARM:DIST? TOT");

    CHECKERR (rsfsh_getViString (instrSession, buffer, buffer2, BUFFER_SIZE, VI_ERROR_PARAMETER1));
    
    sscanf (buffer2, "%lf,%lf",totalHarmonicDistortion_in,totalHarmonicDistortion_in_dB );
    
    return rsfsh_status;
    
}

/*===========================================================================*/
/* Function: Set Harmonics Number
/* Purpose:  This function sets the number of harmonics to measure.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setHarmonicsNumber (ViSession instrSession,
                                            ViInt32 harmonicsNumber)
{
    return rsfsh_setViInt32 (instrSession, 
                             "CALC:MARK:FUNC:HARM:NHAR",
                             harmonicsNumber,
                             1,6,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Harmonics List
/* Purpose:  This function activates and deactivates the AM Modulation Depth
             measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getHarmonicsList (ViSession instrSession,
                                          ViReal64 harmonicsList[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *pbuff=VI_NULL;
    ViInt32     count = 0;
    
    CHECKERR (rsfsh_getViString (instrSession, "CALC:MARK:FUNC:HARM:LIST?", buffer, BUFFER_SIZE, VI_ERROR_PARAMETER1));  
    
    pbuff = strtok (buffer, ",");
    while (pbuff)
    {
        harmonicsList[count++] = atof (pbuff);
        pbuff = strtok(NULL, ",");
    };
    
    return rsfsh_status;     
     
}

/*===========================================================================*/
/* Function: Set AM Modulation Depth Measurement State
/* Purpose:  This function activates and deactivates the AM Modulation Depth
             measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAMModulationDepthMeasurementState (ViSession instrSession,
                                                              ViBoolean state)
{
    return rsfsh_setViBoolean (instrSession, 
                               "CALC:MARK:FUNC:MDEP",
                               state,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get AM Modulation Depth Measurement State
/* Purpose:  This function queries the AM Modulation Depth measurement state.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAMModulationDepthMeasurementState (ViSession instrSession,
                                                              ViBoolean *state)
{
   return rsfsh_getViBoolean (instrSession, 
                              "CALC:MARK:FUNC:MDEP?",
                               state,
                               VI_ERROR_PARAMETER2); 
}

/*===========================================================================*/
/* Function: Get AM Modulation Measurement Results
/* Purpose:  This function queries the results of the AM Modulation Depth
             measurement in %.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAMModulationMeasurementResults (ViSession instrSession,
                                                           ViReal64 *result)
{
   return rsfsh_getViReal64 (instrSession, 
                             "CALC:MARK:FUNC:MDEP:RES?",
                              result,
                              VI_ERROR_PARAMETER2); 
}

/*===========================================================================*/
/* Function: Configure Network Analyzer Measurement
/* Purpose:  This function configures the network analyzer measurement.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureNetworkAnalyzerMeasurement(
    ViSession   instrSession,
    ViInt32 measurementMode,
    ViBoolean   transmissionState,
    ViInt32 measurementFormat,
    ViInt32 measurementPort
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViInt32Range (measurementMode, 0, 5))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViBooleanRange (transmissionState))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViInt32Range (measurementFormat, 0, 3))
        return VI_ERROR_PARAMETER4;
    if (rsfsh_invalidViInt32Range (measurementPort, 1, 2))
        return VI_ERROR_PARAMETER5;
    
    CHECKERR (rsfsh_setNetworkAnalyzerMeasurementMode (instrSession, measurementMode));
    CHECKERR (rsfsh_setNetworkAnalyzerTransmissionState (instrSession, transmissionState));
    CHECKERR (rsfsh_setNetworkAnalyzerTransmissionMeasurementFormat (instrSession, measurementFormat));
    CHECKERR (rsfsh_setNetworkAnalyzerMeasurementPort (instrSession, measurementPort));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Network Analyzer Reflection Measurement
/* Purpose:  This function configures the network analyzer reflection 
/*           measurement.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureNetworkAnalyzerReflectionMeasurement(
    ViSession   instrSession,
    ViBoolean   reflectionState,
    ViInt32 measurementFormat
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViBooleanRange (reflectionState))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (measurementFormat, 0, 8))
        return VI_ERROR_PARAMETER3;
    
    CHECKERR (rsfsh_setNetworkAnalyzerReflectionState (instrSession, reflectionState));
    CHECKERR (rsfsh_setNetworkAnalyzerReflectionMeasurementFormat (instrSession, measurementFormat));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure User Calibrate
/* Purpose:  This function configures the user calibration.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureUserCalibrate(
    ViSession   instrSession,
    ViBoolean   userCalkit,
    ViReal64    userLength,
    ViReal64    offsetLength
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViBooleanRange (userCalkit))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViReal64Range (userLength, 0.00, 1000.00))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViReal64Range (offsetLength, 0.00, 100000.00))
        return VI_ERROR_PARAMETER4;
    
    CHECKERR (rsfsh_setUserCalkit (instrSession, userCalkit));
    CHECKERR (rsfsh_setUserCalkitLength (instrSession, userLength));
    CHECKERR (rsfsh_setUserCalkitOffsetLength (instrSession, offsetLength));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Network Analyzer Measurement Mode
/* Purpose:  This function sets the measurement mode for network analysis 
/*           measurements and distance to fault measurements.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setNetworkAnalyzerMeasurementMode(
    ViSession   instrSession,
    ViInt32 measurementMode
)
{
    return rsfsh_setEnum (instrSession, 
                          "MEAS:MODE",
                          measurementMode,
                          networkAnalyzerModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Network Analyzer Measurement Mode
/* Purpose:  This function queries the measurement mode for network analysis 
/*           measurements and distance to fault measurements.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getNetworkAnalyzerMeasurementMode(
    ViSession   instrSession,
    ViInt32*    measurementMode
)
{
    return rsfsh_getEnum (instrSession, 
                          "MEAS:MODE?",
                          measurementMode,
                          networkAnalyzerModeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Network Analyzer Reflection State
/* Purpose:  This function sets the reflection measurements state.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setNetworkAnalyzerReflectionState(
    ViSession   instrSession,
    ViBoolean   reflectionState
)
{
    return rsfsh_setViBoolean (instrSession, 
                               "MEAS:FUNC:REFL:STAT",
                               reflectionState,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Network Analyzer Reflection State
/* Purpose:  This function queries the reflection measurements state.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getNetworkAnalyzerReflectionState(
    ViSession   instrSession,
    ViBoolean*  reflectionState
)
{
    return rsfsh_getViBoolean (instrSession, 
                               "MEAS:FUNC:REFL:STAT?",
                               reflectionState,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Network Analyzer Reflection Measurement Format
/* Purpose:  This function sets the graphical format in which the 
/*           measurement result are presented.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setNetworkAnalyzerReflectionMeasurementFormat(
    ViSession   instrSession,
    ViInt32 measurementFormat
)
{
    return rsfsh_setEnum (instrSession, 
                          "MEAS:FORM",
                          measurementFormat,
                          measurementFormatArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Network Analyzer Reflection Measurement Format
/* Purpose:  This function queries the graphical format in which the 
/*           measurement result are presented.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getNetworkAnalyzerReflectionMeasurementFormat(
    ViSession   instrSession,
    ViInt32*    measurementFormat
)
{
    return rsfsh_getEnum (instrSession, 
                          "MEAS:FORM?",
                          measurementFormat,
                          measurementFormatArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Network Analyzer Transmission State
/* Purpose:  This function sets the transmission measurement in network 
/*           analysis mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setNetworkAnalyzerTransmissionState(
    ViSession   instrSession,
    ViBoolean   transmissionState
)
{
    return rsfsh_setViBoolean (instrSession, 
                               "MEAS:FUNC:TRAN:STAT",
                               transmissionState,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Network Analyzer Transmission State
/* Purpose:  This function queries the transmission measurement in network 
/*           analysis mode.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getNetworkAnalyzerTransmissionState(
    ViSession   instrSession,
    ViBoolean*  transmissionState
)
{
    return rsfsh_getViBoolean (instrSession, 
                               "MEAS:FUNC:TRAN:STAT?",
                               transmissionState,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Network Analyzer Transmission Measurement Format
/* Purpose:  This function sets the graphical format in which the 
/*           measurement result are presented.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setNetworkAnalyzerTransmissionMeasurementFormat(
    ViSession   instrSession,
    ViInt32 measurementFormat
)
{
    return rsfsh_setEnum (instrSession, 
                          "MEAS:FORM",
                          measurementFormat,
                          transmissionMeasurementFormatArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Network Analyzer Transmission Measurement Format
/* Purpose:  This function queries the graphical format in which the 
/*           measurement result are presented.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getNetworkAnalyzerTransmissionMeasurementFormat(
    ViSession   instrSession,
    ViInt32*    measurementFormat
)
{
    return rsfsh_getEnum (instrSession, 
                          "MEAS:FORM?",
                          measurementFormat,
                          transmissionMeasurementFormatArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Network Analyzer Measurement Port
/* Purpose:  This function sets the measurement port used for two-port 
/*           measurements.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setNetworkAnalyzerMeasurementPort(
    ViSession   instrSession,
    ViInt32 measurementPort
)
{
    return rsfsh_setViInt32 (instrSession, 
                             "MEAS:PORT",
                             measurementPort,
                             1,2,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Network Analyzer Measurement Port
/* Purpose:  This function queries the measurement port used for two-port 
/*           measurements.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getNetworkAnalyzerMeasurementPort(
    ViSession   instrSession,
    ViInt32*    measurementPort
)
{
    return rsfsh_getViInt32 (instrSession, 
                             "MEAS:PORT?",
                             measurementPort,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set User Calkit
/* Purpose:  This function sets the user calibration state.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setUserCalkit(
    ViSession   instrSession,
    ViBoolean   userCalkit
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViBooleanRange (userCalkit))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CALKIT,%ld", userCalkit);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setViBoolean (instrSession, 
                                           "CALK:USER:STAT",
                                           userCalkit,
                                           VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}


/*===========================================================================*/
/* Function: Get User Calkit
/* Purpose:  This function queries the user calibration state.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getUserCalkit(
    ViSession   instrSession,
    ViBoolean*  userCalkit
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!userCalkit)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CALKIT", buffer));
    sscanf (buffer, "%hd", userCalkit);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getViBoolean (instrSession, 
                                           "CALK:USER:STAT?",
                                           userCalkit,
                                           VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set User Calkit Length
/* Purpose:  This function sets the electrical length of the user 
/*           calibration. It is taken into account for phase measurements and 
/*           in the Smith Chart.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setUserCalkitLength(
    ViSession   instrSession,
    ViReal64    userLength
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViReal64Range (userLength, 0.00, 1.00))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CALKITLEN,%Lf", userLength);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        rsfsh_status = rsfsh_setViReal64 (instrSession, 
                                          "CALK:USER:LENG",
                                          userLength,
                                          0.00, 1000.00,
                                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get User Calkit Length
/* Purpose:  This function queries the electrical length of the user 
/*           calibration.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getUserCalkitLength(
    ViSession   instrSession,
    ViReal64*   userLength
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!userLength)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CALKITLEN", buffer));
    sscanf (buffer, "%Lf", userLength);
    break;
    case FSH4:
        rsfsh_status = rsfsh_getViReal64 (instrSession, 
                                          "CALK:USER:LENG?",
                                          userLength,
                                          VI_ERROR_PARAMETER2);
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: User Calkit Rename
/* Purpose:  This function sets the name for the user calibration to save 
/*           the customized settings under a user defined name.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_userCalkitRename(
    ViSession   instrSession,
    ViString    renameUser
)
{
    return rsfsh_setViString (instrSession, 
                              "CALK:USER:NAME",
                              renameUser,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set User Calkit Offset Length
/* Purpose:  This function sets the electrical length offset. It is taken 
/*           into account for phase measurements and in the Smith Chart when 
/*           phase correction for additional cables and adapters has to be 
/*           performed.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setUserCalkitOffsetLength(
    ViSession   instrSession,
    ViReal64    offsetLength
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "CALK:USER:OFFS:LENG",
                              offsetLength,
                              0.00, 100000.00,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get User Calkit Offset Length
/* Purpose:  This function queries the electrical length offset.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getUserCalkitOffsetLength(
    ViSession   instrSession,
    ViReal64*   offsetLength
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "CALK:USER:OFFS:LENG?",
                              offsetLength,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Network Analyzer Markers Mode
/* Purpose:  This function selects the markers mode.
/*===========================================================================*/        
ViStatus _VI_FUNC rsfsh_setNetworkAnalyzerMarkersMode (ViSession instrSession,
                                                       ViInt32 marker,
                                                       ViInt32 mode)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (mode, 0, 7))
        return VI_ERROR_PARAMETER3;
    
    sprintf (buffer, "CALC:MARK%ld:MODE", marker);
    
    return rsfsh_setEnum (instrSession,
                          buffer,
                          mode,
                          markerOutputFormatArr,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Network Analyzer Markers Mode
/* Purpose:  This function queries the markers mode.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getNetworkAnalyzerMarkersMode (ViSession instrSession,
                                                       ViInt32 marker,
                                                       ViInt32 *mode)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (marker, 1, 6))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "CALC:MARK%ld:MODE?", marker);
    
    return rsfsh_getEnum (instrSession,
                          buffer,
                          mode,
                          markerOutputFormatArr,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Configure DTF Measurement                                       */
/* Purpose:  This function configures DTF measurement parameters such as DTF */
/*           Measurement Mode, Cable Model, Cable Length and Cable Length    */
/*           Unit.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureDTFMeasurement (ViSession instrSession,
                                                 ViInt32 DTFMeasurementMode,
                                                 ViString cableModel,
                                                 ViReal64 cableLength,
                                                 ViInt32 cableLengthUnit,
                                                 ViInt32 measurementFormat)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViInt32Range (DTFMeasurementMode, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViString (cableModel, 64))
        return VI_ERROR_PARAMETER3;
    /* Parameter 4 is not range checked */
    if (rsfsh_invalidViInt32Range (cableLengthUnit, 0, 1))
        return VI_ERROR_PARAMETER5;

    CHECKERR (rsfsh_setDTFMeasurementMode (instrSession, DTFMeasurementMode));
    CHECKERR (rsfsh_setDistancetoFaultMeasurementFormat (instrSession, measurementFormat));

    if (DTFMeasurementMode == 0)
        {
        CHECKERR (rsfsh_setCableModel (instrSession, cableModel));
        CHECKERR (rsfsh_setCableLengthUnit (instrSession, cableLengthUnit));
        CHECKERR (rsfsh_setCableLength (instrSession, cableLength));
        }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Cable Model                                                 */
/* Purpose:  This function selects the cable model.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCableModel (ViSession instrSession,
                                       ViString cableModel)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViString (cableModel, 64))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CABLEMOD,%s", cableModel);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Cable Model                                                 */
/* Purpose:  This function returns the cable model.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCableModel (ViSession instrSession,
                                       ViChar _VI_FAR cableModel[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!cableModel)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CABLEMOD", buffer));
    sscanf (buffer, "%64[^\r]", cableModel);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Cable Length                                                */
/* Purpose:  This function defines the cable length. The unit of the length  */
/*           can either be Meter or Feet depending on the Length Unit        */
/*           setting. The cable length is converted and rounded to meters    */
/*           internally.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCableLength (ViSession instrSession,
                                        ViReal64 cableLength)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Not range checked */

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    sprintf (buffer, "CABLELEN,%Lf", cableLength);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Cable Length                                                */
/* Purpose:  This function returns the cable length. The unit of the length  */
/*           can either be Meter or Feet depending on the Length Unit        */
/*           setting. The cable length is converted and rounded to meters    */
/*           internally.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCableLength (ViSession instrSession,
                                        ViPReal64 cableLength)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!cableLength)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "CABLELEN", buffer));
    sscanf (buffer, "%Le", cableLength);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Cable Length Unit                                           */
/* Purpose:  This function sets the cable length unit to meters or feet.     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setCableLengthUnit (ViSession instrSession,
                                            ViInt32 cableLengthUnit)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (cableLengthUnit, 0, 1))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "LENUNIT,%s", (cableLengthUnit == 0) ? "METER" : "FEET");
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Cable Length Unit                                           */
/* Purpose:  This function returns the cable length unit.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCableLengthUnit (ViSession instrSession,
                                            ViPInt32 cableLengthUnit)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!cableLengthUnit)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "LENUNIT", buffer));

    if (strstr (buffer, "METER"))
        *cableLengthUnit = 0;
    else
        *cableLengthUnit = 1;
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Select Distance to Fault Cable Model
/* Purpose:  This function selects the cable model that is used for 
/*           distance-to-fault measurements.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_selectDistancetoFaultCableModel(
    ViSession   instrSession,
    ViString    cableModel
)
{
    return rsfsh_setViString (instrSession,
                              "CALC:DTF:CAB:PRES",
                              cableModel,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set DTF Measurement Mode                                        */
/* Purpose:  This function defines the measurement mode in DTF.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDTFMeasurementMode (ViSession instrSession,
                                               ViInt32 DTFMeasurementMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (DTFMeasurementMode, 0, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "DTFMODE,%ld", DTFMeasurementMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get DTF Measurement Mode                                        */
/* Purpose:  This function returns the measurement mode in DTF.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDTFMeasurementMode (ViSession instrSession,
                                               ViPInt32 DTFMeasurementMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!DTFMeasurementMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "DTFMODE", buffer));
    sscanf (buffer, "%ld", DTFMeasurementMode);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Distance to Fault Measurement Format
/* Purpose:  This function sets the graphical format in which the 
/*           measurement result are presented.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDistancetoFaultMeasurementFormat(
    ViSession   instrSession,
    ViInt32 measurementFormat
)
{
    return rsfsh_setEnum (instrSession, 
                          "MEAS:FORM",
                          measurementFormat,
                          dtfMeasurementFormatArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Distance to Fault Measurement Format
/* Purpose:  This function queries the graphical format in which the 
/*           measurement result are presented.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDistancetoFaultMeasurementFormat(
    ViSession   instrSession,
    ViInt32*    measurementFormat
)
{
    return rsfsh_getEnum (instrSession, 
                          "MEAS:FORM?",
                          measurementFormat,
                          dtfMeasurementFormatArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Calibrate Distance to Fault                                     */
/* Purpose:  This function calibrates the distance to fault measurement.     */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_calibrateDistanceToFault (ViSession instrSession)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer [10];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    CHECKERR (rsfsh_cmdCommand (instrSession, "CAL_DTF"));
    CHECKERR (rsfsh_readInstrData (instrSession, 10, buffer, VI_NULL));
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set Marker Distance to Fault Threshold
/* Purpose:  This function sets the threshold for the DTF list.
/*           All values which exceed the threshold are in the DTF list and 
/*           can be queried with the 
/*           CALCulate:MARKer<1...6>:FUNCtion:DTF:PEAK:RESult? command.
/*           The numeric suffix <1...6> is irrelevant.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setMarkerDistancetoFaultThreshold(
    ViSession   instrSession,
    ViReal64    threshold
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "CALC:MARK:FUNC:DTF:PEAK:THR",
                              threshold,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Marker Distance to Fault Result
/* Purpose:  This function queries the DTF list. It contains the return loss 
/*           and distance from the measurement plane of all reflections that 
/*           exceed the threshold level. The threshold level is defined with 
/*           the CALCulate:MARKer<1...6>:FUNCtion:DTF:PEAK:THReshold
/*           command.
/*           The numeric suffix <1..6> is irrelevant.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getMarkerDistancetoFaultResult(
    ViSession   instrSession,
    ViReal64*   result
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "CALC:MARK:FUNC:DTF:PEAK:RES?",
                              result,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set 3GPP Antenna Diversion                                      */
/* Purpose:  This function defines the antenna diversity mode used for WCDMA */
/* measurements.                                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_set3GPPAntennaDiversion (ViSession instrSession,
                                                 ViInt32 antennaDiversityMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (antennaDiversityMode, 0, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "ANTDIV,%ld", antennaDiversityMode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
         return rsfsh_setEnum (instrSession, 
                              "CDP:ANT",
                              antennaDiversityMode,
                              antennaDiversityModeArr,
                              VI_ERROR_PARAMETER2);
        
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get 3GPP Antenna Diversion                                      */
/* Purpose:  This function queries the antenna diversity mode used for WCDMA */
/* measurements.                                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_get3GPPAntennaDiversion (ViSession instrSession,
                                                 ViInt32 *antennaDiversityMode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!antennaDiversityMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "ANTDIV", buffer));
    sscanf (buffer, "%ld", antennaDiversityMode);
    break;
    case FSH4:
         return rsfsh_getEnum (instrSession, 
                              "CDP:ANT?",
                              antennaDiversityMode,
                              antennaDiversityModeArr,
                              VI_ERROR_PARAMETER2);
        
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set 3GPP Power Mode                                             */
/* Purpose:  his function defines whether the code power is an absolute value*/
/*           or a value related to the power of CPICH.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_set3GPPPowerMode (ViSession instrSession,
                                          ViInt32 powerMode)
{
    return rsfsh_setEnum (instrSession, 
                            "CDP:PREF",
                             powerMode,
                             powerModeArr,
                             VI_ERROR_PARAMETER2);   
}

/*===========================================================================*/
/* Function: Get 3GPP Power Mode                                             */
/* Purpose:  This function returns the absolute value or a value related to  */
/*           the power of CPICH.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_get3GPPPowerMode (ViSession instrSession,
                                          ViInt32 *powerMode)
{   
    return rsfsh_getEnum (instrSession, 
                            "CDP:PREF?",
                             powerMode,
                             powerModeArr,
                             VI_ERROR_PARAMETER2); 
    
}

/*===========================================================================*/
/* Function: Set 3GPP Scrambling Code                                        */
/* Purpose:  This function defines the scrambling code. The scrambling code  */
/* is divided into 2 parts, a primary and a secondary part.                  */
/* From the primary and secondary scrambling codes the common                */
/* scrambling code can be calculated as follows:                             */
/*                                                                           */
/* SCRbs = SCRprimary * 16 + SCRsecondary                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_set3GPPScramblingCode (ViSession instrSession,
                                               ViInt32 primaryScrambleCode,
                                               ViInt32 secondaryScrambleCode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (primaryScrambleCode, 0, 1535))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (secondaryScrambleCode, 0, 15))
        return VI_ERROR_PARAMETER3;
    
    sprintf (buffer, "PSCRCD,%ld", primaryScrambleCode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    
    sprintf (buffer, "SSCRCD,%ld", secondaryScrambleCode);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get 3GPP Scrambling Code                                        */
/* Purpose:  This function queries the scrambling code. The scrambling code  */
/* is divided into 2 parts, a primary and a secondary part.                  */
/* From the primary and secondary scrambling codes the common                */
/* scrambling code can be calculated as follows:                             */
/*                                                                           */
/* SCRbs = SCRprimary * 16 + SCRsecondary                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_get3GPPScramblingCode (ViSession instrSession,
                                               ViInt32 *primaryScrambleCode,
                                               ViInt32 *secondaryScrambleCode)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!primaryScrambleCode)
        return VI_ERROR_PARAMETER2;
    if (!secondaryScrambleCode)
        return VI_ERROR_PARAMETER3;
    
    CHECKERR (rsfsh_getCommand (instrSession, "PSCRCD", buffer));
    sscanf (buffer, "%ld", primaryScrambleCode);
    CHECKERR (rsfsh_getCommand (instrSession, "SSCRCD", buffer));
    sscanf (buffer, "%ld", secondaryScrambleCode);
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Set 3GPP Auto Scrambling Detection
/* Purpose:  This function starts the auto scrambling detection.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_set3GPPAutoScramblingDetection(
    ViSession   instrSession,
    ViInt32     detection
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar buffer [BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
        switch (detection)
        {
            case RSFSH_AUTO_SCRAMBLING_SINGLE:
                sprintf (buffer, "AUTOSDSNGL");
                break;
                
            case RSFSH_AUTO_SCRAMBLING_MULTIPLE:
                sprintf (buffer, "AUTOSDMUL");
                break;
        }
    
    CHECKERR (rsfsh_cmdCommand (instrSession, buffer));
    
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get 3GPP Measurement                                            */
/* Purpose:  This function retrieves the carrier frequency error for WCDMA   */
/* measurements.                                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_get3GPPMeasurement (ViSession instrSession,
                                            ViInt32 measurement,
                                            ViReal64 *result)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      cmdBuffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!result)
        return VI_ERROR_PARAMETER3;
        
    switch (measurement)
        {
        case 0://RSFSH_3GPP_MEAS_CARRFREQERR
            sprintf (cmdBuffer, "CARRFREQERR");         
        break;
        case 1://RSFSH_3GPP_MEAS_CPICHPWR
            sprintf (cmdBuffer, "CPICHPWR");                        
        break;
        case 2://RSFSH_3GPP_MEAS_CPICHSLOTNR
            sprintf (cmdBuffer, "CPICHSLOTNR");                     
        break;
        case 3://RSFSH_3GPP_MEAS_CPICHSYMEVM
            sprintf (cmdBuffer, "CPICHSYMEVM");                     
        break;
        case 4://RSFSH_3GPP_MEAS_PCCPCHPWR
            sprintf (cmdBuffer, "PCCPCHPWR");                       
        break;
        case 5://RSFSH_3GPP_MEAS_PCCPCHPWRREL
            sprintf (cmdBuffer, "PCCPCHPWRREL");                       
        break;
        case 6://RSFSH_3GPP_MEAS_PCCPCHSYMEVM
            sprintf (cmdBuffer, "PCCPCHSYMEVM");                        
        break;
        case 7://RSFSH_3GPP_MEAS_PSCHPWR
            sprintf (cmdBuffer, "PSCHPWR");                     
        break;
        case 8://RSFSH_3GPP_MEAS_PSCHPWRREL
            sprintf (cmdBuffer, "PSCHPWRREL");                     
        break;
        case 9://RSFSH_3GPP_MEAS_SSCHPWR
            sprintf (cmdBuffer, "SSCHPWR");                     
        break;
        case 10://RSFSH_3GPP_MEAS_SSCHPWRREL
            sprintf (cmdBuffer, "SSCHPWRREL");                     
        break;
        case 11://RSFSH_3GPP_MEAS_TOTPWR
            sprintf (cmdBuffer, "TOTPWR");                      
        break;
        case 12://RSFSH_3GPP_MEAS_CPICHEIRAT
            sprintf (cmdBuffer, "CPICHEIRAT");                      
        break;
        case 13://RSFSH_3GPP_MEAS_PCCPCHEIRAT
            sprintf (cmdBuffer, "PCCPCHEIRAT");                      
        break;
        
        default:
            return VI_ERROR_PARAMETER2;
        }
        
        CHECKERR (rsfsh_getCommand (instrSession, cmdBuffer, buffer));
        sscanf (buffer, "%Le", result);
        break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Get 3GPP Synchronization Result                                 */
/* Purpose:  This function retrieves the synchronization result for the WCDMA*/
/* measurement.                                                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_get3GPPSynchronizationResult (ViSession instrSession,
                                                      ViInt32 *synchronizationResult)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!synchronizationResult)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "SYNCRESULT", buffer));
    sscanf (buffer, "%ld", synchronizationResult);
    break;
    case FSH4:
        break;
    }
    
    return rsfsh_status;    
}


/*===========================================================================*/
/* Function: Get 3GPP FDD BS WCDP Result Type                                */
/* Purpose:  This function returns the WCDP measurement Results              */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_get3GPPBSResult (ViSession instrSession,
                                         ViInt32 type, ViReal64 *result)
{
    ViStatus    rsfsh_status = VI_SUCCESS; 
 
    if (rsfsh_invalidViInt32Range (type, 0, 8))
        return VI_ERROR_PARAMETER2;  
    if (!result)
        return VI_ERROR_PARAMETER3;          

    CHECKERR (viQueryf (instrSession, "CALC:MARK:FUNC:WCDP:RES? %s\n","%le", 
                result3GPPBSArr[type], result));
    
    
return rsfsh_status; 
}

/*===========================================================================*/
/* Function: Set Primary Scrambling Code                                     */
/* Purpose:  This function configures the primary scrambling code of the base
             station used to demodulate the signal.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setPrimaryScramblingCode (ViSession instrSession,
                                                  ViInt32 primaryScramblingCode)
{
   return rsfsh_setViInt32 (instrSession, 
                             "CDP:LCOD:PRIM",
                             primaryScramblingCode,
                             0,511,
                             VI_ERROR_PARAMETER2); 
}

/*===========================================================================*/
/* Function: Get Primary Scrambling Code                                     */
/* Purpose:  This function returns the primary scrambling code of the base
             station used to demodulate the signal.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPrimaryScramblingCode (ViSession instrSession,
                                                  ViInt32 *primaryScramblingCode)
{
    return rsfsh_getViInt32 (instrSession, 
                             "CDP:LCOD:PRIM?",
                             primaryScramblingCode,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Secondary Scrambling Code                                   */
/* Purpose:  This function configures the primary scrambling code of the base
             station used to demodulate the signal.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setSecondaryScramblingCode (ViSession instrSession,
                                                    ViInt32 secondaryScramblingCode)
{
      return rsfsh_setViInt32 (instrSession, 
                             "CDP:LCOD:SEC",
                             secondaryScramblingCode,
                             0,15,
                             VI_ERROR_PARAMETER2); 
}

/*===========================================================================*/
/* Function: Get Secondary Scrambling Code                                   */
/* Purpose:  This function returns the secondary scrambling code of the base
             station used to demodulate the signal.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getSecondaryScramblingCode (ViSession instrSession,
                                                    ViInt32 *secondaryScramblingCode)
{
     return rsfsh_getViInt32 (instrSession, 
                             "CDP:LCOD:SEC?",
                             secondaryScramblingCode,
                             VI_ERROR_PARAMETER2); 
}

/*===========================================================================*/
/* Function: Search 3GPP FDD BS Scrambling Code                              */
/* Purpose:  This function initiates a search for the scrambling code of the
             current signal. It is possible to detect multiple scrambling
             codes.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_search3GPPBSScramblingCode (ViSession instrSession)
{
    return rsfsh_sendCmd (instrSession, "CDP:LCOD:SEAR");
    
}

/*===========================================================================*/
/* Function: Get 3GPP FDD BS CDP Scrambling Code Results                     */
/* Purpose:  This function returns result table of the automatic search
             sequence containing the highest power values calculated and the
             corresponding scrambling codes.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_get3GPPBSCDPScramblingCodeResult (ViSession instrSession,
                                                          ViInt32 arraySize,
                                                          ViInt32 scrambingCode[],
                                                          ViReal64 power[],
                                                          ViInt32 *returnedValues)
{
    ViStatus    rsfsh_status = VI_SUCCESS;    
    ViChar*     pbuffer = NULL;
    ViChar*     pstring_value = NULL; 
    ViInt32     idx=0;
    ViUInt32    retCnt;
 
   
    if ((arraySize>0)&&(!scrambingCode))
        return VI_ERROR_PARAMETER2;  
    if ((arraySize>0)&&(!power))
        return VI_ERROR_PARAMETER3;   
    
    CHECKERR (viPrintf(instrSession, "*CLS;:CDP:LCODe:SEAR:LIST?\n"));   
    CHECKERR (rsfsh_readDataUnknownLength(instrSession, &pbuffer, &retCnt));
                                                                            
    if (strlen(pbuffer) > 1)
    {
        pstring_value = strtok(pbuffer, ",");

        while (pstring_value)
        {
            if (idx<arraySize)
            {
                //sscanf (pstring_value, "%ld,%[^,],%Le", &scrambingCode[idx], buf, &power[idx]);
                scrambingCode[idx] = atoi (pstring_value);
                pstring_value = strtok(NULL, ",");   
                pstring_value = strtok(NULL, ",");   
                power[idx] = atof (pstring_value);
                pstring_value = strtok(NULL, ",");   
            }
            //pstring_value = strtok(NULL, ",");   
            //pstring_value = strtok(NULL, ",");   
            //pstring_value = strtok(NULL, ",");          
            idx++;
        }
    }
    if (returnedValues)
    {
        *returnedValues=idx;
    }       
    
    

    if (pbuffer) free(pbuffer);
    return rsfsh_status; 
    
}

/*===========================================================================*/
/* Function: Set C2K BST Power Mode                                          */
/* Purpose:  This function defines whether the code power is an absolute
             value or a value related to the power of CPICH.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setC2KBSTPowerMode (ViSession instrSession,
                                            ViInt32 powerMode)
{
   return rsfsh_setEnum (instrSession, 
                          "CDP:PREF",
                          powerMode,
                          powerC2KModeArr,
                          VI_ERROR_PARAMETER2); 
}

/*===========================================================================*/
/* Function: Get C2K BST Power Mode                                          */
/* Purpose:  This function returns the absolute value or a value related to
             the power of CPICH.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getC2KBSTPowerMode (ViSession instrSession,
                                            ViInt32* powerMode)
{
  return rsfsh_getEnum (instrSession, 
                          "CDP:PREF?",
                          powerMode,
                          powerC2KModeArr,
                          VI_ERROR_PARAMETER2);   
}

/*===========================================================================*/
/* Function: Get C2K BST Code Domain Analyzer Result Summary                 */
/* Purpose:  This function queries the measured and calculated values of the
             code domain power analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getBC2KCodeDomainAnalyzerResultSummary (ViSession instrSession,
                                                                ViInt32 resultType,
                                                                ViReal64 *result)
{
    
    ViStatus    rsfsh_status = VI_SUCCESS; 
    
    if (rsfsh_invalidViInt32Range (resultType, 0, 8))
        return VI_ERROR_PARAMETER2;

     CHECKERR(viQueryf (instrSession, "CALC:MARK:FUNC:CDP:RES? %s\n","%le", 
                resultBC2KArr[resultType], result));
     
     return rsfsh_status; 
}

/*===========================================================================*/
/* Function: Set C2K BST Code Domain Analyzer Results                        */
/* Purpose:  This function selects the measured data that will be displayed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setBC2KCodeDomainAnalyzerResults (ViSession instrSession,
                                                          ViInt32 resultType)
{
    
    ViChar  cmd[BUFFER_SIZE] = "";
    
   switch ( resultType )
    {
        case RSFSH_VAL_BC2K_RES_TYPE_SPECTRUM: // Spectrum Overview
             sprintf(cmd, "CALC:FEED 'XPOW:CPOWer'");
             break;
        case RSFSH_VAL_BC2K_RES_TYPE_CDP: // Code Domain Power
             sprintf(cmd, "CALC:FEED 'XPOW:CDP'");
             break;
        case RSFSH_VAL_BC2K_RES_TYPE_SUMMARY: // Result Summary
             sprintf(cmd, "CALC:FEED 'XTIM:CDP:ERR:SUMM'");
             break;
        case RSFSH_VAL_BC2K_RES_TYPE_CDCT: // Code Domain Channel Table
             sprintf(cmd, "CALC:FEED 'XTIM:CDP:ERR:CTAB'");
             break;
        default:
            return VI_ERROR_PARAMETER2;
               
    } 
  
    return rsfsh_sendCmd (instrSession, cmd);  
}

/*===========================================================================*/
/* Function: Set SEM Sweep Mode                                                   */
/* Purpose:  This function changes from Spectrum Analyzer to Spectrum
             Emission Mask measurement mode and back.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setSEMSweepMode (ViSession instrSession, ViInt32 sweepMode)
{                                                       
          
   
   return rsfsh_setEnum (instrSession, 
                          "SENS:SWE:MODE",
                          sweepMode,
                          SEMArr,
                          VI_ERROR_PARAMETER2);
}   

/*===========================================================================*/
/* Function: Get SEM Sweep Mode                                                   */
/* Purpose:  This function returns the state of the spectrum emission mask.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getSEMSweepMode (ViSession instrSession,
                                     ViInt32 *sweepMode)
{
  
   
    return rsfsh_getEnum (instrSession, 
                          "SENS:SWE:MODE?",
                          sweepMode,
                          SEMArr,
                          VI_ERROR_PARAMETER2); 
}

/// HIFN  This function defines the start frequency of the 3GPP carrier.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]FREQuency:CARRier:STARt
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR startFrequency/This control defines the start frequency of the 3GPP carrier.
ViStatus _VI_FUNC rsfsh_setSEMFrequencyCarrierStart(
	ViSession	instrSession,
	ViReal64	startFrequency
)
{                                                       
   
   return rsfsh_setViReal64 (instrSession, 
                             "SENS:FREQ:CARR:STAR",
                             startFrequency,
                             0.0, 0.0,
                             VI_ERROR_PARAMETER2);
}


/// HIFN  This function returns the start frequency of the 3GPP carrier.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]FREQuency:CARRier:STARt?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR startFrequency/This control returns the start frequency of the 3GPP carrier.
ViStatus _VI_FUNC rsfsh_getSEMFrequencyCarrierStart(
	ViSession	instrSession,
	ViReal64*	startFrequency
)
{                                                       
   
   return rsfsh_getViReal64 (instrSession, 
                             "SENS:FREQ:CARR:STAR?",
                             startFrequency,
                             VI_ERROR_PARAMETER2);
}


/// HIFN  This function defines the stop frequency of the 3GPP carrier.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]FREQuency:CARRier:STOP
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR stopFrequency/This control defines the stop frequency of the 3GPP carrier.
ViStatus _VI_FUNC rsfsh_setSEMFrequencyCarrierStop(
	ViSession	instrSession,
	ViReal64	stopFrequency
)
{                                                       
   
   return rsfsh_setViReal64 (instrSession, 
                             "SENS:FREQ:CARR:STOP",
                             stopFrequency,
                             0.0, 0.0,
                             VI_ERROR_PARAMETER2);
}


/// HIFN  This function returns the stop frequency of the 3GPP carrier.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]FREQuency:CARRier:STOP?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR stopFrequency/This control returns the start frequency of the 3GPP carrier.
ViStatus _VI_FUNC rsfsh_getSEMFrequencyCarrierStop(
	ViSession	instrSession,
	ViReal64*	stopFrequency
)
{                                                       
   
   return rsfsh_getViReal64 (instrSession, 
                             "SENS:FREQ:CARR:STOP?",
                             stopFrequency,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set SEM Preset Standard                                         */
/* Purpose:  This function loads one of the predefined spectrum emission
             masks for a telecommunications standard.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setSEMPresetStandard (ViSession instrSession,
                                              ViString presetStandard)
{
   return rsfsh_setViStringQuoted (instrSession,
                              "ESP:PRES",
							  "'",
                              presetStandard,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set SEM Preset Settings                                         */
/* Purpose:  This function performs an automatic adjustment of the R&S FSH
             settings.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setSEMPresetSettings (ViSession instrSession)
{
   return rsfsh_sendCmd (instrSession, "ESP:PRES:SETT ONCE");    
}

/*===========================================================================*/
/* Function: Set EV-DO Power Reference                                       */
/* Purpose:  This command selects the reference power for relative results.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setEVDOPowerReference (ViSession instrSession,
                                               ViInt32 powerReference)
{
   return rsfsh_setEnum (instrSession, 
                          "CDP:PREF",
                          powerReference,
                          powerEVDOModeArr,
                          VI_ERROR_PARAMETER2);  
}

/*===========================================================================*/
/* Function: Get EV-DO Power Reference                                       */
/* Purpose:  This function returns the reference power for relative results.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getEVDOPowerReference (ViSession instrSession,
                                               ViInt32 *powerReference)
{
   return rsfsh_getEnum (instrSession, 
                          "CDP:PREF?",
                          powerReference,
                          powerEVDOModeArr,
                          VI_ERROR_PARAMETER2);   
}

/*===========================================================================*/
/* Function: Get EV-DO BST Code Domain Analyzer General Results              */
/* Purpose:  This function queries the measured and calculated values of the
             code domain power analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getEVDOCodeDomainAnalyzerGeneralResults (ViSession instrSession,
                                                                 ViInt32 resultType,
                                                                 ViReal64 *result)
{
    ViStatus    rsfsh_status = VI_SUCCESS; 
 
    if (rsfsh_invalidViInt32Range (resultType, 0, 10))
        return VI_ERROR_PARAMETER2;  
    if (!result)
        return VI_ERROR_PARAMETER3;          

     CHECKERR(viQueryf (instrSession, "CALC:MARK:FUNC:CDP:RES? %s\n","%le", 
                resultEVDOArr[resultType], result));
    
    
return rsfsh_status; 
}

/*===========================================================================*/
/* Function: Set EV-DO BST Code Domain Analyzer Results                      */
/* Purpose:  This function selects the measured data that will be displayed.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setEVDOBCodeDomainAnalyzerResults (ViSession instrSession,
                                                           ViInt32 resultType)
{
    ViChar  cmd[BUFFER_SIZE] = "";
    
   switch ( resultType )
    {
        case RSFSH_VAL_EVDOB_RES_TYPE_SPECTRUM: // Spectrum Overview
             sprintf(cmd, "CALC:FEED 'XPOW:CPOWer'");
             break;
  
        case RSFSH_VAL_EVDOB_RES_TYPE_SUMMARY: // Code Domain Channel Table
             sprintf(cmd, "CALC:FEED 'XTIM:CDP:ERR:SUMM'");
             break;
        default: return VI_ERROR_PARAMETER2;
    } 
  
   
    return rsfsh_sendCmd (instrSession, cmd);    
}

/*===========================================================================*/
/* Function: Configure External Gate                                         */
/* Purpose:  This function configures the gate parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureExternalGate (ViSession instrSession,
                                               ViBoolean gating,
                                               ViReal64 gateDelay,
                                               ViReal64 gateLength,
                                               ViReal64 sweepTime)
{
   
    ViStatus    rsfsh_status = VI_SUCCESS;

    if (rsfsh_invalidViReal64Range (gateDelay, 0.0, 100.0))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViReal64Range (gateLength, 10.0e-6, 100.0))
        return VI_ERROR_PARAMETER4;
    
    CHECKERR (rsfsh_setExternalGate (instrSession, gating));
    
    if (gating == VI_TRUE)
    {
        CHECKERR (rsfsh_setExternalGateHoldoff (instrSession, gateDelay));
        CHECKERR (rsfsh_setExternalGateLength (instrSession, gateLength));
        CHECKERR (rsfsh_setExternalGateTime (instrSession, sweepTime));
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set External Gate                                               */
/* Purpose:  This function sets a gated trigger on and off.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setExternalGate (ViSession instrSession,
                                         ViBoolean gating)
{
    return rsfsh_setViBoolean (instrSession, 
                               "SWE:EGAT",
                               gating,
                               VI_ERROR_PARAMETER2);  
}

/*===========================================================================*/
/* Function: Get External Gate                                               */
/* Purpose:  This function returns a gated trigger state.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getExternalGate (ViSession instrSession,
                                         ViBoolean *gating)
{
   return rsfsh_getViBoolean (instrSession, 
                               "SWE:EGAT?",
                               gating,
                               VI_ERROR_PARAMETER2);  
}

/*===========================================================================*/
/* Function: Set External Gate Holdoff                                       */
/* Purpose:  This function sets the gate delay.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setExternalGateHoldoff (ViSession instrSession,
                                                ViReal64 gateDelay)
{
   return rsfsh_setViReal64 (instrSession, 
                              "SWE:EGAT:HOLD",
                              gateDelay,
                              0.0, 100.0,
                              VI_ERROR_PARAMETER2); 
}

/*===========================================================================*/
/* Function: Get External Gate Holdoff                                       */
/* Purpose:  This function returns the gate delay.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getExternalGateHoldoff (ViSession instrSession,
                                                ViReal64 *gateDelay)
{
   return rsfsh_getViReal64 (instrSession, 
                              "SWE:EGAT:HOLD?",
                              gateDelay,
                              VI_ERROR_PARAMETER2); 
}

/*===========================================================================*/
/* Function: Set External Gate Length                                        */
/* Purpose:  This function sets the gate length.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setExternalGateLength (ViSession instrSession,
                                               ViReal64 gateLength)
{
   return rsfsh_setViReal64 (instrSession, 
                              "SWE:EGAT:LENG",
                              gateLength,
                              10.0e-6, 100.0,
                              VI_ERROR_PARAMETER2);   
}

/*===========================================================================*/
/* Function: Get External Gate Length                                        */
/* Purpose:  This function sets the gate length.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getExternalGateLength (ViSession instrSession,
                                               ViReal64 *gateLength)
{
    return rsfsh_getViReal64 (instrSession, 
                              "SWE:EGAT:LENG?",
                              gateLength,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set External GateTime                                           */
/* Purpose:  This function sets the sweep time for the gated trigger.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setExternalGateTime (ViSession instrSession,
                                             ViReal64 sweepTime)
{
   
    return rsfsh_setViReal64 (instrSession, 
                              "SWE:EGAT:TIME",
                              sweepTime,
                              0.0,0.0,
                              VI_ERROR_PARAMETER2); 
    
}

/*===========================================================================*/
/* Function: Get External GateTime                                           */
/* Purpose:  This function returns the sweep time for the gated trigger.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getExternalGateTime (ViSession instrSession,
                                             ViReal64 *sweepTime)
{
    return rsfsh_getViReal64 (instrSession, 
                              "SWE:EGAT:TIME?",
                              sweepTime,
                              VI_ERROR_PARAMETER2);  
}

/// HIFN  This function configures the data capture.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:CAPTure:DATaset[:STATe] ON | OFF
/// HIFN     SYSTem:CAPTure:GPX[:STATe] ON | OFF
/// HIFN     SYSTem:CAPTure:SCReen[:STATe] ON | OFF
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR datasets/This control includes or excludes datasets from the data capture.
/// HIPAR gpx/This control includes or excludes GPX information from the data 
/// HIPAR gpx/capture.
/// HIPAR screenshots/This control includes or excludes screenshots from the data capture.
ViStatus _VI_FUNC rsfsh_configureDataCapture(
	ViSession	instrSession,
	ViBoolean	datasets,
	ViBoolean	gpx,
	ViBoolean	screenshots
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    CHECKERR (rsfsh_setDataCaptureDatasets (instrSession, datasets));
    CHECKERR (rsfsh_setDataCaptureGPX (instrSession, gpx));
	CHECKERR (rsfsh_setDataCaptureScreenshots (instrSession, screenshots));
    
    return rsfsh_status;
}


/// HIFN  This function includes or excludes datasets from the data capture.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:CAPTure:DATaset[:STATe] ON | OFF
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR datasets/This control includes or excludes datasets from the data capture.
ViStatus _VI_FUNC rsfsh_setDataCaptureDatasets(
	ViSession	instrSession,
	ViBoolean	datasets
)
{
   
    return rsfsh_setViBoolean (instrSession, 
                               "SYST:CAPT:DAT",
                               datasets,
                               VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function returns a datasets state.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:CAPTure:DATaset[:STATe]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR datasets/This control returns the datasets state.
ViStatus _VI_FUNC rsfsh_getDataCaptureDatasets(
	ViSession	instrSession,
	ViBoolean*	datasets
)
{
   
    return rsfsh_getViBoolean (instrSession, 
                               "SYST:CAPT:DAT?",
                               datasets,
                               VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function includes or excludes GPX information from the data 
/// HIFN  capture.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:CAPTure:GPX[:STATe] ON | OFF
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR gpx/This control includes or excludes GPX information from the data 
/// HIPAR gpx/capture.
ViStatus _VI_FUNC rsfsh_setDataCaptureGPX(
	ViSession	instrSession,
	ViBoolean	gpx
)
{
   
    return rsfsh_setViBoolean (instrSession, 
                               "SYST:CAPT:GPX",
                               gpx,
                               VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function returns a GPX information state.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:CAPTure:GPX[:STATe]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR gpx/This control returns the GPX information state.
ViStatus _VI_FUNC rsfsh_getDataCaptureGPX(
	ViSession	instrSession,
	ViBoolean*	gpx
)
{
   
    return rsfsh_getViBoolean (instrSession, 
                               "SYST:CAPT:GPX?",
                               gpx,
                               VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function includes or excludes screenshots from the data capture.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:CAPTure:SCReen[:STATe] ON | OFF
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR screenshots/This control includes or excludes screenshots from the data capture.
ViStatus _VI_FUNC rsfsh_setDataCaptureScreenshots(
	ViSession	instrSession,
	ViBoolean	screenshots
)
{
   
    return rsfsh_setViBoolean (instrSession, 
                               "SYST:CAPT:SCR",
                               screenshots,
                               VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function returns a screenshots state.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:CAPTure:SCReen[:STATe]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR screenshots/This control returns the screenshots state.
ViStatus _VI_FUNC rsfsh_getDataCaptureScreenshots(
	ViSession	instrSession,
	ViBoolean*	screenshots
)
{
   
    return rsfsh_getViBoolean (instrSession, 
                               "SYST:CAPT:SCR?",
                               screenshots,
                               VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function configures the saving events.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:[:STATe]
/// HIFN     SYSTem:SOEVent:RECording:STORage SDCard | USB
/// HIFN     SYSTem:SOEVent:SOURce ALLSweeps | DISTanceint | LIM
/// HIFN     SYSTem:SOEVent:LIMits:MODE FAILonly | STARtonfail | STOPonfail
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control turns saving of your current coordinates in case of 
/// HIPAR state/certain events on and off.
/// HIPAR storage/This control sets the storage device to save the coordinates to. 
/// HIPAR source/This control sets the type of event that triggers saving the 
/// HIPAR source/coordinates of your current location. 
/// HIPAR limitsMode/This control sets the limit check condition that must occur in order 
/// HIPAR limitsMode/to save a coordinate.
ViStatus _VI_FUNC rsfsh_configureSavingEvents(
	ViSession	instrSession,
	ViBoolean	state,
	ViInt32	storage,
	ViInt32	source,
	ViInt32	limitsMode
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    CHECKERR (rsfsh_setSavingEventsState (instrSession, state));
	CHECKERR (rsfsh_setSavingEventsStorage (instrSession, storage));
    CHECKERR (rsfsh_setSavingEventsSource (instrSession, source));
	CHECKERR (rsfsh_setSavingEventsLimitsMode (instrSession, limitsMode));
    
    return rsfsh_status;
}


/// HIFN  This function configures the saving events limits.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:DISTance:INTerval
/// HIFN     SYSTem:SOEVent:TIME:INTerval
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR distance/This control sets a distance that you must cover before the R&S FSH 
/// HIPAR distance/saves another coordinate.
/// HIPAR time/This control sets a time interval that must pass before the R&S FSH 
/// HIPAR time/saves another coordinate.
ViStatus _VI_FUNC rsfsh_configureSavingEventsLimits(
	ViSession	instrSession,
	ViInt32	distance,
	ViInt32	time
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    CHECKERR (rsfsh_setSavingEventsDistance (instrSession, distance));
	CHECKERR (rsfsh_setSavingEventsTime (instrSession, time));
    
    return rsfsh_status;
}


/// HIFN  This function turns saving of your current coordinates in case of 
/// HIFN  certain events on and off.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:[:STATe]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control turns saving of your current coordinates in case of 
/// HIPAR state/certain events on and off.
ViStatus _VI_FUNC rsfsh_setSavingEventsState(
	ViSession	instrSession,
	ViBoolean	state
)
{
   
    return rsfsh_setViBoolean (instrSession, 
                               "SYST:SOEV",
                               state,
                               VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function returns the state of the Saving Events state.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:[:STATe]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control returns the state of the Saving Events state.
ViStatus _VI_FUNC rsfsh_getSavingEventsState(
	ViSession	instrSession,
	ViBoolean*	state
)
{
   
    return rsfsh_getViBoolean (instrSession, 
                               "SYST:SOEV?",
                               state,
                               VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function sets the storage device to save the coordinates to. 
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:RECording:STORage SDCard | USB
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR storage/This control sets the storage device to save the coordinates to. 
ViStatus _VI_FUNC rsfsh_setSavingEventsStorage(
	ViSession	instrSession,
	ViInt32	storage
)
{
   
    return rsfsh_setEnum (instrSession, 
                          "SYST:SOEV:REC:STOR",
                          storage,
						  storageArr,
                          VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function returns the storage device to save the coordinates to. 
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:RECording:STORage?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR storage/This control returns the storage device to save the coordinates to. 
ViStatus _VI_FUNC rsfsh_getSavingEventsStorage(
	ViSession	instrSession,
	ViInt32*	storage
)
{
   
    return rsfsh_getEnum (instrSession, 
                          "SYST:SOEV:REC:STOR?",
                          storage,
						  storageArr,
                          VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function sets the type of event that triggers saving the 
/// HIFN  coordinates of your current location.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:SOURce ALLSweeps | DISTanceint | LIM
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR source/This control sets the type of event that triggers saving the 
/// HIPAR source/coordinates of your current location. 
ViStatus _VI_FUNC rsfsh_setSavingEventsSource(
	ViSession	instrSession,
	ViInt32	source
)
{
   
    return rsfsh_setEnum (instrSession, 
                          "SYST:SOEV:SOUR",
                          source,
						  soevSourceArr,
                          VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function returns the type of event that triggers saving the 
/// HIFN  coordinates of your current location.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:SOURce?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR source/This control returns the type of event that triggers saving the 
/// HIPAR source/coordinates of your current location. 
ViStatus _VI_FUNC rsfsh_getSavingEventsSource(
	ViSession	instrSession,
	ViInt32*	source
)
{
   
    return rsfsh_getEnum (instrSession, 
                          "SYST:SOEV:SOUR?",
                          source,
						  soevSourceArr,
                          VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function sets a distance that you must cover before the R&S FSH 
/// HIFN  saves another coordinate.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:DISTance:INTerval
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR distance/This control sets a distance that you must cover before the R&S FSH 
/// HIPAR distance/saves another coordinate.
ViStatus _VI_FUNC rsfsh_setSavingEventsDistance(
	ViSession	instrSession,
	ViInt32	distance
)
{
   
    return rsfsh_setViInt32 (instrSession, 
                             "SYST:SOEV:DIST:INT",
                             distance,
						     0, 0,
                             VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function returns a distance that you must cover before the R&S 
/// HIFN  FSH saves another coordinate.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:DISTance:INTerval?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR distance/This control returns a distance that you must cover before the R&S FSH 
/// HIPAR distance/saves another coordinate.
ViStatus _VI_FUNC rsfsh_getSavingEventsDistance(
	ViSession	instrSession,
	ViInt32*	distance
)
{
   
    return rsfsh_getViInt32 (instrSession, 
                             "SYST:SOEV:DIST:INT?",
                             distance,
                             VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function sets the limit check condition that must occur in order 
/// HIFN  to save a coordinate.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:LIMits:MODE FAILonly | STARtonfail | STOPonfail
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR limitsMode/This control sets the limit check condition that must occur in order 
/// HIPAR limitsMode/to save a coordinate.
ViStatus _VI_FUNC rsfsh_setSavingEventsLimitsMode(
	ViSession	instrSession,
	ViInt32	limitsMode
)
{
   
    return rsfsh_setEnum (instrSession, 
                          "SYST:SOEV:LIM:MODE",
                          limitsMode,
						  soevlimitsModeArr,
                          VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function returns the limit check condition that must occur in 
/// HIFN  order to save a coordinate.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:LIMits:MODE?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR limitsMode/This control returns the limit check condition that must occur in 
/// HIPAR limitsMode/order to save a coordinate.
ViStatus _VI_FUNC rsfsh_getSavingEventsLimitsMode(
	ViSession	instrSession,
	ViInt32*	limitsMode
)
{
   
    return rsfsh_getEnum (instrSession, 
                          "SYST:SOEV:LIM:MODE?",
                          limitsMode,
						  soevlimitsModeArr,
                          VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function sets a time interval that must pass before the R&S FSH 
/// HIFN  saves another coordinate.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:TIME:INTerval
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR time/This control sets a time interval that must pass before the R&S FSH 
/// HIPAR time/saves another coordinate.
ViStatus _VI_FUNC rsfsh_setSavingEventsTime(
	ViSession	instrSession,
	ViInt32	time
)
{
   
    return rsfsh_setViInt32 (instrSession, 
                             "SYST:SOEV:TIME:INT",
                             time,
						     0, 0,
                             VI_ERROR_PARAMETER2); 
    
}


/// HIFN  This function returns a time interval that must pass before the R&S 
/// HIFN  FSH saves another coordinate.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:SOEVent:TIME:INTerval?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR time/This control returns a time interval that must pass before the R&S FSH 
/// HIPAR time/saves another coordinate.
ViStatus _VI_FUNC rsfsh_getSavingEventsTime(
	ViSession	instrSession,
	ViInt32*	time
)
{
   
    return rsfsh_getViInt32 (instrSession, 
                             "SYST:SOEV:TIME:INT?",
                             time,
                             VI_ERROR_PARAMETER2); 
    
}

/*===========================================================================*/
/* Function: File Directories Catalog
/* Purpose:  This function lists the size of memory used by files and the 
/*           size of memory space available togehter with a list of 
/*           subdirectories.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_fileDirectoriesCatalog(
    ViSession   instrSession,
    ViString    directoryPath,
    ViInt32 arraySize,
    ViChar _VI_FAR  catalogList[]
)
{
    ViChar  buffer [BUFFER_SIZE];

    if (!directoryPath)
        sprintf (buffer, ":MMEM:CAT:DIR?");
    else
        sprintf (buffer, ":MMEM:CAT:DIR? '%s'", directoryPath);

    return rsfsh_getCatalog (instrSession,
                             buffer,
                             arraySize,
                             catalogList,
                             VI_ERROR_PARAMETER4);
}

/*===========================================================================*/
/* Function: File Directory
/* Purpose:  This function reads the indicated directory. According to DOS 
/*           convention, wild card characters can be entered in order to 
/*           query eg. a list of all file of a certain type.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_FileDirectory(
    ViSession   instrSession,
    ViString    directory,
    ViInt32 bufferSize,
    ViChar _VI_FAR  output[]
)
{
    ViChar  buffer [BUFFER_SIZE];

    if (!directory)
        sprintf (buffer, ":MMEM:CAT?;*OPC?");
    else
        sprintf (buffer, ":MMEM:CAT? '%s';*OPC?", directory);

    return rsfsh_getCatalog (instrSession,
                             buffer,
                             bufferSize,
                             output,
                             VI_ERROR_PARAMETER4);
}

/*===========================================================================*/
/* Function: File Manager Operations
/* Purpose:  The MMEMory (mass memory) subsystem function provides commands 
/*           which allow for access to the storage media of the instrument 
/*           and for storing and loading various instrument settings.
/*           The File to be Printed/Plotted command stores the HCOPy outputs 
/*           in a file. The various drives can be addressed via the mass 
/*           storage unit specifier <msus> using the conventional DOS syntax. 
/*           The internal hard disk is addressed by "C:", the floppy-disk 
/*           drive installed by "A:". The file names <file_name> are 
/*           indicated as string parameters with the commands being enclosed 
/*           in quotation marks. They correspond to the DOS conventions.
/*           DOS file names consist of max. 8 ASCII characters and an 
/*           extension of up to three characters separated from the file name 
/*           by a colon "." Both, the colon and the extension are optional. 
/*           The colon is not part of the file name. DOS file names do not 
/*           differ between uppercase and lowercase notation. All letters and 
/*           digits are permitted as well as the special characters "_", "^", 
/*           "$", "~", "!", "#", "%", "&", "-", "{","}", "(", ")", "@" and "' 
/*           ". Reserved file names are CLOCK$, CON, AUX, COM1 to COM4, LPT1 
/*           to LPT3, NUL and PRN. The two characters "*" and "?" have the 
/*           function of so-called "wildcards", i.e., they are variables for
/*           selection of several files. The question mark "?" replaces 
/*           exactly one character which may be any, the asterisk means any 
/*           of the remaining characters in the file name. "*.*" thus means 
/*           all files in a directory.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_FileManagerOperations(
    ViSession   instrSession,
    ViInt32 operation,
    ViString    source,
    ViString    destination
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar  buffer [BUFFER_SIZE];

    if (rsfsh_invalidViInt32Range (operation, 0, 7))
        return VI_ERROR_PARAMETER2;
    
    switch (operation)
    {
        case RSFSH_VAL_FILE_NEW:
            sprintf (buffer, "MMEM:MDIR %s", source);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;
            
        case RSFSH_VAL_FILE_COPY:
            sprintf (buffer, "MMEM:COPY %s,%s", source, destination);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;
            
        case RSFSH_VAL_FILE_RENAME:
            sprintf (buffer, "MMEM:MOVE %s,%s", source, destination);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;
            
        case RSFSH_VAL_FILE_DELETE:
            sprintf (buffer, "MMEM:DEL %s", source);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;
            
        case RSFSH_VAL_FILE_RDIR:
            sprintf (buffer, "MMEM:RDIR %s", source);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;
            
        case RSFSH_VAL_FILE_FORMAT:
            sprintf (buffer, "MMEM:INIT %s", source);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;
            
        case RSFSH_VAL_FILE_CDIR:
            sprintf (buffer, "MMEM:CDIR %s", source);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;
            
        case RSFSH_VAL_FILE_CDISC:
            sprintf (buffer, "MMEM:MSIS %s", source);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: File Create
/* Purpose:  This function creates a file with the specified name.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_fileCreate(
    ViSession   instrSession,
    ViString    fileName
)
{
    return rsfsh_setViString (instrSession,
                              "MMEM:FILE",
                              fileName,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set File Date
/* Purpose:  This function sets the date of the indicated file. The sequence 
/*           of entry is year, month, day.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setFileDate(
    ViSession   instrSession,
    ViString fileName,
    ViInt32 year,
    ViInt32 month,
    ViInt32 day
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (year, 1980, 2099))
        return VI_ERROR_PARAMETER3;
    
    if (rsfsh_invalidViInt32Range (month, 1, 12))
        return VI_ERROR_PARAMETER4;
    
    if (rsfsh_invalidViInt32Range (day, 1, 31))
        return VI_ERROR_PARAMETER5;
    
    sprintf (buffer, "MMEM:FILE:DATE %s, %ld, %ld, %ld", fileName, year, month, day);
    
    return rsfsh_sendCmd (instrSession, buffer);

}

/*===========================================================================*/
/* Function: Get File Date
/* Purpose:  This function reads the date of the indicated file. The 
/*           sequence of entry is year, month, day.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getFileDate(
    ViSession   instrSession,
    ViString   fileName,
    ViChar _VI_FAR  date[]
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    sprintf (buffer, "MMEM:FILE:DATE? %s", fileName);
    
    return rsfsh_getViString (instrSession,
                              buffer,
                              date,
                              1024,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set File Time
/* Purpose:  This function sets the date of the indicated file. The sequence 
/*           of entry is year, month, day.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setFileTime(
    ViSession   instrSession,
    ViString fileName,
    ViInt32 hours,
    ViInt32 minutes,
    ViInt32 seconds
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (hours, 0, 23))
        return VI_ERROR_PARAMETER3;
    
    if (rsfsh_invalidViInt32Range (minutes, 0, 59))
        return VI_ERROR_PARAMETER4;
    
    if (rsfsh_invalidViInt32Range (seconds, 0, 59))
        return VI_ERROR_PARAMETER5;
    
    sprintf (buffer, "MMEM:FILE:TIME %s, %ld, %ld, %ld", fileName, hours, minutes, seconds);
    
    return rsfsh_sendCmd (instrSession, buffer);

}

/*===========================================================================*/
/* Function: Get File Time
/* Purpose:  This function queries the date of the indicated file. The 
/*           sequence of entry is year, month, day.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getFileTime(
    ViSession   instrSession,
    ViString    fileName,
    ViChar _VI_FAR time[]
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    sprintf (buffer, "MMEM:FILE:TIME? %s", fileName);
    
    return rsfsh_getViString (instrSession,
                              buffer,
                              time,
                              1024,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Data Set File Operations
/* Purpose:  This function allows to provide data set operation for 
/*           Save/Recall items of memory subsystem.
/*           
/*           The various drives can be addressed via the "mass storage unit 
/*           specifier" <msus> using the conventional DOS syntax. The 
/*           internal hard disk is addressed by "D:", the floppy disk drive 
/*           by "A:".
/*           
/*           For reasons of compatibility with the FSE instruments, 
/*           addressing the hard disk by "C:" is also accepted. Since hard 
/*           disk "C:" is reserved for instrument software, all read and 
/*           write operations are rerouted to hard disk "D:" in normal 
/*           operation (service level 0).
/*           
/*           The file names <file_name> are indicated as string parameters 
/*           with the commands being enclosed in quotation marks. They also 
/*           comply with DOS conventions. DOS file names consist of max. 8 
/*           ASCII characters and an extension of up to three characters
/*           separated from the file name by a dot "." Both, the dot and the 
/*           extension are optional. The dot is not part of the file name. 
/*           DOS file names do not distinguish between uppercase and 
/*           lowercase notation. All letters and digits are permitted as well 
/*           as the special characters "_", "^", "$", "~", "!", "#", "%", 
/*           "&", "-", "{", "}", "(", ")", "@" and "`". Reserved file names 
/*           are CLOCK$, CON, AUX, COM1 to COM4, LPT1 to LPT3,
/*           NUL and PRN. The two characters "*" and "?" have the function 
/*           of so-called "wildcards", i.e., they are variables for
/*           selection of several files. The question mark "?" replaces 
/*           exactly one character, the asterisk means any
/*           of the remaining characters in the file name. "*.*" thus means 
/*           all files in a directory.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_DataSetFileOperations(
    ViSession   instrSession,
    ViInt32 operation,
    ViString    path
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;
    
    switch (operation)
    {
        case RSFSH_VAL_MEM_SAVE:
            sprintf (buffer, "MMEM:STOR:STAT 1,%s", path);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;
            
        case RSFSH_VAL_MEM_RECALL:
            sprintf (buffer, "MMEM:LOAD:STAT 1,%s", path);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;
            
        case RSFSH_VAL_MEM_DELETE:
            sprintf (buffer, "MMEM:CLEAR:STAT 1,%s", path);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;
            
        case RSFSH_VAL_MEM_STARTUP:
            sprintf (buffer, "MMEM:LOAD:AUTO 1,%s", path);
            rsfsh_status = rsfsh_sendCmd (instrSession, buffer);
            break;  
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Display
/* Purpose:  This function configures display.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureDisplay(
    ViSession   instrSession,
    ViReal64    brightness,
    ViInt32 colorScheme,
    ViInt32 dateFormat
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViReal64Range (brightness, 0.00, 1.00))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (colorScheme, 0, 1))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViInt32Range (dateFormat, 0, 1))
        return VI_ERROR_PARAMETER4;
    
    CHECKERR (rsfsh_setDisplayBrightness (instrSession, brightness));
    CHECKERR (rsfsh_setDisplayColorScheme (instrSession, colorScheme));
    CHECKERR (rsfsh_setDisplayDateFormat (instrSession, dateFormat));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Display Color Scheme Default
/* Purpose:  This function resets the screen colors of all display items to 
/*           their default settings, i.e. to the color scheme.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_displayColorSchemeDefault(
    ViSession   instrSession
)
{
    return rsfsh_sendCmd (instrSession, "DISP:CMAP:DEF");
}

/*===========================================================================*/
/* Function: Configure Ethernet
/* Purpose:  This function configures the ethernet.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureEthernet(
    ViSession   instrSession,
    ViBoolean   dhcp,
    ViString    ipAddress,
    ViString    subnetmask,
    ViInt32 port
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViBooleanRange (dhcp))
        return VI_ERROR_PARAMETER2;
    
    CHECKERR (rsfsh_setEthernetDHCPMode (instrSession, dhcp));
    CHECKERR (rsfsh_setEthernetIPAddress (instrSession, ipAddress));
    CHECKERR (rsfsh_setEthernetSubmask (instrSession, subnetmask));
    CHECKERR (rsfsh_setEthernetPort (instrSession, port));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Volume
/* Purpose:  This function configures the volume.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureVolume(
    ViSession   instrSession,
    ViReal64    audioVolume,
    ViReal64    beeperVolume,
    ViReal64    keyVolume
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViReal64Range (audioVolume, 0.00, 1.00))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViReal64Range (beeperVolume, 0.00, 1.00))
        return VI_ERROR_PARAMETER3;
    if (rsfsh_invalidViReal64Range (keyVolume, 0.00, 1.00))
        return VI_ERROR_PARAMETER4;
    
    CHECKERR (rsfsh_setAudioVolume (instrSession, audioVolume));
    CHECKERR (rsfsh_setBeeperVolume (instrSession, beeperVolume));
    CHECKERR (rsfsh_setKeyVolume (instrSession, keyVolume));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Configure Unit
/* Purpose:  This function configures the instrument's default units.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_configureUnit(
    ViSession   instrSession,
    ViInt32 unitLength,
    ViInt32 powerUnit
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    
    if (rsfsh_invalidViInt32Range (unitLength, 0, 1))
        return VI_ERROR_PARAMETER2;
    if (rsfsh_invalidViInt32Range (powerUnit, 0, 8))
        return VI_ERROR_PARAMETER3;
    
    CHECKERR (rsfsh_setUnitLength (instrSession, unitLength));
    CHECKERR (rsfsh_setUnitPower (instrSession, powerUnit));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Display Brightness
/* Purpose:  This function sets the brightness of the display backlight.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDisplayBrightness(
    ViSession   instrSession,
    ViReal64    brightness
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "DISP:BRIG",
                              brightness,
                              0.00, 1.00,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Display Brightness
/* Purpose:  This function queries the brightness of the display backlight.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDisplayBrightness(
    ViSession   instrSession,
    ViReal64*   brightness
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "DISP:BRIG?",
                              brightness,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Display Color Scheme
/* Purpose:  This function sets the color scheme of the display to either 
/*           color or black and white.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDisplayColorScheme(
    ViSession   instrSession,
    ViInt32 colorScheme
)
{
    return rsfsh_setEnum (instrSession, 
                          "DISP:CMAP",
                          colorScheme,
                          colorSchemeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Display Color Scheme
/* Purpose:  This function queries the color scheme of the display to either 
/*           color or black and white.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDisplayColorScheme(
    ViSession   instrSession,
    ViInt32*    colorScheme
)
{
    return rsfsh_getEnum (instrSession, 
                          "DISP:CMAP?",
                          colorScheme,
                          colorSchemeArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Display Date Format
/* Purpose:  This function sets the display date format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setDisplayDateFormat(
    ViSession   instrSession,
    ViInt32 dateFormat
)
{
    return rsfsh_setEnum (instrSession, 
                          "DISP:DATE:FORM",
                          dateFormat,
                          dateFormatArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Display Date Format
/* Purpose:  This function queries the display date format.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getDisplayDateFormat(
    ViSession   instrSession,
    ViInt32*    dateFormat
)
{
    return rsfsh_getEnum (instrSession, 
                          "DISP:DATE:FORM?",
                          dateFormat,
                          dateFormatArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get System Memory Info
/* Purpose:  This function returns the total and used memory parameters.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getSystemMemoryInfo (ViSession instrSession,
                                             ViInt32 *totalMemory,
                                             ViInt32 *usedMemory)
{
    ViStatus rsfsh_status = VI_SUCCESS;
	
	CHECKERR (rsfsh_getViInt32 (instrSession, "SYST:MEM:TOT?", totalMemory, VI_ERROR_PARAMETER2));
	CHECKERR (rsfsh_getViInt32 (instrSession, "SYST:MEM:USED?", usedMemory, VI_ERROR_PARAMETER3));
	
	return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Store Memory Info
/* Purpose:  This function returns the store memory parameters.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getStoreMemoryInfo (ViSession instrSession,
                                            ViInt32 *totalMemory,
                                            ViInt32 *usedMemory,
                                            ViInt32 *freeMemory)
{
    ViStatus rsfsh_status = VI_SUCCESS;

	CHECKERR (rsfsh_getViInt32 (instrSession, "SYST:STOR:TOT?", totalMemory, VI_ERROR_PARAMETER2));
	CHECKERR (rsfsh_getViInt32 (instrSession, "SYST:STOR:USED?", usedMemory, VI_ERROR_PARAMETER3));
	CHECKERR (rsfsh_getViInt32 (instrSession, "SYST:STOR:FREE?", freeMemory, VI_ERROR_PARAMETER4));
	
	return rsfsh_status;
}

/*===========================================================================*/
/* Function: System Reboot
/* Purpose:  This function reboots system.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_SystemReboot (ViSession instrSession)
{
    return rsfsh_sendCmd (instrSession, "SYST:REB");
}

/*===========================================================================*/
/* Function: Get Ethernet MAC Address
/* Purpose:  This function queries the MAC address of the device.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getEthernetMACAddress(
    ViSession   instrSession,
    ViChar _VI_FAR  macAddress[]
)
{
    return rsfsh_getViString (instrSession,
                              "SYST:COMM:LAN:ETH?",
                              macAddress,
                              1024,
                              VI_ERROR_PARAMETER2);
                              
}

/*===========================================================================*/
/* Function: Set Ethernet Submask
/* Purpose:  This function sets the subnet mask address of the R&S FSH.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setEthernetSubmask(
    ViSession   instrSession,
    ViString    subnetmask
)
{
    return rsfsh_setViString (instrSession,
                              "SYST:COMM:LAN:SUBM",
                              subnetmask,
                              VI_ERROR_PARAMETER2);
                              
}

/*===========================================================================*/
/* Function: Get Ethernet Submask
/* Purpose:  This function queries the subnet mask address of the R&S FSH.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getEthernetSubmask(
    ViSession   instrSession,
    ViChar _VI_FAR  subnetmask[]
)
{
    return rsfsh_getViString (instrSession,
                              "SYST:COMM:LAN:SUBM?",
                              subnetmask,
                              1024,
                              VI_ERROR_PARAMETER2);
                              
}

/*===========================================================================*/
/* Function: Set Ethernet IP Address
/* Purpose:  This function sets the IP address of the R&S FSH.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setEthernetIPAddress(
    ViSession   instrSession,
    ViString    ipAddress
)
{
    return rsfsh_setViString (instrSession,
                              "SYST:COMM:SOCK:ADDR",
                              ipAddress,
                              VI_ERROR_PARAMETER2);
                              
}

/*===========================================================================*/
/* Function: Get Ethernet IP Address
/* Purpose:  This function queries the IP address of the R&S FSH.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getEthernetIPAddress(
    ViSession   instrSession,
    ViChar _VI_FAR  ipAddress[]
)
{
    return rsfsh_getViString (instrSession,
                              "SYST:COMM:SOCK:ADDR?",
                              ipAddress,
                              1024,
                              VI_ERROR_PARAMETER2);
                              
}

/*===========================================================================*/
/* Function: Set Ethernet DHCP Mode
/* Purpose:  This function sets the Dynamic Host Configuration Protocol 
/*           (DHCP) state.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setEthernetDHCPMode(
    ViSession   instrSession,
    ViBoolean   dhcpMode
)
{
    return rsfsh_setViBoolean (instrSession, 
                               "SYST:COMM:SOCK:DHCP:STAT",
                               dhcpMode,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Ethernet DHCP Mode
/* Purpose:  This function queries the Dynamic Host Configuration Protocol 
/*           (DHCP) state.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getEthernetDHCPMode(
    ViSession   instrSession,
    ViBoolean*  dhcpMode
)
{
    return rsfsh_getViBoolean (instrSession, 
                               "SYST:COMM:SOCK:DHCP:STAT?",
                               dhcpMode,
                               VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Ethernet Port
/* Purpose:  This function sets the port number for the connection.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setEthernetPort(
    ViSession   instrSession,
    ViInt32 port
)
{
    return rsfsh_setViInt32 (instrSession, 
                             "SYST:COMM:SOCK:PORT",
                             port,
                             0,0,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Ethernet Port
/* Purpose:  This function queries the port number for the connection.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getEthernetPort(
    ViSession   instrSession,
    ViInt32*    port
)
{
    return rsfsh_getViInt32 (instrSession, 
                             "SYST:COMM:SOCK:PORT?",
                             port,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Audio Volume
/* Purpose:  This function sets the volume of the internal speaker.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setAudioVolume(
    ViSession   instrSession,
    ViReal64    audioVolume
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "SYST:AUD:VOL",
                              audioVolume,
                              0.00, 1.00,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Audio Volume
/* Purpose:  This function queries the volume of the internal speaker.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAudioVolume(
    ViSession   instrSession,
    ViReal64*   audioVolume
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "SYST:AUD:VOL?",
                              audioVolume,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Beeper Volume
/* Purpose:  This function sets the volume of the beeper, activated with
/*           CALCulate:LIMit<1|2>:BEEP[:STATe]. The unit is percent.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setBeeperVolume(
    ViSession   instrSession,
    ViReal64    beeperVolume
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "SYST:BEEP:VOL",
                              beeperVolume,
                              0.00, 1.00,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Beeper Volume
/* Purpose:  This function queries the volume of the beeper, activated with
/*           CALCulate:LIMit<1|2>:BEEP[:STATe]. The unit is percent.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getBeeperVolume(
    ViSession   instrSession,
    ViReal64*   beeperVolume
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "SYST:BEEP:VOL?",
                              beeperVolume,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Key Volume
/* Purpose:  This function sets the volume of the keyboard clicking. The 
/*           unit is percent.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setKeyVolume(
    ViSession   instrSession,
    ViReal64    keyVolume
)
{
    return rsfsh_setViReal64 (instrSession, 
                              "SYST:BEEP:KEY:VOL",
                              keyVolume,
                              0.00, 1.00,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Key Volume
/* Purpose:  This function queries the volume of the keyboard clicking. The 
/*           unit is percent.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getKeyVolume(
    ViSession   instrSession,
    ViReal64*   keyVolume
)
{
    return rsfsh_getViReal64 (instrSession, 
                              "SYST:BEEP:KEY:VOL?",
                              keyVolume,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set BNC Mode
/* Purpose:  This function sets the function the BNC socket indicated by the 
/*           numeric suffix is used for.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setBNCMode(
    ViSession   instrSession,
    ViInt32 bnc,
    ViInt32 bncMode
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (bnc, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "SYST:BNC%ld:MODE", bnc);
    
    return rsfsh_setEnum (instrSession,
                          buffer,
                          bncMode,
                          bncModeArr,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get BNC Mode
/* Purpose:  This function queries the function the BNC socket indicated by 
/*           the numeric suffix is used for.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getBNCMode(
    ViSession   instrSession,
    ViInt32 bnc,
    ViInt32*    bncMode
)
{
    ViChar      buffer[BUFFER_SIZE];
    
    if (rsfsh_invalidViInt32Range (bnc, 1, 2))
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "SYST:BNC%ld:MODE?", bnc);
    
    return rsfsh_getEnum (instrSession,
                          buffer,
                          bncMode,
                          bncModeArr,
                          VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Get Auto Off Time
/* Purpose:  This function queries the time when R&S FSH automatically turns 
/*           off when not operating (in battery mode).
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAutoOffTime(
    ViSession   instrSession,
    ViInt32*    autoOffTime
)
{
    return rsfsh_getViInt32 (instrSession,
                             "SYST:POW:AUTO:OFF:TIME?",
                             autoOffTime,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Power Source
/* Purpose:  This function queries whether R&S FSH is battery-powered or 
/*           line-powered.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getPowerSource(
    ViSession   instrSession,
    ViChar _VI_FAR  powerSource[]
)
{
    return rsfsh_getViString (instrSession,
                              "SYST:POW:SOUR?",
                              powerSource,
                              1024,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Remaining Battery Power
/* Purpose:  This function queries the remaining power of the battery.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getRemainingBatteryPower(
    ViSession   instrSession,
    ViInt32*    remainingBatteryPower
)
{
    return rsfsh_getViInt32 (instrSession,
                             "SYST:POW:STAT?",
                             remainingBatteryPower,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Language
/* Purpose:  This function sets the system language.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setLanguage(
    ViSession   instrSession,
    ViString    language
)
{
    return rsfsh_setViStringQuoted (instrSession,
                                    "SYST:LANG",
                                    "'",
                                    language,
                                    VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Language
/* Purpose:  This function queries the system language.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLanguage(
    ViSession   instrSession,
    ViChar _VI_FAR  language[]
)
{
    return rsfsh_getViString (instrSession,
                              "SYST:LANG?",
                              language,
                              1024,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Language Catalog
/* Purpose:  This function lists all available system languages.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getLanguageCatalog(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar _VI_FAR  catalogList[]
)
{
    return rsfsh_getCatalog (instrSession,
                             "SYST:LANG:CAT?",
                             arraySize,
                             catalogList,
                             VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Set Reference Oscillator Source
/* Purpose:  This function selects the source of the frequency reference
             oscillator.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setReferenceOscillatorSource (ViSession instrSession,
                                                      ViInt32 source)
{
   
    return rsfsh_setEnum (instrSession,
                          "ROSC:SOUR",
                          source,
                          sourceArr,
                          VI_ERROR_PARAMETER2);                                                           
}                          

/*===========================================================================*/
/* Function: Get Reference Oscillator Source
/* Purpose:  This function queries the source of the frequency reference
             oscillator.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getReferenceOscillatorSource (ViSession instrSession,
                                                      ViInt32 *source)
{
    return rsfsh_getEnum (instrSession,
                          "ROSC:SOUR?",
                          source,
                          sourceArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Ethernet Remote Desktop Port
/* Purpose:  This function selects the port number for the remote desktop
/*           connection.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setRemoteDesktopPort (ViSession instrSession,
                                              ViInt32 portNumber)
{
    return rsfsh_setViInt32 (instrSession, "SYST:REM:DISP", portNumber, 0, 0, VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Ethernet Remote Desktop Port
/* Purpose:  This function queries the port number for the remote desktop
/*           connection.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getRemoteDesktopPort (ViSession instrSession,
                                              ViInt32 *portNumber)
{
    return rsfsh_getViInt32 (instrSession, "SYST:REM:DISP?", portNumber, VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set GPS State
/* Purpose:  This function activates and deactivates the GPS receiver (R&S
/*           HA-Z240).
/*           Note that the GPS receiver only works if a connection between
/*           the R&S FSH and a GPS signal transmitter is established.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setGPSState (ViSession instrSession, ViBoolean GPS)
{
    return rsfsh_setViBoolean (instrSession,
                    "SYSTem:POSition:GPS:STATe",
                    GPS,
                    VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get GPS State
/* Purpose:  This function returns the GPS receiver (R&S
/*           HA-Z240) state.
/*           Note that the GPS receiver only works if a connection between
/*           the R&S FSH and a GPS signal transmitter is established.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGPSState (ViSession instrSession, ViBoolean *GPS)
{
    return rsfsh_getViBoolean (instrSession,
                    "SYSTem:POSition:GPS:STATe?",
                    GPS,
                    VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get GPS Connected
/* Purpose:  This function queries if the R&S FSH is currently connected to
/*           the GPS receiver (R&S HAZ240).
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGPSConnected (ViSession instrSession,
                                         ViBoolean *GPSConnected)
{
    return rsfsh_getViBoolean (instrSession,
                    "SYSTem:POSition:GPS:CONNected?",
                    GPSConnected,
                    VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get GPS Frequency Correction
/* Purpose:  This function queries the frequency correction factor. This
/*           factor is calculated from a reference signal provided by the GPS
/*           receiver R&S HA-Z240. The reference signal is used to determine
/*           the deviation of the internal clock of the instrument, which can
/*           be turned into a correction factor for the measured frequency.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGPSFrequencyCorrection (ViSession instrSession,
                                                   ViReal64 *correctionFactor)
{
    return rsfsh_getViReal64 (instrSession,
                    "SYSTem:POSition:GPS:CORRection:FREQuency?",
                    correctionFactor,
                    VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get GPS Signal Quality
/* Purpose:  This function queries the quality of the GPS signal.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGPSSignalQuality (ViSession instrSession,
                                             ViInt32 *signalQuality)
{
    return rsfsh_getEnum(instrSession,
                        "SYSTem:POSition:GPS:QUALity?",
                        signalQuality,
                        gpsSignalQualityArr,
                        VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get GPS Satellites
/* Purpose:  This function queries the number of tracked satellites.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGPSSatellites (ViSession instrSession,
                                          ViInt32 *satellites)
{
    return rsfsh_getViInt32 (instrSession,
                    "SYSTem:POSition:GPS:SATellites?",
                    satellites,
                    VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get GPS Latitude
/* Purpose:  This function queries the latitude of the current position of
/*           the R&S FSH.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGPSLatitude (ViSession instrSession,
                                        ViInt32 bufferSize, ViChar latitude[])
{
    return rsfsh_getViString (instrSession,
                             "SYSTem:POSition:LATitude?",
                             latitude,
                             bufferSize,
                             VI_ERROR_PARAMETER3);    
}

/*===========================================================================*/
/* Function: Get GPS Longitude
/* Purpose:  This function queries the longitude of the current position of
/*           the R&S FSH.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGPSLongitude (ViSession instrSession,
                                         ViInt32 bufferSize, ViChar longitude[])
{
    return rsfsh_getViString (instrSession,
                             "SYSTem:POSition:LONGitude?",
                             longitude,
                             bufferSize,
                             VI_ERROR_PARAMETER3);    
}

/*===========================================================================*/
/* Function: Get GPS Altitude
/* Purpose:  This function queries the longitude of the current position of
/*           the R&S FSH.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGPSAltitude (ViSession instrSession,
                                        ViInt32 bufferSize, ViChar altitude[])
{
   return rsfsh_getViString (instrSession,
                             "SYST:POS:ALT?",
                             altitude,
                             bufferSize,
                             VI_ERROR_PARAMETER3); 
}

/*===========================================================================*/
/* Function: Get GPS Position Information
/* Purpose:  This function queries the validity of the position information.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getGPSPositionInformation (ViSession instrSession,
                                                   ViBoolean *positionInformation)
{
    return rsfsh_getViBoolean (instrSession,
                    "SYSTem:POSition:VALid?",
                    positionInformation,
                    VI_ERROR_PARAMETER2);    
}

/*===========================================================================*/
/* Function: Set Unit Length
/* Purpose:  This function sets the instrument's default unit of length.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setUnitLength(
    ViSession   instrSession,
    ViInt32 unitLength
)
{
    return rsfsh_setEnum (instrSession,
                          "UNIT:LENG",
                          unitLength,
                          unitLengthArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Unit Length
/* Purpose:  This function queries the instrument's default unit of length.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getUnitLength(
    ViSession   instrSession,
    ViInt32*    unitLength
)
{
    return rsfsh_getEnum (instrSession,
                          "UNIT:LENG?",
                          unitLength,
                          unitLengthArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Set Unit Power
/* Purpose:  This function sets the default unit.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setUnitPower(
    ViSession   instrSession,
    ViInt32 powerUnit
)
{
    return rsfsh_setEnum (instrSession,
                          "UNIT:POW",
                          powerUnit,
                          powerUnitArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Unit Power
/* Purpose:  This function queries the default unit.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getUnitPower(
    ViSession   instrSession,
    ViInt32*    powerUnit
)
{
    return rsfsh_getEnum (instrSession,
                          "UNIT:POW?",
                          powerUnit,
                          powerUnitArr,
                          VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Calibration Mode
/* Purpose:  This function queries the measurement mode which is calibrated.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCalibrationMode(
    ViSession   instrSession,
    ViInt32*    calibrationMode
)
{
    return rsfsh_getViInt32 (instrSession,
                             "CAL:MODE?",
                             calibrationMode,
                             VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Get Calibration Status
/* Purpose:  This function queries whether the R&S FSH is calibrated for the 
/*           current measurement mode or not.
/*           
/*           This function is not compatible with FSH3 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getCalibrationStatus(
    ViSession   instrSession,
    ViChar _VI_FAR  calibrationStatus[]
)
{
    return rsfsh_getViString (instrSession,
                              "CAL:STAT?",
                              calibrationStatus,
                              1024,
                              VI_ERROR_PARAMETER2);
}

/// HIFN  This function sets the port to be calibrated.
/// HIFN     
/// HIFN     This function is not compatible with FSH3 instrument.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALibration:START?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR method/
/// HIPAR calibration/Returns selected calibration.
ViStatus _VI_FUNC rsfsh_calibrationStart(
	ViSession	instrSession,
	ViInt32	method,
	ViChar _VI_FAR	calibration[]
)
{
    ViChar	cmd[200] = "";

    sprintf (cmd, "CAL:START? %s", methodArr[method]);  

    return rsfsh_getViString (instrSession,
                              cmd,
                              calibration,
                              1024,
                              VI_ERROR_PARAMETER3);
}


/// HIFN  This function resumes calibration after a calibration standard has 
/// HIFN  been connected.
/// HIFN     After the command has been sent, it returns instructions on what to do 
/// HIFN  to continue the calibration process and finish it successfully. 
/// HIFN     
/// HIFN     This function is not compatible with FSH3 instrument.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALibration:CONTinue?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR calibrationContinue/This control displays whether the connected calibration to continue.
ViStatus _VI_FUNC rsfsh_calibrationContinue(
	ViSession	instrSession,
	ViChar _VI_FAR	calibrationContinue[]
)
{
    return rsfsh_getViString (instrSession,
                              "CAL:CONT?",
                              calibrationContinue,
                              1024,
                              VI_ERROR_PARAMETER2);
}


/// HIFN  This function aborts calibration.
/// HIFN     
/// HIFN     This function is not compatible with FSH3 instrument.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALibration:ABORt
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsfsh_calibrationAbort(
	ViSession	instrSession
)
{
    return rsfsh_sendCmd (instrSession, "CAL:ABOR");
}

/*****************************************************************************/
/*-------- INSERT USER-CALLABLE INSTRUMENT-DEPENDENT ROUTINES HERE ----------*/
/*****************************************************************************/


/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus rsfsh_status = VI_SUCCESS;
    ViInt32     acknowledge = 0;

    CHECKERR (viQueryf (instrSession, "%s", "%ld", writeBuffer, &acknowledge));

    /*
        After receiving a SET, GET or CMD command the R&SFSH3 responds with an
        "acknowledge" message. The acknowlede message consists of an ASCII digit
        ("0" for no error) followed by a Carriage Return <cr>. The response message
        indicates if the command or parameter is valid.
    */
    switch (acknowledge)
        {
        case 0:
            rsfsh_status = VI_SUCCESS;
        break;
        case 1:
            rsfsh_status = RSFSH_ERROR_SYNTAX_ERROR;
        break;
        case 2:
            rsfsh_status = RSFSH_ERROR_EXECUTION_ERROR;
        break;
        case 3:
            rsfsh_status = RSFSH_ERROR_DATASET_STORAGE_FULL;
        break;
        case 4:
            rsfsh_status = RSFSH_ERROR_NOT_ALLOWED;
        break;
        case 5:
            rsfsh_status = RSFSH_ERROR_OUT_OF_RANGE;
        break;
        default:
            rsfsh_status = RSFSH_ERROR_UNKNOWN_ERROR;
        }

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_readInstrData (ViSession instrSession,
                                        ViInt32 numberBytesToRead,
                                        ViChar _VI_FAR readBuffer[],
                                        ViPInt32 numBytesRead)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViUInt32    tmpCnt;

    if (numBytesRead)
        *numBytesRead = 0L;

    CHECKERR (viRead (instrSession, (ViBuf)readBuffer, numberBytesToRead, &tmpCnt));

    if (numBytesRead)
        *numBytesRead = (ViInt32)tmpCnt;

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Preset Settings
/* Purpose:  This function selects the preset settings. When selecting the 
/*           Custom Preset Setting a Preset Dataset must be available.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_presetSettings(
    ViSession   instrSession,
    ViInt32 presetSettings
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (rsfsh_invalidViInt32Range (presetSettings, 0, 1))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "PRESETSET,%ld", presetSettings);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;    
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_reset (ViSession instrSession)
{
    ViStatus rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:/*  Initialize the instrument to a known state.  */
        CHECKERR (rsfsh_cmdCommand (instrSession, "PRESET"));
        CHECKERR (rsfsh_defaultInstrSetup (instrSession));
    break;
    case FSH4:
         rsfsh_status = rsfsh_sendCmd (instrSession, "*RST");
    break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Temperature
/* Purpose:  This function returns the current temperature inside the 
/*           instrument.
/*           
/*           This function is not compatible with FSH4 & FSH8 instrument.
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_temperature(
    ViSession   instrSession,
    ViReal64*   temperature
)
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
    case FSH3:
    if (!temperature)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsfsh_getCommand (instrSession, "TEMP", buffer));
    sscanf (buffer, "%Le", temperature);
    break;
    case FSH4:
        break;
    }

    return rsfsh_status;     
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_self_test (ViSession instrSession,
                                    ViPInt16 testResult,
                                    ViChar _VI_FAR testMessage[])
{
    instrSession;
    testResult;
    testMessage;
    return VI_WARN_NSUP_SELF_TEST;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result. If the error query function is not supported by the */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_ERROR_QUERY.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_error_query (ViSession instrSession,
                                      ViPInt32 errorCode,
                                      ViChar _VI_FAR errorMessage[])
{
    ViStatus rsfsh_status = VI_SUCCESS;
    ViChar   msg [256] = "";
    ViInt32  code = 0;
    rsfsh_instrRange   instrPtr;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    switch (instrPtr -> fsh3)
    {
        case FSH3:
        rsfsh_status = VI_WARN_NSUP_ERROR_QUERY;
        break;
        
        case FSH4:
        CHECKERR (viPrintf (instrSession, ":SYST:ERR?\n"));
        CHECKERR (viScanf (instrSession, "%ld,\"%[^\"]", &code, msg));
        if (errorCode != NULL)
            *errorCode = code;
    
        if (errorMessage != NULL)
            strncpy (errorMessage, msg, 255);
        break;
    }
        
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_error_message (ViSession instrSession,
                                        ViStatus statusCode,
                                        ViChar _VI_FAR errMessage[])
{
    ViStatus rsfsh_status = VI_SUCCESS;
    ViInt16 i;

    static rsfsh_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER9 ,  "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},
        /* - instrument driver specific error codes - */
        {RSFSH_ERROR_INSTRUMENT_ERROR,      "ERROR: Instrument status error."},
        {RSFSH_ERROR_UNKNOWN_ERROR,         "ERROR: Unknown Error."},
        {RSFSH_ERROR_SYNTAX_ERROR,          "ERROR: Syntax Error."},
        {RSFSH_ERROR_EXECUTION_ERROR,       "ERROR: Execution Error."},
        {RSFSH_ERROR_DATASET_STORAGE_FULL,  "ERROR: Dataset Storage Full."},
        {RSFSH_ERROR_NOT_ALLOWED,           "ERROR: Not Allowed."},
        {RSFSH_ERROR_OUT_OF_RANGE,          "ERROR: Out of Range."},
        {VI_NULL, VI_NULL}
    };

    rsfsh_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (rsfsh_status == VI_WARN_UNKNOWN_STATUS)
        {
        for (i = 0; statusDescArray[i].stringName; i++)
            {
            if (statusDescArray[i].stringVal == statusCode)
                {
                sprintf (errMessage, "%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
                }
            }
        sprintf (errMessage, "Unknown Error 0x%X", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
        }

    rsfsh_status = VI_SUCCESS;
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get All Errors                                                  */
/* Purpose:  This function retrieves all entries in the error queue.         */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getAllErrors (ViSession instrSession,
                                      ViInt32 length, 
                                      ViChar _VI_FAR errors[])
{
    return rsfsh_getViString (instrSession,
                              "SYST:ERR:ALL?",
                              errors,
                              length,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_revision_query (ViSession instrSession,
                    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    rsfsh_instrRange   instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    switch (instrPtr -> fsh3)
    {
        case FSH3:
    CHECKERR(rsfsh_getCommand (instrSession, "IDN?", buffer));
            break;
        case FSH4:
		default:
            CHECKERR (viQueryf (instrSession, "*IDN?\n", "%s", buffer));
            break;
    }
    sscanf (buffer, "%*[^,],%*[^,],%*[^,],%[^\r\n]", instrRev);
    sprintf (driverRev, "%s", RSFSH_REVISION);

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: System Version                                                  */
/* Purpose:  This function queries the number of the SCPI version, which is  */
/*           relevant for the instrument.                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_systemVersion (ViSession instrSession,
                                       ViInt32 length, 
                                       ViChar _VI_FAR systemVersion[])
{
    return rsfsh_getViString (instrSession,
                              "SYST:VERS?",
                              systemVersion,
                              length,
                              VI_ERROR_PARAMETER3);
}

/// HIFN  This function defines a shift of the system time to select another 
/// HIFN  time zone. 
/// HIFN     
/// HIFN     This function is not compatible with FSH3 instrument.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:TZONee
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR hours/Sets the hours for time shifting.
/// HIPAR minutes/Sets the minutes for time shifting.
ViStatus _VI_FUNC rsfsh_setSystemTimeShift(
	ViSession	instrSession,
	ViInt32	hours,
	ViInt32	minutes
)
{
    ViChar	cmd[250] = "";
	
	if (rsfsh_invalidViInt32Range (hours, 0, 23))
        return VI_ERROR_PARAMETER2;
	if (rsfsh_invalidViInt32Range (minutes, 0, 59))
        return VI_ERROR_PARAMETER3;

    sprintf (cmd, "SYST:TZON %ld,%ld", hours, minutes);  

    return rsfsh_sendCmd (instrSession, cmd);
}


/// HIFN  This function queries a shift of the system time to select another 
/// HIFN  time zone. 
/// HIFN     
/// HIFN     This function is not compatible with FSH3 instrument.
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:TZONee?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR length/Sets the allocated memory length of System Version string.
/// HIPAR timeShift/Returns the time shift.
ViStatus _VI_FUNC rsfsh_getSystemTimeShift(
	ViSession	instrSession,
	ViInt32	length,
	ViChar _VI_FAR	timeShift[]
)
{
    return rsfsh_getViString (instrSession,
                              "SYST:TZON?",
                              timeShift,
                              length,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: Preset Factory                                                  */
/* Purpose:  This function initiates an instrument reset back to factory     */
/*           settings.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_presetFactory (ViSession instrSession)
{
    return rsfsh_sendCmd (instrSession, "SYST:PRES:FACT");
}

/*===========================================================================*/
/* Function: Read To File From Instrument
/* Purpose:  This function is used to read data from the instrument and 
/*           write it to a user specified file on the host computer.
/*           
/*           Attribute(s):
/*           -
/*           
/*           
/*           FORMat[:DATA] REAL,32
/*           MMEMory:DATA? <file_name>
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_ReadToFileFromInstrument(
    ViSession   instrSession,
    ViString    fileName,
    ViInt32 binaryBlockLength,
    ViChar _VI_FAR  binaryBlockData[]
)
{
    ViStatus    rsfsh_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViUInt32    count           = 0,
                lengthOfLength  = 0;
    ViChar      buffer[BUFFER_SIZE],
                format[32],
                *pbuffer, *p2buffer;

    CHECKNULL (fileName, 2);

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    sprintf (buffer, ":MMEM:DATA? '%s'\n", fileName);

    CHECKERR (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) strlen(buffer), VI_NULL));
    CHECKERR (rsfsh_readDataUnknownLength (instrSession, &pbuffer, &count));

    p2buffer = pbuffer;

    if (sscanf (p2buffer, "#%1d", &lengthOfLength) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    sprintf (format, "%%%ldd", lengthOfLength);
    p2buffer += 2;

    if (sscanf (p2buffer, format, &count) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    p2buffer += lengthOfLength;

    if (intfType == VI_INTF_GPIB)
        if ((rsfsh_status = viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL)) < VI_SUCCESS)
            {
            free (pbuffer);
            return rsfsh_status;
            }

    if ((rsfsh_status = rsfsh_checkStatus (instrSession)) < VI_SUCCESS)
        {
        free (pbuffer);
        return rsfsh_status;
        }

    if (count > (ViUInt32) binaryBlockLength)
        {
        rsfsh_status = RSFSH_WARNING_SMALL_BUFFER;
        count = binaryBlockLength;
        memcpy (binaryBlockData, p2buffer, count);
        }
    else
        {
        rsfsh_status = VI_SUCCESS;
        memcpy (binaryBlockData, p2buffer, count);
        }

    free (pbuffer);

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Write From File To Instrument
/* Purpose:  This function is used to read data from the host computer and 
/*           write it to a user specified file in the instrument.
/*           
/*           Attribute(s):
/*           -
/*           
/*           
/*           MMEMory:DATA <file_name>[,<block data>]
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_WriteFromFileToInstrument(
    ViSession   instrSession,
    ViString    fileName,
    ViInt32     binaryBlockLength,
    ViChar _VI_FAR  binaryBlockData[]
)
{
    ViStatus    rsfsh_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViChar*     pbuffer;
    ViChar*     p2buf;

    CHECKNULL (fileName, 2);
    CHECKNULL (binaryBlockData, 4);

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    p2buf = pbuffer = malloc (strlen (fileName) + binaryBlockLength + 1000);

    p2buf += sprintf (pbuffer, ":MMEM:DATA '%s',#%ld%ld",
                               fileName,
                               (ViInt32)(floor(log10(binaryBlockLength)) + 1),
                               binaryBlockLength);

    memcpy (p2buf, binaryBlockData, binaryBlockLength);
//    p2buf[binaryBlockLength] = '\n';

    rsfsh_status = viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) ((p2buf - pbuffer) + binaryBlockLength), VI_NULL);

    if (pbuffer)
        free (pbuffer);

    if (intfType == VI_INTF_GPIB)
        viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL);

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Status Register
/* Purpose:  This function sets the Enable, NTransition, and PTransition 
/*           bits     of the Main, Power, Limit, LMargin, ACP Limit, 
/*           Frequency, status questionable and operating registers.
/*           
/*           This function is not compatible with FSH3 instrument.
/*           
/*           Note:
/*           
/*           (1) For detailed description of Status Reporting System see 
/*           also operating manual.
/*           
/*           
/*           STATus:PRESet
/*           STATus:QUEStionable:POWer:ENABle
/*           STATus:QUEStionable:LIMit<1|2>:ENABle 
/*           STATus:QUEStionable:LMARgin<1|2>:ENABle 
/*           STATus:QUEStionable:SYNC:ENABle 
/*           STATus:QUEStionable:ACPLimit:ENABle 
/*           STATus:QUEStionable:FREQuency:ENABle
/*           STATus:QUEStionable:PNOise:ENABle
/*           STATus:QUEStionable:DIQ:ENABle 
/*           STATus:QUEStionable:POWer:PTRansition
/*           STATus:QUEStionable:LIMit<1|2>:PTRansition 
/*           STATus:QUEStionable:LMARgin<1|2>:PTRansition 
/*           STATus:QUEStionable:SYNC:PTRansition 
/*           STATus:QUEStionable:ACPLimit:PTRansition 
/*           STATus:QUEStionable:FREQuency:PTRansition
/*           STATus:QUEStionable:PNOise:PTRansition
/*           STATus:QUEStionable:DIQ:PTRansition
/*           STATus:QUEStionable:POWer:NTRansition
/*           STATus:QUEStionable:LIMit<1|2>:NTRansition 
/*           STATus:QUEStionable:LMARgin<1|2>:NTRansition 
/*           STATus:QUEStionable:SYNC:NTRansition 
/*           STATus:QUEStionable:ACPLimit:NTRansition 
/*           STATus:QUEStionable:FREQuency:NTRansition 
/*           STATus:QUEStionable:PNOise:NTRansition
/*           STATus:QUEStionable:DIQ:NTRansition
/*           STATus:OPERation:PCALibration:ENABle
/*           STATus:OPERation:PCALibration:PTRansition
/*           STATus:OPERation:PCALibration:NTRansition 
/*           STATus:OPERation:ENABle
/*           STATus:OPERation:PTRansition
/*           STATus:OPERation:NTRansition
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setStatusRegister(ViSession instrSession,
                ViInt32 registerOper, ViInt32 selStatusReg, ViInt32 enable,
                ViInt32 PTransition, ViInt32 NTransition)
{
    ViStatus    rsfsh_status    = VI_SUCCESS;
    
    if (rsfsh_invalidViInt32Range (registerOper, 0, 3))
        return VI_ERROR_PARAMETER2;
    
    switch (registerOper) {
        case 0:
            CHECKERR (viPrintf (instrSession, ":STAT:PRES\n"));
        break;
        case 1:
            if (rsfsh_invalidViInt32Range (selStatusReg, 0, 10))
                return VI_ERROR_PARAMETER3;
            

            CHECKERR (viPrintf (instrSession, ":STAT:QUES%s:ENAB %ld;PTR %ld"
                ";NTR %ld\n", selStatusRegArr[selStatusReg], enable, PTransition,
                NTransition));
        break;
        case 2:
            if (rsfsh_invalidViInt32Range (enable, 0, 65535))
                return VI_ERROR_PARAMETER4;
            if (rsfsh_invalidViInt32Range (PTransition, 0, 65535))
                return VI_ERROR_PARAMETER5;
            if (rsfsh_invalidViInt32Range (NTransition, 0, 65535))
                return VI_ERROR_PARAMETER6;
            
            CHECKERR (viPrintf (instrSession, ":STAT:OPER:ENAB %ld;PTR %ld"
                      ";NTR %ld\n", enable, PTransition, NTransition));
        break;
        case 3:
            if (rsfsh_invalidViInt32Range (enable, 0, 65535))
                return VI_ERROR_PARAMETER4;
            if (rsfsh_invalidViInt32Range (PTransition, 0, 65535))
                return VI_ERROR_PARAMETER5;
            if (rsfsh_invalidViInt32Range (NTransition, 0, 65535))
                return VI_ERROR_PARAMETER6;

            CHECKERR (viPrintf (instrSession, ":STAT:OPER:PCAL:ENAB %ld;PTR %ld"
                ";NTR %ld\n", enable, PTransition, NTransition));
        break;
    }
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Status Register
/* Purpose:  This function contains the commands for the querying status 
/*           reporting system.
/*           
/*           This function is not compatible with FSH3 instrument.
/*           
/*           Note:
/*           
/*           For detailed description of Status Reporting System see also 
/*           operating manual.
/*           
/*           
/*           STATus:PRESetSTATus:OPERation:EVENt?
/*           STATus:OPERation:CONDition?
/*           STATus:QUEStionable:EVENt?
/*           STATus:QUEStionable:CONDition?
/*           STATus:QUEStionable:POWer:EVENt? 
/*           STATus:QUEStionable:POWer:CONDition? 
/*           STATus:QUEStionable:LIMit<1|2>:EVENt?
/*           STATus:QUEStionable:LIMit<1|2>:CONDition? 
/*           STATus:QUEStionable:SYNC:EVENt? 
/*           STATus:QUEStionable:SYNC:CONDition? 
/*           STATus:QUEStionable:ACPLimit:EVENt? 
/*           STATus:QUEStionable:ACPLimit:CONDition? 
/*           STATus:QUEStionable:FREQuency:EVENt? 
/*           STATus:QUEStionable:FREQuency:CONDition? 
/*           STATus:QUEStionable:TRANsducer:EVENt? 
/*           STATus:QUEStionable:TRANsducer:CONDition? 
/*           STATus:QUEStionable?
/*           STATus:QUEStionable:LMARgin<1|2>:EVENt? 
/*           STATus:QUEStionable:LMARgin<1|2>:CONDition? 
/*           STATus:QUEStionable:PNOise:EVENt?
/*           STATus:QUEStionable:PNOise:CONDition?
/*           STATus:QUEStionable:DIQ:EVENt?
/*           STATus:QUEStionable:DIQ:CONDition?
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_getStatusRegister(ViSession instrSession,
                ViInt32 statusReg, ViPInt32 registerValue)
{
    ViStatus    rsfsh_status   = VI_SUCCESS;
   
    if (rsfsh_invalidViInt32Range (statusReg, 0, 28))
                return VI_ERROR_PARAMETER2;
    
    CHECKERR (viQueryf (instrSession, ":STAT:%s?\n", "%ld",
        statusRegArr[statusReg], registerValue));
    
    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Status Queue Next
/* Purpose:  This function returns the earliest entry to the error queue and 
/*           deletes it.
/*           
/*           Positive error numbers indicate device-specific errors, 
/*           negative error numbers are error messages defined by SCPI. If 
/*           the error queue is empty, the error number 0, "no error", is 
/*           returned. This command is identical with the command 
/*           SYSTem:ERRor.
/*           
/*           This function is not compatible with FSH3 instrument.
/*           
/*           
/*           STATus:QUEue[:NEXT]?
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_StatusQueueNext(
    ViSession   instrSession,
    ViReal64*   errorNumber
)
{
    return rsfsh_getViReal64 (instrSession,
                              "STAT:QUE:NEXT?",
                              errorNumber,
                              VI_ERROR_PARAMETER2);
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_close (ViSession instrSession)
{
    rsfsh_instrRange instrPtr = NULL;
    ViSession rmSession;
    ViStatus rsfsh_status = VI_SUCCESS;
    
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    CHECKERR( viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession));
        CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    switch (instrPtr -> fsh3)
    {
    case FSH3:
        
        /*- Go to the LOCAL state -*/
        CHECKERR (rsfsh_cmdCommand (instrSession, "LOCAL"));

        if (instrPtr != NULL)
            free (instrPtr);

        rsfsh_status = viClose (instrSession);
        viClose (rmSession);
        break;
    case FSH4:
        if (instrPtr != NULL)
        free (instrPtr);

        rsfsh_status = viClose (instrSession);
        (void) viClose (rmSession);
        break;
    }

    return rsfsh_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Swap 32bit Integer                                              */
/* Purpose:  Byte order for binary transfers depends on MOTOROLA or INTEL    */
/*           formats.                                                        */
/*===========================================================================*/
void rsfsh_SWAP_32(unsigned char *data, long points)
{
    /* MOTOROLA (byte order 0123) or INTEL (byte order 3210). */
    unsigned int    word = 0x1122;
    unsigned char   *p2word = (unsigned char*)&word;

    /* Instrument returns 4 byte numbers with LSB send first */
    if (*p2word == 0x11)
    {
       unsigned char    src[4];
       long             *p2data = (long*)data;
       int              idx;

       for (idx = 0; idx < points; idx++, p2data++)
          {
              *((long *)src) = *(p2data);
              ((unsigned char *)(p2data))[0] =  ((unsigned char*)(src))[3];
              ((unsigned char *)(p2data))[1] =  ((unsigned char*)(src))[2];
              ((unsigned char *)(p2data))[2] =  ((unsigned char*)(src))[1];
              ((unsigned char *)(p2data))[3] =  ((unsigned char*)(src))[0];
          }
    }

   return;
}


/*===========================================================================*/
/* Function: Check String                                                    */
/* Purpose:  This function checks a string value to see if it is             */
/*           of a defined length and initialized.                            */
/*===========================================================================*/
ViBoolean rsfsh_invalidViString (ViString val, ViUInt32 maxLength)
{
    if (val)
        return (ViBoolean)((strlen (val) > maxLength) ? VI_TRUE : VI_FALSE);

    return VI_TRUE;
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rsfsh_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rsfsh_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rsfsh_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession,
                             ViStatus currentStatus)
{
    rsfsh_instrRange instrPtr;

    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL)
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;

    return currentStatus;
}

/*===========================================================================*/
/* Function: Set FSH4 Measurement Mode                                       */
/* Purpose:  This function switches between the measurement modes by means   */ 
/*           of text parameters.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsfsh_setFSH4MeasurementMode (ViSession instrSession,
                                                ViInt32 measurementMode)

{
    ViStatus    rsfsh_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    if (rsfsh_invalidViInt32Range (measurementMode,0,2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "INST %s", measurementModeArr[measurementMode]);
    CHECKERR (rsfsh_setCommand (instrSession, buffer));

    return rsfsh_status;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument. This     */
/*           function is called by the rsfsh_reset operation and by the      */
/*           rsfsh_init function if the reset option has not been            */
/*           selected. This function is useful for configuring any           */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/
/*===========================================================================*/
ViStatus rsfsh_defaultInstrSetup (ViSession instrSession)
{
    ViStatus            rsfsh_status = VI_SUCCESS;
    rsfsh_instrRange   instrPtr;

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    strcpy (instrPtr -> instrDriverRevision, RSFSH_REVISION);
    instrPtr -> multiMarker = 1;
    instrPtr -> deltaMarker = 1;
    instrPtr -> errorChecking = VI_TRUE;
    

    /*
        CHECKERR (rsfsh_cmdCommand (instrSession, "REMOTE"));
    */

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Cmd Command                                                     */
/* Purpose:  CMD commands are used to initiate an action or a state          */
/*           transition in the instrument like marker-to-peak, preset, remote*/
/*           state, etc.                                                     */
/*===========================================================================*/
ViStatus rsfsh_cmdCommand (ViSession instrSession, ViString cmdCommand)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    CHECKERR(rsfsh_writeInstrData (instrSession, "CMD"));
    CHECKERR(rsfsh_writeInstrData (instrSession, cmdCommand));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Set Command                                                     */
/* Purpose:  SET commands are used to program parameters of the instrument   */
/*           like center frequency, span, etc.                               */
/*===========================================================================*/
ViStatus rsfsh_setCommand (ViSession instrSession, ViString setCommand)
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    CHECKERR(rsfsh_writeInstrData (instrSession, "SET"));
    CHECKERR(rsfsh_writeInstrData (instrSession, setCommand));

    return rsfsh_status;
}

/*===========================================================================*/
/* Function: Get Command                                                     */
/* Purpose:  GET commands are used to query current settings and data from   */
/*           the instrument like center frequency, marker, trace data, etc.  */
/*===========================================================================*/
ViStatus rsfsh_getCommand (ViSession instrSession, ViString getCommand, ViChar value[])
{
    ViStatus    rsfsh_status = VI_SUCCESS;

    CHECKERR(rsfsh_writeInstrData (instrSession, "GET"));
    CHECKERR(rsfsh_writeInstrData (instrSession, getCommand));
    CHECKERR(viScanf (instrSession, "%[^\r\n]", value));

    return rsfsh_status;
}

/*****************************************************************************
 * Function: Delay
 * Purpose:  This function suspends the program for the selected number of 
 *           second.
 *****************************************************************************/
ViStatus rsfsh_delay(
    ViSession   instrSession,
    ViReal64    seconds
)
{
  clock_t StartTime = clock();
  instrSession;
  
  while ((((double)(clock() - StartTime))/CLOCKS_PER_SEC) < (seconds)){
#ifdef _WIN32
  #ifdef _CVI_
          ProcessSystemEvents ();
  #else
          Sleep(0);
  #endif
#endif
  }
  return VI_SUCCESS;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
